package devisor2.foundation.base;

import devisor2.foundation.*;
import devisor2.foundation.elements.*;
import devisor2.grid.GUI.framework.*;
import devisor2.foundation.boundary.*;
import devisor2.grid.options.*;
import java.util.Vector;
import java.util.StringTokenizer;
import java.util.Enumeration;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;

import java.io.*;


/**
 *  This is the first class in a long line of the Domain class hierarchy.
 *  Here, all basic data structures which hold the data of the domain are
 *  defined, and access methods are provided: For everything that can be
 *  included in a domain, numerous ADD and DELETE methods are available.
 *
 *  @author Christian Becker, Dominik Goeddeke
 *  @version bugfixing, but seems quite ok
 */	
public abstract class BasicDomain implements Serializable
{

public class DPoint
{
  double dx,dy;

  public DPoint()
  {
    dx=0.0;
    dy=0.0;
  }

  public DPoint(double udx, double udy)
  {
    dx=udx;
    dy=udy;
  }

  public void set(double udx, double udy)
  {
    dx=udx;
    dy=udy;
  }

  public double getX()
  {
    return dx;
  }

  public double getY()
  {
    return dy;
  }

}
    protected ControlCenter cc;

    /**
     * flag to indicate that the domain was changed
     */
    protected boolean changed;

    /**
     * number of Boundary Edges in the domain
     */
    public int BoundaryEdgeCount;

    /**
     *  number of Boundary Nodes in the domain
     */
    public int BoundaryNodeCount;

    /**
     * the nodes and cells themselves
     */	
    public Vector nodes;
    public Vector edges;
    public Vector cells;
    
    /**
     * number of values in the domain
     */
    public int NodeValueCount;
    public int CellValueCount;
    
    /**
     * the boundary of the domain
     */	
    public DomainBoundary db;
    
    
    /**
     * name of the domain
     */	
    public String name;
    
    /**
     * filename of the domain
     */ 
    protected String domainfilename;

    /**
     * fileformat of the domain (use constants provided)
     */
    protected String domainfileformat;
    

    /**
     * perimter of the domain
     */
    protected int[] perimeter;


int[] step;
int[] step1;
int[] step2;
public int[] ecke1;
int[] ecke2;
int[] roffs;
public int[] validpx;
public int[] validpy;

    
/*********************************************************/


    /**
     * creates a new domain with default values for everything
     */	
    public BasicDomain()
    {
	cc = ControlCenter.getMyself ();

	domainfilename = "noname";
	domainfileformat = "feat";
    	
	cells = new Vector (512,512);		
	nodes = new Vector (512,512);
	edges = new Vector (512,512);
      
	db = new DomainBoundary (this);
      
	perimeter = new int[4];
	
	changed=false;
    }

    /**
     * resets the cell markers to a value of 0
     */
    public void resetCellMarkers()
    {
	Enumeration enum = cells.elements ();
	while (enum.hasMoreElements ())
	    ((Cell)enum.nextElement()).setMarker(0);
    }
  
  
    /**
     * resets the edge markers to a value of 0
     */
    public void resetEdgeMarkers()
    {
	Enumeration enum = edges.elements ();
	while (enum.hasMoreElements ())
	    ((Edge)enum.nextElement()).setMarker(0);
    }
  
  
    /**
     * resets the node markers to a value of 0
     */
    public void resetNodeMarkers()
    {
	Enumeration enum = nodes.elements ();
	while (enum.hasMoreElements ())
	    ((Node)enum.nextElement()).setMarker(0);
    }
  
  
/*********************************************************/

  public void construct_boundary()
  {
	Enumeration enum = cells.elements ();
        int maxbn = 0;
	
	while (enum.hasMoreElements ())
	{
	    Cell c = ((Cell)enum.nextElement());	

	    Node[] n = c.getNodes();
	    
	    for(int idx=0;idx<4;idx++)
	    {
	      if (n[idx].getNPR()>maxbn) maxbn=n[idx].getNPR();
	    }
	 }
	      

int reteps1=cc.dc.tolerance;
cc.dc.tolerance=1;
	      	    
    for(int bn=1;bn<=maxbn;bn++)
    {
	 enum = cells.elements ();
		
	db.addBoundary();
	
	Boundary b = db.getBoundary(bn-1);
	
	while (enum.hasMoreElements ())
	{
	    Cell c = ((Cell)enum.nextElement());	
	    
	    Node[] n = c.getNodes();
	    
	    for(int idx=0;idx<4;idx++)
	    {
	      if ((n[idx].getNPR()==bn) && (n[(idx+1)%4].getNPR()==bn))
	      {
	//        System.out.println("SEG from "+n[idx].getNumber()+" to "+
	//	n[(idx+1)%4].getNumber());
		
		Node n1 = n[idx];
		Node n2 = n[(idx+1)%4];
		
		SegmentLine s = new SegmentLine(n1.getCoord(0),n1.getCoord(1),
		n2.getCoord(0),n2.getCoord(1),SegmentLine.TWOPOINT);
		
		s.updateInnerData ();
		db.addSegment(b,s);
	      }
	    }
	}
    }
    
                double epsi = 0.0005;
		
		int reteps=cc.gc.eps;
            cc.gc.eps = (int)(epsi*(cc.dc.world_x2 - cc.dc.world_x1));

db.continousBoundary();

cc.gc.eps=reteps;



	 int l  = nodes.size();
	 
         for (int i =0;i<l;i++)
	 {
	    Node n = ((Node)nodes.elementAt(i));
	    
	    if (n.getNPR()!=0)
	    {
	       BoundaryNode bn = new BoundaryNode();
	       
	       bn.setCoord(n.getCoord(0),n.getCoord(1));
	       bn.parents=n.parents;
	       
	       bn=db.reconfigureBoundaryNode(bn);
	       
	       
		       if (bn==null) System.out.println("NULLERROR "+i);
	       
	       
	       if (bn!=null) nodes.set(i,bn);
	     }
	 }	 

      cc.dc.tolerance=reteps1;
     
db.updateNumbering();
  }
  
	    
/*********************************************************/

  public int[] getNearestCells(int ipb, int izpb, DPoint mp)
  {
  
    int cc = 0;
    
    Enumeration enum=cells.elements();
    
    while(enum.hasMoreElements())
    {
      Cell c = (Cell)enum.nextElement();
      
      if (c.getParallelBlock()==ipb) cc++;
    }
    
    int[] list = new int[cc];
    double[] dist = new double[cc];
    
    cc=0;
    
    updateCellNeighbours();

    enum=cells.elements();
    
    while(enum.hasMoreElements())
    {
      Cell c = (Cell)enum.nextElement();
      
      int s = (c.getNeighbours()).length;
      
      if ((c.getParallelBlock()==ipb) && (c.getMarker()==0))
      {
        list[cc]=cells.indexOf(c);
	
	Node[] n= ((Cell2D)c).getNodes();
	double mpx=(n[0].getCoord(0)+n[1].getCoord(0)+n[2].getCoord(0)
	  +n[3].getCoord(0))/4.0;
        double mpy=(n[0].getCoord(1)+n[1].getCoord(1)+n[2].getCoord(1)
	  +n[3].getCoord(1))/4.0;
	

        dist[cc]=Math.sqrt((mpx-mp.getX())*(mpx-mp.getX())
          +(mpy-mp.getY())*(mpy-mp.getY()));
	
	boolean isn = false;
	
	for(int k=0;k<s;k++) 
	  if ((c.getNeighbours())[k].getParallelBlock()==izpb) isn=true;
	  
	if (isn) dist[cc]=dist[cc]/10.0;
	
	cc++;
       }
      }
      
      boolean tch = true;
      
      for(;tch;)
      {
        tch=false;
        for (int i=0;i<cc-1;i++)
	  if (dist[i]>dist[i+1])
	  {
	    double tmp=dist[i]; dist[i]=dist[i+1]; dist[i+1]=tmp;
	    int tmp1=list[i]; list[i]=list[i+1]; list[i+1]=tmp1;
	    tch=true;
	  }
       }
       
      return list;
    }  
    
          
  public DPoint getParallelBlockCenter(int ipb)
  {
    double[] mpx = new double[cells.size()];
    double[] mpy = new double[cells.size()];
    int idx = 0;
    
    Enumeration enum=cells.elements();
    
    while (enum.hasMoreElements())
    {
      Cell c = (Cell)enum.nextElement();
      
      if (c.getParallelBlock()==ipb)
      {
        Node[] n= ((Cell2D)c).getNodes();
	mpx[idx]=(n[0].getCoord(0)+n[1].getCoord(0)+n[2].getCoord(0)
	  +n[3].getCoord(0))/4.0;
        mpy[idx]=(n[0].getCoord(1)+n[1].getCoord(1)+n[2].getCoord(1)
	  +n[3].getCoord(1))/4.0;
	idx++;
      }
      
     }
     
       double mx = 0.0;
       double my = 0.0;
       
       for (int index=0; index<idx; index++)
       {
         mx=mx+mpx[index];
	 my=my+mpy[index];
       }     
     
     return (new DPoint(mx/(double)idx,my/(double)idx));
  }


    public void rectifyElement (Node n1, Node n2, Node n3, Node n4)
    /**    
     *  for four given nodes n1,n2,n3,n4 defining the corners
     *  of a quad or macro element, the element is reshaped to a perfect 
     *  rectangle as follows:
     *  n1 .. n4 must be given in correct order according to their orientation
     *  n1 and n2 define one side of the element
     *  n3 is then projected on the orthogonal of the base side
     *  through n2
     *  the missing node n4 is calculated moved accordingly
     *  (i.e. projected on orthogonal through n1)
     */
    {
       	// copy to locals to implove performance
	double a_x = n1.getCoord(0);
       	double a_y = n1.getCoord(1);
        double b_x = n2.getCoord(0);
	double b_y = n2.getCoord(1);
	double c_x = n3.getCoord(0);
	double c_y = n3.getCoord(1);
	// calculate orthogonal vector to vector through n1n2
	double N = (a_y-b_y)*(a_y-b_y) + (b_x-a_x)*(b_x-a_x);
	// calculate scalar product
	double S = (a_y-b_y)*(c_x-a_x) + (b_x-a_x)*(c_y-a_y);
	// and the relative coordinates of the projection
	double p_x = S*(a_y-b_y)/N;
	double p_y = S*(b_x-a_x)/N;
	
	// now project accordingly
	n3.setCoord((int)(p_x+b_x),(int)(p_y+b_y));
	n4.setCoord((int)(p_x+a_x),(int)(p_y+a_y));
	// System.out.println ("Element rectified...");
    }
    
      
  /**
   * returns the maximum number of parallel blocks
   */
  public int getMaxParallelBlock()
  {
    int machs=0;

    Enumeration enum=cells.elements();
    while (enum.hasMoreElements())
    {
      Cell c = (Cell)enum.nextElement();
      
      if (c.getParallelBlock()>machs) 
        machs=c.getParallelBlock();	
    }
    		
    return machs;
  }
    
    /**
     * sets the node numbers according to their index in the
     * node vector
     */
    public void updateNodeNumbers()
    {
	Enumeration enum = nodes.elements ();
	int i=0;
	while (enum.hasMoreElements ())
	{
	    ((Node)enum.nextElement()).setNumber(i);
	    i++;
	}
    }
    

    /**
     * sets the cell numbers according to their index in the
     * cell vector
     */
    public void updateEdgeNumbers()
    {
	Enumeration enum = edges.elements ();
	int i=0;
	while (enum.hasMoreElements ())
	{
	    ((Edge)enum.nextElement()).setNumber(i);
	    i++;
	}
    }
    
    
    /**
     * sets the cell numbers according to their index in the
     * cell vector
     */
    public void updateCellNumbers()
    {
	Enumeration enum = cells.elements ();
	int i=0;
	while (enum.hasMoreElements ())
	{
	    ((Cell)enum.nextElement()).setNumber(i);
	    i++;
	}
    }
    
    
    /**
     * sets the nodes in the cells according to their edges
     */
    public void updateCellNodes()
    {
	Enumeration e = cells.elements();
	while (e.hasMoreElements()) 
	{
	    ((Cell)(e.nextElement())).updateCellNodes();
	}
    }
    
    
    /**
     * sets the nodes in the cells according to their edges
     */
    public void updateCellNeighbours()
    {
	Enumeration e = cells.elements();
	while (e.hasMoreElements()) 
	{
	    ((Cell)(e.nextElement())).updateCellNeighbours();
	}
    }
    
    
    /**
     * update all numbers
     */ 
    public void updateAll()
    {
	updateCellNumbers();
	updateNodeNumbers();
	updateEdgeNumbers();
    }
    
/*********************************************************/
    
    
    /**
     * returns the number of nodes in the domain
     * @return number of nodes
     */
    public int getNodeCount()
    {
	return nodes.size();
    }
    
    
    /**
     * returns the number of boundary nodes in the domain
     * @return number of boundary nodes
     */  
    public int getBoundaryNodeCount()
    {
	Enumeration enum = nodes.elements ();
	int i=0;
	while (enum.hasMoreElements ())
	{
	    if (enum.nextElement() instanceof BoundaryNode)
		i++;
	}
	return i;
    }
    
  
    /**
     * returns the number of cells in the domain
     * @return number of cells
     */
    public int getCellCount()
    {
	return cells.size();
    }
    
    
    /**
     * returns the number of edges in the domain
     * @return number of edges
     */
    public int getEdgeCount()
    {
	return edges.size();
    }

    public Vector getNodes ()
    {
	return nodes;
    }

    public Vector getEdges ()
    {
	return edges;
    }

    public Vector getCells ()
    {
	return cells;
    }

    public int getBoundaryEdgeCount ()
    {
	return BoundaryEdgeCount;
    }
    /**
     *  @return null if no such node exists, the node with the given number otherwise
     */
    public Node getNode (int number)
    {
	if (number <= getNodeCount ())
	{
	    return (Node)nodes.elementAt (number);
	}
	else
	    return null;
    }


    /**
     *  @return null if no such edge exists, the edge with the given number otherwise
     */
    public Edge getEdge(int number)
    {
	if (number <= getEdgeCount())
	{
	    return (Edge)edges.elementAt(number);
	}
	else
	    return null;
    }
    

    /**
     *  @return null if no such edge exists, the edge with the given number otherwise
     */
    public Cell getCell(int number)
    {
	if (number <= getCellCount())
	{
	    return (Cell)cells.elementAt(number);
	}
	else
	    return null;
    }        
/*********************************************************/
    
    
    /**
     *  adds the given GridItem to the domain's corresponding vectors
     *  every time appending at the end (for segments, the working boundary
     *  is used
     *  @param item - a node, cell, edge, or segment
     */
    public void add (GridItem item)
    {
	if (item instanceof Node)
	    addNode ((Node)item);
	if (item instanceof Edge)
	    addEdge ((Edge)item);
	if (item instanceof Cell2D)
	    addCell ((Cell2D)item);
    }


//      /**
//       * adds a node/cell at a specific position to the domain
//       * @param cell/nodenumber number of the cell/node 
//       * @param n element to be inserted
//       */	
//      public Node addNode(int nodenumber, Node n)
//      {
//  	nodes.insertElementAt(n,nodenumber);
//  	NodeCount++;
	
//  	changed=true;
	
//  	return n;
//      }
    
//      public Node addNode(int nodenumber, Node n,int i)
//      {
//  	nodes.insertElementAt(n,nodenumber);
//  	NodeCount++;
	
//  	changed=true;
	
//  	return n;
//      }

    /**
     * adds a node/cell to the domain and checks for its existance
     * @param n element to be inserted
     */	  
    public Node adjustNode(Node neu)
    {
	Node n = getNodeWithSameCoords (neu);
	if (n == null)
	{
	    addNode(neu);
	    changed = true;
	    return neu;
	}
	else
	{
	    return n;
	}
    }
    
    
//      /**
//       * adds a cell at a specific position to the domain
//       * @param cellnumber number of the cell/node 
//       * the segment should be inserted
//       * @param n element to be inserted
//       */
//      public Cell addCell(int cellnumber, Cell n)
//      {
//  	cells.insertElementAt(n,cellnumber);
//  	CellCount++;
//  	n.updateCellNodes();
//  	changed=true;
	
//  	return n;
//      }
    
  
//      /**
//       * adds a edge at a spezific position to the domain
//       * @param edgenumber number of the cell/node 
//       * the segment should be inserted
//       * @param n element to be inserted
//       */
//      public Edge addEdge(int edgenumber, Edge e)
//      {
//  	edges.insertElementAt(e,edgenumber);
//  	EdgeCount++;
//  	changed=true;
	
//  	return e;
//      }
    
    
//      /**
//       * adds an edge consisting of nodes at index n1,n2 in the nodes
//       * vector  (1 == first index)
//       */	
//      public Edge addEdge(int n1, int n2)
//      {
//  	Edge e = new Edge((Node)nodes.elementAt(n1-1),
//  			  (Node)nodes.elementAt(n2-1));
	
//  	edges.insertElementAt(e,0);
//  	EdgeCount++;
	
//  	changed=true;
	
//  	return e;
//      }
    
    
    /**
     * adds a edge to the domain and checks for its existance
     * @param a,b edge definition to be inserted
     */
    public Edge adjustEdge(Node a, Node b)
    {
	Edge e = searchEdge(a,b);
	if (e == null)
	{
	    e = new Edge(a,b);
	    addEdge(e);
	    changed = true;
	}
	return e;
    }

    /**
     * adds a edge to the domain and checks for its existance
     * @param a,b edge definition to be inserted
     */
    public Edge adjustEdge(Edge e)
    {
	if (!edges.contains (e))
	{
	    addEdge(e);
	    changed = true;
	}
	return e;
    }
    
    
    /**
     * adds a node/cell at the end of the according list to the domain
     * @param n element to be inserted
     */		
    public Node addNode(Node n)
    {
	nodes.add (n);
	changed = true;
	return n;
    }
    
//      public Node addNode(Node n, int i)
//      {
//  	return addNode(NodeCount,n,i);
//      }
    
    /**
     * adds a cell at the end of the according list to the domain
     * @param n element to be inserted
     */
    public Cell addCell(Cell n)
    {
	cells.addElement (n);
	n.updateCellNodes();
	changed=true;
	return n;
    }
    
    
    /**
     * adds a edge at the end of the according list to the domain
     * @param n element to be inserted
     */
    public Edge addEdge(Edge e)
    {
	edges.addElement (e);
	changed = true;
	return e;
    }	
    
    
    /**
     * sets the name
     * of this domain
     */
    public void setName(String s)
    {
	name = s;
    }
    
    /**
     *  removes the given GridItem from the domain
     */
    public void del (GridItem item)
    {
	if (item instanceof Node)
	    delNode ((Node)item);
	if (item instanceof Cell2D)
	    delCell ((Cell2D)item);
	if (item instanceof Edge)
	    delEdge ((Edge)item);
	if (item instanceof Segment)
	{
	    Segment s = (Segment)item;
	    db.delSegment (s);
	}
    }
    
    
    /**
     * deletes an node element of the domain
     * @param n element to be deleted
     */
    public void delNode(Node n)
    {
	if (nodes.contains (n))
	    if (n.getParents()!=null)
	    {
		Edge[] adges = n.getParents();
		for(int e=0;e<adges.length;e++)
		{
		    if (adges[e]!=null)
		    {
			Cell[] zells = (Cell[])adges[e].getParents();
			if (zells!=null)
			{
			    for(int c=0;c<zells.length;c++)
			    {
				if (zells[c]!=null) 
				{
				    adges[e].removeParent(zells[c]);
				    
				    for(int k=0;k<zells[c].getNPT();k++)
				    {
					((Cell2D)zells[c]).getEdge(k).removeParent(zells[c]);
				    }
				    cells.removeElement(zells[c]);
				}
			    }
			}
			edges.removeElement(adges[e]);
		    }
		}				
		
	    }  						
	nodes.removeElement(n);
	changed=true;
    }


    /**
     * deletes an cell element of the domain, and recursively all afterwards
     *  isolated edges
     * @param c element to be deleted
     */
    public void delCell(Cell c)
    {
	if (cells.contains (c))
	    for(int k=0;k<c.getNPT();k++)
	    {
		Edge e = ((Cell2D)c).getEdge(k);
		e.removeParent(c);
		if (e.getParents() == null)
		    delEdge (e);
	    }
	cells.removeElement(c);
	changed=true;
    }

    /**
     *  deletes an edge of the domain, but WITHOUT deleting its nodes,
     *  which will remain isolated afterwards. If the cell does not exist,
     *  nothing happens
     *
     *  @param e - edge to be deleted
     */
    public void delEdge (Edge e)
    {
	if (edges.contains (e))
        {
	    Node n1 = e.getNode (0);
	    Node n2 = e.getNode (1);
	    n1.removeParent (e);
	    n2.removeParent(e);
	    edges.removeElement (e);
	    changed = true;
	}
    }


    /**
     * gets the name of the domain
     * @return String
     */
    public String getName()
    {
	return name;
    }
    
    
//      /**
//       * gets an node element of the domain
//       * @param cell-/nodenumber position of the segment
//       * @return segment
//       */
//      public Node getNode(int nodenumber)
//      {
//  	return ((Node)nodes.elementAt(nodenumber));
//      }
    
    
//      /**
//       * gets an cell element of the domain
//       * @param cell-/nodenumber position of the segment
//       * @return segment
//       */
//      public Cell getCell(int cellnumber)
//      {
//  	return ((Cell)cells.elementAt(cellnumber));
//      }
    
    
//      /**
//       * gets an edge element of the domain
//       * @param cell-/nodenumber position of the segment
//       * @return segment
//       */
//      public Edge getEdge(int edgenumber)
//      {
//  	return ((Edge)edges.elementAt(edgenumber));
//      }
    
    
//      public double getEps()
//      {
	
//  	//    double[] ska = {dc.snapx,dc.snapy};
//  	//    double[] nska = ((Transformation2D)tf).rescale(ska);
	
//  	double eps=0.1;
	
//  	//    if (nska[0]<nska[1])
//  	//      eps=nska[0];
//  	//    else
//  	//      eps=nska[1];
	
//  	//System.out.println("getEps: "+dc.snapx+" "+dc.snapy+" ==> "+eps);
	
//  	return eps;
	
    //    }
    
    
    /**
     * gets the edge the node n resides on
     */ 
    public Edge getEdgeForNode(Node n, double eps)
    {
	boolean found=false;
	Edge temp;
	Enumeration enum = edges.elements ();
	while (enum.hasMoreElements ())
	{
	    temp=(Edge)enum.nextElement ();
	    found=temp.isOnEdge(n,eps);
	    if (found)
		return temp;
	}
	return null;
    }
    
    
    /**
     * gets the Node in the Domain with the same coords
     * @param n node to be searched
     * @return the node with the same coords, 
     *         null if such a node does not exist
     */
    public Node getNodeWithSameCoords(Node n)
    {
	boolean found=false;
	int i;
	int eps = 20;
	Enumeration enum = nodes.elements ();
	while (enum.hasMoreElements ())
	{
	    Node temp = (Node)enum.nextElement ();
	    found=(Math.abs(temp.getCoord(0)-n.getCoord(0))<eps) && (Math.abs(temp.getCoord(1)-n.getCoord(1))<eps);
	    if (found)
		return temp;
	}
	return null;    
    }
    
    
    /**
     * searches for an edge with nodes a and b. If so an edge exists
     * the reference is returned else null
     */
    public Edge searchEdge(Node a, Node b)
    {
	
	Edge[] temp = a.getParents();
	
	if (temp!=null)
	    for(int i=0; i<temp.length; i++)
	    {
		if (((a==temp[i].getNode(0)) && (b==temp[i].getNode(1))) ||
		    ((b==temp[i].getNode(0)) && (a==temp[i].getNode(1))))
		    {
			return temp[i];
		    }
	    }
	return null;
    }
    
    
  //    /**
//       * gets the index of  an cell of the domain
//       * @param temp cell to be searched
//       * @return index of the cell
//       */    
//      public int getCellIndex(Node temp)
//      {
//  	boolean found=false;
//  	int i;
//  	Polygon p ;
//  	double skal=100000;
	
//  	for(i=0;i<CellCount;i++)
//  	{
//  	    Cell2D v = (Cell2D)cells.elementAt(i);
	    
//  	    v.updateCellNodes();
	    
//  	    p = new Polygon();
//  	    p.addPoint((int)(v.getNodes()[0].getCoord(0)*skal),(int)(skal*v.getNodes()[0].getCoord(1)));
//  	    p.addPoint((int)(v.getNodes()[1].getCoord(0)*skal),(int)(skal*v.getNodes()[1].getCoord(1)));
//  	    p.addPoint((int)(v.getNodes()[2].getCoord(0)*skal),(int)(skal*v.getNodes()[2].getCoord(1)));
//  	    p.addPoint((int)(v.getNodes()[3].getCoord(0)*skal),(int)(skal*v.getNodes()[3].getCoord(1)));
	    
//  	    found=p.contains((int)(temp.getCoord(0)*skal),(int)(skal*temp.getCoord(1)));
	    
//  	    if (found) break;
//  	}
	
//  	if (found)
//  	    return i;
//  	else
//  	    return -1;
//      }
    
    
  public void drawGrid(Graphics g)
  {
    int x,y;
    double dx,dy;
      
    g.setColor(Color.lightGray);
       
int[] tmp=new int[2];
int[] tmp1;

int[] offs;

if (ecke1==null)
{
      step = new int[2];
step1 = new int[2];
step2 = new int[2];
step[0] = cc.gc.SingleFromScreenToWorld(4*cc.gc.vsnapx);
step[1] = cc.gc.SingleFromScreenToWorld(8);
step1[0] = cc.gc.SingleFromScreenToWorld(4);
step1[1] = cc.gc.SingleFromScreenToWorld(4);
step2[1] = cc.gc.SingleFromScreenToWorld(4*cc.gc.vsnapy);
step2[0] = cc.gc.SingleFromScreenToWorld(8);  



ecke1 = cc.gc.fromScreenToWorld(0,cc.dc.screen_height);
ecke2 = cc.gc.fromScreenToWorld(cc.dc.screen_width,0);

step[0]=cc.gc.fromUserToWorld(0.2);
step2[1]=cc.gc.fromUserToWorld(0.2);
}

int b1=ecke1[0];
int b2=ecke1[1];
ecke1[0]=cc.dc.world_x1;
ecke1[1]=cc.dc.world_y1;

while(ecke1[0]>b1) ecke1[0]=ecke1[0]-step[0];
while(ecke1[1]>b2) ecke1[1]=ecke1[1]-step2[1];

      for(x=ecke1[0];x<ecke2[0];x+=step[0])
        for(y=ecke1[1];y<ecke2[1];y+=step[1])
	{
	  tmp=cc.gc.fromWorldToScreen(x,y);
	  tmp1=cc.gc.fromWorldToScreen(x,y+step[1]/2);
	  
	  g.drawLine(tmp[0],tmp[1],tmp1[0],tmp1[1]);
	  
        }
	


      for(y=ecke1[1];y<ecke2[1];y+=step2[1])
        for(x=ecke1[0];x<ecke2[0];x+=step2[0])
	{
	  tmp=cc.gc.fromWorldToScreen(x,y);
	  tmp1=cc.gc.fromWorldToScreen(x+step2[0]/2,y);

          g.drawLine(tmp[0],tmp[1],tmp1[0],tmp1[1]);
	    

	}


validpx = new int[6132];   // ich weiss, ist nicht die feine englisch ....
validpy = new int[6132];
int vpx=0;

      y=0;
      for(x=ecke1[0];x<ecke2[0];x+=step[0]/10)
      {
	  tmp=cc.gc.fromWorldToScreen(x,y);
	  
	  if ((tmp[0]>=0)&&(tmp[0]<=cc.dc.screen_width)) 
	    validpx[vpx++]=tmp[0];
      }

      x=0;	
      vpx=0;
      for(y=ecke1[1];y<ecke2[1];y+=step2[1]/10)
      {
	  tmp=cc.gc.fromWorldToScreen(x,y);
	  
	  if ((tmp[1]>=0)&&(tmp[1]<=cc.dc.screen_height)) 
	    validpy[vpx++]=tmp[1];
      }
      
      
  }
      
    /**
     * draws the whole domain
     * @param g graphics context
     * @return none
     */	
    public void draw(Graphics g)
    {
	Enumeration enum;
	if (cc.dc.draw_grid) 
	{
	    drawGrid(g);
	}
	
	if (cc.dc.draw_boundary)
	    db.draw(g); 
            
	// Cells : the cell decides itself how to be drawn according
	// to the given DrawControl object.
	// Must be drawn before nodes
	// otherwise node may be hidden.
	if (cc.dc.draw_element)
	{
	    enum = cells.elements ();
	    while (enum.hasMoreElements ())
		((Cell)enum.nextElement()).draw(g);
	}
	// Nodes
	if (cc.dc.draw_node)
	{
	    enum = nodes.elements ();
	    while (enum.hasMoreElements ())
		((Node)enum.nextElement()).draw(g);
	}
	// Edges
	if (cc.dc.draw_edge)
	{
	    enum = edges.elements ();
	    while (enum.hasMoreElements ())
		((Edge)enum.nextElement()).draw(g);
	}
	if (cc.dc.fill_element)
	{   
	    enum = edges.elements ();
	    while (enum.hasMoreElements ())
		((Edge)enum.nextElement()).fill(g);
	}




    if (cc.dc.draw_perimeter)
    {

      int[] px1=cc.gc.fromWorldToScreen(cc.dc.world_x1,cc.dc.world_y1);
      int[] px2=cc.gc.fromWorldToScreen(cc.dc.world_x2,cc.dc.world_y2);
   
      g.setColor(Color.GREEN);
      g.drawLine(px1[0],px1[1],px2[0],px1[1]);
      g.drawLine(px2[0],px1[1],px2[0],px2[1]);
      g.drawLine(px2[0],px2[1],px1[0],px2[1]);
      g.drawLine(px1[0],px2[1],px1[0],px1[1]);
   
    }
   
  }

  


  /**
   * updates all boundary nodes of the domain
   * @return none
   */	
//  public void updateBoundaryNode()
//  {
//    int i;
//      
//    for(i=0; i<NodeCount; i++)
//      if (getNode(i) instanceof BoundaryNode ) 
 //       ((BoundaryNode)getNode(i)).updateBoundaryNode(db);	    
//
//  }
  
  
    /**
     * returns the perimeter of the domain
     * @return double[4] erg (xmin,ymin,xmax,ymax)
     */	
    public int[] getPerimeter()
    {
	return getPerimeter(false);
    }
    
    
    public int[] getPerimeter(boolean trf)
    {
	int[] erg;
	int index;
	Node n;
	int x,y,z;
	
	      erg=db.getPerimeter();
	// erg=new int[4];
      	
	//    updateBoundaryNode();
	
	Enumeration enum = nodes.elements ();
	while (enum.hasMoreElements ())
	{
	    n=(Node)enum.nextElement ();
	    
	    if (!trf)
	    {
		x=n.getCoord(0); y=n.getCoord(1); 
	    }
	    else
	    {

		System.out.println("getPerimeter(true) yet not implemented");
		x=n.getCoord(0); y=n.getCoord(1);
		double[] dummy = {x,y};
		//          double[] ergg=tf.transform(dummy);
		int[] ergg=new int[2];
		x=ergg[0];
		y=ergg[1];
		
	    }
	    
	    if (x<erg[0]) erg[0]=x;
	    if (y<erg[1]) erg[1]=y;
	    if (x>erg[2]) erg[2]=x;
	    if (y>erg[3]) erg[3]=y;
	    	    
	}
	
        if (cc.dc.world_x1<erg[0]) erg[0]=cc.dc.world_x1;
        if (cc.dc.world_x2>erg[2]) erg[2]=cc.dc.world_x2;
        if (cc.dc.world_y1<erg[1]) erg[1]=cc.dc.world_y1;
        if (cc.dc.world_y2>erg[3]) erg[3]=cc.dc.world_y2;
	
	return erg;
    }
    
    
    /**
     * scales a domain to adapt the grid size to the gui frame size
     */	
    public void centerDomain()
    {
	centerDomain(true);
    }
    
    
    public void setDomainPerimeter(int[] p)
    {
	perimeter=p;
    }
    
    
    public void centerDomain(boolean modus)
    {
	if (modus) perimeter = getPerimeter();
	
	int xlen=Math.abs(perimeter[0])+Math.abs(perimeter[2]);
	int ylen=Math.abs(perimeter[1])+Math.abs(perimeter[3]);
	
	int mx = (perimeter[2]+perimeter[0])/2;
	int my = (perimeter[3]+perimeter[1])/2;
	
        cc.dc.act_x1=perimeter[0]-xlen/10;
        cc.dc.act_y1=perimeter[1]-ylen/10;

        cc.dc.act_x2=perimeter[2]+xlen/10;
        cc.dc.act_y2=perimeter[3]+ylen/10;

        cc.gc.setTransformation();

step = new int[2];
step1 = new int[2];
step2 = new int[2];
step[0] = cc.gc.SingleFromScreenToWorld(4*cc.gc.vsnapx);
step[1] = cc.gc.SingleFromScreenToWorld(8);
step1[0] = cc.gc.SingleFromScreenToWorld(4);
step1[1] = cc.gc.SingleFromScreenToWorld(4);
step2[1] = cc.gc.SingleFromScreenToWorld(4*cc.gc.vsnapy);
step2[0] = cc.gc.SingleFromScreenToWorld(8);  

step[0]=cc.gc.fromUserToWorld(0.2);
step2[1]=cc.gc.fromUserToWorld(0.2);



ecke1 = cc.gc.fromScreenToWorld(0,cc.dc.screen_height);
ecke2 = cc.gc.fromScreenToWorld(cc.dc.screen_width,0);


            	
    }
    
    public void centerDomain(Rectangle box)
    {
//	if (modus) perimeter = getPerimeter();
	
	int[] perimeter = new int[4];
	
//	perimeter[0]=(int)(box.x/step[0])*step[0];

	perimeter[0]=box.x;
	perimeter[1]=box.y-box.height;
	perimeter[2]=box.x+box.width;
	perimeter[3]=box.y;
		
	int xlen=Math.abs(perimeter[0])+Math.abs(perimeter[2]);
	int ylen=Math.abs(perimeter[1])+Math.abs(perimeter[3]);
	
	int mx = (perimeter[2]+perimeter[0])/2;
	int my = (perimeter[3]+perimeter[1])/2;
	
        cc.dc.act_x1=((int)(perimeter[0]*cc.globalscale2))/cc.globalscale2;  
        cc.dc.act_y1=((int)(perimeter[1]*cc.globalscale2))/cc.globalscale2;

        cc.dc.act_x2=((int)(perimeter[2]*cc.globalscale2))/cc.globalscale2;
        cc.dc.act_y2=((int)(perimeter[3]*cc.globalscale2))/cc.globalscale2;

        cc.gc.setTransformation();


    }    
    /**
     *  Performs a check if the domain is containing elements only supported 
     *  by the FEAST file format.
     *
     *  @return true if the domain needs FEAST format to be saved correctly,
     *          false otherwise
     */
    public boolean isFeastable ()
    {
	Enumeration elemenum = cells.elements ();
	while (elemenum.hasMoreElements ())
	{
	    if (elemenum.nextElement () instanceof Macro)
		return true;
	}
	return false;
    } 


    /**
     *  returns the filename of the domain
     */
    public String getFilename()
    {
	return domainfilename;
    }
    
    /**
     *  returns the file format of the domain (UCD, FEAT or FEAST)
     */
    public String getFileformat ()
    {
	return domainfileformat;
    }


    /**
     *  returns if the domain has been changed recently
     */
    public boolean hasChanged ()
    {
	return changed;
    }

 
    /** constant for the UCD file format */
    public static final String UCD = "UCD";

    /** constant for the FEAT file format */
    public static final String FEAT = "FEAT";

    /** constant for the FEAST file format */
    public static final String FEAST = "FEAST";



  
}
