package devisor2.foundation.base;

import javax.swing.*;
import javax.swing.event.*;

import devisor2.foundation.elements.*;
import devisor2.foundation.boundary.*;
import devisor2.grid.options.*;

import java.io.*;
import java.util.*;



/**
 *  This class is an extended version of the original BasicDomain class
 *  which supports loading and saving the domain instance in one of 
 *  the following formats: UCD, FEAT or FEAST. <br>
 *  See the MainActionListener for a description of how this class
 *  is used in contrast to the original Domain class.
 *
 *  @version FEAT+FEAST
 *  @author Christian Becker
 */
public abstract class DomainIO extends DomainStatistics
{

    ControlCenter cc = ControlCenter.getMyself ();
    
    /**
     *  This method handles loading a domain. Basically, it performs
     *  some checks and verifies the existence of the specified file.
     *  
     *  @param fileName - the full qualified path to the domain file
     *  @param format - one of the constants UCD, FEAT or FEAST
     *  @param para - flag to indicate if the prm file should be loaded
     *  @param grid - flag to indicate that the tri file should be loaded
     */
    public int loadDomain(String fileName, String format, boolean para, boolean grid) 
        {
    	try 
    	{
    	    FileReader readFile = new FileReader(fileName);
    	    BufferedReader in = new BufferedReader(readFile);

    	    domainfilename=fileName;
    	    domainfileformat=format;

    	    if ( format.equals (UCD) ) 
    	    {
    	//	loadDomainUCD(in);
    		updateCellNumbers();
    		updateNodeNumbers();
    		return 0;
    	    }

    	    if ( format.equals("prm") || format.equals("tri")) 
    	    {
    		in.close();

    		loadDomainFEAT(fileName, para, grid,false,0.0,0.0);

    	    domainfilename=fileName;
    		domainfileformat=format;

    		updateCellNumbers();
    		updateNodeNumbers(); 
    		return 0;
    	    }

    	    if ( format.equals ("feast") ) 
    	    {
    		in.close();

    		loadDomainFEAST(fileName);

    	    domainfilename=fileName;
    		domainfileformat=format;

    		updateCellNumbers();
    		updateNodeNumbers();
    		return 0;
    	    }

//   // 	    double[] erg = getPerimeter (false);
//   // 	    getDrawControl().setRefPoint(erg);
    	}
    	catch (IOException e) 
    	{
    	    JOptionPane.showMessageDialog (null, "IO Error loading the Domain", "ERROR", JOptionPane.ERROR_MESSAGE);
    	    return -1;
    	}
    	catch ( SecurityException e ) 
    	{
    	    JOptionPane.showMessageDialog (null, "Security Error loading the Domain", "ERROR", JOptionPane.ERROR_MESSAGE);
    	    return -1;
    	}
    	JOptionPane.showMessageDialog (null, "input format not supported", "ERROR", JOptionPane.ERROR_MESSAGE);
    	return -1;
        }

        /**
         *  loads the given domain (convenience version)
         *
         *  @param fileName - full qualified path to the domain file(s)
         *  @param format - UCD, FEAT or FEAST
         */
        public int loadDomain(String fileName, String format) 
        {
    	try 
    	{
    	    FileReader readFile = new FileReader(fileName);
    	    BufferedReader in = new BufferedReader(readFile);
	    
  	    System.out.println("FORMAT "+format);
	    
    	    domainfilename=fileName;
    	    domainfileformat=format;
	    
    	    if ( format.equals(UCD) ) 
    	    {
    //		loadDomainUCD(in);
    		updateCellNumbers();
    		updateNodeNumbers();
    		return 0;
    	    }
    	    if ( format.equals("prm") || format.equals("tri")) 
    	    {
    		in.close();
	
    		loadDomainFEAT(fileName, true, true,false,0.0,0.0);
    		updateCellNumbers();
    		updateNodeNumbers();
    		return 0;
    	    }
    	    if ( format.equals("feast") ) 
    	    {
    		in.close();
    		loadDomainFEAST(fileName);
    		updateCellNumbers();
    		updateNodeNumbers();
    		return 0;
    	    }
    	//    double[] erg = getPerimeter(false);
    	    //   getDrawControl().setRefPoint(erg);
    	}
    	catch (IOException e) 
    	{
    	    JOptionPane.showMessageDialog (null, "IO Error loading the Domain", "ERROR", JOptionPane.ERROR_MESSAGE);
    	    return -1;
    	}
    	catch ( SecurityException e ) 
    	{
    	    JOptionPane.showMessageDialog (null, "Security Error loading the Domain", "ERROR", JOptionPane.ERROR_MESSAGE);
    		return -1;
    	}
  //  	JOptionPane.showMessageDialog (null, "Input format error loading the Domain", "ERROR", JOptionPane.ERROR_MESSAGE);
    	return 0;
        }
    
    

        /**
         *  Saves this domain with the specified filename and format.
         *
         *  @param fileName - full qualified path
         *  @param format one of the constants UCD, FEAT or FEAST
         *  @param check - flag to toggle the check for already existing files
         */
        public int saveDomain(String fileName, String format, boolean check) 
        {
    	domainfilename=fileName;
    	domainfileformat=format;
	
  	System.out.println("SAVEDOMAIN "+fileName);
	
    	if ( format.equals(UCD) ) 
    	{
    	    if (check)
    	    {
    		int cf=checkFilename(fileName)+checkFilename(fileName+".inp");
		
    		if (cf!=0)
    		{
    		    //  box = new AlertBox(root,this,350,150,"FILEEXISTS",
    		    //"File exists. Overwrite?","OK","Cancel");
    		    //		    box.show();
    		    return 1;
    		}
    		else
    		{
    		    domainfilename=fileName;
    		    domainfileformat=format;
		    
  //  		    saveDomainUCD(fileName);
    		    return 0;
    		}
    	    }
    	    else
    	    {
    		domainfilename=fileName;
    		domainfileformat=format;
		
  //  		saveDomainUCD(fileName);
    		return 0;
    	    }
	    
    	}
	
    	if ( format.equals("tri") || format.equals("prm") ) 
    	{
    	    if (check)
    	    {
    		int cf=checkFilename(fileName)+checkFilename(fileName+".tri")+
    		    checkFilename(fileName+".prm");
		
    		if (cf!=0)
    		{
    		    //		    box = new AlertBox(root,this,350,150,"FILEEXISTS",
    		    //	       "File exists. Overwrite?","OK","Cancel");
    		//		    box.show();
    		    return 1;
    		}
    		else
    		{
    		    domainfilename=fileName;
    		    domainfileformat=format;
		    
    		    saveDomainFEAT(fileName);
    		    return 0;
    		}
    	    }
    	    else
    	    {
    		domainfilename=fileName;
    		domainfileformat=format;
		
    		saveDomainFEAT(fileName);
    		return 0;
    	    }
    	}
	
    	if ( format.equals("feast") ) 
    	{
    	    if (check)
    	    {
    		int cf=checkFilename(fileName)+checkFilename(fileName+".feast");
		
    		if (cf!=0)
    		{
    		    //			box = new AlertBox(root,this,350,150,"FILEEXISTS",
    		    //"File exists. Overwrite?","OK","Cancel");
    		    //			box.show();
    		    return 1;
    		}
    		else
    		{
    		    domainfilename=fileName;
    		    domainfileformat=format;
    		    saveDomainFEAST(fileName);
    		    return 0;
    		}
              }
    	    else
    	    {
    		domainfilename=fileName;
    		domainfileformat=format;
		
    		saveDomainFEAST(fileName);
    		return 0;
    	    }
  	 }
    	    return -1;
  	
        }


        /**
         *  Checks if the given file already exists on the system.
         *
         *  @param fileName - name of the file to query
         *  @param return 0 if file does not exist, 1 otherwise
         */
        private int checkFilename(String dateiname)
        {
    	int exists = 1;
    	// all very straight forward, just try blindly to open the file and
    	// catch the exception of there is one :-)
    	try
    	{
    	    BufferedReader in;
    	    in = new BufferedReader (new FileReader (dateiname));
    	    in.close();
    	}
    	catch(IOException exep)
    	{
    	    exists = 0;
    	}
    	return exists;
        }

        public static int checkFile(String dateiname)
        {
    	int exists = 1;
    	// all very straight forward, just try blindly to open the file and
    	// catch the exception of there is one :-)
    	try
    	{
    	    BufferedReader in;
    	    in = new BufferedReader (new FileReader (dateiname));
    	    in.close();
    	}
    	catch(IOException exep)
    	{
    	    exists = 0;
    	}
    	return exists;
        }
	

//  //      /*******************************************************************/
//  //      // ROUTINES FOR UCD FILE FORMAT
//  //      /******************************************************************/

//  //      /**
//  //       *  loads the whole domain in UCD format
//  //       * @param String FileName
//  //       * @return some int < 0 if not successful, > 0 otherwise
//  //       */		
//  //      private int loadDomainUCD (BufferedReader in)
//  //      {
//  //  	NodeCount = 0;
//  //  	CellCount = 0;
//  //  	Node[] vertices;
//  //  	Cell[] grid;
//  //  	int i, j, k, count;
//  //  	int[] erg = new int[6];
//  //  	double[] derg = new double[3];
//  //  	int nNodes, nCells;
//  //  	boolean flag = true;
//  //  	StringTokenizer st;
//  //  	String[] string = new String[4];
//  //  	String buffer;
	
//  //  	try 
//  //  	{
//  //  	    buffer = in.readLine();
//  //  	    scanLineUCD(buffer, "s", derg, erg, string);  
//  //  	    while (string[0].startsWith("#")) 
//  //  	    {
//  //  		buffer = in.readLine();
//  //  		scanLineUCD(buffer, "s", derg, erg, string);  
//  //  	    }

//  //  	    scanLineUCD(buffer, "iiiii", derg, erg, string);  
	    
//  //  	    nNodes = erg[0];
//  //  	    nCells = erg[1];
//  //  	    NodeValueCount = erg[2];
//  //  	    CellValueCount = erg[3];
	    
//  //  	    int[] nodeNumber = new int[nNodes];
//  //  	    for (i=0; i<nNodes; i++) 
//  //  	    {
//  //  		buffer = in.readLine();
//  //  		scanLineUCD(buffer, "iddd", derg, erg, string);  
//  //  		count = erg[0];
//  //  		nodeNumber[i] = count;
//  //  		addNode( new Node(derg[0], derg[1], derg[2], NodeValueCount, count) );
//  //  	    }
	    
//  //  	    int max = 0;
//  //  	    int min = 0;
//  //  	    for (j=0; j<nNodes; j++) 
//  //  	    {
//  //  		max = Math.max(max, nodeNumber[j]);
//  //  		min = Math.min(min, nodeNumber[j]);
//  //  	    }
	    
//  //  	    int maxi = max + 2 - min;
//  //  	    min--;
//  //  	    max++;
//  //  	    //	    System.err.println(nNodes);
//  //  	    //	    System.err.println(max);
//  //  	    int[] indexNode = new int[maxi];
//  //  	    for (j=0; j<maxi; j++)
//  //  		indexNode[i] = -1;
//  //  	    for (j=0; j<nNodes; j++)
//  //  		indexNode[nodeNumber[j]] = j;
	    
//  //  	    int[] cellNumber = new int[nCells];
//  //  	    String type;
//  //  	    int nBoundaryCells = 0;
//  //  	    for (i=0; i<nCells; i++) 
//  //  	    {
//  //  		buffer = in.readLine();
//  //  		scanLineUCD(buffer, "iis", derg, erg, string);  
//  //  		cellNumber[i] = erg[0];
//  //  		type = string[0];
//  //  		type.trim();
		
//  //  		if (type.equals("quad")) 
//  //  		{
//  //  		    scanLineUCD(buffer, "iisiiii", derg, erg, string);  
//  //  		    Node[] n = new Node[4];
//  //  		    for (j=0; j<4; j++) 
//  //  		    {
//  //  			if ( (erg[j+2]<max) && (erg[j+2]>min) && (indexNode[ erg[j+2] ] != -1) )
//  //  			    n[j] = (Node)nodes.elementAt( indexNode[ erg[j+2] ] );
//  //  			else 
//  //  			{
//  //  			    //			    System.out.println("False Node number in quad number " + cellNumber[i]);
//  //                return -1;
//  //  			}
//  //  		    }

//  //  		    Edge[] edg = new Edge[4];
//  //  		    Edge help;
		    
//  //  		    for (j=0; j<4; j++) 
//  //  		    {
//  //  			help = (Edge)searchEdge(n[j], n[(j+1)%4]);
//  //  			if (help==null) 
//  //  			{
//  //  			    edg[j] = new Edge(n[j], n[(j+1)%4]);
//  //  			}
//  //  			else 
//  //  			{
//  //  			    edg[j] = help;
//  //  			}
//  //  			addEdge(edg[j]);
//  //  		    }
//  //  		    Quad q = new Quad(i, edg[0], edg[1], edg[2], edg[3]);
//  //  		    addCell( q );
//  //  		}

//  //  		if (type.equals("tri")) 
//  //  		{
//  //  		    scanLineUCD(buffer, "iisiii", derg, erg, string);  
//  //  		    Node[] n = new Node[3];
//  //  		    for (j=0; j<3; j++) 
//  //  		    {
//  //  			if ( (erg[j+2]<max) && (erg[j+2]>min) && 
//  //  			     (indexNode[ erg[j+2] ] != -1) )
//  //  			    n[j] = (Node)nodes.elementAt( indexNode[ erg[j+2] ] );
//  //  			else 
//  //  			{
//  //  			    //			    System.out.println("False Node number in tri number " + cellNumber[i]);
//  //  			    return -1;
//  //  			}
//  //  		    }
//  //  		    Edge[] edg = new Edge[3];
//  //  		    Edge help;
//  //  		    for (j=0; j<3; j++) 
//  //  		    {
//  //  			help = (Edge)searchEdge(n[j], n[(j+1)%3]);
//  //  			if (help==null) 
//  //  			{
//  //  			    edg[j] = new Edge(n[j], n[(j+1)%3]);
//  //  			}
//  //  			else 
//  //  			{
//  //  			    edg[j] = help;
//  //  			}
//  //  			addEdge(edg[j]);
//  //  		    }
//  //  		    Tri tri = new Tri(i, edg[0], edg[1], edg[2]);
//  //  		    addCell( tri );
//  //  		}
		
//  //  		if (type.equals("line")) 
//  //  		{
//  //  		    scanLineUCD(buffer, "iisii", derg, erg, string);  
//  //  		    Node[] n = new Node[2];
//  //  		    for (j=0; j<2; j++) 
//  //  		    {
//  //  			if ( (erg[j+2]<max) && (erg[j+2]>min) && (indexNode[ erg[j+2] ] != -1) )
//  //  			    n[j] = (Node)nodes.elementAt( indexNode[ erg[j+2] ] );
//  //  			else 
//  //  			{
//  //  			    //			    System.out.println("False Node number in line number " + cellNumber[i]);
//  //  			    return -1;
//  //  			}
//  //  		    }
//  //  		    BoundaryEdgeCount++;
//  //  		    Edge edg = new Edge(n[0], n[1]);
//  //  		    edg.boundaryEdge = true;
//  //  		    edg.BoundaryNumber = erg[1];
//  //  		    addEdge(edg);
//  //  		}
//  //  	    }
	    
//  //  	    updateCellNodes();
//  //  	    updateCellNeighbours();
	    
//  //  	    max = 0;
//  //  	    min = 0;
//  //  	    for (j=0; j<nCells-BoundaryEdgeCount; j++) 
//  //  	    {
//  //  		max = Math.max(max, cellNumber[j]);
//  //  		min = Math.min(min, cellNumber[j]);
//  //  	    }
	    
//  //  	    maxi = max + 2 - min;
//  //  	    min--;
//  //  	    max++;
//  //  	    int[] indexCell = new int[maxi];
//  //  	    for (j=0; j<maxi; j++)
//  //  	    {
//  //  		indexCell[j] = -1;
//  //  	    }
//  //  	    for (j=0; j<nCells-BoundaryEdgeCount; j++) 
//  //  	    {
//  //  		indexCell[cellNumber[j]] = j;
//  //  	    }

//  //  	    int nValues;
//  //  	    String stri;
//  //  	    int userCounter;
//  //  	    /** Read node values **/
//  //  	    dc.nodeValueName = null;
//  //  	    dc.nodeValueUnit = null;
//  //  	    if (NodeValueCount != 0)
//  //  	    {
//  //  		nValues = 0;
//  //  		try 
//  //  		{
//  //  		    buffer = in.readLine();
//  //  		    scanLineUCD(buffer, "i", derg, erg, string);  
//  //  		    nValues = erg[0];
//  //  		    if ( nValues < 1) 
//  //  		    {
//  //  			//			System.out.println("Simple Mesh read.");
//  //  			centerDomain();	
//  //  			return -1;
//  //  		    }
//  //  		}
//  //  		catch ( Exception e  ) 
//  //  		{
//  //  		    //		    System.err.println("Simple Mesh read");
//  //  		    //		    System.err.println("Mesh without values available.");
//  //  		    centerDomain();	
//  //  		    return -1;
//  //  		}
  
//  //  		stri = "i";
//  //  		for (j=0; j<nValues; j++) 
//  //  		{
//  //  		    stri += "i";
//  //  		}
		
//  //  		erg = new int[nValues+1];
//  //  		dc.nodeValueType = new int[nValues][];
//  //  		scanLineUCD(buffer, stri, derg, erg, string);  
//  //  		for (j=0; j<nValues; j++) 
//  //  		{
//  //  		    dc.nodeValueType[j] = new int[erg[j+1]];
//  //  		    for (int jjg=0; jjg<erg[j+1]; jjg++) 
//  //  		    {
//  //  			dc.nodeValueType[j][jjg] = j + jjg;
//  //  		    }
//  //  		}
		
//  //  		dc.nodeValueName = new String[NodeValueCount];
//  //  		dc.nodeValueUnit = new String[NodeValueCount];
//  //  		userCounter = 0;
//  //  		for (j=0; j<nValues; j++) 
//  //  		{
//  //  		    buffer = in.readLine();
//  //  		    String name = buffer.substring(0, buffer.lastIndexOf(","));
//  //  		    String unit = "";
//  //  		    if (buffer.lastIndexOf(",") >= buffer.length()-1) 
//  //  		    {
//  //  			unit = "none";
//  //  		    }
//  //  		    else 
//  //  		    {
//  //  			unit= buffer.substring(buffer.lastIndexOf(",")+1);
//  //  		    }
		    
//  //  		    if (dc.nodeValueType[j].length > 1) 
//  //  		    {
//  //  			for (k=0; k<dc.nodeValueType[j].length; k++) 
//  //  			{
//  //  			    dc.nodeValueName[userCounter] = name + " " + k;
//  //  			    dc.nodeValueUnit[userCounter++] = unit;
//  //  			}
//  //  		    }
//  //  		    else 
//  //  		    {
//  //  			dc.nodeValueName[userCounter] = name;
//  //  			dc.nodeValueUnit[userCounter++] = unit;
//  //  		    }
//  //  		}
		
//  //  		derg = new double[NodeValueCount];
//  //  		stri = "i";
		
//  //  		for (j=0; j<NodeValueCount; j++) 
//  //  		{
//  //  		    stri += "d";
//  //  		}
		
//  //  		for (k=0; k<nNodes; k++) 
//  //  		{
//  //  		    buffer = in.readLine();
//  //  		    scanLineUCD(buffer, stri, derg, erg, string);  
		    
//  //  		    for (j=0; j<NodeValueCount; j++) 
//  //  		    {
//  //  			((Node)nodes.elementAt(indexNode[erg[0]])).setValue(j, derg[j]);
//  //  		    }
//  //  		}
		
//  //  		calculateNodeMinMax();
//  //  	    }
	    
//  //  	    /** Read Cell Values **/
//  //  	    dc.cellValueName = null;
//  //  	    dc.cellValueUnit = null;
//  //  	    if (CellValueCount != 0)
//  //  	    {
//  //  		try 
//  //  		{
//  //  		    buffer = in.readLine();
//  //  		    nValues = 0;
//  //  		    scanLineUCD(buffer, "i", derg, erg, string);  
//  //  		    nValues = erg[0];
//  //  		    if ( nValues < 1) 
//  //  		    {
//  //  			//			System.out.println("No Cell Values");
//  //  			//			System.err.println("Mesh with Node values available.");
//  //  			centerDomain();	
//  //  			return -1;
//  //  		    }
//  //  		}
//  //  		catch ( Exception e  ) 
//  //  		{
//  //  		    //		    System.err.println("No Cell Values");
//  //  		    //		    System.err.println("Mesh with Node values available.");
//  //  		    centerDomain();	
//  //  		    return -1;
//  //  		}
		
//  //  		stri = "i";
//  //  		for (j=0; j<nValues; j++) 
//  //  		{
//  //  		    stri += "i";
//  //  		}
		
//  //  		erg = new int[nValues+1];
//  //  		dc.cellValueType = new int[nValues][];
//  //  		scanLineUCD(buffer, stri, derg, erg, string);  
//  //  		for (j=0; j<nValues; j++)
//  //  		{
//  //  		    dc.cellValueType[j] = new int[erg[j+1]];
//  //  		    for (int jjg=0; jjg<erg[j+1]; jjg++) 
//  //  		    {
//  //  			dc.cellValueType[j][jjg] = j + jjg;
//  //  		    }
//  //  		}
		
//  //  		dc.cellValueName = new String[CellValueCount];
//  //  		dc.cellValueUnit = new String[CellValueCount];
		
//  //  		userCounter = 0;
//  //  		for (j=0; j<nValues; j++) 
//  //  		{
//  //  		    buffer = in.readLine();
//  //  		    String name = buffer.substring(0, buffer.lastIndexOf(","));
//  //  		    String unit = "";
//  //  		    if (buffer.lastIndexOf(",") >= buffer.length()-1) 
//  //  		    {
//  //  			unit = "none";
//  //  		    }
//  //  		    else 
//  //  		    {
//  //  			unit= buffer.substring(buffer.lastIndexOf(",")+1);
//  //  		    }
		    
//  //  		    if (dc.cellValueType[j].length > 1) 
//  //  		    {
//  //  			for (k=0; k<dc.cellValueType[j].length; k++) 
//  //  			{
//  //  			    dc.cellValueName[userCounter] = name + " " + k;
//  //  			    dc.cellValueUnit[userCounter++] = unit;
//  //  			}
//  //  		    }
//  //  		    else 
//  //  		    {
//  //  			dc.cellValueName[userCounter] = name;
//  //  			dc.cellValueUnit[userCounter++] = unit;
//  //  		    }
//  //  		}
		
//  //  		derg = new double[CellValueCount];
//  //  		stri = "i";
//  //  		for (j=0; j<CellValueCount; j++) 
//  //  		{
//  //  		    stri += "d";
//  //  		}
		
//  //  		for (k=0; k<nCells; k++) 
//  //  		{
//  //  		    buffer = in.readLine();
//  //  		    scanLineUCD(buffer, stri, derg, erg, string);
//  //  		    ((Cell)cells.elementAt(indexCell[erg[0]])).defValue(CellValueCount);
		    
//  //  		    for (j=0; j<CellValueCount; j++) 
//  //  		    {
//  //  			((Cell)cells.elementAt(indexCell[erg[0]])).setValue(j, derg[j]);
//  //  		    }
//  //  		}
		
//  //  		calculateCellMinMax();
//  //  	    }
	    
//  //  	    /** General work on the domain**/
//  //  	    centerDomain();	
//  //  	}
//  //  	catch ( NumberFormatException e  ) 
//  //  	{
//  //  	    System.out.println("Not an UCD file : " + e);
//  //  	    return -1;
//  //  	}
//  //  	catch (IOException e) 
//  //  	{
//  //  	    System.out.println("Could not load file : " + e);
//  //  	    return -1;
//  //  	}
//  //  	catch ( SecurityException e ) 
//  //  	{
//  //  	    System.out.println("403 Forbidden : " + e);
//  //  	    return -1;
//  //  	}
//  //  	catch ( Exception e ) 
//  //  	{
//  //  	    System.out.println("403 Forbidden : " + e);
//  //  	    return -1;
//  //  	}
	
//  //  	return 0;
//  //      }

//  //      /** 
//  //       *  saves the domain in UCD format to the given filename
//  //       *  @param fN - full qualified path and filename of the desired target
//  //       */
//  //      private void saveDomainUCD(String fN)
//  //      {
//  //  	int sc,maxpb;
//  //  	Segment s;
//  //  	String fileName;
	
//  //  	try
//  //  	{
//  //  	    if (fN.indexOf(".inp")>0)
//  //  		fileName=fN.substring(0,fN.indexOf(".inp"));
//  //  	    else 
//  //  		fileName=fN;
	    
//  //  	    FileOutputStream writeFile = new FileOutputStream(fileName+".inp");
//  //  	    DataOutputStream out = new DataOutputStream( writeFile );
	    
//  //  	    //	    System.out.println("Filename:"+fileName+".inp");
	    
//  //  	    int cn = getNodeCount();
//  //  	    int cc = getCellCount();
//  //  	    int cb = 0;
	    
//  //  	    Edge ee;
//  //  	    for (int i=0; i < EdgeCount; i++)
//  //  	    {
//  //  		ee = getEdge(i);
//  //  		if (ee.boundaryEdge == true)
//  //  		    cb++;
//  //  	    }
//  //  	    BoundaryEdgeCount = cb;
	    
//  //  	    out.writeBytes("#UCD file created by DeViSorGrid2\n");
//  //  	    out.writeBytes(cn + " ");
	    
//  //  	    int tmpcc = cc + cb;
//  //  	    out.writeBytes(tmpcc + " ");
//  //  	    out.writeBytes("0 0 0 \n");
	  
//  //  	    Node knot;
//  //  	    Cell zell;
//  //  	    int index;
	    
//  //  	    for(index=0;index<cn; index++)
//  //  	    {
//  //  		knot=getNode(index);
//  //  		out.writeBytes(knot.getNumber() + " " + knot.getX(0) + " " + 
//  //  			       knot.getX(1) + " " + knot.getX(2) + "\n");
//  //  	    }
	    
//  //  	    for(index=0;index<cc; index++)
//  //  	    {
//  //  		zell=getCell(index);
//  //  		out.writeBytes(zell.getNumber() + " 0 ");
//  //  		int nbn = zell.getNPT();
//  //  		switch (nbn)
//  //  		{
//  //  		case 4 :
//  //  		    out.writeBytes("quad ");
//  //  		    for (int i=0; i<4; i++)
//  //  		    {
//  //  			out.writeBytes(zell.nodes[i].getNumber() + " ");
//  //  		    }
//  //  		    out.writeBytes("\n");
//  //  		    break;
//  //  		case 3 :
//  //  		    out.writeBytes("tri ");
//  //  		    for (int i=0; i<3; i++)
//  //  		    {
//  //  			out.writeBytes(zell.nodes[i].getNumber() + " ");
//  //  		    }
//  //  		    out.writeBytes("\n");
//  //  		    break;
//  //  		case 2 :
//  //  		    out.writeBytes("line ");
//  //  		    for (int i=0; i<2; i++)
//  //  		    {
//  //  			out.writeBytes(zell.nodes[i].getNumber() + " ");
//  //  		    }
//  //  		    out.writeBytes("\n");
//  //  		    break;
//  //  		default :
//  //  		    //		    System.out.println("Not implemented yet :-(");
//  //  		}
//  //  	    }
	  
//  //  	    /*
//  //  	     * Boundaries through Edges
//  //  	     */
//  //  	    int tmpCount = 0;
//  //  	    for (int i=0; i < EdgeCount; i++)
//  //  	    {
//  //  		ee = getEdge(i);
//  //  		if (ee.boundaryEdge == true)
//  //  		{
//  //  		    out.writeBytes(tmpCount + " " + ee.BoundaryNumber + " line ");
//  //  		    tmpCount++;
//  //  		    out.writeBytes( ee.getNode(0).getNumber() + " " + ee.getNode(1).getNumber() + "\n");
//  //  		}
//  //  	    }
//  //  	    out.close();
//  //  	    //	    cc.somethingwaschanged=false;
	    
//  //  	}
//  //  	catch(IOException e)
//  //  	{
//  //  	    //	    System.err.println("IO-Error: "+e);
//  //  	}
//  //      }
    

//  //      private void scanLineUCD(String t, String format, double[] dret, int[] iret, String[] sret)
//  //      {
//  //  	int toks = 0;
//  //  	int itoks = 0;
//  //  	int dtoks = 0;
//  //  	int stoks = 0;
//  //  	char test;
	
//  //  	int start=0;
//  //  	int end=0;
//  //  	boolean minus = false;
	
//  //  	while (t.charAt(start)==' ') 
//  //  	    start++;
//  //  	end=start;
	
//  //  	String subs = "test";
	
//  //  	while ((end<t.length()) && (toks<format.length()))
//  //  	{
//  //  	    test=t.charAt(end);
	    
//  //  	    if  ( (test==' ') || (minus && (test=='-')) )
//  //  	    {
//  //  		subs=t.substring(start,end);
		
//  //  		if (minus && (test=='-')) 
//  //  		    start=end;
//  //  		else 
//  //  		    start=end+1;
		
//  //  		minus = false;
		
//  //  		if (start>=t.length())
//  //  		    end=start-1;
//  //  		else
//  //  		{
//  //  		    while (t.charAt(start)==' ') start++;
		    
//  //  		    end=start;
//  //  		}
		
//  //  		if (format.charAt(toks)=='i')
//  //  		    iret[itoks++]=(new Integer(subs)).intValue();
//  //  		else
//  //  		{ 
//  //  		    if (format.charAt(toks)=='d')
//  //  			dret[dtoks++]=(new Double(subs)).doubleValue();
//  //  		    else
//  //  			sret[stoks++]=subs;
//  //  		}
					
//  //  		toks++;
		
//  //  	    }
//  //  	    else 
//  //  	    {
//  //  		end++;
		
//  //  		if ( (test == 'E') || (test == 'e') || (test == ' ') )
//  //  		    minus = false;
//  //  		else
//  //  		    minus = true;
//  //  	    }		
//  //  	}
	
//  //  	if (toks<format.length())
//  //  	{
//  //  	    subs=t.substring(start,end);
//  //  	    if (format.charAt(toks)=='i')
//  //  		iret[itoks++]=(new Integer(subs)).intValue();
//  //  	    else
//  //  	    { 
//  //  		if (format.charAt(toks)=='d')
//  //  		    dret[dtoks++]=(new Double(subs)).doubleValue();
//  //  		else
//  //  		    sret[stoks++]=subs;
//  //  	    }
//  //  	}
	
//  //      }
    
    
        private String printIntAsDouble(int udigit)
        {
	
	
  	int gs = cc.globalscaledigits;
	
  	if (udigit<0)
  	{
  	int digit=-udigit;
          String tmp = (new Integer(digit)).toString();
	
  	if (tmp.length()>gs)
  	{
  	   int i = tmp.length()-gs;
  	   String ret="-"+tmp.substring(0,i)+"."+tmp.substring(i);
  	return ret;
  	}
  	else
  	{
  	  int i=gs-tmp.length();
  	  String vorlage = "000000000000000000";
	  
  	  String ret="-0."+vorlage.substring(0,i)+tmp;
  	return ret;
  	}
  	}
  	else
  	{
  	int digit=udigit;
          String tmp = (new Integer(digit)).toString();
	
  	if (tmp.length()>gs)
  	{
  	   int i = tmp.length()-gs;
  	   String ret=tmp.substring(0,i)+"."+tmp.substring(i);
  	return ret;
  	}
  	else
  	{
  	  int i=gs-tmp.length();
  	  String vorlage = "000000000000000000";
	  
  	  String ret="0."+vorlage.substring(0,i)+tmp;
  	return ret;
  	}
  	}	
	
        }
        /*********************************************************************/
        /* ROUTINES FOR FEAT FORMAT */
        /*********************************************************************/
		
        /**
         * saves the domain in the FEAT format (name.prm, name.tri)
         * @param fileName file name
         */	
        public void saveDomainFEAT(String fN)
        {
    	Boundary be;
    	int sc;
    	Segment s;
    	String fileName;
	
    	try
    	{
    	    // remove eventually existing file extensions
    	    if (fN.indexOf(".tri")>0)
    		fileName=fN.substring(0,fN.indexOf(".tri"));
    	    else 
    		if (fN.indexOf(".prm")>0)
    		    fileName=fN.substring(0,fN.indexOf(".prm"));
    		else
    		    fileName=fN;
								
	    boolean onlytri=(fN.indexOf(".tri")>0);

    	    FileOutputStream writeFile;
    	    DataOutputStream out;
    	    int bc = db.getBoundaryCount();
	    	    
	    if (!onlytri)
	    {
	    			

    	    writeFile = new FileOutputStream(fileName+".prm");
    	    out = new DataOutputStream( writeFile );
	    
    	    out.writeBytes("NBCT\n");
    	    out.writeBytes(bc+"\n");
	    
    	    for(int i=0; i<bc; i++)
    	    {
    		out.writeBytes("IBCT\n");
    		out.writeBytes((i+1)+"\n");
		
    		be=db.getBoundary(i);
	
    		out.writeBytes("NCOMP\n");
		
    		sc=be.getSegmentCount();
		
    		out.writeBytes(sc+"\n");
		
    		out.writeBytes("ITYP NSPLINE NPAR\n");
		
    		for (int j=0; j<sc; j++)
    		{
    		    s=be.getSegment(j);
		    
    		    if (s instanceof SegmentLine)
    		    {
    			out.writeBytes("1 1 2\n");
    		    }
    		    else 
    			if (s instanceof SegmentCircle)
    			{
    			    out.writeBytes("2 1 3\n");
    			}
    			else
    			    			    System.out.println("Internal Error");
    		}
		
    	    }
	    
    	    out.writeBytes("PARAMETERS\n");
    	    int[] prm;
	    
    	    for(int i=0; i<bc; i++)
    	    {
    		be=db.getBoundary(i);
    		sc=be.getSegmentCount();
    		for (int j=0; j<sc; j++)
    		{
    		    s=be.getSegment(j);
		    
    		    if (s instanceof SegmentLine)
    		    {
    			prm=s.getParams();
    			out.writeBytes(printIntAsDouble(prm[0])+" "+printIntAsDouble(prm[1])+"\n");
    			out.writeBytes(printIntAsDouble(prm[2])+" "+printIntAsDouble(prm[3])+"\n");
    		    }
    		    else 
    			if (s instanceof SegmentCircle)
    			{
    			    prm=s.getParams();
    			    out.writeBytes(printIntAsDouble(prm[0])+" "+printIntAsDouble(prm[1])+"\n");
    			    out.writeBytes(printIntAsDouble(prm[2])+" 0.0E0\n");
			    
    			    out.writeBytes((prm[3]/180.0*Math.PI)+" "+(prm[4]/180.0*Math.PI)+"\n");
    			}
    			else
    			    			    System.out.println("Internal Error");
    		}
    	    }
	    
    	    out.close();
	    
    	    System.out.println("File "+fileName+".prm saved!");
	    }
	    
    	    writeFile = new FileOutputStream(fileName+".tri");
    	    out = new DataOutputStream( writeFile );
	    
    	    bc = db.getBoundaryCount();
    	    int cn = getNodeCount();
    	    int cec = getCellCount();
	    
    	    out.writeBytes("Gitterbeschreibung fuer "+fileName+"\n");
    	    out.writeBytes("Parametrisierung PARCX, PARCY, TMAXC\n");
    	    out.writeBytes(cec+"  "+cn+" 0 "+" 4 " + bc + "   NEL NVT NMT NVE NBCT\n");
	    
    	    out.writeBytes("DCORVG\n");
	    
    	    Node nobby;
    	    int index;

            Enumeration nodeenum = nodes.elements();
	    	    
    	    while (nodeenum.hasMoreElements())
    	    {
                nobby = (Node)nodeenum.nextElement();
		
    		if (nobby instanceof BoundaryNode)
		    if (!onlytri)
    		      out.writeBytes(((BoundaryNode)nobby).getParaValue()+" 0.0\n");
		    else
                      out.writeBytes(printIntAsDouble(nobby.getCoord(0))+" "+printIntAsDouble(nobby.getCoord(1))+"\n");
    		else
    		    out.writeBytes(printIntAsDouble(nobby.getCoord(0))+" "+printIntAsDouble(nobby.getCoord(1))+"\n");
    	    }
	    
    	    out.writeBytes("KVERT\n");
    	    Cell zell;
    	    int nn;
	    
    	    updateNodeNumbers();

            Enumeration cellenum = cells.elements();
	    
    	    while (cellenum.hasMoreElements ())
    	    {

                zell = (Cell2D)cellenum.nextElement();
		
    		if ((zell instanceof Macro) || (zell instanceof Quad))
    		{
    		    out.writeBytes((((Cell2D)zell).getNode(0).getNumber()+1)+" "+
    				   (((Cell2D)zell).getNode(1).getNumber()+1)+" "+
    				   (((Cell2D)zell).getNode(2).getNumber()+1)+" "+
    				   (((Cell2D)zell).getNode(3).getNumber()+1)+"\n");
    		}
    		else
    		    System.out.println("not implemented yet :-(");
    	    }
	    
    	    out.writeBytes("KNPR\n");
	    
            nodeenum = nodes.elements();
	    
    	    while (nodeenum.hasMoreElements())
    	    {
                nobby = (Node)nodeenum.nextElement();
		
		if (!onlytri)
		{
    		  if (nobby instanceof BoundaryNode)
    		     out.writeBytes((((BoundaryNode)nobby).getBoundaryNumber()+1)+" ");
    		  else
    		     out.writeBytes("0 ");
		}
		else
    		  if (nobby instanceof BoundaryNode)
    		     out.writeBytes((((BoundaryNode)nobby).getBoundaryNumber()+1)+" ");
		  else       		    
		     out.writeBytes(nobby.getNPR()+" ");
		
    	    }
	    
    	    out.writeBytes("\nKMM\n");
	    
    	    double[] diffmin = new double[bc];
    	    double[] diffmax = new double[bc];
    	    int[] idxmin = new int[bc];
    	    int[] idxmax = new int[bc];
    	    int bn;
    	    double t;
	    
    	    for(index=0;index<bc;index++)
    	    {
    		diffmin[index]=524244.0;
    		diffmax[index]=0.0;
    	    }

            nodeenum = nodes.elements();
	    index=0;
	    
    	    while (nodeenum.hasMoreElements())
    	    {
                nobby = (Node)nodeenum.nextElement();
		
    		if (nobby instanceof BoundaryNode)
    		{
    		    t=((BoundaryNode)nobby).getParaValue();
    		    bn=((BoundaryNode)nobby).getBoundaryNumber();
		    		    
    		    if (t<diffmin[bn])
    		    {
    			diffmin[bn]=t;
    			idxmin[bn]=index;
    		    }
		    
    		    if (t>diffmax[bn])
    		    {
    			diffmax[bn]=t;
    			idxmax[bn]=index;
    		    }
    		}
		index++;
    	    }
	    
    	    for (index=0;index<bc;index++) 
    		out.writeBytes((idxmin[index]+1)+" "+(idxmax[index]+1)+" ");
	    
    	    out.writeBytes("\n");
	    
    	    out.close();
	    
    	    	    System.out.println("File "+fileName+".tri saved!");
    	    changed=false;
	    
	    
    	}
    	catch(IOException e)
    	{
    	    System.err.println("IO-Error: "+e);
    	}
        }
    
    
        /**
         * loads a domain from files in  FEAT format (name.prm, name.tri)
         * @param fileName file name
         * @param para boolean load name.prm file (parametrisation)
         * @param grid boolean load name.tri file (grid description)
         * @return =0 load succesfull, <>0 load error
         */			
        public int loadDomainFEAT(String fN, boolean para, boolean grid, boolean merge, double uoffx, double uoffy)
	{
	  return loadDomainFEAT(fN,para,grid,merge, uoffx,uoffy,true);
	}
	
        public int loadDomainFEAT(String fN, boolean para, boolean grid, 
	  boolean merge, double uoffx, double uoffy,  boolean redraw)
        {
    	int erg,bec,index,nec,jdx,typ,nspline,npar;
    	double offx,offy;
          ControlCenter cc = ControlCenter.getMyself ();
	
    	if (!merge)
    	{
    	    offx=0.0;
    	    offy=0.0;
    	}
    	else
    	{
    	    offx=uoffx;
    	    offy=uoffy;
    	}

    	String dateiname;
    	
    	int offbound= db.getBoundaryCount();
    	offbound=0;
    	int offnode= getNodeCount();
	
    	int offseg=0;
	
	boolean onlytri = (fN.indexOf(".tri")>0);
	
    	if (fN.indexOf(".tri")>0)
    	    dateiname=fN.substring(0,fN.indexOf(".tri"));
    	else 
    	    if (fN.indexOf(".prm")>0)
    		dateiname=fN.substring(0,fN.indexOf(".prm"));
    	    else
    		dateiname=fN;
	
    	Edge[] temp;
	
    	//	tf = new Transformation2D();
	
	
	if (!onlytri)
	{
	System.out.println("LOAD "+dateiname+".prm");
	
    	try
    	{
    	    BufferedReader in;
    	    StreamTokenizer eingabe;
	    
    	    if (para)
    	    {
    		in = new BufferedReader(new FileReader(dateiname+".prm"));
		
    		eingabe = new StreamTokenizer(in);
		
    		eingabe.ordinaryChar('/');
    		eingabe.slashSlashComments(true);
    		eingabe.slashStarComments(true);
    		eingabe.parseNumbers();
		
    		erg=eingabe.nextToken();
		
    		if (!((erg==eingabe.TT_WORD) && (eingabe.sval.equals("NBCT"))))
    		{ 
    		    in.close(); return 110; 
    		}
		
    		erg=eingabe.nextToken();
		
    		if (!(erg==eingabe.TT_NUMBER))
    		{ 
    		    in.close(); return 111; 
    		}
			
    		bec=(int)eingabe.nval;
		
    		for (index=0;index<bec;index++)
    		{
    		    if (!merge)
    			db.addBoundary();
		    
    		    erg=eingabe.nextToken();
		    
    		    if (!((erg==eingabe.TT_WORD) && (eingabe.sval.equals("IBCT"))))
    		    { 
    			in.close(); return 112; 
    		    }
		    
    		    erg=eingabe.nextToken();
		    
    		    if (!((erg==eingabe.TT_NUMBER) && ((int)eingabe.nval!=(index-1))))
    		    { 
    			in.close(); return 113; 
    		    }
			
    		    erg=eingabe.nextToken();
		    
    		    if (!((erg==eingabe.TT_WORD) && (eingabe.sval.equals("NCOMP"))))
    		    { 
    			in.close();  return 114; 
    		    } 
			
    		    erg=eingabe.nextToken();
		    
    		    if (!(erg==eingabe.TT_NUMBER))
    		    { 
    			in.close();  return 115;
    		    }
		    
    		    nec=(int)eingabe.nval;
		    
    		    erg=eingabe.nextToken();
    		    erg=eingabe.nextToken();
    		    erg=eingabe.nextToken();
		    
    		    offseg=db.getBoundary(index).getSegmentCount();
		    
    		    for(jdx=offseg;jdx<offseg+nec;jdx++)
    		    {
    			erg=eingabe.nextToken();
    			if (!(erg==eingabe.TT_NUMBER))
    			{ 
    			    in.close(); return 116; 
    			} 
			
  			typ=(int)eingabe.nval;
			
    			erg=eingabe.nextToken();
    			if (!(erg==eingabe.TT_NUMBER))
    			{ 
    			    in.close(); return 117; 
    			} 
			
    			nspline=(int)eingabe.nval;
			
    			erg=eingabe.nextToken();
    			if (!(erg==eingabe.TT_NUMBER))
    			{ 
    			    in.close(); return 118; 
    			} 
    			npar=(int)eingabe.nval;
			
  			switch(typ)
  			{
  			
 			  case 1:
  			    db.addSegment(db.getBoundary(offbound+index),new SegmentLine(0,0,1,0,true));
  			    break;
  			  case 2:
  			    db.addSegment(db.getBoundary(offbound+index),new SegmentCircle(0,0,1,0,0));
  			    break;
  			  case 3:
  			    System.out.println("not yet implem.");
  			default:
  			   { 
  				in.close(); return 119; 
  			    } 
  			}
    		    }
    		}
		
    		erg=eingabe.nextToken();
    		if (!((erg==eingabe.TT_WORD) && (eingabe.sval.equals("PARAMETERS"))))
    		{ 
    		    in.close();return 120; 
    		} 
		
    		eingabe.ordinaryChar('d');
    		eingabe.ordinaryChar('D');
    		eingabe.ordinaryChar('e');
    		eingabe.ordinaryChar('E');
    		eingabe.ordinaryChar('+');
    		eingabe.ordinaryChar('-');
		
    		Boundary b;
    		Segment s;
    		int[] p = new int[16];
		
    		for(index=offbound;index<offbound+bec;index++)
    		{
    		    b=db.getBoundary(index);
    		    nec=b.getSegmentCount();
		    
    		    for(jdx=offseg;jdx<nec;jdx++)
    		    {
    			s=b.getSegment(jdx);
    			if (s instanceof SegmentLine)
    			{
    			    p[0]=(int)(cc.globalscale*(nextExpToken(eingabe)+offx));
    			    p[1]=(int)(cc.globalscale*(nextExpToken(eingabe)+offy));
    			    p[2]=(int)(cc.globalscale*nextExpToken(eingabe));
    			    p[3]=(int)(cc.globalscale*nextExpToken(eingabe));
			    
    			    s.setParams(p);
    			}
    			else 
    			    if (s instanceof SegmentCircle)
    			    {
    				p[0]=(int)(cc.globalscale*(nextExpToken(eingabe)+offx));
    				p[1]=(int)(cc.globalscale*(nextExpToken(eingabe)+offy));
    				p[2]=(int)(cc.globalscale*nextExpToken(eingabe));
    				p[3]=(int)(nextExpToken(eingabe)/Math.PI*180);
    				p[3]=(int)(nextExpToken(eingabe)/Math.PI*180);
    				p[4]=(int)(nextExpToken(eingabe)/Math.PI*180);
								
    				s.setParams(p);
    			    }
    			    else
    			    { 
    				in.close();  return 121; 
    			    } 
    		    }
    		}
		
    		in.close();
    	    }
		   }
    	catch(IOException e)
    	{  	
    	    //	    System.err.println("Fehler beim Dateilesen");
    	    return 100;
    	}

	}
	
	
		    	try
    	{   
	    	    BufferedReader in;
    	    StreamTokenizer eingabe;
	    
	    
    	    if (grid)
    	    {
    		in = new BufferedReader(new FileReader(dateiname+".tri"));

    		eingabe = new StreamTokenizer(in);
		
    		boolean trigen = false;
		
    		do 
    		{
    		    erg=eingabe.nextToken();
		    
    		    if ((erg==eingabe.TT_WORD) && (eingabe.sval.equals("trigen2d")))
    		    {
    			trigen=true;
    			//	System.out.println("TRIGEN mode enabled!");
    		    }
	    
    		} while (eingabe.lineno()<3);
		
    		int nel,nvt,nmt,nve,nbct;
		
    		if (!(erg==eingabe.TT_NUMBER))
    		{ 
    		    in.close(); return 122; 
    		}

    		nel=(int)eingabe.nval;
		
    		erg=eingabe.nextToken();
    		if (!(erg==eingabe.TT_NUMBER))
    		{ 
    		    in.close(); return 123; 
    		}
		
    		nvt=(int)eingabe.nval;

    		erg=eingabe.nextToken();
    		if (!(erg==eingabe.TT_NUMBER))
    		{ 
    		    in.close(); return 124; 
    		}

    		nmt=(int)eingabe.nval;
		
    		erg=eingabe.nextToken();
    		if (!(erg==eingabe.TT_NUMBER))
    		{ 
    		    in.close(); return 125; 
    		}

    		nve=(int)eingabe.nval;
		
    		erg=eingabe.nextToken();
    		if (!(erg==eingabe.TT_NUMBER))
    		{ 
    		    in.close(); return 126;
    		}

    		nbct=(int)eingabe.nval;
		
    		if (!trigen)
    		{
    		    erg=eingabe.nextToken();		
    		    if (!((erg==eingabe.TT_WORD) && (eingabe.sval.equals("NEL"))))
    		    { 
    			in.close(); return 127; 
    		    }

    		    erg=eingabe.nextToken();		
    		    if (!((erg==eingabe.TT_WORD) && (eingabe.sval.equals("NVT"))))
    		    { 
    			in.close(); return 128; 
    		    }

    		    erg=eingabe.nextToken();		
    		    if (!((erg==eingabe.TT_WORD) && (eingabe.sval.equals("NMT"))))
    		    { 
    			in.close(); return 129; 
    		    }

    		    erg=eingabe.nextToken();		
    		    if (!((erg==eingabe.TT_WORD) && (eingabe.sval.equals("NVE"))))
    		    { 
    			in.close(); return 130; 
    		    }

    		    erg=eingabe.nextToken();		
    		    if (!((erg==eingabe.TT_WORD) && (eingabe.sval.equals("NBCT"))))
    		    { 
    			in.close(); return 131; 
    		    }
    		}

    		erg=eingabe.nextToken();		
    		if (!((erg==eingabe.TT_WORD) && (eingabe.sval.equals("DCORVG"))))
    		{ 
    		    in.close(); return 132;
    		}
	
    		eingabe.ordinaryChar('d');
    		eingabe.ordinaryChar('D');
    		eingabe.ordinaryChar('e');
    		eingabe.ordinaryChar('E');
    		eingabe.ordinaryChar('+');
    		eingabe.ordinaryChar('-');
		
    		double[][] tempn = new double[3][nvt];
		
    		for(index=0;index<nvt;index++)
    		{
    		    tempn[0][index]=nextExpToken(eingabe);
    		    tempn[1][index]=nextExpToken(eingabe);
    		    tempn[2][index]=0;
    		}
		
    		eingabe.wordChars('E','E');
    		eingabe.wordChars('e','e');
    		eingabe.wordChars('D','D');
    		eingabe.wordChars('d','d');
		
    		erg=eingabe.nextToken();
		
    		if (!((erg==eingabe.TT_WORD) && (eingabe.sval.equals("KVERT"))))
    		    if (eingabe.sval.equals("KV"))   
    		    {
    			erg=eingabe.nextToken();
    		    }
    		    else
    		    { 
    			in.close(); return 133; 
    		    }

    		int[][] tempc = new int[nve][nel];
		
    		for(index=0;index<nel;index++)
    		{
    		    for(jdx=0;jdx<nve;jdx++)
    		    {
    			erg=eingabe.nextToken();
    			if (!(erg==eingabe.TT_NUMBER))
    			{ 
    			    in.close(); return 134; 
    			}
    			tempc[jdx][index]=(int)eingabe.nval;
    		    }	
    		}
		
    		erg=eingabe.nextToken();		
    		if (!((erg==eingabe.TT_WORD) && (eingabe.sval.equals("KNPR"))))
    		{
    		    in.close(); return 135; 
    		}
		
    		int[] tempni = new int[nvt];
	
    		for(index=0;index<nvt;index++)
    		{
    		    erg=eingabe.nextToken();
    		    if (!(erg==eingabe.TT_NUMBER))
    		    {
    			in.close(); return 136; 
    		    }
    		    tempni[index]=(int)eingabe.nval;
    		}
		
    		erg=eingabe.nextToken();		
    		if (!((erg==eingabe.TT_WORD) && (eingabe.sval.equals("KMM"))))
    		{ 
    		    in.close(); return 137; 
    		}

    		in.close();
		
		if (!onlytri)
		{
    		for(index=0; index<nvt;index++)
    		    if (tempni[index]==0)
    			addNode(new Node((int)(cc.globalscale*(tempn[0][index]+offx)),
  			                 (int)(cc.globalscale*(tempn[1][index]+offy)),0));
    		    else
    			addNode(new BoundaryNode(tempn[0][index]+(double)offseg,
    						 cc.d.db.getBoundary(tempni[index]-1+offbound)));
		}
		else
		{
    		  for(index=0; index<nvt;index++)
		  {
		    Node nobby = new Node((int)(cc.globalscale*(tempn[0][index]+offx)),
  			           (int)(cc.globalscale*(tempn[1][index]+offy)),0);
                    nobby.setNPR(tempni[index]);
    	            addNode(nobby);
		  }
					 
		}
		
    		for(index=0; index<nel; index++)
    		    if (nve==3)
    		    {
    			/*
    			  Node[] temp = {getNode(tempc[0][index]-1),
    			  getNode(tempc[1][index]-1),
    			  getNode(tempc[2][index]-1)};
    			  addCell(new Tri(0,temp));
    			*/
    		    }
    		    else
    		    {
    			temp = new Edge[4];
			
    			for (int i=0;i<4;i++)
    			{
  			    // FLICKEN
  			      Node a = getNode(tempc[i][index]-1+offnode);
  			      Node b = getNode(tempc[(i+1)%4][index]-1+offnode);
  			    // ENDE FLICKEN
			    
    			    temp[i]=searchEdge(a,b);
			    
    			    if (temp[i]==null)
    				temp[i]=addEdge(new Edge(a,b));
    			} 
			
    			addCell(new Quad(0,temp[0],temp[1],temp[2],temp[3]));
    		    }
		
    		tempn=null;
    		tempc=null;
    		tempni=null;	
    	    }
    	}
    	catch(IOException e)
    	{  	
    	    //	    System.err.println("Fehler beim Dateilesen");
    	    return 100;
    	}
		    
	    cc.dc.world_x1=0;
	    cc.dc.world_x2=0;
	    cc.dc.world_y1=0;
	    cc.dc.world_y2=0;
	    
	    int[] peri = getPerimeter();
	    
	    
	    cc.dc.world_x1=peri[0];
	    cc.dc.world_x2=peri[2];
	    cc.dc.world_y1=peri[1];
	    cc.dc.world_y2=peri[3];
	    	    
    	    if (redraw) centerDomain();
    	    updateAll();
   
    	    return 0;		
	    


        }
    

//      /**********************************************************************/
//      /*  ROUTINES FOR FEAST FORMAT */
//      /**********************************************************************/
    
      /**
       *  saves the domain in FEAST format
       *  @param fN - full qualified path+filename of the target file
       */
  private void saveDomainFEAST (String fN)
  {
    Boundary be;
    int sc,maxpb;
    Segment s;
    String fileName;
	
    try
    {
	    
      maxpb=0;
	    
      if (fN.indexOf(".feast")>0)
        fileName=fN.substring(0,fN.indexOf(".feast"));
      else 
        fileName=fN;
	    
      FileOutputStream writeFile = new FileOutputStream(fileName+".feast");
      DataOutputStream out = new DataOutputStream( writeFile );
	    
      int cn = getNodeCount();
      int cco = getCellCount();
      int ce = getEdgeCount();
	    
      int cb = db.getBoundaryCount();
	    
      out.writeBytes("FEAST\n");
      out.writeBytes("2D\n");
      out.writeBytes("2\n");
      out.writeBytes("2\n");
	    
      out.writeBytes("# FEAST macro grid created by DeViSoRGrid2\n");
	    
      maxpb=getMaxParallelBlock();
	    
      out.writeBytes(maxpb+" # count of parallel blocks\n");
      out.writeBytes(cn+" # number of macro nodes\n");
      out.writeBytes(cco+" # number of macros\n");
      out.writeBytes(ce+" # number of macro edges\n");
	    
      out.writeBytes(cb+" # number of boundaries\n");
	    
      for(int i=0; i<cb; i++)
      {
        Boundary b =db.getBoundary(i);
		
        int segc=b .getSegmentCount();
		
        out.writeBytes(segc+" # segment number of boundary "+i+"\n");
		
        for (int j=0; j<segc; j++)
        {
          Segment seg=b.getSegment(j);
		    
          if (seg instanceof SegmentLine)
          {
            out.writeBytes("0     # SegmentLine "+i+" "+j+"\n");
            int[] prm=seg.getParams();
			
            out.writeBytes((cc.gc.fromWorldToUser(prm[0]))+" "+
	      (cc.gc.fromWorldToUser(prm[1]))+"\n");
            out.writeBytes((cc.gc.fromWorldToUser(prm[2]))+" "+
	      (cc.gc.fromWorldToUser(prm[3]))+"\n");
          }
          else if (seg instanceof SegmentCircle)
          {
            out.writeBytes("1     # SegmentCircle "+i+" "+j+"\n");
            int[] prm=seg.getParams();
			
            out.writeBytes(cc.gc.fromWorldToUser(prm[0])+" "+
	      cc.gc.fromWorldToUser(prm[1])+"\n");
            out.writeBytes(cc.gc.fromWorldToUser(prm[2])+" 0.0E0\n");
            out.writeBytes((prm[3]/180.0*Math.PI)+" "+(prm[4]/180.0*Math.PI)+"\n");
          }
		    
        }
		
      }
	    
      out.writeBytes("#\n");
	    
      Node nobby;
      int index;
	    
      for(index=0;index<cn; index++)
      {
        nobby=getNode(index);
		
        if (nobby instanceof BoundaryNode)
          out.writeBytes(((BoundaryNode)nobby).getParaValue()+" 0.0 "+(((BoundaryNode)nobby).getBoundaryNumber()+1)+"\n");
        else 
          out.writeBytes(cc.gc.fromWorldToUser(nobby.getCoord(0))+" "
	  +cc.gc.fromWorldToUser(nobby.getCoord(1))+" 0 \n");
      }
	    
      out.writeBytes("#\n");
	    
      Edge e;
	    
      updateNodeNumbers();
	    
      for(index=0;index<ce;index++)
      {
        e=getEdge(index);
		
        out.writeBytes((e.getNode(0).getNumber()+1)+" "+(e.getNode(1).getNumber()+1)+" "+e.getStatus()+" "+e.getBC()+"\n");
      }
	    
      out.writeBytes("#\n");
	    
      Cell zell;
      int nn;
	    
      updateEdgeNumbers();
      updateCellNumbers();
      updateCellNodes();
	    
      for(index=0;index<cco;index++)
      {
        zell=getCell(index);
		
        if ((zell instanceof Macro) || (zell instanceof Quad))
        {
          out.writeBytes((((Cell2D)zell).getNode(0).getNumber()+1)+" "+(((Cell2D)zell).getNode(1).getNumber()+1)+" "+(((Cell2D)zell).getNode(2).getNumber()+1)+" "+(((Cell2D)zell).getNode(3).getNumber()+1)+"\n");
          out.writeBytes((((Cell2D)zell).getEdge(0).getNumber()+1)+" "+(((Cell2D)zell).getEdge(1).getNumber()+1)+" "+(((Cell2D)zell).getEdge(2).getNumber()+1)+" "+ (((Cell2D)zell).getEdge(3).getNumber()+1)+"\n");
		    
          int[] eddes = new int[4];
		    
          for(int i=0;i<4;i++)
          {
            Edge eddy1=((Cell2D)zell).getEdge(i);
			
            Cell zell1= new Quad();
			
            Object[] parents1 = eddy1.getParents();
			
            if (parents1.length>1)
            {
              if ((Cell2D)parents1[0]==((Cell2D)zell))
                zell1=(Cell2D)parents1[1];
              else
                zell1=(Cell2D)parents1[0];
			    
              eddes[i]=zell1.getNumber()+1;
			    
            }
            else
              eddes[i]=0;
          }
		    
          out.writeBytes(eddes[0]+" "+eddes[1]+" "+eddes[2]+" "+eddes[3]+"\n");
		    
          int[] digidx = new int[Math.max(cco,4)];
          int[][] digs = new int[Math.max(cco,4)][8];
		    
          for (int i=0;i<cco;i++)
          {
            digidx[i]=0;
            digs[i][0]=-1; digs[i][1]=-1; digs[i][2]=-1; digs[i][3]=-1;
            digs[i][4]=-1; digs[i][5]=-1; digs[i][6]=-1; digs[i][7]=-1;
          }
		    
          for(int i=0;i<4;i++)
          {
            Edge eddy1=((Cell2D)zell).getEdge(i);
            Edge eddy2=((Cell2D)zell).getEdge((i+1)%4);
			
            Cell zell1= new Quad();
            Cell zell2= new Quad();
			
            Object[] parents1 = eddy1.getParents();
            Object[] parents2 = eddy2.getParents();
			
            if (parents1.length>1)
            {
              if ((Cell2D)parents1[0]==((Cell2D)zell))
                zell1=(Cell2D)parents1[1];
              else
                zell1=(Cell2D)parents1[0];
            }
			
            if (parents2.length>1)
            {
              if ((Cell2D)parents2[0]==((Cell2D)zell))
                zell2=(Cell2D)parents2[1];
              else
                zell2=(Cell2D)parents2[0];
            }
			
            Node a1=eddy1.getNode(0);
            Node a2=eddy1.getNode(1);
            Node b1=eddy2.getNode(0);
            Node b2=eddy2.getNode(1);
			
            Node n = new Node();
			
            if (a1==b1) 
              n=a1;
            else 
              if (a1==b2) 
                n=a1;
              else 
                if (a2==b1) 
                  n=a2;
                else 
                  if (a2==b2) 
                    n=a2;
                  else
                    System.out.println("FEHLER");

            Object[] parentes = n.getParents();
			
            for(int j=0;j<parentes.length;j++)
            {
              Object[] marentes = ((Edge)parentes[j]).getParents();
              if (marentes!=null)
                for(int k=0;k<marentes.length;k++)
              {
                if ( ((Cell2D)marentes[k]!=zell) 
                  && ((Cell2D)marentes[k]!=zell1)
                  && ((Cell2D)marentes[k]!=zell2) )
                {
                  int newidx = ((Cell2D)marentes[k]).getNumber();
                  boolean da = false;
                  for (int l=0;(l<digidx[i]) && (!da);l++)
                  {
                    da=(digs[i][l]==newidx);
                  }

                  if (!da)
                  {
                    digs[i][digidx[i]]=newidx;
                    digidx[i]++;
                  }
                }
              }
            }
          }
		    
          for (int i=0;i<4;i++)
            if (digidx[i]==0) 
              out.writeBytes("0 ");
            else 
              if (digidx[i]==1) 
                out.writeBytes((digs[i][0]+1)+" ");
              else  
                out.writeBytes((-digidx[i])+" ");
          out.writeBytes("\n");

          for (int i=0;i<4;i++)
            if (digidx[i]>1)
            {  
              for (int j=0;j<digidx[i]; j++) 
                out.writeBytes((digs[i][j]+1)+" " );
			    
              out.writeBytes("\n");
            }
		    
        }
        else
          System.out.println("not yet implemented");
		
        out.writeBytes((((Cell2D)zell).getParallelBlock())+" \n");
        out.writeBytes((((Cell2D)zell).getMatrixBlock())+" \n");
		
        if (zell instanceof Macro )
          out.writeBytes((((Macro)zell).getRefinement(0))
            +" "+(((Macro)zell).getAnisotropyMode())
            +" "+(((Macro)zell).getAnisotropyFactors()[0])
            +" "+(((Macro)zell).getAnisotropyFactors()[1])
            +" "+(((Macro)zell).getAnisotropyFactors()[2])+"\n");
        else
          out.writeBytes("1\n");
		
        out.writeBytes("#\n");
		
      }
	    
      out.close();
	    
//System.out.println("File "+fileName+".feast saved!");
      changed=false;
	    
    }
    catch(IOException e)
    {
      System.err.println("IO-Error: "+e);
    }
  }
    

//      /**
//       *  loads the domain description from the specified file
//       *  @param fN - full qualified path and file name
//       */
  public int loadDomainFEAST(String fN)
  {
    int erg,bec,index,nec,jdx,typ,nspline,npar;

    String dateiname,buffer;
    	
    if (fN.indexOf(".feast")>0)
      dateiname=fN.substring(0,fN.indexOf(".feast"));
    else
      dateiname=fN;

    try
    {
      BufferedReader in;
      StreamTokenizer eingabe;
				
      in = new BufferedReader(new FileReader(dateiname+".feast"));

      double[] testr = new double[16];
      int[] testi = new int[16];
      String[] tests = new String[16];
		
      while (in.ready())
      {
        buffer=in.readLine();
        buffer=in.readLine();
        buffer=in.readLine();
        buffer=in.readLine();
        buffer=in.readLine();

        buffer=in.readLine();
        scanLine(buffer,"i",testr,testi,tests);
			
        buffer=in.readLine();
        scanLine(buffer,"i",testr,testi,tests);
        int tempnn=testi[0];

        buffer=in.readLine();
        scanLine(buffer,"i",testr,testi,tests);
        int tempc=testi[0];

        buffer=in.readLine();
        scanLine(buffer,"i",testr,testi,tests);
        int tempe=testi[0];


        buffer=in.readLine();
        scanLine(buffer,"i",testr,testi,tests);
        int tempb=testi[0];

        for(int i=0;i<tempb;i++)
        {
          db.addBoundary();

          buffer=in.readLine();
          scanLine(buffer,"i",testr,testi,tests);
          int temps=testi[0];
	  
          for(int j=0;j<temps;j++)
          {
            buffer=in.readLine();
            scanLine(buffer,"i",testr,testi,tests);
            int id=testi[0];

            if (id==0)    // SegmentLine
            {
              buffer=in.readLine();
              scanLine(buffer,"dd",testr,testi,tests);
              double sx=testr[0];
              double sy=testr[1];

              buffer=in.readLine();
              scanLine(buffer,"dd",testr,testi,tests);
              double ex=testr[0];
              double ey=testr[1];

              db.addSegment(db.getBoundary(i),new SegmentLine(cc.gc.fromUserToWorld(sx),
	                                        cc.gc.fromUserToWorld(sy),
						cc.gc.fromUserToWorld(ex),
						cc.gc.fromUserToWorld(ey),
						SegmentLine.GRADIENT));

            }
            else if (id==1)    // SegmentCircle
            {
              buffer=in.readLine();
              scanLine(buffer,"dd",testr,testi,tests);
              double sx=testr[0];
              double sy=testr[1];

              buffer=in.readLine();
              scanLine(buffer,"dd",testr,testi,tests);
              double rx=testr[0];
              double ry=testr[1];

              buffer=in.readLine();
              scanLine(buffer,"dd",testr,testi,tests);
              double ax=testr[0];
              double ay=testr[1];

              db.addSegment(db.getBoundary(i),new SegmentCircle(cc.gc.fromUserToWorld(sx),
	                                          cc.gc.fromUserToWorld(sy),
						  cc.gc.fromUserToWorld(rx),
						  (ax/Math.PI*180.0),
						  (ay/Math.PI*180.0)));
            }
            else
              System.out.println("Yet not supported!");
          }
        }

        buffer=in.readLine();

        for(int i=0;i<tempnn;i++)
        {
          buffer=in.readLine();
          scanLine(buffer,"ddi",testr,testi,tests);
          if (testi[2]==0)
            addNode(new Node(cc.gc.fromUserToWorld(testr[0]),
	                     cc.gc.fromUserToWorld(testr[1]),0));
          else
            addNode(new BoundaryNode(testr[0],cc.d.db.getBoundary(testi[2]-1))); 
        }

        buffer=in.readLine();

        for(int i=0;i<tempe;i++)
        {
          buffer=in.readLine();
          scanLine(buffer,"iiii",testr,testi,tests);
          addEdge(new Edge(getNode(testi[0]-1),getNode(testi[1]-1),
            testi[2],testi[3]));
        }

        buffer=in.readLine();
						
        for(int i=0;i<tempc;i++)
        {
          buffer=in.readLine();
          buffer=in.readLine();
          scanLine(buffer,"iiii",testr,testi,tests);
          addCell(new Macro(0,5,5,getEdge(testi[0]-1),getEdge(testi[1]-1),
            getEdge(testi[2]-1),getEdge(testi[3]-1)));

          buffer=in.readLine();
          buffer=in.readLine();
          scanLine(buffer,"iiii",testr,testi,tests);

          for(int j=0; j<4;j++)
            if (testi[j]<0.0) buffer=in.readLine();

          buffer=in.readLine();
          scanLine(buffer,"i",testr,testi,tests);
          getCell(i).setParallelBlock(testi[0]);

          buffer=in.readLine();
          scanLine(buffer,"i",testr,testi,tests);
          ((Macro)getCell(i)).setMatrixBlock(testi[0]);

          buffer=in.readLine();
          scanLine(buffer,"iiddd",testr,testi,tests);

          ((Macro)getCell(i)).setRefinement(0,testi[0]);
          ((Macro)getCell(i)).setRefinement(1,testi[0]);

          ((Macro)getCell(i)).setAnisotropy(testi[1],
            testr[2],testr[3],testr[4]);

          buffer=in.readLine();
        }
      }

      in.close();

	    cc.dc.world_x1=0;
	    cc.dc.world_x2=0;
	    cc.dc.world_y1=0;
	    cc.dc.world_y2=0;
	    
	    int[] peri = getPerimeter();
	    
	    
	    cc.dc.world_x1=peri[0];
	    cc.dc.world_x2=peri[2];
	    cc.dc.world_y1=peri[1];
	    cc.dc.world_y2=peri[3];
	    
    	    centerDomain();
    	    updateAll();
   
      return 0;

    }
    catch(IOException e)
    {
      System.out.println("Fehler auffetreten");

      return -1;
    }
  }				

    
    
    
    
        /********************************************************************/
        /*  PARSING ROUTINES */
        /********************************************************************/

 
        /**
         *  extracts the next double number in exponential format from the
         *  given Tokenizer
         *  @param in - the Tokenizer to extract from
         *  @return the extracted double value
         */		
        private double nextExpToken(StreamTokenizer in)
        {
    	try
    	{
    	    double base;
    	    int erg;
		
    	    erg = in.nextToken();
		
    	    if (erg==in.TT_WORD)
    		return 0.0;
    	    else
    	    {
    		double vz;			
    		if (erg=='-')
    		{
    		    vz=-1.0;
    		    erg=in.nextToken();
    		}
    		else
    		    vz=1.0;
				
    		base=vz*in.nval;
		
    		erg=in.nextToken();
		
    		if ((erg=='e') || (erg=='E') || (erg=='D') || (erg=='d')) 
    		{
    		    erg=in.nextToken();
		    
    		    double signum;
		    
    		    if (erg=='+')
    			signum=1.0;
    		    else 
    			if (erg=='-')
    			    signum=-1.0;
  			else
    			{
    			    in.pushBack();
    			    signum=1.0;
    			}
		    
    		    erg=in.nextToken();
		    
    		    return base*Math.pow(10.0,signum*in.nval);
    		}
    		else
    		{
    		    in.pushBack();
    		    return base;
    		}
    	    }
	    
    	}
    	catch(IOException e)
    	{
    	    //	    System.out.println("Could not load file...");
    	    return 0.0;
    	}
        }


 


//      /**
//       *  this method scans one line as String and decomposes it into
//       *  double, int and String components which are returned in the
//       *  corresponding arrays
//       *  @param t - the String to scan
//       *  @param format - a String to toggle the number format
//       *  @param dret - an array of double values
//       *  @param iret - an array of int values
//       *  @param sret - an array of String values
//       */
  public void scanLine(String t, String format, double[] dret, 
    int[] iret, String[] sret)
  {
    int toks = 0;
    char test;

    int start=0;
    int end=0;

    while (t.charAt(start)==' ') start++;
		
    end=start;
		
    while ((end<t.length()) && (toks<format.length()))
    {
      test=t.charAt(end);

      if  (test==' ')
      {
        String subs=t.substring(start,end);
				
        start=end+1;

        if (start>=t.length())
          end=start-1;
        else
        {
          while (t.charAt(start)==' ') start++;

          end=start;
        }

        if (format.charAt(toks)=='i')
          iret[toks++]=(new Integer(subs)).intValue();
        else
        { 
          if (format.charAt(toks)=='d')
            dret[toks++]=(new Double(subs)).doubleValue();
          else
            sret[toks++]=subs;
        }
					
//	toks++;
                                
      }
      else
        end++;
		
    }

    if (toks<format.length())
    {
      String subs=t.substring(start,end);
      if (format.charAt(toks)=='i')
        iret[toks++]=(new Integer(subs)).intValue();
      else
      { 
        if (format.charAt(toks)=='d')
          dret[toks++]=(new Double(subs)).doubleValue();
        else
          sret[toks++]=subs;
      }
    }

  }


  

}// end of class



