package devisor2.foundation.base;

import devisor2.foundation.elements.*;
import devisor2.foundation.boundary.*;
import devisor2.foundation.boundary.*;

import java.io.*;
import java.util.*;



/**
 *  This class is an extended version of the original BasicDomain class
 *  which additionally provides support for giving statistics about 
 *  the domain. For convenience, the all-important methods returns
 *  a completely formatted String which just has to be passed to
 *  the displaying GUI dialog. <br>
 *  See the MainActionListener for a description of how this class
 *  is used in contrast to the original Domain class.
 *
 *  @version 0.9 (all but BoundaryNodes supported)
 *  @author Christian Becker, Dominik Goeddeke
 */
public abstract class DomainStatistics extends BasicDomain 
{


    /**
     *  the all-important method :-)
     */
    public String getStatistics ()
    {
	StringBuffer ges = new StringBuffer();

	StringBuffer sb = new StringBuffer();

	Enumeration enum = db.getBoundaries().elements ();
	while (enum.hasMoreElements())
	{
	    Boundary b = (Boundary)enum.nextElement ();
	    sb.append("   #Segments on Boundary "+(b.getNumber())+": "+ b.getSegmentCount()+"\n");
	}

	double ar = 0.0;
	double lar = 0.0;
	
	double hmin = 10000000000000.0;
	double hmax = 0.0;
	
	double gar = 0.0;
	double ghmin = 10000000000000.0;
	double ghmax = 0.0;
	
	enum = cells.elements ();
	while (enum.hasMoreElements ())
	{
	    Node[] nodez = ((Cell2D)enum.nextElement()).getNodes();
	    
	    double[] ee = new double[4];
	    
	    for (int e=0;e<=3;e++)
		ee[e] = Math.sqrt((nodez[e].getCoord(0)-nodez[(e+1)%4].getCoord(0))*
				  (nodez[e].getCoord(0)-nodez[(e+1)%4].getCoord(0)) +
				  (nodez[e].getCoord(1)-nodez[(e+1)%4].getCoord(1))*
				  (nodez[e].getCoord(1)-nodez[(e+1)%4].getCoord(1)));
	    
	    hmin = 10000000000000.0;
	    hmax = 0.0;
	    ar = 0.0;
	    
	    for (int e=0;e<=3;e++)
	    {
		if (ee[e]<hmin) hmin=ee[e];
		if (ee[e]>hmax) hmax=ee[e];
		
		if (ee[e]>ee[(e+1)%4])
		    lar=ee[e]/ee[(e+1)%4];
		else
		    lar=ee[(e+1)%4]/ee[e];
		
		if (lar>ar) ar=lar;
		
	    }
	    
	    if (hmin<ghmin) ghmin=hmin;
	    if (hmax>ghmax) ghmax=hmax;
	    if (ar>gar) gar=ar;
	    
	}

	ges.append ("Statistics for the current domain\n" + "+++++++++++++++++++++++++++++++++\n \n");
	ges.append ("General information:\n");
	ges.append ("hmin = " + ghmin + "\n");
	ges.append ("hmax = "+ghmax+"\n");
	ges.append ("AR = " + gar + "\n");
	ges.append ("\n");
	ges.append ("File and Format: \n");
	ges.append ("Filename: " + domainfilename + "\n");
	ges.append ("Fileformat: " + domainfileformat + "\n");
	ges.append ("\n");
	ges.append ("Object statistics\n");
	ges.append ("#Cells        :" + getCellCount() + "\n");
	ges.append ("#Nodes(All): " + getNodeCount() + "\n");
	ges.append ("#Nodes(Bound.)  :"+getBoundaryNodeCount()+"\n");
	ges.append ("#Nodes(Inner) :"+ (getNodeCount() - getBoundaryNodeCount())+"\n");
	ges.append ("#Edges        :"+getEdgeCount()+"\n");
	ges.append ("#Boundaries   :"+db.getBoundaryCount()+"\n");
	ges.append (sb.toString()+"\n");
	ges.append ("===========================================\n");

	return ges.toString ();
    }


    /**
     *  prints out the whole domain without boundary
     */
    public void printDomain ()
    {
	printNodeVector ();
	printEdgeVector ();
	printCellVector ();
	db.printDomainBoundary ();
    }

    public void printNodeVector ()
    {
	System.out.println ("current Node vector...");
	if (nodes == null)
	{
	    System.out.println (" Domain Node Vector is null");
	}
	else
	{
	    Enumeration enum = nodes.elements ();
	    while (enum.hasMoreElements ())
	    {
		Node n = (Node) enum.nextElement ();
		System.out.print ("Index: " + nodes.indexOf (n));
		System.out.print (", Number: " + n.getNumber ());
		//System.out.print (" (x,y)=("+n.getCoord(0)+","+n.getCoord(1)+")");
		if (n.getFathers() == null)
		{
		    System.out.println (" No parents");
		}
		else
		{
		    System.out.print (" Parents:");
		    Enumeration edges = n.getFathers().elements ();
		    if (!edges.hasMoreElements())
		    {
			System.out.print ("none");
		    }
		    while (edges.hasMoreElements())
		    {
			Edge e = (Edge)edges.nextElement ();
			System.out.print (" "+e.getNumber ());
		    }
		}
		if (n instanceof BoundaryNode)
		{
		    BoundaryNode b = (BoundaryNode)n;
		    System.out.print ("; on "+b.getBoundary().getNumber()+" @segment "+b.getSegment()+" w/ par "+b.getParaValue());
		}
		System.out.println ();
	    }
	}
    }

    public void printEdgeVector ()
    {
	System.out.println ("current Edge vector");
	if (edges == null)
	{
	    System.out.println ("Domain Edge Vector is null");
	}
	else
	{
	    Enumeration enum = edges.elements ();
	    while (enum.hasMoreElements ())
	    {
		Edge e = (Edge) enum.nextElement ();
		System.out.print ("Index: " + edges.indexOf (e));
		System.out.print (", Number: " + e.getNumber ());
		System.out.print (", n0 = "+e.getNode(0).getNumber());
		System.out.print (", n1 = "+e.getNode(1).getNumber());
		if (e.getFathers() == null)
		{
		    System.out.println (", no parents");
		}
		else
		{
		    System.out.print (", Parents:");
		    Enumeration cells = e.getFathers().elements ();
		    while (cells.hasMoreElements())
		    {
			Cell c = (Cell)cells.nextElement ();
			System.out.print (" "+c.getNumber ());
		    }
		    System.out.println ();
		}
	    }
	}
    }


    public void printCellVector ()
    {
	System.out.println ("current Cell vector");
	if (cells == null)
	{
	    System.out.println (" Domain Cell Vector is null");
	}
	else
	{
	    Enumeration enum = cells.elements ();
	    while (enum.hasMoreElements ())
	    {
		Cell c = (Cell) enum.nextElement ();
		System.out.print ("Index: " + cells.indexOf (c));
		System.out.print (", Number: " + c.getNumber ());
		if (c.getParent() == null)
		    System.out.print (", no parent ");
		else
		    System.out.print (", Parent " + c.getParent().getNumber ());
		if (c instanceof Tri)
		    System.out.println (", Type: Tri");
		if (c instanceof Quad)
		    System.out.println (", Type: Quad");
		if (c instanceof Macro)
		    System.out.println (", Type: Macro");
		//print neighbour cells by number
		if (c.getNeighbours() == null)
		{
		    System.out.println ("No neighbours");
		}
		else
		{
		    System.out.print ("Neighbours: ");
		    for (int i=0; i<c.getNeighbours().length-1;i++)
		    {
			System.out.print (c.getNeighbours()[i].getNumber ());
		    }	
		    System.out.println (c.getNeighbours()[c.getNeighbours().length-1].getNumber ());
		}
		// print children cells by number
		if (c.getChildren() == null)
		{
		    System.out.println ("No children");
		}
		else
		{
		    System.out.print ("children: ");
		    for (int i=0; i<c.getChildren().length-1;i++)
		    {
			System.out.print (c.getChildren()[i].getNumber ());
		    }	
		    System.out.println (c.getChildren()[c.getChildren().length-1].getNumber ());
		}
		// print nodes by number
		if (c.getNodes() == null)
		{
		    System.out.println ("No nodes");
		}
		else
		{
		    System.out.print ("nodes: ");
		    for (int i=0; i<c.getNodes().length-1;i++)
		    {
			System.out.print (c.getNodes()[i].getNumber () + " ");
		    }	
		    System.out.println (c.getNodes()[c.getNodes().length-1].getNumber ());
		}
	    }
	}
    }


}
