/**
 * This class SegmentFile describes the segmenttyp  File.
 *
 * @author Ch. Becker <BR>
 *
 * This module is part of the DeViSoR project
 * by Ch.Becker/C.Waguet
 *
 * @version 0.00.02, 3/22/98
 */
 
package devisor2.foundation.boundary;

import devisor2.foundation.elements.*;
import devisor2.foundation.base.*;
import java.awt.*;
import java.awt.geom.*;
import java.io.*;

public abstract class SegmentFile extends Segment implements java.io.Serializable
{

/**
 * name of the parametrisation file
 */
  public String name;
  public String path;


  AffineTransform gtf;

/**
 * link to the external sources 
 */
  parq2d fcall;

  double geps;

/**
 * boundary number
 */
  public int BoundaryNumber;

/**
 * resolution of the drawing
 */
  public int drawstep;

/*********************************************************/


/**
 *  creates the segment type file
 *  @param String n Name of the parametrisation file
 *  @param int bn boundary number
 *  @param int ds drawing step size
 */
  public SegmentFile(String n, String n2,int bn, int ds)
  {
    super(1.0);
    name=n;
    path=n2;
    BoundaryNumber=bn;
    drawstep=ds;
    gtf= new AffineTransform();
    geps=0.2;

    fcall = new parq2d(path+"lib"+name+".so");

    tmax=fcall.Tmax(bn);
		
  }
	

  public static void compile(String n, String n2)
  {
    try
    {
      FileOutputStream writeFile = new FileOutputStream(n2+n+".c");
      DataOutputStream out = new DataOutputStream( writeFile );
		
      out.writeBytes("#include <jni.h>\n");
      /*out.writeBytes("#include \"parq2d.h\"\n");*/

      out.writeBytes("double parx(double *p1, long int *n);\n");
      out.writeBytes("double pary(double *p1, long int *n);\n");
      out.writeBytes("double tmax(long int *n);\n");

      out.writeBytes(
   "JNIEXPORT jdouble JNICALL Java_devisor_foundation_boundary_parq2d_ParX\n");

      out.writeBytes("  (JNIEnv* ep, jobject this, jdouble t, jint n)\n");
      out.writeBytes("{\n");
      out.writeBytes("	return parx(&t,&n);\n");
      out.writeBytes("}\n");

      out.writeBytes(
   "JNIEXPORT jdouble JNICALL Java_devisor_foundation_boundary_parq2d_ParY\n");

      out.writeBytes("  (JNIEnv* ep, jobject this , jdouble t, jint n)\n");
      out.writeBytes("{\n");
      out.writeBytes("	return pary(&t,&n);\n");
      out.writeBytes("}\n");

      out.writeBytes(
   "JNIEXPORT jdouble JNICALL Java_devisor_foundation_boundary_parq2d_Tmax\n");

      out.writeBytes("  (JNIEnv* ep, jobject this, jint n)\n");
      out.writeBytes("{\n");
      out.writeBytes("	return tmax(&n);\n");
      out.writeBytes("}\n");

      out.close();
		
      Runtime RT = null;
      Process PR = null;
		
      RT = Runtime.getRuntime();
      PR = RT.exec("f77 -c -ext_names=plain "+n2+n+".f -o "+n2+n+"f.o");

      //System.out.println("f77 -c -ext_names=plain "+n2+n+".f -o "+n2+n+"f.o");

      PR = RT.exec(
        "cc -G -I/usr/local/jdk/include  -I/usr/local/jdk/include/solaris "
        +n2+n+"f.o "+n2+n+".c -o "+n2+"lib"+n+".so");

    }
    catch (IOException e)
    {
      System.out.println("Compile-Error");
    }

  }


/**
 *  creates the segment type file with draw step size 256
 *  @param String n Name of the parametrisation file
 *  @param int bn boundary number
 */
  public SegmentFile(String n, String n2,int bn)
  {
    super(1.0);
    //    new SegmentFile(n,n2,bn,256);
  }

	
/*********************************************************/


  public void performTransformation(AffineTransform tf, boolean preserve)
  {
    gtf=tf;
  }


/**
 * @see Segment.isOnBoundary
 */
  public double[] isOnBoundary(Node n, double eps)
  {
    double[] erg = new double[3];

    double tx = n.getCoord(0);
    double ty = n.getCoord(1);
    geps=eps;
                  
    double t=0.0;
		
    boolean found=false;

    for(int index=0;(index<drawstep*10)&&(!found);index++)
    {
      double cx=fcall.ParX(t,BoundaryNumber);
      double cy=fcall.ParY(t,BoundaryNumber);

      found=(Math.sqrt((cx-tx)*(cx-tx)+(cy-ty)*(cy-ty))<eps);
                      
      t=t+tmax/(drawstep*10-1);
    }

    if (found)
      erg[0]=-1.0;
    else
      erg[0]=0.0;

    return erg;
  }
	

/**
 * @see Segment.getPolyCoords
 */	
  public int[] getPolyCoords(AffineTransform tf)
  {
    int[] erg = new int[4];

    System.out.println("Yet not implemented 1!!");

    return erg;
  }


 //     /**
//       * @see DomainBoundary.getPerimeter
//       */
//      public int[] getPerimeter()
//      {
//  	int[] erg = new int[4];
//  	int[] help = new int[2];
//  	double t;
//  	int index;
	
//  	erg[0]=524244;
//  	erg[1]=524244;
//  	erg[2]=-524244;
//  	erg[3]=-524244;
	
//  	t=0.0;
	
//  	for(index=0;index<drawstep;index++)
//  	    {
//  		help[0]=(int)(fcall.ParX(t,BoundaryNumber)*100000);
//  		help[1]=(int)(fcall.ParY(t,BoundaryNumber)*100000);
		
//  		if (help[0]<erg[0]) erg[0]=help[0];
//  		if (help[1]<erg[1]) erg[1]=help[1];
//  		if (help[0]>erg[2]) erg[2]=help[0];
//  		if (help[1]>erg[3]) erg[3]=help[1];
		
//  		t=t+tmax/(drawstep-1);
//  	    }
	
//  	return erg;
	
//      }

    public abstract Rectangle getPerimeter ();
    
	
/**
 * @see Segment.getCoords
 */	
  public Node getCoords(double t)
  {
    Node temp=new Node((int)(fcall.ParX(t,BoundaryNumber)*100000),
    (int)(fcall.ParY(t,BoundaryNumber)*100000));
		
    //System.out.println("t="+t);
    return temp;
  }


/**
 * draws the segment
 */
  public void draw(Graphics g)
  {
    double[] help=new double[3];
    double[] x1;
    double[] x2;
		
    int[] hx = new int[drawstep];
    int[] hy = new int[drawstep];
		
    int index;
    double t;
		
    t=0.0;
		
    for(index=0;index<drawstep;index++)
    {
      help[0]=fcall.ParX(t,BoundaryNumber);
      help[1]=fcall.ParY(t,BoundaryNumber);

//      x2 = gtf.transform( help );
//      x1 = tf.transform( x2 );
//      hx[index]=(int)x1[0];
//      hy[index]=Display.VISY-(int)x1[1];
                       
      t=t+tmax/(drawstep-1);
    }

    g.setColor(Color.black);
		
//    for(index=0; index<(drawstep-1); index++)
//      g.drawLine(hx[index],hy[index],hx[index+1],hy[index+1]);

  }
	

  public double getParameterCoord(Node n)
  {
    double[] erg = new double[3];

    double tx = n.getCoord(0);
    double ty = n.getCoord(1);
                  
    double t=0.0;
		
    boolean found=false;

    for(int index=0;(index<drawstep*10)&&(!found);index++)
    {
      double cx=fcall.ParX(t,BoundaryNumber);
      double cy=fcall.ParY(t,BoundaryNumber);

      found=(Math.sqrt((cx-tx)*(cx-tx)+(cy-ty)*(cy-ty))<geps);
                      
      t=t+tmax/(drawstep*10-1);
    }

    if (found)
      return t;
    else
      return 0.0;

  }


  public void swapPerimeter()
  {
  }


  public void adjustPerimeter(Node n, int index)
  {
  }

  public void setParams(int ux[])
  {
  }



}

