package devisor2.foundation.boundary;

import devisor2.foundation.elements.*;
import devisor2.foundation.*;
import devisor2.grid.options.*;
import devisor2.grid.GUI.framework.*;
import devisor2.grid.backend.*;
import java.awt.*;
import java.awt.geom.*;
import java.io.*;

/**
 *  This class represents the segment type LINE, a straight line between two
 *  given points. As it can be messed with on the grid, it indirectly extends
 *  the GridItem class
 *
 *  @author Dominik Goeddeke, Christian Becker
 */
public class SegmentLine extends Segment implements java.io.Serializable
{
    
    
    /**
     * start point of the line
     */
    private int x,y;
    
    /**
     * gradient of the line
     */
    private int sx,sy;

    /**
     *  a sort of bounding box or rectangular area for picking purposes
     */
    private Polygon poly;
    private Polygon dpoly;
    
    
    /**
     *  constant to indicate that a SegmentLine is initialised via startpoint
     *  and gradient vector
     */
    public static final boolean GRADIENT = true;
    /**
     *  constant to indicate that a SegmentLine is initialised via startpoint
     *  and end point coordinates
     */
    public static final boolean TWOPOINT = false;
  
    
    /*********************************************************/


    /**
     *  creates the segment type Line
     * @param stmax length of the line
     * @param tx,ty start point of the line
     * @param tsx,tsy gradient of the line or end point
     * @param mode GRADIENT or TWOPOINT
     */
    public SegmentLine(int tx, int ty, int tsx, int tsy, boolean mode)
    {
	super(1.0);
	x=tx; y=ty;
	if (mode == GRADIENT)
	{
	    sx=tsx; 
	    sy=tsy;
	}
	else
	{
	    sx = tsx - tx;
	    sy = tsy - ty;
	}
	updateInnerData ();
    }

    
    
    /**
     * creates an empty SegmentLine
     */
    public SegmentLine()
    {
	super(0);
	x=0; y=0;
	sx=0; sy=0;
	updateInnerData ();
    }

    /***************************GETTERS AND SETTERS*******************/
    
    /**
     *  sets the parameters of the line
     */
    public void setParameters(double ux[])
    {
	x = (int)ux[0];
	y = (int)ux[1];
	sx = (int)ux[2];
	sy = (int)ux[3];
	updateInnerData ();
    }
    
    /**
     *  gets the parameters of the line
     */
    public double[] getParameters()
    {
        double[] ux = new double[4];
	
	ux[0]=x;
	ux[1]=y;
	ux[2]=sx;
	ux[3]=sy;
	
	return ux;
    }

    /**
     *  sets the parameters of the line
     */
    public void setParams(int[] ux)
    {
	x = ux[0];
	y = ux[1];
	sx = ux[2];
	sy = ux[3];
	updateInnerData ();
    }
    
    /**
     *  gets the parameters of the line
     */
    public int[] getParams()
    {
        int[] ux = new int[4];
	
	ux[0]=x;
	ux[1]=y;
	ux[2]=sx;
	ux[3]=sy;
	
	return ux;
    }
    
    /**
     *  sets the startpoint of the line
     */
    public void setStartPoint (int x1, int y1)
    {
	x = x1;
	y = y1;
	updateInnerData ();
    }
    
    /**
     *  sets the endpoint of the line
     */
    public void setEndPoint (int x2,int y2)
    {
	sx = x2-x;
	sy = y2 - y;
	updateInnerData ();
    }

    /**
     *  returns the startpoint coordinates
     */
    public int[] getStartPoint ()
    {
	int[] p = {x,y};
	return p;
    }

    /**
     *  returns the endpoint coordinates
     */
    public int[] getEndPoint ()
    {
	int[] p = {x+sx,y+sy};
	return p;
    }


    
    
    /*********************************************************/
    
    /**
     *  
     * @see Segment.isOnBoundary
     */
    public boolean isOnBoundary(int x, int y)
    {
	return isHit (x,y);
    }


    
    
    /**
     * @see DomainBoundary.getIntersection
     */	
    public int[] getIntersection(int[] xx)
    {
	//     double l = (-(xx[0] *y) + xx[2] * y + x 
	//       *xx[1] - xx[2] *xx[1] - x *xx[3] + xx[0] *xx[3])/
	//       (sy *xx[0] - sy * xx[2] - sx * xx[1] + sx * xx[3]);
	
	//     double m = (sy * x - sy * xx[0] - sx * y + sx * xx[1])/
	//       (-(sy * xx[0]) + sy * xx[2] + sx * xx[1] - sx * xx[3]);
	
	//     double[] erg = new double[5];
	
	//     if ((l>=0.0) && (l<=1.0) && (m>=0.0) && (m<1.0))
	//     {
	//       erg[0]=-1;
	//       erg[1]=x+l*sx;
	//       erg[2]=y+l*sy;
	//     }
	//     else
	//       erg[0]=0;
	
	//     return erg;
	return null;
    }
    
    
    /**
     * @see Segment.getParameterValue
     */
    public double getParameterValue (Node n)
    {
	   double l;
	   double m;
	   //System.out.println (n);
    	   if (sx == 0)
	       l=-1;
	   else
	       l = ((double)n.getCoord(0)-x)/sx;
	   if (sy == 0)
	       m=-1;
	   else
	       m = ((double)n.getCoord(1)-y)/sy;
	   //System.out.println ("l="+l+", m="+m);
	   if (((l<0) || (l>1.0)) && (m!=-1))
	   {
	       return Math.min(m,1.0);
	   }
	   else
	   {
	       return Math.min(l,1.0);
	   }
    }
      
    
    /**
     * returns the cartesian coordinates according to the
     * parameter value t
     */
    public Node getCartesianCoords(double t)
    {
	int tx,ty;
	
	tx=(int)(x+(t*sx));
	ty=(int)(y+(t*sy));
	
	Node tmp = new Node(tx,ty);
	
	return tmp;
    }
    
    
    /**
     * @see Segment.swapPerimeter
     */
    public void swapPerimeter()
    {
	x=x+sx;
	y=y+sy;
	sx=-sx;
	sy=-sy;
    }
    
    
    /**
     * @see Segment.adjustPerimeter
     */
    public void adjustPerimeter(Node n, int index)
    {
	if (index==0)
	{
	    sx=sx+x-n.getCoord(0);
	    sy=sy+y-n.getCoord(1);
	    x=n.getCoord(0);
	    y=n.getCoord(1);
	}
	else
	{
	    sx=n.getCoord(0)-x;
	    sy=n.getCoord(1)-y;
	}
    }
    
  /**
     *  Is the item inside a given rectangular area?
     *  All in WORLD COORDINATES!!!
     *
     *  @param topleft - the topleft corner of the rectangle
     *  @param bottomright - the bottom right corner of the rectangle
     *  @return true if the item is completely inside the rectangle
     */
    public boolean isInside (int[] topleft, int[] bottomright)
    {
	return (topleft[0] < xmin && bottomright[0] > xmax && topleft[1] > ymax && bottomright[1] < ymin);
    }

    /**
     *  Determines if the item has been hit by a click on the grid
     *  ALL IN WORLD COORDINATES to avoid the snap mechanism
     *
     *  @param x,y - coordinates of the event point where the click
     *         occurred
     *  @return true if the item has been hit
     */
    public boolean isHit (int cx, int cy)
    {
	return poly.contains (cx,cy);
    }

    /**
     *  updates the inner variables xmin, xmax, ymin, ymax and the polygonal
     *  representation of the line object including the orientation marker
     */
    public void updateInnerData ()
    {
	int[] coord = {x,y,(int)(x+sx),(int)(y+sy)};
	xmax = Math.max (coord[0], coord[2]);
	xmin = Math.min (coord[0], coord[2]);
	ymax = Math.max (coord[1], coord[3]);
	ymin = Math.min (coord[1], coord[3]);

	// get normal vector of this line and scale down to length 1
	double[] n = {sy, sx};
	if (Math.abs (sx) > Math.abs (sy))
	    n[1] = -n[1];
	else
	    n[0] = -n[0];
	double l = Math.sqrt (n[0]*n[0] + n[1]*n[1]);
	n[0] /= l;
	n[1] /= l;
	// get scaled versions of sx and sy
	l = Math.sqrt (((double)sx*(double)sx+(double)sy*(double)sy));
	double[] s = {sx,sy};
	double ssx = (s[0] /= l);
	double ssy = (s[1] /= l);
	// create parallel polygon around line with distance click_tolerance
	poly = new Polygon ();
	poly.addPoint ((int)(coord[0] - cc.dc.tolerance*ssx+n[0]*cc.dc.tolerance), (int)(coord[1]-cc.dc.tolerance*ssy+n[1]*cc.dc.tolerance));
	poly.addPoint ((int)(coord[2]+cc.dc.tolerance*ssx+n[0]*cc.dc.tolerance), (int)(coord[3]+cc.dc.tolerance*ssy+n[1]*cc.dc.tolerance));
	poly.addPoint ((int)(coord[2]+cc.dc.tolerance*ssx-n[0]*cc.dc.tolerance), (int)(coord[3]+cc.dc.tolerance*ssy-n[1]*cc.dc.tolerance));
	poly.addPoint ((int)(coord[0]- cc.dc.tolerance*ssx-n[0]*cc.dc.tolerance), (int)(coord[1]-cc.dc.tolerance*ssy-n[1]*cc.dc.tolerance));
	// update orientation marker variables
	int side = cc.gc.SingleFromScreenToWorld (5);
	Node tmp1 = getCartesianCoords (0.9);
	Node tmp2 = getCartesianCoords (0.85);
	double tmpdx = tmp2.getCoord(0) - tmp1.getCoord(0);
	double tmpdy = tmp2.getCoord(1) - tmp1.getCoord(1);
	l = Math.sqrt (Math.pow (tmpdx,2)+ Math.pow (tmpdy,2));
	tmpdx /= l;
	tmpdy /= l;
	orientation_marker_1 = new int[2];
	orientation_marker_1[0] = tmp1.getCoord(0);
	orientation_marker_1[1] = tmp1.getCoord(1);
	orientation_marker_2 = new int[2];
	orientation_marker_2[0] = (int)(tmp1.getCoord(0) + tmpdx*side + n[0]*side);
	orientation_marker_2[1] = (int)(tmp1.getCoord(1) + tmpdy*side + n[1]*side);
	orientation_marker_3 = new int[2];
	orientation_marker_3[0] = (int)(tmp1.getCoord(0) + tmpdx*side - n[0]*side);
	orientation_marker_3[1] = (int)(tmp1.getCoord(1) + tmpdy*side - n[1]*side);
    }


    /**
     * draw the segment. @see Segment.draw
     */
    public void draw(Graphics g)
    {
	if (marked)
	    g.setColor (cc.dc.color_selection);
	else
	    g.setColor(cc.dc.boundary_palette[getBoundaryNumber()%16]);
	// create points out of gradient and transform manually
	int[] coord = {x,y,(int)(x+sx*tmax),(int)(y+sy*tmax)};
	int[] p1 = cc.gc.fromWorldToScreen(coord[0], coord[1]);
	int[] p2 = cc.gc.fromWorldToScreen(coord[2], coord[3]);
	// perform actual painting
	if (marked)
	{
	    Graphics2D g2d = (Graphics2D)g;
	    g2d.setStroke (new BasicStroke (4));
	    g2d.drawLine (p1[0], p1[1], p2[0], p2[1]);
	    g2d.setStroke (new BasicStroke (1));
	}
	else
	{
	    g.drawLine(p1[0],p1[1],p2[0],p2[1]);
	}
	if (cc.dc.number_boundary)
	{
	    int mpx = (int)((p1[0]+p2[0])/2);
	    int mpy = (int)((p1[1]+p2[1])/2);
	    g.setColor (cc.dc.color_boundarynumber);
	    g.drawString (((getSegmentNumber()+1)+","+(getBoundaryNumber()+1)), mpx,mpy);
	}
	drawOrientationMarker (g);
   }

    /**
     *  returns the perimeter of the Line element, that is the smallest 
     *  bounding box completely containing the Line
     */
    public Rectangle getPerimeter ()
    {
	return new Rectangle (xmin, ymin, xmax-xmin, ymax-ymin);
    }

    /**
     *  transforms the line segment by transforming the two endpoints
     */
    public void performTransformation (GridTransformation trf)
    { 	
	if (!transformed)
  	{
	    AffineTransform trafo = trf.getMatrix ();
  	    transformed=true;
	    double[] tmp1 = {x,y};
	    double[] ntmp1 = new double[2];
	    double[] tmp2 = {x+sx,y+sy};
	    double[] ntmp2 = new double[2];
	    trafo.transform (tmp1,0,ntmp1,0,1);
	    trafo.transform (tmp2,0,ntmp2,0,1);
  	    x = (int)ntmp1[0];
  	    y = (int)ntmp1[1];
	    sx = (int)(ntmp2[0] - ntmp1[0]);
	    sy = (int)(ntmp2[1] - ntmp1[1]);
	    updateInnerData ();
 	}
    }

    /**
     *  This method mirrors the line at the given line by mirroring the
     *  two endpoints
     *  @param n - the normalized normal of the line (length 1)
     *  @param p - an arbitrary point on the line
     */
    public void performMirror (double nx, double ny, double px, double py)
    {
	if (!transformed)
        {
	    int[] coord = {x,y,(int)(x+sx),(int)(y+sy)};
	    transformed = true;
	    // vector v from line to startpoint must point TOWARDS the start
	    double vx = px - coord[0];
	    double vy = py - coord[1];
	    // scalarproduct of n and v is distance from start to line
	    double sp = nx*vx + ny*vy;
	    // perform translation
	    if (sp > 0)
	    {
		x += 2*sp*nx;
		y += 2*sp*ny;
	    }
	    else
		if (sp < 0)
		{
		    x += 2*sp*nx;
		    y += 2*sp*ny;
		}
	    // vector v from line to endpoint must point TOWARDS the end
	    vx = px - coord[2];
	    vy = py - coord[3];
	    // scalarproduct of n and v is distance from node to line
	    sp = nx*vx + ny*vy;
	    // perform translation
	    if (sp > 0)
	    {
		coord[2] += 2*sp*nx;
		coord[3] += 2*sp*ny;
	    }
	    else
		if (sp < 0)
		{
		    coord[2] += 2*sp*nx;
		    coord[3] += 2*sp*ny;
		}
	    sx = coord[2] - x;
	    sy = coord[3] - y;
	    updateInnerData ();
	}
    }

    /**
     *  creates a full clone of the SegmentLine object, including boundary
     *  information
     */
    public Object clone ()
    {
	SegmentLine newline = new SegmentLine (x,y,sx,sy,GRADIENT);
	newline.boundary = boundary;
	newline.setMarker (this.isMarked());
	newline.tmax = tmax;
	return newline;
    }

    /**
     *  @return a String representation of the line object
     */
    public String toString ()
    {
	String s = "SegmentLine from "+x+","+y+" with gradient "+sx+","+sy;
	if (boundary == null)
	    s += " with boundary null";
	else
	    s += " @boundaryno "+getBoundaryNumber()+" segno "+getSegmentNumber()+" tmax "+tmax+" @boundary "+boundary;;
	return s;
    }

  
}

