package devisor2.foundation.elements;

import java.awt.*;
import java.awt.geom.*;
import java.io.*;
import java.util.Vector;

import devisor2.grid.options.*;
import devisor2.grid.GUI.framework.*;
import devisor2.grid.backend.*;

/**
 *  The Macro class represents a Quad which can refine itself nonuniformly.
 *
 *  @author Dominik Goeddeke, Christian Becker
 *  @version 0.5 half way there
 */
public class Macro extends Quad implements java.io.Serializable
{

    /**		
     * Number of nodes of a Macro.
     */
    static final int NPT = 4;

    /**
     * level of refinement in x direction
     */
    protected int n;

    /**
     * level of refinement in y direction
     */
    protected int m;
    
    protected int aniso;
    
    protected double[] anisofaktor;
    
    /**
     * Empty constructor.
     * Calls the empty constructor of Cell,
     * and defines the array of nodes of the cell.
     * @see Cell
     */
    public Macro() 
    {
	super();
	edges = new Edge[NPT];
	n=1;
	m=1;
	aniso=0;
	anisofaktor=new double[3];
	anisofaktor[0]=0.5;
	anisofaktor[1]=0.5;
	anisofaktor[2]=0.5;
    }
    
    
    /**
     * Calls one constructor of Cell,
     * and defines the array of edges of the cell.
     * @see Cell
     * @param int i number of the cell
     * @param int nn,mm level of refinements
     * @param Edge e1,e2,e3,e4 edges of the cell
     */
    public Macro(int i, int nn, int mm, Edge e1, Edge e2,
		 Edge e3, Edge e4) 
    {
	super(0);
	parallelblock=i;
	Edge[] ed = {e1,e2,e3,e4};
	setCell(ed);
	n=nn;
	m=mm;
	aniso=0;
	anisofaktor=new double[3];
	anisofaktor[0]=0.5;
	anisofaktor[1]=0.5;
	anisofaktor[2]=0.5;
	
    }
    
    
    public void setAnisotropy(int aiso, double r1, double r2, double r3)
    {
	aniso=aiso;
	anisofaktor[0]=r1;
	anisofaktor[1]=r2;
	anisofaktor[2]=r3;
    }
    
    
    public int getAnisotropyMode()
    {
	return aniso; 
    }
    
    
    public double[] getAnisotropyFactors()
    {
	return anisofaktor; 
    }
    
    
    public int getRefinement(int index)
    {
	if (index==0)
	    return n-1;
	else
	    return m-1;
    }
    
    
    public void setRefinement(int index, int nr)
    {
	if (index==0)
	    n=nr+1;
	else
	    m=nr+1;
    }
    
    
    /**
     * Returns the ith edge of the cell modulo NPT,
     * i.e. for a Quad modulo 4.
     * @see Edge
     */  
    public Edge getEdge( int i ) 
    {
      return edges[i%NPT];
    }
  

    /**
     * Draws the perimeter of the macro.
     */
    public void draw(Graphics g) 
    {
	// set Color
        g.setPaintMode ();
	
 	if (marked)
	    g.setColor (cc.dc.color_selection);
	else
	    g.setColor(cc.dc.parallel_palette[(parallelblock-1)%16]);
	    
	// get nodes first with respect to order
	Node[] nodes = updateCellNodes ();
	// manually transform to screen coords
	int[] n0 = cc.gc.fromWorldToScreen(nodes[0].getCoord(0), nodes[0].getCoord(1));
	int[] n1 = cc.gc.fromWorldToScreen(nodes[1].getCoord(0), nodes[1].getCoord(1));
	int[] n2 = cc.gc.fromWorldToScreen(nodes[2].getCoord(0), nodes[2].getCoord(1));
	int[] n3 = cc.gc.fromWorldToScreen(nodes[3].getCoord(0), nodes[3].getCoord(1));
		    	
	// paint lines
	if (marked)
	{
	    Graphics2D g2d = (Graphics2D)g;
	    g2d.setStroke (new BasicStroke (4));
	    g2d.drawLine (n0[0],n0[1],n1[0],n1[1]);
	    g2d.drawLine (n1[0],n1[1],n2[0],n2[1]);
	    g2d.drawLine (n2[0],n2[1],n3[0],n3[1]);
	    g2d.drawLine (n3[0],n3[1],n0[0],n0[1]);
	    g2d.setStroke (new BasicStroke (1));
	}
	else
	{
	    g.drawLine (n0[0],n0[1],n1[0],n1[1]);
	    g.drawLine (n1[0],n1[1],n2[0],n2[1]);
	    g.drawLine (n2[0],n2[1],n3[0],n3[1]);
	    g.drawLine (n3[0],n3[1],n0[0],n0[1]);
	}
        double hx1,hx2,hy1,hy2;
        int step,i;
	
        step=0;

        double[] f = new double[256];
        double[] fs = new double[256];

        f[0]=0.0; f[1]=1.0;

        int oldref=2;

        if (aniso!=0)
        {
          for(i = 1;i<=(n-1);i++)
          {

            int fakidx=0;

            int ref = 2*(oldref-1)+1;

            fs[0]=f[0];
            boolean toggle = false;
            for(int j=1;j<=ref-2;j++)
            {
              if (!toggle)
                fs[j]=f[j/2]+anisofaktor[fakidx++]*(f[j/2+1]-f[j/2]);
              else
                fs[j]=f[j/2];

              toggle=!toggle;

              fakidx=Math.min(fakidx,2);
           }

           fs[ref-1]=f[oldref-1];
                  
           oldref=ref;
      
           for(int j=0;j<=255;j++) f[j]=fs[j];
         }

         if ( ((aniso & 4)==4))
           for(int j=0;j<=255;j++) f[j]=1.0-fs[j];

         if ( ((aniso & 8)==8))
           for(int j=0;j<=255;j++) fs[j]=1.0-fs[j];

       }

        if (cc.dc.cross_element)
        {	
          g.setColor(cc.dc.parallel_palette[(matrixblock-1)%16]);

          g.drawLine(n0[0], n0[1],
                     n2[0], n2[1]);

          for(int i1=1;i1<5;i1++)
          {
            g.drawLine(n0[0]-i1, n0[1],
                       n2[0]-i1, n2[1]);
            g.drawLine(n0[0] + i1, n0[1],
                       n2[0]+i1, n2[1]);
          }

          g.drawLine(n1[0], n1[1],
                     n3[0], n3[1]);

          for(int i1=1;i1<5;i1++)
          {      
            g.drawLine(n1[0]-i1, n1[1],
                       n3[0]-i1, n3[1]);
            g.drawLine(n1[0] + i1, n1[1],
                       n3[0]+i1, n3[1]);
          }

        }



     if ((cc.dc.highlight_rect) && isRect())
     {
       g.setColor(cc.dc.color_rect);

       int [] x = {n0[0],n1[0], n2[0],n3[0]};
       int [] y = {n0[1],n1[1], n2[1], n3[1]};

       Polygon poly = new Polygon( x, y, 4 );

       g.fillPolygon( poly );

     }
    
	g.setColor(cc.dc.parallel_palette[(parallelblock-1)%16]);

	  
       hx1=(n1[0]-n0[0])/Math.pow(2.0,(double)(n-1));
       hx2=(n2[0]-n3[0])/Math.pow(2.0,(double)(n-1));
       hy1=(n1[1]-n0[1])/Math.pow(2.0,(double)(m-1));
       hy2=(n2[1]-n3[1])/Math.pow(2.0,(double)(m-1));
	  
       step=(int)Math.pow(2.0,(double)(n-1));

       if ((aniso & 1) == 1)
       {	  
         for(i=1;i<step;i++)
         {
           g.drawLine((int)(n0[0]+f[i]*(n1[0]-n0[0])),
                      (int)(n0[1]+f[i]*(n1[1]-n0[1])),
                      (int)(n3[0]+f[i]*(n2[0]-n3[0])),
                      (int)(n3[1]+f[i]*(n2[1]-n3[1])));
         }
       }
       else
       {
         for(i=1;i<step;i++)
         {
           g.drawLine((int)(n0[0]+i*hx1),(int)(n0[1]+i*hy1),
                      (int)(n3[0]+i*hx2),(int)(n3[1]+i*hy2));
         }
       }

       hx1=(n3[0]-n0[0])/Math.pow(2.0,(double)(n-1));
       hx2=(n2[0]-n1[0])/Math.pow(2.0,(double)(n-1));
       hy1=(n3[1]-n0[1])/Math.pow(2.0,(double)(m-1));
       hy2=(n2[1]-n1[1])/Math.pow(2.0,(double)(m-1));
	  
       step=(int)Math.pow(2.0,(double)(n-1));

       if ((aniso & 2) == 2)
       {        
         for(i=1;i<step;i++)
         {
           g.drawLine((int)(n0[0]+fs[i]*(n3[0]-n0[0])),
                      (int)(n0[1]+fs[i]*(n3[1]-n0[1])),
                      (int)(n1[0]+fs[i]*(n2[0]-n1[0])),
                      (int)(n1[1]+fs[i]*(n2[1]-n1[1])));

         }
       }
       else
       {

         for(i=1;i<step;i++)
         {
           g.drawLine((int)(n0[0]+i*hx1),(int)(n0[1]+i*hy1),
                      (int)(n1[0]+i*hx2),(int)(n1[1]+i*hy2));
         }
       }


        if (cc.dc.fill_element)
        {
          g.setColor(cc.dc.parallel_palette[(parallelblock-1)%16]);

          int [] x = {n0[0],n1[0],n2[0],n3[0]};
          int [] y = {n0[1],n1[1],n2[1],n3[1]};
          Polygon poly = new Polygon( x, y, 4 );
          g.fillPolygon( poly );
         }

	// paint numbering
	if (cc.dc.number_element)
	{
	    g.setColor (cc.dc.color_elementnumber);
	    int middleX = (int)((n0[0]+n1[0]+n2[0]+n3[0])/4);
	    int middleY = (int)((n0[1]+n1[1]+n2[1]+n3[1])/4);
	    g.setFont(cc.dc.font_numbering);
	    g.drawString((number+1)+"/"+parallelblock+
	      "/"+matrixblock,middleX,middleY);
	}

        g.setXORMode (Color.WHITE);

	int[] n4 = new int[2];
	
	n4[0]=(n0[0]+n1[0]+n2[0]+n3[0])/4;
	n4[1]=(n0[1]+n1[1]+n2[1]+n3[1])/4;

	    g.setColor (Color.BLACK);
	    g.drawLine (n0[0],n0[1],n4[0],n4[1]);
	    g.drawLine (n0[0]-1,n0[1]-1,n4[0]-1,n4[1]-1);
	    g.drawLine (n0[0]+1,n0[1]+1,n4[0]+1,n4[1]+1);

        g.setPaintMode ();

  }

    public int getNPT()
    {
	return NPT;
    }

    /**
     * Draws the Macro filled with its color.
     * @see Cell
     */
    public void fill(Graphics g)
    {
//    g.setColor(color);
	
//    double[] x0 = tf.transform( nodes[0].getX() );
//    double[] x1 = tf.transform( nodes[1].getX() );
//    double[] x2 = tf.transform( nodes[2].getX() );
//    double[] x3 = tf.transform( nodes[3].getX() );
      
//    int [] x = {(int)x0[0], (int)x1[0], (int)x2[0], (int)x3[0]};
//    int [] y = {Display.VISY-(int)x0[1], Display.VISY-(int)x1[1],
//      Display.VISY-(int)x2[1], Display.VISY-(int)x3[1]};

//    Polygon poly = new Polygon( x, y, 4 );
     
//    g.fillPolygon( poly );
    }
  

    /**
     *  updates the inner variables xmin, xmax, ymin, ymax and the polygonal
     *  representation of the macro object
     */
    public void updateInnerData ()
    {
	Node[] nds = updateCellNodes ();
	poly_world = new Polygon ();
	poly_world.addPoint (nds[0].coord[0], nds[0].coord[1]);
	poly_world.addPoint (nds[1].coord[0], nds[1].coord[1]);
	poly_world.addPoint (nds[2].coord[0], nds[2].coord[1]);
	poly_world.addPoint (nds[3].coord[0], nds[3].coord[1]);
	xmax = Math.max (Math.max (Math.max (poly_world.xpoints[0], poly_world.xpoints[1]), poly_world.xpoints[2]), poly_world.xpoints[3]);
	xmin = Math.min (Math.min (Math.min (poly_world.xpoints[0], poly_world.xpoints[1]), poly_world.xpoints[2]),poly_world.xpoints[3]);
	ymax = Math.max (Math.max (Math.max (poly_world.ypoints[0], poly_world.ypoints[1]), poly_world.ypoints[2]), poly_world.ypoints[3]);
	ymin = Math.min (Math.min (Math.min (poly_world.ypoints[0], poly_world.ypoints[1]), poly_world.ypoints[2]),poly_world.ypoints[3]);
    }

    /**
     *  Is the item inside a given rectangular area?
     *  All in WORLD COORDINATES!!!
     *
     *  @param topleft - the topleft corner of the rectangle
     *  @param bottomright - the bottom right corner of the rectangle
     *  @return true if the item is completely inside the rectangle
     */
    public boolean isInside (int[] topleft, int[] bottomright)
    {
	return (topleft[0] < xmin && bottomright[0] > xmax && topleft[1] > ymax && bottomright[1] < ymin);
    }


    /**
     *  Determines if the item has been hit by a click on the grid
     *  ALL IN SCREEN COORDINATES to avoid the snap mechanism
     *
     *  @param x,y - coordinates of the event point where the click
     *         occurred
     *  @return true if the item has been hit
     */
    public boolean isHit (int x, int y)
    {
	return poly_world.contains (x,y);
    }
    
    /**
     *  returns the perimeter of the Macro element, that is the smallest 
     *  bounding box completely containing the Macro
     */
    public Rectangle getPerimeter ()
    {
	return new Rectangle (xmin, ymin, xmax-xmin, ymax-ymin);
    }

    /**
     *  transforms the Macro element by changing all nodes
     */
    public void performTransformation (GridTransformation trafo)
    {
	if (!transformed)
	{
	    transformed = true;
	    Node[] nodes = updateCellNodes ();
	    nodes[0].performTransformation (trafo);
	    nodes[1].performTransformation (trafo);
	    nodes[2].performTransformation (trafo);
	    nodes[3].performTransformation (trafo);
	    updateInnerData ();
	}

    }

    /**
     *  clones this Macro element shallowly, which means no nodes are 
     *  recursivly cloned as well. See the GridCloner class in the backend 
     *  package for more information on how cloning of entire collections 
     *  is performed.
     */
    public Object clone ()
    {
	Macro mac = new Macro ();
	mac.setNumber (number);
	mac.setMarker (this.isMarked ());
	mac.m = m;
	mac.n = n;
	mac.aniso = aniso;
	mac.anisofaktor = anisofaktor;
	return mac;
    }

   public double[] getParameters ()
   {
	double[] params = new double[22];
	params[0] = edges[0].getParameters ()[0];
	params[1] = edges[0].getParameters ()[1];
	params[2] = edges[0].getParameters ()[2];
	params[3] = edges[0].getParameters ()[3];
	params[4] = edges[1].getParameters ()[0];
	params[5] = edges[1].getParameters ()[1];
	params[6] = edges[1].getParameters ()[2];
	params[7] = edges[1].getParameters ()[3];
	params[8] = edges[2].getParameters ()[0];
	params[9] = edges[2].getParameters ()[1];
	params[10] = edges[2].getParameters ()[2];
	params[11] = edges[2].getParameters ()[3];
	params[12] = edges[3].getParameters ()[0];
	params[13] = edges[3].getParameters ()[1];
	params[14] = edges[3].getParameters ()[2];
	params[15] = edges[3].getParameters ()[3];
	params[16] = n;
	params[17] = m;
	params[18] = aniso;
	params[19] = anisofaktor[0];
	params[20] = anisofaktor[1];
	params[21] = anisofaktor[2];
	return params;
    }

    public void setParameters (double[] params)
    {
	double[] paras = new double[4];
	paras[0] = params[0];
	paras[1] = params[1];
	paras[2] = params[2];
	paras[3] = params[3];
	edges[0].setParameters (paras);
	paras[0] = params[4];
	paras[1] = params[5];
	paras[2] = params[6];
	paras[3] = params[7];
	edges[1].setParameters (paras);
	paras[0] = params[8];
	paras[1] = params[9];
	paras[2] = params[10];
	paras[3] = params[11];
	edges[2].setParameters (paras);
	paras[0] = params[12];
	paras[1] = params[13];
	paras[2] = params[14];
	paras[3] = params[15];
	edges[3].setParameters (paras);
	n = (int)params[16];
	m = (int)params[17];
	aniso = (int)params[18];
	anisofaktor[0] = params[19];
	anisofaktor[1] = params[20];
	anisofaktor[2] = params[21];
	updateInnerData ();
    }

    public String toString ()
    {
	Node[] nds = updateCellNodes ();
	String s = "Macro "+getNumber();
	for (int i=0; i<nds.length; i++)
	    s += " "+nds[i];
	return s;
    }


}


