package devisor2.foundation.elements;

import java.awt.*;
import java.awt.geom.*;
import java.io.*;
import java.util.Vector;

import devisor2.grid.options.*;
import devisor2.grid.GUI.framework.*;
import devisor2.grid.backend.*;


/**
 * The class Tri represents
 * the cells of the discrete domain which have 3 nodes. 
 * This class is derived from Cell2D.
 * @see Cell
 * @see Quad
 * @see Macro
 */

public class Tri extends Cell2D  implements java.io.Serializable
{

    /**		
    * Number of nodes of a Tri.
    */
    static final int NPT = 3;

    /**
     * Empty constructor.
     * Calls the empty constructor of Cell,
     * and defines the array of nodes of the cell.
     * @see Cell
     */
    public Tri() 
    {
	super();
	cc = ControlCenter.getMyself ();
	edges = new Edge[NPT];
	poly_world = new Polygon ();
    }

    public Tri(Edge e1, Edge e2, Edge e3) 
    {
	super();
	cc = ControlCenter.getMyself ();
	edges = new Edge[NPT];
	edges[0] = e1;
	edges[1] = e2;
	edges[2] = e3;
	for (int i=0; i<NPT; i++)
	    edges[i].addParent(this);
	updateInnerData ();
    }
    
    /**
     * Calls one constructor of Cell,
     * and defines the array of nodes of the cell.
     * @see Cell
     */
    public Tri(int i, Edge e1, Edge e2, Edge e3) 
    {
	super(i);
	setCell(e1, e2, e3);
	updateInnerData ();
    }

    ///////////////////////////////////////////////

    public double[] getParameters ()
    {
	double[] params = new double[12];
	params[0] = edges[0].getParameters ()[0];
	params[1] = edges[0].getParameters ()[1];
	params[2] = edges[0].getParameters ()[2];
	params[3] = edges[0].getParameters ()[3];
	params[4] = edges[1].getParameters ()[0];
	params[5] = edges[1].getParameters ()[1];
	params[6] = edges[1].getParameters ()[2];
	params[7] = edges[1].getParameters ()[3];
	params[8] = edges[2].getParameters ()[0];
	params[9] = edges[2].getParameters ()[1];
	params[10] = edges[2].getParameters ()[2];
	params[11] = edges[2].getParameters ()[3];
	return params;
    }
    public void setParameters (double[] params)
    {
	double[] paras = new double[4];
	paras[0] = params[0];
	paras[1] = params[1];
	paras[2] = params[2];
	paras[3] = params[3];
	edges[0].setParameters (paras);
	paras[0] = params[4];
	paras[1] = params[5];
	paras[2] = params[6];
	paras[3] = params[7];
	edges[1].setParameters (paras);
	paras[0] = params[8];
	paras[1] = params[9];
	paras[2] = params[10];
	paras[3] = params[11];
	edges[2].setParameters (paras);
	updateInnerData ();
    }


    public int getNPT()
    {
	return NPT;
    }

    /**
     * Defines the array of nodes of the cell.
     * @see Node
     */
    public void setCell ( Edge e1, Edge e2, Edge e3 ) 
    {
	if (edges != null) 
	{
	    for (int i=0; i<NPT; i++)
		edges[i].removeParent(this);
	}
	edges = new Edge[NPT];
	edges[0] = e1;
	edges[1] = e2;
	edges[2] = e3;
	for (int i=0; i<NPT; i++)
	    edges[i].addParent(this);
	updateInnerData ();
    }
    
 
  //    /**
//       * Defines the edges of the cell,
//       * and set the number of the cell to i.
//       * @see Face
//       * @see Cell
//       */
//      public void setCell ( int i, Edge[] e ) 
//      {
//  	//if (edges != null) 
//  	//{
//  	//    for (int j=0; j<NPT; j++)
//  	//	edges[j].removeParent(this);
//  	//}
//  	edges = e;
//  	number = i;
//  	for (int j=0; j<NPT; j++)
//  	    edges[j].addParent(this);
//  	updateInnerData ();
//      }
    
    
  //    /**
//       * Defines the edges of the cell,
//       * and set the number of the cell to 0.
//       * @see Face
//       * @see Cell
//       */
//      public void setCell (Edge[] e)
//      {
//  	setCell(0,e);
//      }
    

    /**
     * Returns the ith edge of the cell modulo NPT,
     * i.e. for a Tri modulo 3.
     * @see Edge
     */  
    public Edge getEdge( int i ) 
    {
	return edges[i%NPT];
    }
    
    
  //    /**
//       * sets the i-th edge of the cell to e
//       */
//      public void setEdge( int i, Edge e ) 
//      {
//  	int j = i%NPT;
//  	if (edges[j]!=null)
//  	    edges[j].removeParent(this);
//  	edges[j] = e;
//  	edges[j].addParent(this);
//  	updateInnerData ();
//      }
    
    
    /**
     *  updates the inner variables xmin, xmax, ymin, ymax and the polygonal
     *  representation of the tri object
     */
    public void updateInnerData ()
    {
	Node[] nds = updateCellNodes ();
	poly_world = new Polygon ();
	poly_world.addPoint (nds[0].coord[0], nds[0].coord[1]);
	poly_world.addPoint (nds[1].coord[0], nds[1].coord[1]);
	poly_world.addPoint (nds[2].coord[0], nds[2].coord[1]);
	//System.out.println ("tri poly world:");
	//for (int i=0; i<3; i++)
	//    System.out.println (poly_world.xpoints[i]+","+poly_world.ypoints[i]);
	xmax = Math.max (Math.max (poly_world.xpoints[0], poly_world.xpoints[1]), poly_world.xpoints[2]);
	xmin = Math.min (Math.min (poly_world.xpoints[0], poly_world.xpoints[1]), poly_world.xpoints[2]);
	ymax = Math.max (Math.max (poly_world.ypoints[0], poly_world.ypoints[1]), poly_world.ypoints[2]);
	ymin = Math.min (Math.min (poly_world.ypoints[0], poly_world.ypoints[1]), poly_world.ypoints[2]);
    }
    
    /**
     * Draws the perimeter of the Tri.
     */
    public void draw(Graphics g) 
    {      
	cc = ControlCenter.getMyself ();
	// set Color
	if (marked)
	    g.setColor (cc.dc.color_selection);
	else
	    g.setColor(cc.dc.color_element);
	// get nodes first with respect to order
	Node[] nodes = updateCellNodes ();
	// manually transform to screen coords
	int[] n0 = cc.gc.fromWorldToScreen(nodes[0].getCoord(0), nodes[0].getCoord(1));
	int[] n1 = cc.gc.fromWorldToScreen(nodes[1].getCoord(0), nodes[1].getCoord(1));
	int[] n2 = cc.gc.fromWorldToScreen(nodes[2].getCoord(0), nodes[2].getCoord(1));
	// paint lines
	if (marked)
	{
	    Graphics2D g2d = (Graphics2D)g;
	    g2d.setStroke (new BasicStroke (4));
   	    g2d.drawLine (n0[0],n0[1],n1[0],n1[1]);
	    g2d.drawLine (n1[0],n1[1],n2[0],n2[1]);
	    g2d.drawLine (n2[0],n2[1],n0[0],n0[1]);
	    g2d.setStroke (new BasicStroke (1));
	}
	else
	{
	    g.drawLine (n0[0],n0[1],n1[0],n1[1]);
	    g.drawLine (n1[0],n1[1],n2[0],n2[1]);
	    g.drawLine (n2[0],n2[1],n0[0],n0[1]);
	}
	// paint numbering
	if (cc.dc.number_element)
	{
	    g.setColor (cc.dc.color_elementnumber);
	    int middleX = (int)((n0[0]+n1[0]+n2[0])/3);
	    int middleY = (int)((n0[1]+n1[1]+n2[1])/3);
	    g.setFont(cc.dc.font_numbering);
	    g.drawString((number+1)+" ",middleX,middleY);
	}
	
	
//    double[] x0 = tf.transform( nodes[0].getX() );
//    double[] x1 = tf.transform( nodes[1].getX() );
//    double[] x2 = tf.transform( nodes[2].getX() );
	
//    if (dc.filled) 
//    {
//      if (dc.fillingMode == dc.SIMPLE_FILLING_MODE)
//        fill(g, tf, dc);
//    }

//    if (dc.celldraw)
//    {	  
//      g.setColor(Color.red);
//      g.drawLine((int)x0[0], Display.VISY-(int)x0[1],
//        (int)x2[0], Display.VISY-(int)x2[1]);
//      g.drawLine((int)x0[0], Display.VISY-(int)x0[1],
//        (int)x1[0], Display.VISY-(int)x1[1]);
//      g.drawLine((int)x1[0], Display.VISY-(int)x1[1],
//        (int)x2[0], Display.VISY-(int)x2[1]);
//    }
  }
  
  
    /**
     * Draws the Tri filled with its color.
     * @see Cell
     */
    public void fill(Graphics g) 
    {
//    g.setColor(color);
	
//    double[] x0 = tf.transform( nodes[0].getX() );
//    double[] x1 = tf.transform( nodes[1].getX() );
//    double[] x2 = tf.transform( nodes[2].getX() );
      
//    int [] x = {(int)x0[0], (int)x1[0], (int)x2[0]};
//    int [] y = {Display.VISY-(int)x0[1],
//      Display.VISY-(int)x1[1], Display.VISY-(int)x2[1]};

//    Polygon poly = new Polygon( x, y, 3 );
      
//    g.fillPolygon( poly );
  }

    /**
     *  Is the tri completely inside a given rectangular area?
     *  All in WORLD COORDINATES!!!
     *
     *  @param topleft - the topleft corner of the rectangle
     *  @param bottomright - the bottom right corner of the rectangle
     *  @return true if the item is completely inside the rectangle
     */
    public boolean isInside (int[] topleft, int[] bottomright)
    {
	return (topleft[0] < xmin && bottomright[0] > xmax && topleft[1] > ymax && bottomright[1] < ymin);
    }


    /**
     *  Determines if the item has been hit by a click on the grid
     *  ALL IN WORLD COORDINATES 
     *
     *  @param x,y - coordinates of the event point where the click
     *         occurred in world coordinates
     *  @return true if the item has been hit
     */
    public boolean isHit (int x, int y)
    {
	return poly_world.contains (x,y);
    }


    /**
     *  returns the perimeter of the Tri element, that is the smallest 
     *  bounding box completely containing the Tri
     */
    public Rectangle getPerimeter ()
    {
	return new Rectangle (xmin, ymin, xmax-xmin, ymax-ymin);
    }

    /**
     *  transforms the Tri element by changing all nodes
     */
    public void performTransformation (GridTransformation trafo)
    {
	if (!transformed)
	{
	    transformed = true;
	    Node[] nodes = updateCellNodes ();
	    nodes[0].performTransformation (trafo);
	    nodes[1].performTransformation (trafo);
	    nodes[2].performTransformation (trafo);
	    updateInnerData ();
	}
    }

    /**
     *  mirrors the Tri element by changing all nodes
     */
    public void performMirror (double nx, double ny, double px, double py)
    {
	if (!transformed)
	{
	    transformed = true;
	    Node[] nodes = updateCellNodes ();
	    nodes[0].performMirror (nx,ny,px,py);
	    nodes[1].performMirror (nx,ny,px,py);
	    nodes[2].performMirror (nx,ny,px,py);
	    updateInnerData ();
	}
    }

    /**
     *  clones this Tri element shallowly, which means no nodes are recursivly
     *  cloned as well. See the GridCloner class in the backend package for 
     *  more information on how cloning of entire collections is performed.
     */
    public Object clone ()
    {
	Tri t = new Tri ();
	t.setNumber (number);
	t.setMarker (this.isMarked ());
	return t;
    }

    public String toString ()
    {
	Node[] nds = updateCellNodes ();
	String s = "Tri "+getNumber();
	for (int i=0; i<nds.length; i++)
	    s += " "+nds[i];
	return s;
    }

    /**
     *  calculates and returns the smallest inner angle of the element.
     *  @return a double representing an angle in degrees!
     */
    public double getSmallestInnerAngle ()
    {
	Node[] nodes = updateCellNodes();
	double[] vec1 = new double[2];
	double[] vec2 = new double[2];
	double[] vec3 = new double[3];
	double[] vec4 = new double[3];
	vec1[0] = nodes[1].getCoord(0) - nodes[0].getCoord(0);
	vec1[1] = nodes[1].getCoord(1) - nodes[0].getCoord(1);
	vec2[0] = nodes[2].getCoord(0) - nodes[0].getCoord(0);
	vec2[1] = nodes[2].getCoord(1) - nodes[0].getCoord(1);
	vec3[0] = nodes[1].getCoord(0) - nodes[2].getCoord(0);
	vec3[1] = nodes[1].getCoord(1) - nodes[2].getCoord(1);
	vec4[0] = -vec2[0];
	vec4[1] = -vec2[1];
	double l1 = Math.sqrt (Math.pow(vec1[0],2)+Math.pow(vec1[1],2));
	double l2 = Math.sqrt (Math.pow(vec2[0],2)+Math.pow(vec2[1],2));
	double l3 = Math.sqrt (Math.pow(vec3[0],2)+Math.pow(vec3[1],2));
	double l4 = Math.sqrt (Math.pow(vec4[0],2)+Math.pow(vec4[1],2));
	double a1 = Math.toDegrees(Math.acos((vec1[0]*vec2[0]+vec1[1]*vec2[1])/(l1*l2)));
	double a2 = Math.toDegrees(Math.acos((vec3[0]*vec4[0]+vec3[1]*vec4[1])/(l3*l4)));
	double a3 = 180 -a1-a2;
	return Math.min (Math.min(a1,a2),a3);
    }

    /**
     *  calculates the aspect ratio of the element, that is the quotient of
     *  the shortest and the longest edge
     */
    public double getAspectRatio ()
    {
	Edge[] ed = getEdges ();
	double[] lengths = new double[3];
	for (int i=0; i<3; i++)
	    lengths[i] = ed[i].getLength ();
	double min = lengths[0];
	for (int i=1; i<3; i++)
	    if (lengths[i] < min)
		min = lengths[i];
	double max = lengths[0];
	for (int i=1; i<3; i++)
	    if (lengths[i] > max)
		max = lengths[i];
	return (min/max);
    }

    /**
     *  calculates and returns the area (size) of the element
     */
    public double getArea ()
    {
	Edge[] ed = getEdges ();
	double[] lengths = new double[3];
	for (int i=0; i<3; i++)
	    lengths[i] = ed[i].getLength ();
	double tmp1 = Math.pow (lengths[0]+lengths[1],2)-Math.pow (lengths[2],2);
	double tmp2 = Math.pow (lengths[2],2) - Math.pow (lengths[0]-lengths[1],2);
	return 0.25*Math.sqrt (tmp1)*Math.sqrt(tmp2);
    }

  
}










