package devisor2.grid.GUI.dialogs;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.table.*;

import java.util.*;

import devisor2.grid.options.*;

/**
 *  The underlying table model for the JTable on the accelerators tab
 *  of the OptionsDialog class. <br>
 *  <br>
 *  In the constructor, the global Options instance (in ControlCenter)
 *  is searched for all known accelerator string entries, therefore
 *  all accelerator - related options must start with "accelerator".
 *  Otherwise, they are not recognized. <br>
 *  The Table is made uneditable, so that the user must use the edit panel
 *  provided in the Dialog. <br>
 *  <br>
 *  The implementation is pretty straightforward, see the API 
 *  (AbstractTableModel) for details.
 *
 *  @author Dominik Goeddeke
 *  @version 1.0 (fully tested)
 */
class AcceleratorTableModel extends AbstractTableModel
{
    /**
     *  the names of the columns
     */
    private String[] columnNames;

    /**
     *  the data stored in the table, see below for explanation
     */
    private Vector data;
    /**
     *  the usual control center reference
     */
    private ControlCenter cc;

    /**
     *  creates a new instance, searches the global Options instance for
     *  all keys starting with "accelerator" and takes these as the data 
     *  source.
     */
    public AcceleratorTableModel ()
    {
	data = new Vector ();
	cc = ControlCenter.getMyself ();
	columnNames = new String[2];
	columnNames[0] = (String)cc.rb.getObject ("options_OptionsDialog_acceleratorPanel_tableleftcolumn");
	columnNames[1] = (String)cc.rb.getObject ("options_OptionsDialog_acceleratorPanel_tablerightcolumn");
	Enumeration opEnum = cc.op.keys ();
	while (opEnum.hasMoreElements ())
	    {
		String s = (String)opEnum.nextElement ();
		if ((s.length() > 12) && (s.substring(0,11).equals("accelerator")))
		    {
			String[] pair = new String[2];
			pair[0] = s;
			pair[1] = cc.op.get(s);
			data.add (pair);
		    }
	    }
    }

    /**
     *  returns if the cell at (row,col) is editable
     */
    public boolean isCellEditable(int row, int column) 
    {
	return false;
    }

    /**
     *  returns the number of columns in the table
     */
    public int getColumnCount() 
    {
        return columnNames.length;
    }

    /**
     *returns the number of rows in the table
     */
    public int getRowCount() 
    {
        return data.size ();
    }

    /**
     *  returns the name of the column at the given position
     */
    public String getColumnName(int col) 
    {
        return columnNames[col];
    }

    /**
     *  returns the value stored in the table at the given position
     */
    public Object getValueAt(int row, int col) 
    {
	if (col == 0)
	    return ((String[])data.elementAt(row))[0];
	else
	    return ((String[])data.elementAt(row))[1];
    }

    /**
     *  returns the data source of the TableModel.
     *  Data structure: the index in the vector corresponds with the row
     *  in the table, each entry is a two-dimensional array of strings,
     *  first entry is the accelerator key from the Options instance,
     *  second entry is the assigned shortcut.
     *
     *  @return a vector of String[2] objects, the first element being a key,
     *          the second element being the value
     */
    public Vector getData ()
    {
	return data;
    }




}
