package devisor2.grid.GUI.dialogs;

import java.util.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

import devisor2.grid.options.*;
import devisor2.grid.GUI.framework.*;
import devisor2.foundation.elements.*;
import devisor2.foundation.boundary.*;


/**
 *  This class is the DockableDialog instance for Boundary Node input. 
 *  It is used both for direct input of Nodes and for Property Queries 
 *  of Nodes. On two tab sheets, either the parametric description or the
 *  cartesian coordinates can be edited. <br>
 *
 *  @author Dominik Goeddeke
 *  @version 0.9 (release candidate)
 *  
 */

public class BoundaryNodeDialog extends DockableDialog implements ActionListener, FocusListener
{

    // the source (for property mode)
    private Node source;
    // the bn added 
    private BoundaryNode lastnode;
  
    /** 
     *  returns the initial Position of the dialog
     */
    public Point getInitialPosition ()
    {
	return new Point (parent.getLocationOnScreen().x - 50, parent.getLocationOnScreen().y);
    }

    /**
     *  sets the textfields initial values
     */
    public void setCoords (double x, double y, int nodeprop)
    {
	coordxTF.setText (Double.toString (x));
	coordyTF.setText (Double.toString (y));
	nodepropTF.setText (Integer.toString (nodeprop));
    }

    /**
     *  sets the source variable for the dialog (in case of property mode)
     */
    public void setSource (Node source)
    {
	if (source != null)	 
	{
	    this.source = source;
	    x = cc.gc.fromWorldToUser (source.getCoord(0));
	    y = cc.gc.fromWorldToUser(source.getCoord(1));
	    nodeprop = source.getNPR();
	    setCoords (x,y,nodeprop);
	    if (source instanceof BoundaryNode)
	    {
		BoundaryNode bsource = (BoundaryNode)source;
		Boundary b = bsource.getBoundary();
		actparTF.setText (Double.toString (bsource.getParaValue()));
		aktpar = bsource.getParaValue();
		boundaryCombo.setSelectedItem (Integer.toString (b.getNumber()+1));
		bnumber = b.getNumber();
		
		maxparTF.setText (Double.toString (b.getParaMax()));
		maxpar = b.getParaMax ();
		tabs.setEnabledAt (1,true);
		
		tabs.setSelectedIndex (1);
	    }
	    else
	    {
		tabs.setEnabledAt (1,false);
		tabs.setSelectedIndex (0);
	    }
	}
	else
	    this.source = null;
    }

    /**
     *  @return the number of the selected tabsheet
     */
    public int getSelectedTab ()
    {
	return tabs.getSelectedIndex ();
    }

    /**
     *  sets the selected tab of the dialog to the given index
     */
    public void setSelectedTab (int index)
    {
	tabs.setSelectedIndex (index);
    }




    /**
     *  the constructor just sets references correctly and creates the GUI
     */
    public BoundaryNodeDialog (MainFrame parent)
    {
	super (parent);
	JPanel content = new JPanel ();
	content.setBorder (new EmptyBorder (10,10,10,10));
	// just ordinary GUI layouting
	GridBagLayout layout = new GridBagLayout ();
	GridBagConstraints constraints = new GridBagConstraints ();
	content.setLayout (layout);
	// the parametric tab
	JPanel parametric = new JPanel ();
	parametric.setLayout (layout);
	parametric.setBorder (new EmptyBorder (5,5,5,5));
JLabel actparLabel = new JLabel ((String)cc.rb.getObject ("dialogs_boundarynode_actpar"));
	constraints.gridy = 0;
	constraints.gridx = 0;
	layout.setConstraints (actparLabel, constraints);
	parametric.add (actparLabel);

	actparTF = new JTextField (20);
	constraints.gridx = 1;
	constraints.gridwidth = 3;
	layout.setConstraints (actparTF, constraints);
	parametric.add (actparTF);

	JLabel maxparLabel = new JLabel ((String)cc.rb.getObject ("dialogs_boundarynode_maxpar"));
	constraints.gridy = 1;
	constraints.gridx = 0;
	constraints.gridwidth = 1;
	layout.setConstraints (maxparLabel, constraints);
	parametric.add (maxparLabel);
	maxparTF = new JTextField (4);
	maxparTF.setEditable (false);
	constraints.gridx = 1;
	layout.setConstraints (maxparTF, constraints);
	parametric.add (maxparTF);
	JLabel boundaryLabel = new JLabel ((String)cc.rb.getObject ("dialogs_boundarynode_boundary"));
	constraints.gridx = 2;
	layout.setConstraints (boundaryLabel, constraints);
	parametric.add (boundaryLabel);
	boundaryCombo = new JComboBox ();
	boundaryCombo.setEditable (false);
	boundaryCombo.setActionCommand (BOUNDARY);
	boundaryCombo.addActionListener (this);
	constraints.gridx = 3;
	layout.setConstraints (boundaryCombo, constraints);
  	parametric.add (boundaryCombo);

	JPanel cassetterecorder = new JPanel ();
	cassetterecorder.setLayout (new FlowLayout ());
	fastrewindButton = new JButton ((String)cc.rb.getObject ("dialogs_boundarynode_fastrewind"));
	fastrewindButton.setToolTipText ((String)cc.rb.getObject ("dialogs_boundarynode_fastrewind_tooltip"));
	fastrewindButton.setActionCommand (FASTREWIND);
	fastrewindButton.addActionListener (this);
	cassetterecorder.add (fastrewindButton);
	rewindButton = new JButton ((String)cc.rb.getObject ("dialogs_boundarynode_rewind"));
	rewindButton.setToolTipText ((String)cc.rb.getObject ("dialogs_boundarynode_rewind_tooltip"));
	rewindButton.setActionCommand (REWIND);
	rewindButton.addActionListener (this);
	cassetterecorder.add (rewindButton);
	forwardButton = new JButton ((String)cc.rb.getObject ("dialogs_boundarynode_forward"));
	forwardButton.setToolTipText ((String)cc.rb.getObject ("dialogs_boundarynode_forward_tooltip"));
	forwardButton.setActionCommand (FORWARD);
	forwardButton.addActionListener (this);
	cassetterecorder.add (forwardButton);
	fastforwardButton = new JButton ((String)cc.rb.getObject ("dialogs_boundarynode_fastforward"));
	fastforwardButton.setToolTipText ((String)cc.rb.getObject ("dialogs_boundarynode_fastforward_tooltip"));
	fastforwardButton.setActionCommand (FASTFORWARD);
	fastforwardButton.addActionListener (this);
	cassetterecorder.add (fastforwardButton);
	constraints.gridy = 2;
	constraints.gridx = 0;
	constraints.gridwidth = 4;
	layout.setConstraints (cassetterecorder, constraints);
	parametric.add (cassetterecorder);
	// the edit flag
	editCheck = new JCheckBox ((String)cc.rb.getObject ("dialogs_boundarynode_editcheck"));
	constraints.gridy = 3;
	constraints.gridx = 1;
	constraints.gridwidth = 2;
	layout.setConstraints (editCheck, constraints);
	parametric.add (editCheck);

	// the cartesian tab
	JPanel cart = new JPanel ();
	cart.setLayout (layout);
	cart.setBorder (new EmptyBorder (10,10,10,10));
	JLabel coordLabel = new JLabel ((String)cc.rb.getObject ("dialogs_boundarynode_coord"));
	constraints.gridy = 0;
	constraints.gridx = 0;
	constraints.gridwidth = 2;
	layout.setConstraints (coordLabel, constraints);
	cart.add (coordLabel);
	coordxTF = new JTextField (10);
	constraints.gridy = 1;
	constraints.gridx = 0;
	constraints.gridwidth = 1;
	layout.setConstraints (coordxTF, constraints);
	cart.add (coordxTF);
	
	coordyTF = new JTextField (10);
	constraints.gridy = 2;
	constraints.gridx = 0;
	layout.setConstraints (coordyTF, constraints);
	cart.add (coordyTF);


        JLabel coordLabel1 = new JLabel ((String)cc.rb.getObject ("dialogs_boundarynode_nodeprop"));
	constraints.gridy = 3;
	constraints.gridx = 0;
	constraints.gridwidth = 2;
	layout.setConstraints (coordLabel1, constraints);
	cart.add (coordLabel1);
	
	nodepropTF = new JTextField (10);
	constraints.gridy = 3;
	constraints.gridx = 1;
	layout.setConstraints (nodepropTF, constraints);
	cart.add (nodepropTF);
	
	
	//the tabbed pane
	tabs = new JTabbedPane ();
	tabs.addTab((String)cc.rb.getObject ("dialogs_boundarynode_coord_tabheader"), null, cart, (String)cc.rb.getObject ("dialogs_boundarynode_coord_tooltip"));
	tabs.addTab((String)cc.rb.getObject ("dialogs_boundarynode_para_tabheader"), null, parametric, (String)cc.rb.getObject ("dialogs_boundarynode_para_tooltip"));
	constraints.fill = GridBagConstraints.BOTH;
	constraints.weightx = constraints.weighty = 1.0;
	constraints.gridx = 0;
	constraints.gridy = 1;
	constraints.gridwidth = 4;
	constraints.gridheight = 4;
	layout.setConstraints (tabs, constraints);
	content.add (tabs);

	// buttons
	JPanel buttons = new JPanel ();
	buttons.setLayout (new FlowLayout ());
	okButton = new JButton ((String)cc.rb.getObject ("general_Ok"));
	okButton.setMnemonic ( ((Integer)cc.rb.getObject("general_Ok_mnemonic")).intValue());
	okButton.setActionCommand (OK);
	okButton.addActionListener (this);
	buttons.add (okButton);
	applyButton = new JButton ((String)cc.rb.getObject ("general_Apply"));
	applyButton.setMnemonic ( ((Integer)cc.rb.getObject("general_Apply_mnemonic")).intValue());
	applyButton.setActionCommand (APPLY);
	applyButton.addActionListener (this);
	buttons.add (applyButton);
	cancelButton = new JButton ((String)cc.rb.getObject ("general_Cancel"));
	cancelButton.setMnemonic ( ((Integer)cc.rb.getObject("general_Cancel_mnemonic")).intValue());
	cancelButton.setActionCommand (CANCEL);
	cancelButton.addActionListener (this);
	buttons.add (cancelButton);

	// add them all to the dialog
	getRootPane().setDefaultButton (applyButton);
		
	constraints.gridy = 6;
	constraints.gridx = 1;
	constraints.gridwidth = 2;
	layout.setConstraints(buttons, constraints);
	content.add (buttons);
	// finish the GUI
	getContentPane ().add (content);
	addComponentListener (listener);

	pack ();
    }


    /**
     *  updates the boundary combobox.
     */
    public void refreshGUI ()
    {
	boundaryCombo.removeActionListener (this);
	boundaryCombo.removeAllItems ();
	if (cc.gc.workingBoundary == null)
	{
	    boundaryCombo.addItem ("-1");
	    boundaryCombo.setSelectedItem ("-1");
	    maxparTF.setText ("0");
	}
	else
	{
	    for (int i=0; i<cc.d.db.getBoundaryCount(); i++)
		boundaryCombo.addItem (Integer.toString(i+1));
	    boundaryCombo.setSelectedItem (Integer.toString (cc.gc.workingBoundary.getNumber ()-1));
	    bnumber = cc.gc.workingBoundary.getNumber();
	    maxparTF.setText (Double.toString(cc.gc.workingBoundary.getParaMax()));
	    maxpar = cc.gc.workingBoundary.getParaMax();
	}
	boundaryCombo.addActionListener (this);
    }


    /**
     *  This class handles its own action event. The actionPerformed method 
     *  just updates the ControlCenter's EdgeStatusInfo 
     */
    public void actionPerformed (ActionEvent e)
    {
	if (e.getActionCommand().equals (OK))
	{
	    if (verify ())
	    {
		if (source != null)
		{
		    performUpdateNode ();
		}
		else
		{
		    performAddNode ();
		}
		cc.dm.hide (cc.dm.BOUNDARYNODEDIALOG);
	    }
	    return;
	}
	if (e.getActionCommand().equals (APPLY))
	{
	    if (verify ())
	    {
		if (source != null)
		{
		    performUpdateNode ();
		}
		else
		{
		    performAddNode ();
		}
	    }
	    return;
	}
	if (e.getActionCommand().equals (CANCEL))
	{
	    cc.dm.hide (cc.dm.BOUNDARYNODEDIALOG);
	    return;
	}
	if (e.getActionCommand().equals (FASTREWIND))
	{
	    aktpar -= 0.1;
	    if (aktpar < 0)
		aktpar = 0;
	    actparTF.setText (Double.toString (aktpar));
	    return;
	}
	if (e.getActionCommand().equals (REWIND))
	{
	    aktpar -= 0.01;
	    if (aktpar < 0)
		aktpar = 0;
	    actparTF.setText (Double.toString (aktpar));
	    return;
	}
	if (e.getActionCommand().equals (FORWARD))
	{
	    aktpar += 0.01;
	    try
	    {
		bnumber = Integer.parseInt ((String)boundaryCombo.getSelectedItem())-1;
	    }
	    catch (NumberFormatException ex)
	    {
		JOptionPane.showMessageDialog (parent,  (String)cc.rb.getObject ("dialogs_boundarynode_error_boundnum"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
	    }
	    Boundary b = cc.d.db.getBoundary (bnumber);
	    if (aktpar > b.getParaMax())
		aktpar = b.getParaMax ();
	    actparTF.setText (Double.toString (aktpar));
	    return;
	}
	if (e.getActionCommand().equals (FASTFORWARD))
	{
	    aktpar += 0.1;
	    try
	    {
		bnumber = Integer.parseInt ((String)boundaryCombo.getSelectedItem())-1;
	    }
	    catch (NumberFormatException ex)
	    {
		JOptionPane.showMessageDialog (parent,  (String)cc.rb.getObject ("dialogs_boundarynode_error_boundnum"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
	    }
	    Boundary b = cc.d.db.getBoundary (bnumber);
	    if (aktpar > b.getParaMax())
		aktpar = b.getParaMax ();
	    actparTF.setText (Double.toString (aktpar));
	    return;
	}
	if (e.getActionCommand().equals (BOUNDARY))
	{
	    try
	    {
		bnumber = Integer.parseInt ((String)boundaryCombo.getSelectedItem())-1;
	    }
	    catch (Exception ex)
	    {
		JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject ("dialogs_multiplebn_error_boundary"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
		return;
	    }
	    if (bnumber < cc.d.db.getBoundaryCount() && bnumber >= 0)
	    {
		maxparTF.setText (Double.toString (cc.d.db.getBoundary(bnumber).getParaMax()));
	    }
	    else
	    {
		maxparTF.setText ("");
		JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject ("dialogs_multiplebn_error_nosuchboundary"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
	    }
	    return;
	}
    }

    /**
     *  the boring verification code is outsourced into this method
     */
    private boolean verify ()
    {
	boolean allOK = true;
	// (1) inner node input
	if (tabs.getSelectedIndex ()== 0)
	{
	    // the two text fields contain valid doubles
	    try
	    {
		x = Double.parseDouble (coordxTF.getText ());
	    }
	    catch (NumberFormatException ex)
	    {
		JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject ("dialogs_boundarynode_error_coord_x"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
		allOK = false;
	    }
	    try
	    {
		y = Double.parseDouble (coordyTF.getText ());
	    }
	    catch (NumberFormatException ex)
	    {
		JOptionPane.showMessageDialog (parent,  (String)cc.rb.getObject ("dialogs_boundarynode_error_coord_y"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
		allOK = false;
	    }
	    try
	    {
		nodeprop = Integer.parseInt(nodepropTF.getText ());
	    }
	    catch (NumberFormatException ex)
	    {
		JOptionPane.showMessageDialog (parent,  (String)cc.rb.getObject ("dialogs_boundarynode_error_nodeprop"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
		allOK = false;
	    }
	    	    return allOK;
	}
	// (2) boundary node input via parametrisation
	if (tabs.getSelectedIndex()==1)
	{
	    try
	    {
		bnumber = Integer.parseInt ((String)boundaryCombo.getSelectedItem())-1;
	    }
	    catch (Exception ex)
	    {
		JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject ("dialogs_multiplebn_error_boundary"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
		allOK = false;
	    }
	    // the textfields on para tab contain ints and doubles
	    try
	    {
		aktpar = Double.parseDouble (actparTF.getText ());
	    }
	    catch (NumberFormatException ex)
	    {
		JOptionPane.showMessageDialog (parent,  (String)cc.rb.getObject ("dialogs_boundarynode_error_actpar"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
		allOK = false;
	    }
	    // verify if inside range
	    if (bnumber < cc.d.db.getBoundaryCount())
	    {
		maxpar = cc.d.db.getBoundary(bnumber).getParaMax();
		maxparTF.setText (Double.toString (maxpar));
	    }
	    else
	    {
		JOptionPane.showMessageDialog (parent,  (String)cc.rb.getObject ("dialogs_boundarynode_error_boundrange"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
		allOK = false;
	    }
	    if (aktpar < 0 || aktpar > maxpar)
	    {
		JOptionPane.showMessageDialog (parent,(String)cc.rb.getObject ("dialogs_boundarynode_error_actrange"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
		allOK = false;
	    }
	    return allOK;
	}
	return allOK;
    }


    /**
     *  adds a new node to the domain with the parameter values from
     *  the dialog textfields
     */
    private void performAddNode ()
    {
	DrawingArea da = parent.getDrawingArea ();
	int xx = cc.gc.fromUserToWorld(x);
	int yy = cc.gc.fromUserToWorld(y);
	// (1) inner node via coords
	if (tabs.getSelectedIndex ()== 0)
	{
	    lastnode = null;
	    Node newnode = new Node (xx,yy);
	    newnode.setNPR(nodeprop);
	    cc.d.addNode(newnode);
	    cc.d.updateNodeNumbers ();
	    Vector tmpvec = new Vector ();
	    tmpvec.add (newnode);
	    cc.um.notifyUndo (cc.um.UNDO_ADDNODE, tmpvec, null);
	    da.repaint ();
	    return;
	}
	// (2) boundary node input via parametrisation
	if (tabs.getSelectedIndex()==1)
	{
	    if (editCheck.isSelected())
	    {
		if (lastnode == null)
		{
		    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject("dialogs_boundarynode_error_editlast"), (String)cc.rb.getObject ("errors_notfounderror"), JOptionPane.ERROR_MESSAGE);
		}
		else
		{
		    lastnode.setBoundary (cc.d.db.getBoundary(bnumber));
		    lastnode.setParaValue (aktpar);
		    lastnode.updateBoundaryNode ();
		    cc.d.updateNodeNumbers ();
		    da.repaint ();
		}
		return;
	    }
	    else
	    {
		lastnode = new BoundaryNode (aktpar, cc.d.db.getBoundary(bnumber));
		cc.d.addNode(lastnode);
		cc.d.updateNodeNumbers ();
		Vector tmpvec = new Vector ();
		tmpvec.add (lastnode);
		cc.um.notifyUndo (cc.um.UNDO_ADDNODE, tmpvec, null);
		da.repaint();
		return;
	    }
	}
    }
    /**
     *  updates the node referenced by the source variable with
     *  the parameter values from the dialog textfields
     */
    private void performUpdateNode ()
    {
	DrawingArea da = parent.getDrawingArea ();
	// (1) inner node via coords
	if (tabs.getSelectedIndex ()== 0)
	{
	    lastnode = null;
	    if (source instanceof BoundaryNode)
	    { 
		JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject("dialogs_boundarynode_error_useparatext"), (String)cc.rb.getObject ("dialogs_boundarynode_error_usepara"), JOptionPane.ERROR_MESSAGE);
	    }
	    else
	    {
		// prepare undo
		int[] info = new int[3];
		info[0] = source.getCoord (0);
		info[1] = source.getCoord (1);
		info[2] = source.getNPR ();
		Vector tmpvec = new Vector ();
		tmpvec.add (source);
		cc.um.notifyUndo (cc.um.UNDO_PROPERTYNODE, tmpvec, info);
		// update source node
		int xx = cc.gc.fromUserToWorld(x);
		int yy = cc.gc.fromUserToWorld(y);
		cc.d.updateNodeNumbers ();
		source.setCoord(xx,yy);
		source.setNPR(nodeprop);
	    }
	    da.repaint ();
	    return;
	}
	// (2) boundary node input via parametrisation
	if (tabs.getSelectedIndex()==1)
	{
	    if (editCheck.isSelected())
	    {
		if (lastnode == null)
		{
		    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject("dialogs_boundarynode_error_editlast"), (String)cc.rb.getObject ("errors_notfounderror"), JOptionPane.ERROR_MESSAGE);
		}
		else
		{
		    // prepare undo
		    Object[] info = new Object[2];
		    info[0] = lastnode.getBoundary ();
		    info[1] = new Double (lastnode.getParaValue ());
		    Vector tmpvec = new Vector ();
		    tmpvec.add (lastnode);
		    cc.um.notifyUndo (cc.um.UNDO_PROPERTYBNODE, tmpvec, info);
		    // update last boundary node
		    lastnode.setBoundary (cc.d.db.getBoundary(bnumber));
		    lastnode.setParaValue (aktpar);
		    lastnode.updateBoundaryNode ();
		    cc.d.updateNodeNumbers ();
		    da.repaint ();
		}
		return;
	    }
	    else
	    {
		if (!(source instanceof BoundaryNode))
		{
		    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject("dialogs_boundarynode_error_regularparametric"), (String)cc.rb.getObject ("dialogs_boundarynode_error_para"), JOptionPane.ERROR_MESSAGE);

		}
		else
		{
		    BoundaryNode bsource = (BoundaryNode)source;
		    // prepare undo
		    Object[] info = new Object[2];
		    info[0] = bsource.getBoundary ();
		    info[1] = new Double (bsource.getParaValue ());
		    Vector tmpvec = new Vector ();
		    tmpvec.add (bsource);
		    cc.um.notifyUndo (cc.um.UNDO_PROPERTYBNODE, tmpvec, info);
		    // update source boundary node
		    bsource.setBoundary (cc.d.db.getBoundary(bnumber));
		    bsource.setParaValue (aktpar);
		    bsource.updateBoundaryNode ();
		    cc.d.updateNodeNumbers ();
		    da.repaint();
		}
		return;
	    }
	}
    }

    public void focusLost (FocusEvent e)
    {
	try
	{
	    bnumber = Integer.parseInt ((String)boundaryCombo.getSelectedItem())-1;
	}
	catch (NumberFormatException ex)
	{
	    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject ("dialogs_multiplebn_error_boundary"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
	    return;
	}
	if (bnumber < cc.d.db.getBoundaryCount() && bnumber > 0)
	{
	    maxparTF.setText (Double.toString (cc.d.db.getBoundary(bnumber).getParaMax()));
	}
	else
	{
	    maxparTF.setText ("");
	    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject ("dialogs_multiplebn_error_nosuchboundary"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
	}
    }

    public void focusGained (FocusEvent e)
    {
	return;
    }


    // some storage for coordinates (from coord textfields)
    private double x,y;
    private int nodeprop;
    // some storage for parametrisation (from textfields)
    private double aktpar;
    private int bnumber;
    private double maxpar;


    // keys for the action commands

    private static final String OK = "OK";;
    private static final String CANCEL = "CANCEL";
    private static final String APPLY = "APPLY";
    private static final String FASTREWIND = "FASTREWIND";
    private static final String REWIND = "REWIND";
    private static final String FORWARD = "FORWARD";
    private static final String FASTFORWARD = "FASTFORWARD";
    private static final String BOUNDARY = "BOUNDARY";
  


    // GUI items
    private JTabbedPane tabs;
    private JButton okButton;
    private JButton cancelButton;
    private JButton applyButton;
    private JButton fastforwardButton;
    private JButton forwardButton;
    private JButton rewindButton;
    private JButton fastrewindButton;
    private JComboBox boundaryCombo;
    private JTextField maxparTF;
    private JTextField actparTF;
    private JTextField coordxTF;
    private JTextField coordyTF;
    private JTextField nodepropTF;
    private JCheckBox editCheck;


}
