package devisor2.grid.GUI.dialogs;

import java.util.*;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import javax.swing.*;
import javax.swing.border.*;

import devisor2.grid.options.*;
import devisor2.grid.GUI.framework.*;
import devisor2.grid.GUI.event.*;
import devisor2.foundation.elements.*;
import devisor2.grid.backend.*;



/**
 *  This class is the DockableDialog instance to change the working boundary.
 *  <br>
 *  See the GridListener and MainActionListener for a detailed description 
 *  of how the dialog is connected to the application.
 *
 *  @author Dominik Goeddeke
 *  @version done
 *  
 */

public class ChangeBoundaryDialog extends DockableDialog implements ActionListener
{


    /** 
     *  returns the initial Position of the dialog
     */
    public Point getInitialPosition ()
    {
	return new Point (parent.getLocationOnScreen().x - 50, parent.getLocationOnScreen().y);
    }

    /**
     *  the constructor just sets references correctly and creates the GUI
     */
    public ChangeBoundaryDialog (MainFrame parent)
    {
	super (parent);
	setModal (true);
	setTitle ((String)cc.rb.getObject("dialogs_changeb_caption"));
	JPanel content = new JPanel ();
	content.setBorder (new EmptyBorder (10,10,10,10));
	// just ordinary GUI layouting
	GridBagLayout layout = new GridBagLayout ();
	GridBagConstraints constraints = new GridBagConstraints ();
	content.setLayout (layout);
	constraints.fill = GridBagConstraints.BOTH;
	constraints.weightx = constraints.weighty = 1.0;
	// combo
	JLabel label = new JLabel ((String)cc.rb.getObject ("dialogs_changeb_label"));
	constraints.gridy = 0;
	constraints.gridx = 0;
	layout.setConstraints (label, constraints);
	content.add (label);
	selectBoundaryCombo = new JComboBox ();
	selectBoundaryCombo.setToolTipText ((String)cc.rb.getObject("GUI_framework_ToolBar_selectboundarytooltip"));
	selectBoundaryCombo.setEditable (false);
	content.add (selectBoundaryCombo);

	// buttons
	JPanel buttons = new JPanel ();
	buttons.setLayout (new FlowLayout ());
	okButton = new JButton ((String)cc.rb.getObject ("general_Ok"));
	okButton.setMnemonic ( ((Integer)cc.rb.getObject("general_Ok_mnemonic")).intValue());
	okButton.setActionCommand (OK);
	okButton.addActionListener (this);
	buttons.add (okButton);
	cancelButton = new JButton ((String)cc.rb.getObject ("general_Cancel"));
	cancelButton.setMnemonic ( ((Integer)cc.rb.getObject("general_Cancel_mnemonic")).intValue());
	cancelButton.setActionCommand (CANCEL);
	cancelButton.addActionListener (this);
	buttons.add (cancelButton);
	// add them all to the dialog
	getRootPane().setDefaultButton (okButton);
	constraints.gridy = 3;
	constraints.gridx = 0;
	constraints.gridwidth = 2;
	layout.setConstraints(buttons, constraints);
	content.add (buttons);
	// finish the GUI
	getContentPane ().add (content);
	addComponentListener (listener);
	//	updateCombo ();
	pack ();
    }

    /**
     *  updates the boundary comboboxes
     */
    public void refreshGUI ()
    {
	selectBoundaryCombo.removeAllItems ();
	if (cc.d.db.getBoundaryCount () == 0)
	{
	    selectBoundaryCombo.addItem ("-1");
	    selectBoundaryCombo.setSelectedItem ("-1");
	}
	else
	{
	    for (int i=0; i<cc.d.db.getBoundaryCount(); i++)
		selectBoundaryCombo.addItem (Integer.toString(i+1));
	    selectBoundaryCombo.setSelectedItem (Integer.toString (cc.gc.workingBoundary.getNumber ()-1));
	}
    }


    /**
     *  This class handles its own action event. 
     */
    public void actionPerformed (ActionEvent e)
    {
	if (e.getActionCommand().equals (OK))
	{
	    try
	    {
		cc.gc.workingBoundary = cc.d.db.getBoundary(Integer.parseInt ((String)selectBoundaryCombo.getSelectedItem())-1);
	    }
	    catch (Exception exc) {}
	    cc.dm.hide (cc.dm.CHANGEBOUNDARYDIALOG);
	    return;
	}
	if (e.getActionCommand().equals (CANCEL))
	{
	    cc.dm.hide (cc.dm.CHANGEBOUNDARYDIALOG);
	    return;
	}
    }

    // keys for the action commands

    private static final String OK = "OK";
    private static final String CANCEL = "CANCEL";
    
    // buttons

    private JButton okButton;
    private JButton cancelButton;
    private JComboBox selectBoundaryCombo;

}
