package devisor2.grid.GUI.dialogs;

import java.util.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

import devisor2.grid.options.*;
import devisor2.grid.GUI.framework.*;
import devisor2.foundation.boundary.*;
import devisor2.foundation.elements.*;

/**
 *  This class is the DockableDialog instance for Circle input. It is used 
 *  both for direct input of Line Segments and for Property Queries of Lines.
 *  See the GridListener class for usage instructions, and of course the 
 *  technical paper about dialogs in general.
 *
 *  @author Dominik Goeddeke
 *  @version 1.0
 *  
 */

public class CircleDialog extends DockableDialog implements ActionListener
{
    // the source (i.e. the segment the dialog works on in property mode)
    private SegmentCircle source;

    // the intended orientation for the circle
    private boolean orientation;


    /**
     *  returns the initial Position of the dialog
     */
    public Point getInitialPosition ()
    {
	return new Point (parent.getLocationOnScreen().x - 50, parent.getLocationOnScreen().y);
    }



    /**
     *  the constructor just sets references correctly and creates the GUI
     */
    public CircleDialog (MainFrame parent)
    {
	super (parent);
	JPanel content = new JPanel ();
	content.setBorder (new EmptyBorder (10,10,10,10));
	// just ordinary GUI layouting
	GridBagLayout layout = new GridBagLayout ();
	GridBagConstraints constraints = new GridBagConstraints ();
	constraints.fill = GridBagConstraints.BOTH;
	constraints.weightx = constraints.weighty = 1.0;
	// middlepoint	
	JLabel centerLabel = new JLabel ((String)cc.rb.getObject ("dialogs_circle_center"));
	center_x_TF = new JTextField (5);
	center_y_TF = new JTextField (5);
	// radius
	JLabel radiusLabel = new JLabel ((String)cc.rb.getObject("dialogs_circle_radius"));
	radiusTF = new JTextField (5);
	// start angle
	JLabel angle1Label = new JLabel ((String)cc.rb.getObject ("dialogs_circle_startangle"));
	angle1TF = new JTextField (5);
	// end angle
	JLabel angle2Label = new JLabel ((String)cc.rb.getObject ("dialogs_circle_endangle"));
	angle2TF = new JTextField (5);
	// buttons
	JPanel buttons = new JPanel ();
	buttons.setLayout (new FlowLayout ());
	okButton = new JButton ((String)cc.rb.getObject ("general_Ok"));
	okButton.setMnemonic ( ((Integer)cc.rb.getObject("general_Ok_mnemonic")).intValue());
	okButton.setActionCommand (OK);
	okButton.addActionListener (this);
	buttons.add (okButton);
	applyButton = new JButton ((String)cc.rb.getObject ("general_Apply"));
	applyButton.setMnemonic ( ((Integer)cc.rb.getObject("general_Apply_mnemonic")).intValue());
	applyButton.setActionCommand (APPLY);
	applyButton.addActionListener (this);
	buttons.add (applyButton);
	cancelButton = new JButton ((String)cc.rb.getObject ("general_Cancel"));
	cancelButton.setMnemonic ( ((Integer)cc.rb.getObject("general_Cancel_mnemonic")).intValue());
	cancelButton.setActionCommand (CANCEL);
	cancelButton.addActionListener (this);
	buttons.add (cancelButton);
	// add them all to the dialog
	getRootPane().setDefaultButton (applyButton);
	content.setLayout (layout);
	constraints.gridy = 0;
	constraints.gridx = 0;
	layout.setConstraints(centerLabel, constraints);
	content.add (centerLabel);
	constraints.gridx = 1;
	layout.setConstraints(center_x_TF, constraints);
	content.add (center_x_TF);
	constraints.gridx = 2;
	layout.setConstraints(center_y_TF, constraints);
	content.add (center_y_TF);

	constraints.gridy = 1;
	constraints.gridx = 0;
	layout.setConstraints(radiusLabel, constraints);
	content.add (radiusLabel);
	constraints.gridx = 1;
	constraints.gridwidth = 2;
	layout.setConstraints(radiusTF, constraints);
	content.add (radiusTF);

	constraints.gridy = 2;
	constraints.gridx = 0;
	constraints.gridwidth = 1;
	layout.setConstraints(angle1Label, constraints);
	content.add (angle1Label);
	constraints.gridx = 1;
	constraints.gridwidth = 2;
	layout.setConstraints(angle1TF, constraints);
	content.add (angle1TF);

	constraints.gridy = 3;
	constraints.gridx = 0;
	constraints.gridwidth = 1;
	layout.setConstraints(angle2Label, constraints);
	content.add (angle2Label);
	constraints.gridx = 1;
	constraints.gridwidth = 2;
	layout.setConstraints(angle2TF, constraints);
	content.add (angle2TF);

	constraints.gridy = 4;
	constraints.gridx = 0;
	constraints.gridwidth = 3;
	layout.setConstraints(buttons, constraints);
	content.add (buttons);
	// finish the GUI
	getContentPane ().add (content);
	addComponentListener (listener);
	//setSize (300,150);
	pack ();
    }

    /**
     *  this method can be used to update the GUI of the dialog dynamically.
     *  it is called by the DialogManager e.g. when some comboboxes containing
     *  boundary information need to be refreshed
     */
    public void refreshGUI ()
    {
	// no need fort this in this class :-)
	return;
    }


    /**
     *  sets the source line the dialog instance works on
     */
    public void setSource (SegmentCircle circle)
    {
	source = circle;
	if (circle != null)
	{
	    center_x_TF.setText (Double.toString (cc.gc.fromWorldToUser(circle.getMidPoint()[0])));
	    center_y_TF.setText (Double.toString (cc.gc.fromWorldToUser (circle.getMidPoint()[1])));
	    angle1TF.setText (Double.toString (circle.getStartAngle()));
	    angle2TF.setText (Double.toString (circle.getEndAngle()));
	    radiusTF.setText (Double.toString (cc.gc.fromWorldToUser (circle.getRadius())));
	}
    }

    /**
     *  sets the values of the centerpoint textfield
     *
     *  @param x,y - the centerpoint's user coordinates
     */
    public void setCenter (double x, double y)
    {
	center_x_TF.setText (Double.toString (x));
	center_y_TF.setText (Double.toString (y));
    }

    /**
     *  sets the values of the angles textfield
     *
     *  @param start, end - start- and endangle of the circle
     */
    public void setAngles (double start, double end)
    {
	angle1TF.setText (Double.toString (start));
	angle2TF.setText (Double.toString (end));
    }

    /**
     *  sets the value for the radius textfield
     *
     *  @param radius - the radius for the circle (user coord)
     */
    public void setRadius (double radius)
    {
	radiusTF.setText (Double.toString (radius));
    }

    /**
     *  sets the orientation flag
     *  @param or - mathematically positive or negative orientation, 
     *  see constants below in this class
     */
    public void setOrientation (boolean ori)
    {
	orientation = ori;
    }

    /**
     *  This class handles its own action event. 
     */
    public void actionPerformed (ActionEvent e)
    {
	if (e.getActionCommand().equals (OK))
	{
	    if (verify ())
	    {
		if (source != null)
		{
		    performUpdateCircle ();
		}
		else
		{
		    performAddCircle ();
		}
		cc.dm.hide (cc.dm.CIRCLEDIALOG);
	    }
	    return;
	}
	if (e.getActionCommand().equals (APPLY))
	{
	    if (verify ())
	    {
		if (source != null)
		{
		    performUpdateCircle ();
		}
		else
		{
		    performAddCircle ();
		}
	    }
	    return;
	}
	if (e.getActionCommand().equals (CANCEL))
	{
	    cc.dm.hide (cc.dm.CIRCLEDIALOG);
	    return;
	}
    }

   /**
     *  the boring verification code is outsourced into this method
     */
    private boolean verify ()
    {
	boolean allOK = true;
	try
	{
	    double xu = Double.parseDouble (center_x_TF.getText ());
	    x = cc.gc.fromUserToWorld(xu);
	}
	catch (NumberFormatException ex)
	{
	    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject ("dialogs_circle_error_x"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
	    allOK = false;
	}
	try
	{
	    double yu = Double.parseDouble (center_y_TF.getText ());
	    y = cc.gc.fromUserToWorld(yu);
	}
	catch (NumberFormatException ex)
	{
	    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject ("dialogs_circle_error_y"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
	    allOK = false;
	}
	try
	{
	    double ru = Double.parseDouble (radiusTF.getText ());
	    r = cc.gc.fromUserToWorld(ru);
	    if (r <= 0)
		throw new NumberFormatException ();
	}
	catch (NumberFormatException ex)
	{
	    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject ("dialogs_circle_error_radius"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
	    allOK = false;
	}
	try
	{
	    s = Double.parseDouble (angle1TF.getText ());

	}
	catch (NumberFormatException ex)
	{
	    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject ("dialogs_circle_error_start"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
	    allOK = false;
	}
	try
	{
	    e = Double.parseDouble (angle2TF.getText ());
	}
	catch (NumberFormatException ex)
	{
	    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject ("dialogs_circle_error_end"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
	    allOK = false;
	}
	// check for correct orientation
	if (orientation == POS_ORIENTATION)
	{
	    if (s > e)
	    {
		JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject ("dialogs_circle_error_orientation"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
		allOK = false;
	    }
	}
	else
	{
	    if (s < e)
	    {
		JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject ("dialogs_circle_error_orientation"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
		allOK = false;
	    }
	}
	return allOK;
    }


    /**
     *  adds a new circle segment to the domain with the parameter values from
     *  the dialog textfields
     */
    private void performAddCircle ()
    {
	DrawingArea da = parent.getDrawingArea ();
	// Create a new circle object and add it to the domain
	SegmentCircle newcirc = new SegmentCircle (x,y,r,s,e);
	cc.d.db.addSegment (cc.gc.workingBoundary,newcirc);
	Vector tmpvec = new Vector ();
	tmpvec.add (newcirc);
	cc.um.notifyUndo (cc.um.UNDO_ADDCIRCLE, tmpvec, null);
	// give visual feedback
	da.repaint ();
    }

    /**
     *  updates the segment circle referenced by the source variable with
     *  the parameter values from the dialog textfields
     */
    private void performUpdateCircle ()
    {
	DrawingArea da = parent.getDrawingArea ();
	// prepare undo
	int[] info = new int[5];
	info[0] = source.getMidPoint ()[0];
	info[1] = source.getMidPoint ()[1];
	info[2] = source.getRadius ();
	info[3] = (int)source.getStartAngle ();
	info[4] = (int)source.getEndAngle ();
	Vector tmpvec = new Vector ();
	tmpvec.add (source);
	cc.um.notifyUndo (cc.um.UNDO_PROPERTYCIRC, tmpvec, info);
	// update circle
	source.setMidPoint (x,y);
	source.setRadius (r);
	source.setStartAngle (s);
	source.setEndAngle (e);	
	da.repaint ();
    }

    // keys for the action commands

    private static final String OK = "OK";
    private static final String CANCEL = "CANCEL";
    private static final String APPLY = "APPLY";

    // buttons

    private JButton okButton;
    private JButton cancelButton;
    private JButton applyButton;
    private JTextField radiusTF;
    private JTextField angle1TF;
    private JTextField angle2TF;
    private JTextField center_x_TF;
    private JTextField center_y_TF;

    // the values from the dialog's textfields
    private int x,y,r;
    private double s,e;


    // constants for the orientation
    public static final boolean POS_ORIENTATION = true;
    public static final boolean NEG_ORIENTATION = false;

}
