package devisor2.grid.GUI.dialogs;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import devisor2.grid.options.*;

/** 
 *  This class is purely convenience: It just extends the default color 
 *  chooser of the swing API to suit our needs. <br>
 *  More precisely: Everything we wanted to be the same in all instances of 
 *  the dialog was hard-coded, everything we wanted to change at times 
 *  was put into the parameter list of the <code>showDialog()<\code> methos.
 *  <br> <br>
 *
 *  @author Dominik Goeddeke
 *  @version fully tested
 */

public class ColorPicker extends JColorChooser
{
    /**
     *  The constructor just overrides the old constructor. There's no need 
     *  to use it directly, the showDialog method does the trick
     */
    public ColorPicker (Color initialColor)
    {
	super (initialColor);
	setPreviewPanel (new JPanel ());
    }

    /**
     *  displays a modal Color dialog.
     *  @param title - the title string for the dialog
     *  @param initialColor - the initially selected Color
     *  @param parent - the component the dialog is centered upon
     *
     *  @return a Color object with value initialColor if the user pressed 
     *  Cancel or the selected color respectively
     */
    public static Color show (String title, Color initialColor, Component parent )
    {

	Color c = showDialog (parent, title, initialColor);
	if (c == null)
	    c = initialColor;
	return c;
    }

}
