package devisor2.grid.GUI.dialogs;

import java.util.*;
import javax.swing.*;
import java.awt.*;
import devisor2.foundation.base.*;
import devisor2.grid.GUI.help.*;
import devisor2.grid.GUI.framework.MainFrame;
import devisor2.grid.backend.spellchecker.*;
import devisor2.grid.options.*;

public class DialogManager
{

    private MainFrame parent;

    private ControlCenter cc;

    public static final int DIALOGCOUNT = 18;

    public static final int BOUNDARYNODEDIALOG = 0;
    public static final int CHANGEBOUNDARYDIALOG = 1;
    public static final int EDGESTATUSDIALOG = 2;
    public static final int LINEDIALOG = 3;
    public static final int LOADBALANCINGDIALOG = 4;
    public static final int MERGEBOUNDARIESDIALOG = 5;
    public static final int MIRRORDIALOG = 6;
    public static final int MULTIPLEBNDIALOG = 7;
    public static final int PARALLELBLOCKDIALOG = 8;
    public static final int PERIMETERDIALOG = 9;
    public static final int QUADDIALOG = 10;
    public static final int REFINEDIALOG = 11;
    public static final int ROTATEDIALOG = 12;
    public static final int SCALEDIALOG = 13;
    public static final int SPELLCHECKERDIALOG = 14;
    public static final int TRIDIALOG = 15;
    public static final int CIRCLEDIALOG = 16;
    public static final int MACRODIALOG = 17;


    private DockableDialog[] dialogs;


    public DialogManager (MainFrame parent)
    {
	dialogs = new DockableDialog[DIALOGCOUNT];
	this.parent = parent;
	cc = ControlCenter.getMyself ();
    }

    public void initDialogs ()
    {
	// important: DO NOT MESS AROUND WITH THE ORDER OF THESE INITS!!!
	dialogs[BOUNDARYNODEDIALOG] = new BoundaryNodeDialog (parent);
	dialogs[CHANGEBOUNDARYDIALOG] = new ChangeBoundaryDialog (parent);
	dialogs[EDGESTATUSDIALOG] = new EdgeStatusDialog (parent);
	dialogs[LINEDIALOG] = new LineDialog (parent);
	dialogs[LOADBALANCINGDIALOG] = new LoadBalancingDialog (parent);
	dialogs[MERGEBOUNDARIESDIALOG] = new MergeBoundariesDialog (parent);
	dialogs[MIRRORDIALOG] = new MirrorDialog (parent);
	dialogs[MULTIPLEBNDIALOG] = new MultipleBNDialog (parent);
	dialogs[PARALLELBLOCKDIALOG] = new ParallelBlockDialog (parent);
	dialogs[PERIMETERDIALOG] = new PerimeterDialog (parent);
	dialogs[QUADDIALOG] = new QuadDialog (parent);
	dialogs[REFINEDIALOG] = new RefineDialog (parent);
	dialogs[ROTATEDIALOG] = new RotateDialog (parent);
	dialogs[SCALEDIALOG] = new ScaleDialog (parent);
	dialogs[SPELLCHECKERDIALOG] = new SpellCheckerDialog (parent);
	dialogs[TRIDIALOG] = new TriDialog (parent);
	dialogs[CIRCLEDIALOG] = new CircleDialog (parent);
	dialogs[MACRODIALOG] = new MacroDialog (parent);

	for (int i=0; i<dialogs.length; i++)
	{
	    // extract docking state from Options
	    dialogs[i].setDocked (new Boolean (cc.op.get ("dockstate_"+Integer.toString(i))).booleanValue ());
	    // extract position from Options if neccessary
	    if (dialogs[i].isDocked())
	    {
		int x = parent.getLocationOnScreen().x + Integer.parseInt (cc.op.get ("dockposition_x_"+Integer.toString(i)));
		int y = parent.getLocationOnScreen().y + Integer.parseInt (cc.op.get ("dockposition_y_"+Integer.toString(i)));
		dialogs[i].setPositionOnScreen (new Point (x,y));
	    }
	    else
	    {
		dialogs[i].setPositionOnScreen (dialogs[i].getInitialPosition());
		dialogs[i].setDocked (true);
	    }
	}

	OptionsDialog.initialize (parent);
	InfoDialog.initialize (parent);

    }

    public void refreshDialogs ()
    {
	String[] captions = new String[DIALOGCOUNT];
	boolean visible[] = new boolean[DIALOGCOUNT];
	for (int i=0; i<dialogs.length; i++)
	{
	    captions[i] = dialogs[i].getTitle ();
	    visible[i] = dialogs[i].isVisible ();
	    hide (i);
	}
	initDialogs ();
	for (int i=0; i<dialogs.length; i++)
	{
	    dialogs[i].setTitle (captions[i]);
	    if (visible[i])
		dialogs[i].setVisible (true);
	}
    }

    public void hide (int dialog)
    {
	if (dialogs[dialog].isDocked())
	{
	    
	    cc.op.set ("dockstate_"+Integer.toString(dialog), "true");
	    cc.op.set ("dockposition_x_"+Integer.toString(dialog), Integer.toString (dialogs[dialog].getPositionOnScreen().x - parent.getLocationOnScreen().x));
	    cc.op.set ("dockposition_y_"+Integer.toString(dialog), Integer.toString (dialogs[dialog].getPositionOnScreen().y - parent.getLocationOnScreen().y));
	}
	else
	{
	    cc.op.set ("dockstate_"+Integer.toString(dialog), "false");
	    cc.op.set ("dockposition_x_"+Integer.toString(dialog), Integer.toString (dialogs[dialog].getInitialPosition().x - parent.getLocationOnScreen().x));
	    cc.op.set ("dockposition_y_"+Integer.toString(dialog), Integer.toString (dialogs[dialog].getInitialPosition().y - parent.getLocationOnScreen().y));
	}
	dialogs[dialog].setVisible (false);
    }

    public void hideAll ()
    {
	for (int i=0; i<dialogs.length; i++)
	{
	    hide (i);
	}
    }

    public DockableDialog get (int index)
    {
	return dialogs[index];
    }

    public void refreshAllGUI ()
    {
	for (int i=0; i<dialogs.length; i++)
	{
	    dialogs[i].refreshGUI ();
	}
    }

}

