package devisor2.grid.GUI.dialogs;

import java.util.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

import devisor2.grid.options.*;
import devisor2.grid.GUI.framework.*;
import devisor2.foundation.elements.*;
import devisor2.foundation.*;


/**
 *  This class is the DockableDialog instance to set the status of edges. <br>
 *  When it is displayed via the corresponding menu entry, the application
 *  switches to EDGE STATUS MODE, and every click on an edge sets the edge's
 *  status to the values set in the dialog. <br>
 *  To avoid looking up all these values every time, they are stored in the 
 *  ControlCenter instance in an array. See there for details. Whenever 
 *  changes are made in the dialog, this instance is automatically updated. 
 *  <br>
 *  Take a look at our technical paper about dialogs if you want to learn
 *  more, to be found in the /manual/specs directory of your installation.
 *
 *  @author Dominik Goeddeke
 *  @version 1.0 fully tested
 *  
 */

public class EdgeStatusDialog extends DockableDialog implements ActionListener
{
    private int status;
    private int bc;

    /**
     *  this method can be used to update the GUI of the dialog dynamically.
     *  it is called by the DialogManager e.g. when some comboboxes containing
     *  boundary information need to be refreshed
     */
    public void refreshGUI ()
    {
	// no need fort this in this class :-)
	return;
    }


    /** 
     *  returns the initial Position of the dialog
     */
    public Point getInitialPosition ()
    {
	return new Point (parent.getLocationOnScreen().x + parent.getWidth(), parent.getLocationOnScreen().y);
    }


    /**
     *  the constructor just sets references correctly and creates the GUI
     */
    public EdgeStatusDialog (MainFrame parent)
    {
	super (parent);
	setTitle ((String)cc.rb.getObject ("dialogs_edgestatus_title"));
	JPanel content = new JPanel ();
	// just ordinary GUI layouting
	GridBagLayout layout = new GridBagLayout ();
	GridBagConstraints constraints = new GridBagConstraints ();
	constraints.fill = GridBagConstraints.BOTH;
	constraints.weightx = constraints.weighty = 1.0;
	// the edgestatus button group
	ButtonGroup group1 = new ButtonGroup ();
	JPanel jstatus = new JPanel ();
	jstatus.setLayout (new FlowLayout ());
	jstatus.setBorder (BorderFactory.createTitledBorder ((String)cc.rb.getObject ("dialogs_edgestatus_status_caption")));
	noneButton = new JRadioButton ((String)cc.rb.getObject ("dialogs_edgestatus_status_none"));
	noneButton.setActionCommand (NONE);
	noneButton.addActionListener (this);
	group1.add (noneButton);
	jstatus.add (noneButton);
	realButton = new JRadioButton ((String)cc.rb.getObject ("dialogs_edgestatus_status_real"));
	realButton.setActionCommand (REAL);
	realButton.addActionListener (this);
	group1.add (realButton);
	jstatus.add (realButton);
	innerButton = new JRadioButton ((String)cc.rb.getObject ("dialogs_edgestatus_status_inner"));
	innerButton.setActionCommand (INNER);
	innerButton.addActionListener (this);
	group1.add (innerButton);
	jstatus.add (innerButton);
	mirrorButton = new JRadioButton ((String)cc.rb.getObject ("dialogs_edgestatus_status_mirror"));
	mirrorButton.setActionCommand (MIRROR);
	mirrorButton.addActionListener (this);
	group1.add (mirrorButton);
	jstatus.add (mirrorButton);
	// the bc button group
	ButtonGroup group2 = new ButtonGroup ();
	JPanel jbc = new JPanel ();
	jbc.setLayout (new FlowLayout ());
	jbc.setBorder (BorderFactory.createTitledBorder ((String)cc.rb.getObject ("dialogs_edgestatus_bc_caption")));
	
	
//	bcnoneButton = new JRadioButton ((String)cc.rb.getObject ("dialogs_edgestatus_bc_none"));
//	bcnoneButton.setActionCommand (BCNONE);
//	bcnoneButton.addActionListener (this);
//	group2.add (bcnoneButton);
//	bc.add (bcnoneButton);

	dirichletButton = new JRadioButton ((String)cc.rb.getObject ("dialogs_edgestatus_bc_dirichlet"));
	dirichletButton.setActionCommand (DIRICHLET);
	dirichletButton.addActionListener (this);
	group2.add (dirichletButton);
	jbc.add (dirichletButton);
	neumannButton = new JRadioButton ((String)cc.rb.getObject ("dialogs_edgestatus_bc_neumann"));
	neumannButton.setActionCommand (NEUMANN);
	neumannButton.addActionListener (this);
	group2.add (neumannButton);
	jbc.add (neumannButton);
	// the buttons
	JPanel buttons = new JPanel ();
	buttons.setLayout (new FlowLayout ());
	automaticButton = new JButton ((String)cc.rb.getObject ("dialogs_edgestatus_automatic"));
	automaticButton.setToolTipText ((String)cc.rb.getObject ("dialogs_edgestatus_automatic_tooltip"));
	automaticButton.setMnemonic ( ((Integer)cc.rb.getObject("dialogs_edgestatus_automatic_mnemonic")).intValue());
	automaticButton.setActionCommand (AUTOMATIC);
	automaticButton.addActionListener (this);
	buttons.add (automaticButton);
	
	applyButton = new JButton ((String)cc.rb.getObject ("dialogs_edgestatus_apply"));
	applyButton.setToolTipText ((String)cc.rb.getObject ("dialogs_edgestatus_apply_tooltip"));
	applyButton.setMnemonic ( ((Integer)cc.rb.getObject("dialogs_edgestatus_automatic_mnemonic")).intValue());
	applyButton.setActionCommand (APPLY);
	applyButton.addActionListener (this);
	buttons.add (applyButton);
		
	closeButton = new JButton ((String)cc.rb.getObject ("dialogs_edgestatus_close"));
	closeButton.setToolTipText ((String)cc.rb.getObject ("dialogs_edgestatus_close_tooltip"));
	closeButton.setMnemonic ( ((Integer)cc.rb.getObject("dialogs_edgestatus_close_mnemonic")).intValue());
	closeButton.setActionCommand (CLOSE);
	closeButton.addActionListener (this);
	buttons.add (closeButton);
	// add them all to the dialog
	getRootPane().setDefaultButton (closeButton);
	content.setLayout (layout);
	content.setBorder (new EmptyBorder(10,10,10,10));
	constraints.gridy = 0;
	layout.setConstraints(jstatus, constraints);
	content.add (jstatus);
	constraints.gridy = 1;
	layout.setConstraints(jbc, constraints);
	content.add (jbc);
	constraints.gridy = 2;
	layout.setConstraints (buttons, constraints);
	content.add (buttons);
	getContentPane().add (content);
	// finish the GUI
	addComponentListener (listener);
	status=Edge.STATUS_REAL;
	bc=0;
	updateButtons();
	pack ();
    }

    /**
     *  sets the Status radio Button to the given parameter 
     *  (see parameter constants in class Edge)
     */
    public void setStatus (int stat)
    {
	status = stat;
	updateButtons ();
    }

    /**
     *  sets the boundary contition radio Button to the given parameter 
     *  (see parameter constants in class Edge)
     */
    public void setBC (int bcond)
    {
	bc = bcond;
	updateButtons ();
    }

    /**
     *  returns the current status set in the radiobuttons
     */
    public int getStatus ()
    {
	if (innerButton.isSelected ())
	    return Edge.STATUS_INNER;
	if (realButton.isSelected ())
	    return Edge.STATUS_REAL;
	if (mirrorButton.isSelected ())
	    return Edge.STATUS_MIRRORB;
	return Edge.STATUS_NONE;
    }

    /**
     * returns the current Boundary condition set in the radiobuttons
     */
    public int getBC ()
    {
	if (dirichletButton.isSelected ())
	    return Edge.BC_DIRICHLET;
	if (neumannButton.isSelected ())
	    return Edge.BC_NEUMANN;
	return Edge.BC_NONE;
    }

    /**
     *  sets the buttons according to the inner status and bc variables
     */
    private void updateButtons ()
    {
	switch (status)
	{
	case (Edge.STATUS_NONE) : 
	    {
		noneButton.setSelected (true);
		disableBC ();
		break;
	    }
	case (Edge.STATUS_INNER) :
	    {
		innerButton.setSelected (true);
		disableBC ();
		break;
	    }
	case (Edge.STATUS_REAL) :
	    {
		realButton.setSelected (true);
		
		if (bc==Edge.BC_NEUMANN)
		{
		  neumannButton.setSelected (true);
		  dirichletButton.setSelected (false);
		}
		if (bc==Edge.BC_DIRICHLET)
		{
		  neumannButton.setSelected (false);
		  dirichletButton.setSelected (true);
		}		
		
		enableBC ();
		break;
	    }
	case (Edge.STATUS_MIRRORB) :
	    {
		mirrorButton.setSelected (true);
		disableBC ();
		break;
	    }
	}
    }

    /**
     *  disables all buttons for boundary condition
     */
    private void disableBC ()
    {
//	bcnoneButton.setEnabled (false);
	dirichletButton.setEnabled (false);
	neumannButton.setEnabled (false);
    }

    /**
     *  enables all buttons for boundary condition
     */
    private void enableBC ()
    {
//	bcnoneButton.setEnabled (true);
	dirichletButton.setEnabled (true);
	neumannButton.setEnabled (true);
    }

    /**
     *  This class handles its own action event. The actionPerformed method 
     *  just enables or disables the boundary condition radiobuttons
     */
    public void actionPerformed (ActionEvent e)
    {
	if (e.getActionCommand().equals (NONE))
	{
	    if (noneButton.isSelected ())
		disableBC ();
	    else
		enableBC ();
	    return;
	}
	if (e.getActionCommand().equals (INNER))
	{
	    if (innerButton.isSelected ())
		disableBC ();
	    else
		enableBC ();
	    return;
	}
	if (e.getActionCommand().equals (REAL))
	{
	    if (realButton.isSelected ())
		enableBC ();
	    else
		disableBC ();
	    return;
	}
	if (e.getActionCommand().equals (MIRROR))
	{
	    if (mirrorButton.isSelected ())
		disableBC ();
	    else
		enableBC ();
	    return;
	}
	if (e.getActionCommand ().equals (CLOSE))
	{
	    cc.dm.hide (cc.dm.EDGESTATUSDIALOG);
	    return;
	}
	
	if (e.getActionCommand ().equals (APPLY))
	{
	//    hideDialog ();
	    
	    int status = 0;
	    int bc = 0;
	    
            if (innerButton.isSelected()) status=1;
            if (realButton.isSelected()) status=2;
            if (mirrorButton.isSelected()) status=3;

            if (dirichletButton.isSelected()) bc=0;
            if (neumannButton.isSelected()) bc=1;
      	    
	    Vector selectedItems = parent.getDrawingArea().getListener().getSelectedItems();
	 
	    Enumeration enum = selectedItems.elements();
	    
	    while (enum.hasMoreElements())
	    {
	      GridItem c = (GridItem)enum.nextElement ();
	      
	      if (c instanceof Edge)
	      {	    
	        ((Edge)c).setStatus(status);
	        ((Edge)c).setBC(bc);
	      }
	    }
	    
	    parent.getDrawingArea().repaint();
	    
	    return;
	}
	
	if (e.getActionCommand ().equals (AUTOMATIC))
	{
	    
          int ce = cc.d.getEdgeCount();

          Edge ed;
          cc.d.updateAll();

          for(int index=0;index<ce;index++)
          {
            ed=cc.d.getEdge(index);
			
            Object[] parents1 = ed.getParents();

            if (dirichletButton.isSelected()) 
	      ed.setBC(0);
           else 
	      ed.setBC(1);

           if (parents1.length==1)
             ed.setStatus(1);
           else
             if ( ((Cell2D)parents1[0]).getParallelBlock()!=
               ((Cell2D)parents1[1]).getParallelBlock())
               ed.setStatus(2);
             else
               ed.setStatus(0);

           }

           parent.getDrawingArea().repaint();

//	    hideDialog ();
	    return;
	}
   }

    // keys for the action commands
    private static final String NONE = "NONE";
    private static final String REAL = "REAL";
    private static final String INNER = "INNER";
    private static final String MIRROR = "MIRROR";
    private static final String DIRICHLET = "DIRICHLET";
    private static final String NEUMANN = "NEUMANN";
    private static final String BCNONE = "BCNONE";
    private static final String CLOSE = "CLOSE";
    private static final String AUTOMATIC = "AUTOMATIC";
    private static final String APPLY = "APPLY";

    // buttons
    private JRadioButton noneButton;
    private JRadioButton realButton;
    private JRadioButton innerButton;
    private JRadioButton mirrorButton;
 //   private JRadioButton bcnoneButton;
    private JRadioButton dirichletButton;
    private JRadioButton neumannButton;
    private JButton closeButton;
    private JButton automaticButton;
    private JButton applyButton;
    
   
}
