package devisor2.grid.GUI.dialogs;


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import devisor2.grid.GUI.framework.*;
import devisor2.grid.options.*;


/** 
 *  This class defines our basic info dialog with version information, 
 *  developers, revision history etc. It is linked to the GUI via the 
 *  MainMenu's About... entry. <br>
 *  <br>
 *  This is a regular dialog, so it is used just like every other dialog as 
 *  well, if you want to know more, take a look at the technical paper
 *  which explains dialogs to be found in the /manual/specs directory of
 *  your DeViSoRGriD 2 installation. <br>
 *  In this class, the version number is managed!
 *
 *  @author Dominik Goeddeke
 *  @version 1.0 (quite finished)
 */
public class InfoDialog extends JDialog 
{

    public static final int version_level1 = 2;
    public static final int version_level2 = 1;
    public static final int version_level3 = 0;

    public static final String release_date = "October 17th, 2003";

    /** 
     *  the info dialog is displayed via two static methods of this class
     *  (see below), this class variable just references the instance 
     *  which is ready for display
     */
    private static InfoDialog infodialog;

    /** 
     *  the MainFrame object of this dialog. important because the dialog is
     *  centered on it 
     */
    private MainFrame parent;

    /**
     *  the ControlCenter reference
     */
    private ControlCenter cc;


    /** 
     *  creates a new instance of the dialog
     *  @param JFrame parent the parent frame the dialog is centered on 
     */
    public InfoDialog (MainFrame parent)
    {
	super (parent, true);
	this.parent = parent;
	cc = ControlCenter.getMyself ();
	layoutGui ();
	setLocationRelativeTo (parent);
	setDefaultCloseOperation (DO_NOTHING_ON_CLOSE);
	setResizable (false);
	pack ();
    }

    /** 
     *  creates the gui (nothing spectacular here) 
     */
    private void layoutGui ()
    {
    	// the basics
    	setTitle ((String)cc.rb.getObject ((String)"GUI_dialogs_InfoDialog_frametitle"));
    	JPanel panel = new JPanel ();
    	panel.setLayout (new BorderLayout ());
	panel.add (Box.createRigidArea (new Dimension(20,20)), BorderLayout.WEST);
	panel.add (Box.createRigidArea (new Dimension(20,20)), BorderLayout.EAST);
	// north we have the logo
	JLabel logo = new JLabel (new ImageIcon(cc.op.get(Options.general_devisorhome)+"images/devisorlogo.gif"));
	panel.add (logo, BorderLayout.NORTH);
	// in the middle some text
	JPanel center = new JPanel ();
	GridBagLayout layout = new GridBagLayout ();
	GridBagConstraints constraints = new GridBagConstraints ();
	JLabel label;
	center.setLayout (layout);
	constraints.fill = GridBagConstraints.HORIZONTAL;
	constraints.weightx = 1.0;
	// programm info
	JPanel about = new JPanel ();
	about.setLayout (new BoxLayout (about, BoxLayout.Y_AXIS));
	about.setBorder (BorderFactory.createTitledBorder ((String)cc.rb.getObject("GUI_dialogs_InfoDialog_about")));
	label = new JLabel ((String)cc.rb.getObject("GUI_dialogs_InfoDialog_abouttext1"));
	label.setAlignmentX (Component.CENTER_ALIGNMENT);
	about.add (label);
	label = new JLabel ((String)cc.rb.getObject("GUI_dialogs_InfoDialog_abouttext2"));
	label.setAlignmentX (Component.CENTER_ALIGNMENT);
	about.add (label);
	label = new JLabel ((String)cc.rb.getObject("GUI_dialogs_InfoDialog_abouttext3"));
	label.setAlignmentX (Component.CENTER_ALIGNMENT);
	about.add (label);
	label = new JLabel ((String)cc.rb.getObject("GUI_dialogs_InfoDialog_abouttext4"));
	label.setAlignmentX (Component.CENTER_ALIGNMENT);
	about.add (label);
	constraints.gridy = 0;
	layout.setConstraints(about, constraints);
	center.add (about);
	// version info
	JPanel version = new JPanel ();
	version.setLayout (new BoxLayout (version, BoxLayout.Y_AXIS));
	version.setBorder (BorderFactory.createTitledBorder ((String)cc.rb.getObject("GUI_dialogs_InfoDialog_version")));
	label = new JLabel (version_level1+"."+version_level2+"."+version_level3);
	label.setAlignmentX (Component.CENTER_ALIGNMENT);
	version.add (label);
	label = new JLabel (release_date);
	label.setAlignmentX (Component.CENTER_ALIGNMENT);
	version.add (label);
	constraints.gridy = 1;
	layout.setConstraints(version, constraints);
	center.add (version);
	// developers info
	JPanel dev = new JPanel ();
	dev.setLayout (new BoxLayout (dev, BoxLayout.Y_AXIS));
	dev.setBorder (BorderFactory.createTitledBorder ((String)cc.rb.getObject("GUI_dialogs_InfoDialog_developers")));
	label = new JLabel ("Christian Becker");
	label.setAlignmentX (Component.CENTER_ALIGNMENT);
	dev.add (label);
	label = new JLabel ("Dominik Gddeke");
	label.setAlignmentX (Component.CENTER_ALIGNMENT);
	dev.add (label);
	label = new JLabel ("Patrick Otto");
	label.setAlignmentX (Component.CENTER_ALIGNMENT);
	dev.add (label);
	dev.add (new JLabel ("   "));
	label = new JLabel ("Institute of Numerics and Applied Mathematics");
	label.setAlignmentX (Component.CENTER_ALIGNMENT);
	dev.add (label);
	label = new JLabel ("University of Dortmund, Germany");
	label.setAlignmentX (Component.CENTER_ALIGNMENT);
	dev.add (label);
	constraints.gridy = 2;
	layout.setConstraints(dev, constraints);
	center.add (dev);
	// contact info
	JPanel contact = new JPanel ();
	contact.setLayout (new BoxLayout (contact, BoxLayout.Y_AXIS));
	contact.setBorder (BorderFactory.createTitledBorder ((String)cc.rb.getObject("GUI_dialogs_InfoDialog_contact")));
	label = new JLabel ((String)cc.rb.getObject("GUI_dialogs_InfoDialog_contact_text1"));
	label.setAlignmentX (Component.CENTER_ALIGNMENT);
	contact.add (label);
	label = new JLabel ((String)cc.rb.getObject("GUI_dialogs_InfoDialog_contact_text2"));
	label.setAlignmentX (Component.CENTER_ALIGNMENT);
	contact.add (label);
	label = new JLabel ("http://www.featflow.de");
	label.setAlignmentX (Component.CENTER_ALIGNMENT);
	contact.add (label);
	label = new JLabel ((String)cc.rb.getObject("GUI_dialogs_InfoDialog_contact_text3"));
	label.setAlignmentX (Component.CENTER_ALIGNMENT);
	contact.add (label);
	label = new JLabel ("devisor@featflow.de");
	label.setAlignmentX (Component.CENTER_ALIGNMENT);
	contact.add (label);
	constraints.gridy = 3;
	layout.setConstraints(contact, constraints);
	center.add (contact);
	panel.add (center, BorderLayout.CENTER);
	// a button goes south
	JButton button = new JButton ("OK");
	button.addActionListener (new ActionListener () {
		public void actionPerformed (ActionEvent e) {
		    dispose ();
		}
	    });
	button.setAlignmentX(Component.CENTER_ALIGNMENT);
	button.setSize (150, 40);
	panel.add (button, BorderLayout.SOUTH);
	getRootPane().setDefaultButton (button);
	getContentPane().add(panel);
	setResizable (false);
    }

    /** 
     *  this is one of the two important methods in this class.
     *  it instantiates a new dialog object and sets up its gui
     */
    public static void initialize(MainFrame parent) {
	infodialog = new InfoDialog (parent);
    }
  
    /**
     * if initialize() has been called before (i.e. the dialog has been 
     * instanciated), this method just lets the dialog pop up modally on the
     * screen.
     */
    public static void showDialog() {
        if (infodialog != null) 
        {
	    infodialog.setLocationRelativeTo (infodialog.parent);
            infodialog.setVisible(true);
        } 
	else 
        {
            System.err.println("InfoDialog requires you to call initialize "
                               + "before calling showDialog.");
        }
    }

}
