package devisor2.grid.GUI.dialogs;

import java.util.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

import devisor2.grid.options.*;
import devisor2.grid.GUI.framework.*;
import devisor2.foundation.boundary.*;
import devisor2.foundation.elements.*;


/**
 *  This class is the DockableDialog instance for Line input. It is used both
 *  for direct input of Line Segments and for Property Queries of Lines.<br>
 *  See the GridListener class for usage instructions, and of course the 
 *  technical paper about dialogs in general.
 *
 *
 *  @author Dominik Goeddeke
 *  @version 0.5 (GUI, no real functionality)
 *  
 */

public class LineDialog extends DockableDialog implements ActionListener
{

    // the source (the line this dialog works on)
    private SegmentLine source;

    /**
     *  this method can be used to update the GUI of the dialog dynamically.
     *  it is called by the DialogManager e.g. when some comboboxes containing
     *  boundary information need to be refreshed
     */
    public void refreshGUI ()
    {
	// no need fort this in this class :-)
	return;
    }


    /** 
     *  returns the initial Position of the dialog
     */
    public Point getInitialPosition ()
    {
	return new Point (parent.getLocationOnScreen().x - 50, parent.getLocationOnScreen().y);
    }


    /**
     *  the constructor just sets references correctly and creates the GUI
     */
    public LineDialog (MainFrame parent)
    {
	super (parent);
	JPanel content = new JPanel ();
	content.setBorder (new EmptyBorder (10,10,10,10));
	// just ordinary GUI layouting
	GridBagLayout layout = new GridBagLayout ();
	GridBagConstraints constraints = new GridBagConstraints ();
	constraints.fill = GridBagConstraints.BOTH;
	constraints.weightx = constraints.weighty = 1.0;
	// startpoint	
	JLabel startLabel = new JLabel ((String)cc.rb.getObject ("dialogs_line_startpoint"));
	start_x_TF = new JTextField (5);
	start_y_TF = new JTextField (5);
	// endpoint	
	JLabel endLabel = new JLabel ((String)cc.rb.getObject ("dialogs_line_endpoint"));
	end_x_TF = new JTextField (5);
	end_y_TF = new JTextField (5);
	// buttons
	JPanel buttons = new JPanel ();
	buttons.setLayout (new FlowLayout ());
	applyButton = new JButton ((String)cc.rb.getObject ("general_Apply"));
	applyButton.setMnemonic ( ((Integer)cc.rb.getObject("general_Apply_mnemonic")).intValue());
	applyButton.setActionCommand (APPLY);
	applyButton.addActionListener (this);
	buttons.add (applyButton);
	okButton = new JButton ((String)cc.rb.getObject ("general_Ok"));
	okButton.setMnemonic ( ((Integer)cc.rb.getObject("general_Ok_mnemonic")).intValue());
	okButton.setActionCommand (OK);
	okButton.addActionListener (this);
	buttons.add (okButton);
	cancelButton = new JButton ((String)cc.rb.getObject ("general_Cancel"));
	cancelButton.setMnemonic ( ((Integer)cc.rb.getObject("general_Cancel_mnemonic")).intValue());
	cancelButton.setActionCommand (CANCEL);
	cancelButton.addActionListener (this);
	buttons.add (cancelButton);
	// add them all to the dialog
	getRootPane().setDefaultButton (applyButton);
	content.setLayout (layout);
	
	constraints.gridy = 0;
	constraints.gridx = 0;
	layout.setConstraints (startLabel, constraints);
	content.add (startLabel);
	constraints.gridx = 1;
	layout.setConstraints (start_x_TF, constraints);
	content.add (start_x_TF);
	constraints.gridx = 2;
	layout.setConstraints (start_y_TF, constraints);
	content.add (start_y_TF);

	constraints.gridy = 1;
	constraints.gridx = 0;
	layout.setConstraints (endLabel, constraints);
	content.add (endLabel);
	constraints.gridx = 1;
	layout.setConstraints (end_x_TF, constraints);
	content.add (end_x_TF);
	constraints.gridx = 2;
	layout.setConstraints (end_y_TF, constraints);
	content.add (end_y_TF);

	constraints.gridy = 2;
	constraints.gridx = 0;
	constraints.gridwidth = 3;
	layout.setConstraints(buttons, constraints);
	content.add (buttons);
	// finish the GUI
	getContentPane ().add (content);
	addComponentListener (listener);
	pack ();
    }


    /**
     *  sets the source line the dialog instance works on
     */
    public void setSource (SegmentLine line)
    {
	source = line;
	if (line != null)
	{ 
	    int[] start = line.getStartPoint ();
	    int[] end = line.getEndPoint ();
	    start_x_TF.setText (Double.toString (cc.gc.fromWorldToUser(start[0])));
	    end_x_TF.setText (Double.toString(cc.gc.fromWorldToUser(end[0])));
	    start_y_TF.setText (Double.toString(cc.gc.fromWorldToUser(start[1])));
	    end_y_TF.setText (Double.toString(cc.gc.fromWorldToUser(end[1])));
	}
    }

    /**
     *  sets the values of the startpoint textfield
     *
     *  @param x,y - the startpoint's world coordinates
     */
    public void setStart (double x, double y)
    {
	start_x_TF.setText (Double.toString (x));
	start_y_TF.setText (Double.toString (y));
    }

    /**
     *  sets the values of the endpoint textfield
     *
     *  @param x,y - the endpoint's world coordinates
     */
    public void setEnd (double x, double y)
    {
	end_x_TF.setText (Double.toString (x));
	end_y_TF.setText (Double.toString (y));
    }

    /**
     *  This class handles its own action event. The actionPerformed method 
     *  just updates the ControlCenter's EdgeStatusInfo 
     */
    public void actionPerformed (ActionEvent e)
    {
	if (e.getActionCommand().equals (OK))
	{
	    if (verify ())
	    {
		if (source != null)
		{
		    performUpdateLine ();
		}
		else
		{
		    performAddLine ();
		}
	    }
	    cc.dm.hide (cc.dm.LINEDIALOG);
	    return;
	}
	if (e.getActionCommand().equals (APPLY))
	{
	    if (verify ())
	    {
		if (source != null)
		{
		    performUpdateLine ();
		}
		else
		{
		    performAddLine ();
		}
	    }
	    return;
	}
	if (e.getActionCommand().equals (CANCEL))
	{
	    cc.dm.hide (cc.dm.LINEDIALOG);
	    return;
	}
    }

    /**
     *  the boring verification code is outsourced into this method
     */
    private boolean verify ()
    {
	boolean allOK = true;
	try
	{
	    x1 = Double.parseDouble (start_x_TF.getText ());
	}
	catch (NumberFormatException ex)
	{
	    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject ("dialogs_line_error_x1"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
	    allOK = false;
	}
	try
	{
	    y1 = Double.parseDouble (start_y_TF.getText ());
	}
	catch (NumberFormatException ex)
	{
	    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject ("dialogs_line_error_y1"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
	    allOK = false;
	}
	try
	{
	    x2 = Double.parseDouble (end_x_TF.getText ());
	}
	catch (Exception ex)
	{
	    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject ("dialogs_line_error_x2"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
	    allOK = false;
	}
	try
	{
	    y2 = Double.parseDouble (end_y_TF.getText ());
	}
	catch (Exception ex)
	{
	    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject ("dialogs_line_error_y2"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
	    allOK = false;
	}
	if (x1 == x2 && y1 == y2)
	{
	    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject("gridlistener_error_samepointtwice"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
	    allOK = false;
	}
	return allOK;
    }


    /**
     *  adds a new line segment to the domain with the parameter values from
     *  the dialog textfields
     */
    private void performAddLine ()
    {
	DrawingArea da = parent.getDrawingArea ();
	// retransform
	int xx1 = cc.gc.fromUserToWorld(x1);
	int yy1 = cc.gc.fromUserToWorld(y1);
	int xx2 = cc.gc.fromUserToWorld(x2);
	int yy2 = cc.gc.fromUserToWorld(y2);
	// create new line and add it to the working boundary
	SegmentLine newline = new SegmentLine (xx1,yy1,xx2,yy2, SegmentLine.TWOPOINT);
	cc.d.db.addSegment (cc.gc.workingBoundary,newline);
	Vector tmpvec = new Vector ();
	tmpvec.add (newline);
	cc.um.notifyUndo (cc.um.UNDO_ADDLINE, tmpvec, null);
	// give visual feedback
	da.repaint ();
    }

    /**
     *  updates the segment line referenced by the source variable with
     *  the parameter values from the dialog textfields
     */
    private void performUpdateLine ()
    {
	DrawingArea da = parent.getDrawingArea ();
	// update the line object to be found in source variable
	int xx1 = cc.gc.fromUserToWorld(x1);
	int yy1 = cc.gc.fromUserToWorld(y1);
	int xx2 = cc.gc.fromUserToWorld(x2);
	int yy2 = cc.gc.fromUserToWorld(y2);
	// prepare undo
	int[] info = new int[4];
	info[0] = source.getStartPoint ()[0];
	info[1] = source.getStartPoint ()[1];
	info[2] = source.getEndPoint ()[0];
	info[3] = source.getEndPoint ()[1];
	Vector tmpvec = new Vector ();
	tmpvec.add (source);
	cc.um.notifyUndo (cc.um.UNDO_PROPERTYLINE, tmpvec, info);
	// update line
	source.setStartPoint (xx1,yy1);
	source.setEndPoint (xx2,yy2);
	da.repaint ();
    }

    // keys for the action commands

    private static final String OK = "OK";
    private static final String CANCEL = "CANCEL";
    private static final String APPLY = "APPLY";

    // buttons

    private JButton okButton;
    private JButton cancelButton;
    private JButton applyButton;
    private JTextField start_x_TF;
    private JTextField start_y_TF;
    private JTextField end_x_TF;
    private JTextField end_y_TF;

    // some storage for textfield entries
    private double x1,y1,x2,y2;

}
