package devisor2.grid.GUI.dialogs;

import java.util.*;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

import devisor2.grid.options.*;
import devisor2.grid.GUI.framework.*;
import devisor2.foundation.boundary.*;
import devisor2.foundation.elements.*;
import devisor2.foundation.base.*;
import devisor2.foundation.base.BasicDomain.DPoint;


/**
 *  This class is the DockableDialog instance for Line input. It is used both
 *  for direct input of Line Segments and for Property Queries of Lines.<br>
 *  See the GridListener class for usage instructions, and of course the 
 *  technical paper about dialogs in general.
 *
 *
 *  @author Dominik Goeddeke
 *  @version 0.5 (GUI, no real functionality)
 *  
 */

public class LoadBalancingDialog extends DockableDialog implements ActionListener
{

    public class LoadFactors
    {
	int nedges;
	int[] von;
	int[] bis;
	Vector[] fak;
	Vector[] idx;
	int[] loadmove;
  
	public LoadFactors(int n)
	{
	    nedges=n;
	    //System.out.println("INIT "+n);
	    von = new int[n+1];
	    bis = new int[n+1];
	    loadmove = new int[n+1];
	    
	    fak=new Vector[n+1];
	    idx=new Vector[n+1];
	    
	    for (int i=0;i<=n;i++)
	    {
		fak[i]= new Vector();
		idx[i]= new Vector();
	    }
	}

	public void setHeader(int idx, int uvon, int ubis)
	{
	    //System.out.println("SH "+idx+" "+uvon+" "+ubis);
	    von[idx]=uvon;
	    bis[idx]=ubis;
	}

	public void addEntryFak(int index, double ufak)
	{
	    //    System.out.println("ADD "+index+" "+fak.length+" "+ufak);
	    fak[index].addElement(new Double(ufak));
	}

	public void addEntryIdx(int index, int uidx)
	{
	    //    System.out.println("ADD "+index);
	    idx[index].addElement(new Integer(uidx));
	}
	
	public void printEntry(int index)
	{
	    System.out.println("EDGE "+von[index]+" "+bis[index]);
	    for(int i=0;i<fak[index].size();i++)
	    {
		System.out.println(((Double)fak[index].elementAt(i)).doubleValue()+" "+
				   ((Integer)idx[index].elementAt(i)).intValue());
	    }
	}

	public double calcValue(int index, int[] load)
	{
	    double sum=0;
	    
	    for(int i=0;i<fak[index].size();i++)
	    {
		//System.out.println("CLAL "+(index)+" | "+
		//+(((Integer)idx[index].elementAt(i)).intValue()-1)+" "
		//+(((Double)fak[index].elementAt(i)).doubleValue()));
		
		sum=sum+load[((Integer)idx[index].elementAt(i)).intValue()-1]*
		    ((Double)fak[index].elementAt(i)).doubleValue();
	    }
	    
	    if (sum>0) sum=(int)(sum+0.5);
	    if (sum<0) sum=-(int)(-sum+0.5);
	    
	    return sum;
	}
	
	public int getNumEdges()
	{
	    return nedges;
	}
	
	public int getLoadInfoFrom(int index)
	{
	    return von[index];
	}
	
	public int getLoadInfoTo(int index)
	{
	    return bis[index];
	}
	
	public int getLoadInfo(int index)
	{
	    return loadmove[index];
	}
	
	
	public void calcAllValue(int[] load)
	{
	    for(int i=0; i<=nedges; i++)
	    {
		loadmove[i]=(int)calcValue(i,load);
	    }
	}
	
    }
    

    
    static Runtime RT = null;
    static Process PR = null;
    
    LoadFactors lfc;
  
    /**
     *  this method can be used to update the GUI of the dialog dynamically.
     *  it is called by the DialogManager e.g. when some comboboxes containing
     *  boundary information need to be refreshed
     */
    public void refreshGUI ()
    {
	// no need fort this in this class :-)
	return;
    }


    /** 
     *  returns the initial Position of the dialog
     */
    public Point getInitialPosition ()
    {
	return new Point (parent.getLocationOnScreen().x - 50, parent.getLocationOnScreen().y);
    }


    /**
     *  the constructor just sets references correctly and creates the GUI
     */
    public LoadBalancingDialog(MainFrame parent)
    {
	super (parent);
	setTitle ((String)cc.rb.getObject("dialogs_loadbalancing_title"));
	JPanel content = new JPanel ();
	content.setBorder (new EmptyBorder (10,10,10,10));
	// just ordinary GUI layouting
	GridBagLayout layout = new GridBagLayout ();
	GridBagConstraints constraints = new GridBagConstraints ();
	constraints.fill = GridBagConstraints.BOTH;
	constraints.weightx = constraints.weighty = 1.0;
	// startpoint	
	JLabel Label1 = new JLabel ((String)cc.rb.getObject ("dialogs_loadbalancing_mpi2sol"));
	mpl2sol_TF = new JTextField (32);
	// endpoint	
	JLabel Label2 = new JLabel ((String)cc.rb.getObject ("dialogs_loadbalancing_filename"));
	filename_TF = new JTextField (32);
	// msg area
	msg_TA=new JTextArea(25,80);
	// buttons
	JPanel buttons = new JPanel ();
	buttons.setLayout (new FlowLayout ());
	calcButton = new JButton ((String)cc.rb.getObject ("dialogs_loadbalancing_calc"));
	calcButton.setMnemonic ( ((Integer)cc.rb.getObject("dialogs_loadbalancing_calc_mnemonic")).intValue());
	calcButton.setActionCommand (CALC);
	calcButton.addActionListener (this);
	buttons.add (calcButton);
	
	performButton = new JButton ((String)cc.rb.getObject ("dialogs_loadbalancing_perform"));
	performButton.setMnemonic ( ((Integer)cc.rb.getObject("dialogs_loadbalancing_perform_mnemonic")).intValue());
	performButton.setActionCommand (PERFORM);
	performButton.addActionListener (this);
	buttons.add (performButton);
		
	cancelButton = new JButton ((String)cc.rb.getObject ("general_Cancel"));
	cancelButton.setMnemonic ( ((Integer)cc.rb.getObject("general_Cancel_mnemonic")).intValue());
	cancelButton.setActionCommand (CANCEL);
	cancelButton.addActionListener (this);
	buttons.add (cancelButton);
	// add them all to the dialog
	getRootPane().setDefaultButton (cancelButton);
	content.setLayout (layout);
	
	constraints.gridy = 0;
	constraints.gridx = 0;
	layout.setConstraints (Label1, constraints);
	content.add (Label1);
	constraints.gridx = 1;
	layout.setConstraints (mpl2sol_TF, constraints);
	content.add (mpl2sol_TF);

	constraints.gridy = 1;
	constraints.gridx = 0;
	layout.setConstraints (Label2, constraints);
	content.add (Label2);
	constraints.gridx = 1;
	layout.setConstraints (filename_TF, constraints);
	content.add (filename_TF);

        filename_TF.setText("/tmp/loadbaldev");
        mpl2sol_TF.setText(System.getProperty("user.home")+"/feast/feast/bin/mpl2sol");
	
	constraints.gridy = 2;
	constraints.gridx = 1;
	//constraints.gridwidth = 2;
	layout.setConstraints (msg_TA, constraints);
	content.add (msg_TA);
	
	constraints.gridy = 3;
	constraints.gridx = 0;
	constraints.gridwidth = 3;
	layout.setConstraints(buttons, constraints);
	content.add (buttons);
	// finish the GUI
	getContentPane ().add (content);
	addComponentListener (listener);
	pack ();
    }


    void parseLine(int findex,String line)
    {
	int modus = 0;
	StringBuffer number = new StringBuffer();
	StringBuffer index = new StringBuffer();
	
	//System.out.println("PARSE "+findex+" "+line);
	
	for(int i =0;i<line.length();i++)
	{
	    char c = line.charAt(i);
	    
	    if ((modus==0) && ((Character.isDigit(c))||(c=='-')))
	    {
		modus=1;
		number=new StringBuffer(c);
	    }
	    if ((modus==4) && (c=='i'))
	    {
		modus=2;
		index=new StringBuffer();
	    }
	    if ((modus==1) && ((Character.isDigit(c))||(c=='E')||(c=='-')))
	    {
		number=number.append(c);
	    }
	    if ((modus==2) && (Character.isDigit(c)))
	    {
		index=index.append(c);
	    }
	    if ((modus==1) && (c=='.'))
	    {
		number=number.append(c);
	    }
	    
	    if ((modus==1) && (c=='*'))
	    {
		modus=4;
		
		lfc.addEntryFak(findex,(new Double(number.toString())).doubleValue());
	    }
	    if ((modus==2) && ((c=='+')||(c=='-')))
	    {
		modus=1;
		number=new StringBuffer(c+"0000");
		lfc.addEntryIdx(findex,(new Integer(index.toString())).intValue());
	    }
	}
	
        lfc.addEntryIdx(findex,(new Integer(index.toString())).intValue());
    }


    public void readLoadFile()
    {
	Domain dd = cc.d;
        int mpar = dd.getMaxParallelBlock();
	
	int[] testload = new int[mpar+1];
	
	try
	{
	    FileInputStream writeFile =
		new FileInputStream(filename_TF.getText()+".sol");
	    DataInputStream out = new DataInputStream( writeFile );
	    
	    String line;
	    
	    double[] testr = new double[16];
	    int[] testi = new int[16];
	    String[] tests = new String[16];    
	    
	    String anz = out.readLine();
	    
	    int ianz = (new Integer(anz)).intValue();
	    
	    for(int i=0;i<mpar;i++) testload[i]=0;
	    
	    int cc = dd.getCellCount();
	    
	    for(int i=0;i<cc;i++) testload[dd.getCell(i).getParallelBlock()-1]++;
	    //     for(int i=0;i<mpar;i++) testload[i]=1;
	    //System.out.println("LOAD "+mpar);
	    //for(int i=0;i<mpar;i++) System.out.println("LOAD "+testload[i]);
	    
	    lfc = new LoadFactors(ianz);
	    
	    int findex = -1;
	    
	    StringBuffer coll = new StringBuffer("");
	    
	    while ((line=out.readLine())!=null)
	    {
		//         System.out.println(line);
		
		if (line.indexOf("EDGE")>=0)
		{
		    //          System.out.println(coll);
		    
		    if (findex!=-1)
		    { 
			parseLine(findex,coll.toString());
			coll=new StringBuffer();
			
			//         System.out.println("TXT "+lfc.calcValue(findex,testload));
			//            lfc.printEntry(findex);
		    }
		    
		    findex++;
		    
		    dd.scanLine(line,"sii",testr,testi,tests);
		    //          System.out.println("EDGE "+testi[1]+" "+testi[2]);
		    
		    lfc.setHeader(findex,testi[1],testi[2]);
		    
		}
		else if (line.indexOf("s1 = ")>=0)
		{
		    //          System.out.println("s1");
		    coll=new StringBuffer(line.substring(8));
		    
		}
		else if (line.indexOf("t0 = ")>=0)
		{
		    //          System.out.println("t0");
		    if (line.charAt(8)=='-')
			coll=coll.append(line.substring(8));
		    else
			coll=coll.append("+"+line.substring(8));
		}
		else if (line.indexOf(";")>=0)
		{
		    if ((line.indexOf("s1")>=0) && (line.length()>3))
			coll=coll.append(line.substring(0,line.length()-4));
		    else
			if (line.length()>1)
			    coll=coll.append(line.substring(0,line.length()-1));
		    
		}
		else
		{
		    if (line.indexOf("s1")>=0)
			coll=coll.append(line.substring(0,line.length()-3));
		    else
			coll=coll.append(line);
		}
	    }
	    out.close();
	    
	    //          System.out.println(coll);
	    parseLine(findex,coll.toString());
	    //          lfc.printEntry(findex);
	    //          System.out.println("TXT "+lfc.calcValue(findex,testload));
	}
	catch(IOException exep)
	{
	    System.err.println("IO-Error: "+exep);
	    msg_TA.append("IO-Error: "+exep+"\n");
	}
	
	
	lfc.calcAllValue(testload);
	
	dd.resetCellMarkers();
	
	for(int i=0;i<lfc.getNumEdges(); i++)
	{
	    int a,b,c;
	    
	    a=lfc.getLoadInfoFrom(i);
	    b=lfc.getLoadInfoTo(i);
	    c=lfc.getLoadInfo(i);
	    
	    if (c==0) msg_TA.append("Loadmove "+c+" von p"+a+" nach p"+b+"\n");
	    
	    if (c>0)
	    {
		msg_TA.append("Loadmove "+c+" von p"+a+" nach p"+b+"\n");
		
		DPoint mp = dd.getParallelBlockCenter(b);
		
		int[] cl = dd.getNearestCells(a,b,mp);
		
		msg_TA.append("   cells: ");
		for(int j=0;j<c;j++) 
		{
		    msg_TA.append(cl[j]+" ");
		    dd.getCell(cl[j]).setMarker(1);
		    //	       dd.getCell(cl[j]).setParallelBlock(b);
		}
		
		msg_TA.append("  \n");
		
	    } 
	}
	
	
	/*	for(int i=1;i<=mpar;i++)
		{
		DPoint mp = dd.getParallelBlockCenter(i);
		System.out.println("MP "+i+" = "+mp.getX()+" "+mp.getY());
		}
		
		DPoint mp = dd.getParallelBlockCenter(1);
		
		int[] cl = dd.getNearestCells(4,mp);
		
		System.out.println("CELLIDX ");
		for(int i=0;i<cl.length;i++) System.out.print(cl[i]+" ");
		System.out.println("  ");
	*/
	
	
    }  


    public void performLoadFile()
    {
	Domain dd = cc.d;
        int mpar = dd.getMaxParallelBlock();
	
	int[] testload = new int[mpar+1];
	
	for(int i=0;i<mpar;i++) testload[i]=0;
	
	int cc = dd.getCellCount();
	
	int maxload = 0;
	
	for(int i=0;i<lfc.getNumEdges(); i++)
	{
	    int c=lfc.getLoadInfo(i);
	    if (c>maxload) maxload=c;
	}
	
	
	for(int i=0;i<cc;i++) testload[dd.getCell(i).getParallelBlock()-1]++;
      	
	lfc.calcAllValue(testload);
	
	dd.resetCellMarkers();
	
	for (int j=0;j<maxload; j++)	
	    for(int i=0;i<lfc.getNumEdges(); i++)
	    {
		int a,b,c;
		
		a=lfc.getLoadInfoFrom(i);
		b=lfc.getLoadInfoTo(i);
		c=lfc.getLoadInfo(i);
		
		if (c>0)
		{
		    
		    DPoint mp = dd.getParallelBlockCenter(b);
		    
		    int[] cl = dd.getNearestCells(a,b,mp);
		    
		    msg_TA.append("   cells: ");
		    //             for(int j=0;j<c;j++) 
		    if (j<c) 
		    {
			msg_TA.append("Transfer Load "+cl[j]+" von p"+a+" nach p"+b+"\n");
			msg_TA.append(cl[0]+" ");
			dd.getCell(cl[0]).setMarker(1);
			dd.getCell(cl[0]).setParallelBlock(b);
		    }
		    
		    msg_TA.append("  \n");
		    
		} 
	    }
	
	msg_TA.append("===========================================\n");
	
	
    }  
    
    public void calcLoadFile()
    {
	Domain dd = cc.d;
	
	int maxpb=dd.getMaxParallelBlock();
	int cc = dd.getCellCount();
	
	dd.updateCellNeighbours();
	
	int igraph[][] = new int[maxpb][maxpb];
	int edges[] = new int[20*maxpb*maxpb];
	
	int iedge=0;
	
	{
	    
	    for(int k=1;k<=maxpb;k++)
		for(int i=0;i<cc;i++)
		{
		    Cell c = dd.getCell(i);
		    
		    if (c.getParallelBlock()==k)
		    {
			int s = (c.getNeighbours()).length;
			
			for(int j=0;j<s;j++)
			{
			    int iz=(c.getNeighbours())[j].getParallelBlock();
			    
			    //		System.out.println("NEIGH "+k+" "+iz);
			    
			    boolean found = false;
			    
			    for(int l=0;l<iedge;l++)
			    {
				found=((edges[2*l]==k) && (edges[2*l+1]==iz));
				if (found) break;
			    }
			    
			    if ((!found)&&(k!=iz))
			    {
				edges[2*iedge]=k;
				edges[2*iedge+1]=iz;
				iedge++;
			    }
			    
			}			      
		    }
		}
	    
	    //	 for (int i=0; i<iedge; i++)
	    //	 System.out.println("EDGES "+edges[2*i]+" "+edges[2*i+1]);
	    
	    for(int i=0;i<iedge;i++)
	    {
		igraph[edges[2*i]-1][edges[2*i+1]-1]=-1;
	    }
	    
	    
	    for(int i=1;i<=maxpb;i++)
	    {
		int k=0;
		
		for(int j=1;j<=maxpb;j++)
		{
		    if (igraph[i-1][j-1]==-1) k=k+1;
		}
		
		igraph[i-1][i-1]=k;
		
	    }
	    
	    try
	    {
		FileOutputStream writeFile =
		    new FileOutputStream(filename_TF.getText()+".mpl");
		DataOutputStream out = new DataOutputStream( writeFile );
		
		out.writeBytes("with(linalg,linsolve):\n");
		out.writeBytes("readlib(C):\n");
		out.writeBytes("\n");
		out.writeBytes("A:=matrix([\n");
		
		for(int i=1;i<=maxpb;i++)
		{
		    out.writeBytes(" [");
		    
		    for(int j=1;j<maxpb;j++)
			out.writeBytes(igraph[j-1][i-1]+",");
		    
		    if (i==maxpb)
			out.writeBytes(igraph[maxpb-1][i-1]+"]]):\n");
		    else
			out.writeBytes(igraph[maxpb-1][i-1]+"],\n");
		    
		}
		
		out.writeBytes("\n");
		
		String s = "(";
		for(int j=1;j<maxpb;j++) s=s+"i"+(j)+"+";
		s=s+"i"+(maxpb)+")/"+(maxpb);
		
		out.writeBytes("b:=vector([\n");
		
		for(int i=1;i<maxpb;i++)
		{
		    out.writeBytes("i"+(i)+"-"+s+",\n");
		}
		
		out.writeBytes("i"+(maxpb)+"-"+s+"]):");
		
		out.writeBytes("\n");
		out.writeBytes("s:=linsolve(A,b):\n");
		out.writeBytes("\n");
		
		
		for(int i=0;i<iedge;i++)
		{
		    out.writeBytes("printf(`%s %d  %d\\n`,EDGE,"+(edges[2*i])
				   +","+(edges[2*i+1])+");\n");
		    out.writeBytes("C(evalf(s["+(edges[2*i])+"]-s["
				   +(edges[2*i+1])+"]));\n");
		}
		
		out.writeBytes("quit:\n");
		out.close();
		
	    }
	    catch(IOException exep)
	    {
		System.err.println("IO-Error: "+exep);
	    }
	    
	    
	    msg_TA.append("Start mpl2sol ...\n");
	    
	    try
	    {
		RT=Runtime.getRuntime();
		PR=RT.exec("csh "+mpl2sol_TF.getText()+" "+filename_TF.getText());
		
		msg_TA.append("mpl2sol messages:\n");
		msg_TA.append("===========================================\n");
		
		if (PR != null)
		{
		    DataInputStream IS = new DataInputStream(PR.getInputStream());
		    
		    String s;
		    
		    while ((s = IS.readLine()) != null)
			msg_TA.append(s+"\n");
		    
		    readLoadFile();
		    
		} 
	    }
	    catch (IOException ee)
	    {
		System.out.println("Aufruffehler");
		msg_TA.append("Aufruffehler\n");
	    }
	    
	    msg_TA.append("mpl2sol finished!\n");
	    msg_TA.append("==============================================\n");
	    
	}	
    }
    

    /**
     *  This class handles its own action event. The actionPerformed method 
     *  just updates the ControlCenter's EdgeStatusInfo 
     */
    public void actionPerformed (ActionEvent e)
    {
	if (e.getActionCommand().equals (CALC))
	{
	     msg_TA.setText("");
	     calcLoadFile();
	     //cc.d.construct_boundary();
	     return;
	}
	if (e.getActionCommand().equals (PERFORM))
	    {
		msg_TA.setText("");
		performLoadFile();
		return;
	}
	if (e.getActionCommand().equals (CANCEL))
	{
	    cc.dm.hide (cc.dm.LOADBALANCINGDIALOG);
	    return;
	}
    }






    // keys for the action commands

    private static final String CALC = "CALC";
    private static final String CANCEL = "CANCEL";
    private static final String PERFORM = "PERFORM";

    // buttons

    private JButton calcButton;
    private JButton cancelButton;
    private JButton performButton;
    private JTextField mpl2sol_TF;
    private JTextField filename_TF;
    private JTextArea msg_TA;

    

}
