package devisor2.grid.GUI.dialogs;

import java.util.*;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

import devisor2.grid.options.*;
import devisor2.grid.GUI.framework.*;
import devisor2.foundation.elements.*;
import devisor2.foundation.*;


/**
 *  This class is the DockableDialog instance to add Macros and to display
 *  Macro properties in Query Mode.
 *
 *  @author Dominik Goeddeke
 *  @version 0.9
 *  
 */

public class MacroDialog extends DockableDialog implements ActionListener
{

    /**
     *  the Macro element acting as source for updating
     */
    private Macro source;

    /**
     *  this method can be used to update the GUI of the dialog dynamically.
     *  it is called by the DialogManager e.g. when some comboboxes containing
     *  boundary information need to be refreshed
     */
    public void refreshGUI ()
    {
	// no need fort this in this class :-)
	return;
    }


    /** 
     *  returns the initial Position of the dialog
     */
    public Point getInitialPosition ()
    {
	return new Point (parent.getLocationOnScreen().x + parent.getWidth(), parent.getLocationOnScreen().y + (int)(parent.getHeight()/2));
    }

  /**
     *  sets the given int as text for the Node1TF
     */
    public void setNode1 (int index)
    {
	node1TF.setText (Integer.toString (index));
    }
    
    /**
     *  sets the given int as text for the Node2TF
     */
    public void setNode2 (int index)
    {
	node2TF.setText (Integer.toString (index));
    }

    /**
     *  sets the given int as text for the Node3TF
     */
    public void setNode3 (int index)
    {
	node3TF.setText (Integer.toString (index));
    }

    /**
     *  sets the given int as text for the Node4TF
     */
    public void setNode4 (int index)
    {
	node4TF.setText (Integer.toString (index));
    }

    /**
     *  sets the source element (the Macro element the dialog works on.
     *  Note that if source is set explicitly to null, the dialog
     *  works for directinput, if a Macro element of the domain is referenced,
     *  the dialog works in property change mode
     *
     *  @param source - the source macro element for property mode
     */
    public void setSource (Macro source)
    {
	this.source = source;
	if (source != null)
	{
	    Node[] nodes = source.updateCellNodes ();
	    setNode1 (nodes[0].getNumber()+1);
	    setNode2 (nodes[1].getNumber()+1);
	    setNode3 (nodes[2].getNumber()+1);
	    setNode4 (nodes[3].getNumber()+1);
	    pblockTF.setText (Integer.toString (source.getParallelBlock ()));
	    mblockTF.setText (Integer.toString (source.getMatrixBlock ()));
	    double[] fac = source.getAnisotropyFactors();
	    factor1TF.setText (Double.toString (fac[0]));
	    factor2TF.setText (Double.toString (fac[1]));
	    factor3TF.setText (Double.toString (fac[2]));
	    refinementTF.setText (Integer.toString (source.getRefinement(0)));
	    int  aimo = source.getAnisotropyMode();
	    if ((aimo & 1) == 1)
		x_left_rb.setSelected(true);
	    else 
		if ((aiso & 4) == 4)
		    x_right_rb.setSelected(true);
		else
		    x_none_rb.setSelected(true);
	    if ((aiso & 2) == 2)
		y_down_rb.setSelected(true);
	    else 
		if ((aiso & 4) == 4)
		    y_up_rb.setSelected(true);
		else	  
		    y_none_rb.setSelected(true);
	    setSelectedTab (1);
	}
    }

    /**
     *  @return the number of the selected tabsheet
     */
    public int getSelectedTab ()
    {
	return tabs.getSelectedIndex ();
    }

    /**
     *  sets the selected tab of the dialog to the given index
     */
    public void setSelectedTab (int index)
    {
	tabs.setSelectedIndex (index);
    }

  
    /**
     *  the constructor sets references correctly and creates the GUI
     */
    public MacroDialog (MainFrame parent)
    {
	super (parent);
	JPanel content = new JPanel ();
	// just ordinary GUI layouting
	GridBagLayout layout = new GridBagLayout ();
	GridBagConstraints constraints = new GridBagConstraints ();
	content.setLayout (layout);
	content.setBorder (new EmptyBorder (10,10,10,10));
	constraints.fill = GridBagConstraints.BOTH;
	constraints.weightx = constraints.weighty = 1.0;

	// the nodes tab
	JPanel nodes = new JPanel ();
	GridBagLayout nodeslayout = new GridBagLayout ();
	GridBagConstraints nodesconstraints = new GridBagConstraints ();
	nodes.setLayout (nodeslayout);
	nodes.setBorder (new EmptyBorder (10,10,10,10));
	nodesconstraints.fill = GridBagConstraints.BOTH;
	nodesconstraints.weightx = nodesconstraints.weighty = 1.0;
	// first node
	JLabel node1Label = new JLabel ((String)cc.rb.getObject ("dialogs_macro_node1"));
	nodesconstraints.gridy = 0;
	nodesconstraints.gridx = 0;
	nodeslayout.setConstraints (node1Label, nodesconstraints);
	nodes.add (node1Label);
	node1TF = new JTextField (6);
	nodesconstraints.gridx = 1;
	nodeslayout.setConstraints (node1TF, nodesconstraints);
	nodes.add (node1TF);
	// second node
	JLabel node2Label = new JLabel ((String)cc.rb.getObject ("dialogs_macro_node2"));
	nodesconstraints.gridy = 1;
	nodesconstraints.gridx = 0;
	nodeslayout.setConstraints (node2Label, nodesconstraints);
	nodes.add (node2Label);
	node2TF = new JTextField (6);
	nodesconstraints.gridx = 1;
	nodeslayout.setConstraints (node2TF, nodesconstraints);
	nodes.add (node2TF);
	// third node
	JLabel node3Label = new JLabel ((String)cc.rb.getObject ("dialogs_macro_node3"));
	nodesconstraints.gridy = 2;
	nodesconstraints.gridx = 0;
	nodeslayout.setConstraints (node3Label, nodesconstraints);
	nodes.add (node3Label);
	node3TF = new JTextField (6);
	nodesconstraints.gridx = 1;
	nodeslayout.setConstraints (node3TF, nodesconstraints);
	nodes.add (node3TF);
	// forth node
	JLabel node4Label = new JLabel ((String)cc.rb.getObject ("dialogs_macro_node4"));
	nodesconstraints.gridy = 3;
	nodesconstraints.gridx = 0;
	nodeslayout.setConstraints (node4Label, nodesconstraints);
	nodes.add (node4Label);
	node4TF = new JTextField (6);
	nodesconstraints.gridx = 1;
	nodeslayout.setConstraints (node4TF, nodesconstraints);
	nodes.add (node4TF);

	// the blocks tab
	JPanel blocks = new JPanel ();
	GridBagLayout blocklayout = new GridBagLayout ();
	GridBagConstraints blockconstraints = new GridBagConstraints ();
	blockconstraints.fill = GridBagConstraints.BOTH;
	blockconstraints.weightx = blockconstraints.weighty = 1.0;
	// parallel block edit
	JLabel PBLabel = new JLabel ((String)cc.rb.getObject ("dialogs_macro_parallelblock"));
	pblockTF = new JTextField (10);
	// parallel block edit
	JLabel MBLabel = new JLabel ((String)cc.rb.getObject ("dialogs_macro_matrixblock"));
	mblockTF = new JTextField (10);
	// macro refinement edit
	JLabel MRLabel = new JLabel ((String)cc.rb.getObject ("dialogs_macro_macrorefinement"));
	refinementTF = new JTextField (10);
	// macro x anisotrophy
	JLabel xaniLabel = new JLabel ((String)cc.rb.getObject ("dialogs_macro_anisotrophy_x"));
	ButtonGroup xanibg = new ButtonGroup ();
	x_none_rb = new JRadioButton ((String)cc.rb.getObject ("dialogs_macro_anisotrophy_x_none"));
	xanibg.add (x_none_rb);
	x_left_rb = new JRadioButton ((String)cc.rb.getObject ("dialogs_macro_anisotrophy_x_left"));
	xanibg.add (x_left_rb);
	x_right_rb = new JRadioButton ((String)cc.rb.getObject ("dialogs_macro_anisotrophy_x_right"));
	xanibg.add (x_right_rb);
	// macro y anisotrophy
	JLabel yaniLabel = new JLabel ((String)cc.rb.getObject ("dialogs_parallelblock_anisotrophy_y"));
	ButtonGroup yanibg = new ButtonGroup ();
	y_none_rb = new JRadioButton ((String)cc.rb.getObject ("dialogs_parallelblock_anisotrophy_y_none"));
	yanibg.add (y_none_rb);
	y_down_rb = new JRadioButton ((String)cc.rb.getObject ("dialogs_parallelblock_anisotrophy_y_down"));
	yanibg.add (y_down_rb);
	y_up_rb = new JRadioButton ((String)cc.rb.getObject ("dialogs_parallelblock_anisotrophy_y_up"));
	yanibg.add (y_up_rb);
	// macro refinement factors
	JLabel factorLabel = new JLabel ((String)cc.rb.getObject ("dialogs_parallelblock_factors"));
	factor1TF = new JTextField (5);
	factor2TF = new JTextField (5);
	factor3TF = new JTextField (5);
	// add them all to the dialog
	blocks.setLayout (blocklayout);
	blocks.setBorder (new EmptyBorder (10,10,10,10));
	blockconstraints.gridy = 0;
	blockconstraints.gridx = 0;
	blocklayout.setConstraints(PBLabel, blockconstraints);
	blocks.add (PBLabel);
	blockconstraints.gridy = 0;
	blockconstraints.gridx = 1;
	blockconstraints.gridwidth = 3;
	blocklayout.setConstraints(pblockTF, blockconstraints);
	blocks.add (pblockTF);
	blockconstraints.gridy = 1;
	blockconstraints.gridx = 0;
	blocklayout.setConstraints(MBLabel, blockconstraints);
	blocks.add (MBLabel);
	blockconstraints.gridy = 1;
	blockconstraints.gridx = 1;
	blockconstraints.gridwidth = 3;
	blocklayout.setConstraints(mblockTF, blockconstraints);
	blocks.add (mblockTF);
	blockconstraints.gridwidth = 1;
	blockconstraints.gridy = 2;
	blockconstraints.gridx = 0;
	blocklayout.setConstraints(MRLabel, blockconstraints);
	blocks.add (MRLabel);
	blockconstraints.gridy = 2;
	blockconstraints.gridx = 1;
	blockconstraints.gridwidth = 3;
	blocklayout.setConstraints(refinementTF, blockconstraints);
	blocks.add (refinementTF);
	blockconstraints.gridwidth = 1;
	blockconstraints.gridy = 3;
	blockconstraints.gridx = 0;
	blocklayout.setConstraints(xaniLabel, blockconstraints);
	blocks.add (xaniLabel);
	blockconstraints.gridx = 1;
	blocklayout.setConstraints(x_none_rb, blockconstraints);
	blocks.add (x_none_rb);
	blockconstraints.gridx = 2;
	blocklayout.setConstraints(x_left_rb, blockconstraints);
	blocks.add (x_left_rb);
	blockconstraints.gridx = 3;
	blocklayout.setConstraints(x_right_rb,blockconstraints);
	blocks.add (x_right_rb);
	blockconstraints.gridy = 4;
	blockconstraints.gridx = 0;
	blocklayout.setConstraints(yaniLabel, blockconstraints);
	blocks.add (yaniLabel);
	blockconstraints.gridx = 1;
	blocklayout.setConstraints(y_none_rb, blockconstraints);
	blocks.add (y_none_rb);
	blockconstraints.gridx = 2;
	blocklayout.setConstraints(y_down_rb, blockconstraints);
	blocks.add (y_down_rb);
	blockconstraints.gridx = 3;
	blocklayout.setConstraints(y_up_rb, blockconstraints);
	blocks.add (y_up_rb);
	blockconstraints.gridy = 5;
	blockconstraints.gridx = 0;
	blocklayout.setConstraints(factorLabel, blockconstraints);
	blocks.add (factorLabel);
	blockconstraints.gridx = 1;
	blocklayout.setConstraints(factor1TF, blockconstraints);
	blocks.add (factor1TF);
	blockconstraints.gridx = 2;
	blocklayout.setConstraints(factor2TF, blockconstraints);
	blocks.add (factor2TF);
	blockconstraints.gridx = 3;
	blocklayout.setConstraints(factor3TF, blockconstraints);
	blocks.add (factor3TF);
	// initial values	
	pblockTF.setText("1");
	mblockTF.setText("1");
        refinementTF.setText("1");
        factor1TF.setText("0.5");
        factor2TF.setText("0.5");
        factor3TF.setText("0.5");
        x_none_rb.setSelected(true);
        y_none_rb.setSelected(true);

	//the tabbed pane
	tabs = new JTabbedPane ();
	tabs.addTab((String)cc.rb.getObject ("dialogs_macro_nodes_tabheader"), null, nodes, (String)cc.rb.getObject ("dialogs_macro_nodes_tooltip"));
	tabs.addTab((String)cc.rb.getObject ("dialogs_macro_blocks_tabheader"), null, blocks, (String)cc.rb.getObject ("dialogs_macro_blocks_tooltip"));
	constraints.fill = GridBagConstraints.BOTH;
	constraints.weightx = constraints.weighty = 1.0;
	constraints.gridx = 0;
	constraints.gridy = 0;
	constraints.gridwidth = 4;
	constraints.gridheight = 4;
	layout.setConstraints (tabs, constraints);
	content.add (tabs);

	// buttons
	JPanel buttons = new JPanel ();
	buttons.setLayout (new FlowLayout ());
	okButton = new JButton ((String)cc.rb.getObject ("general_Ok"));
	okButton.setMnemonic ( ((Integer)cc.rb.getObject("general_Ok_mnemonic")).intValue());
	okButton.setActionCommand (OK);
	okButton.addActionListener (this);
	buttons.add (okButton);
	applyButton = new JButton ((String)cc.rb.getObject ("general_Apply"));
	applyButton.setMnemonic ( ((Integer)cc.rb.getObject("general_Apply_mnemonic")).intValue());
	applyButton.setActionCommand (APPLY);
	applyButton.addActionListener (this);
	buttons.add (applyButton);
	cancelButton = new JButton ((String)cc.rb.getObject ("general_Cancel"));
	cancelButton.setMnemonic ( ((Integer)cc.rb.getObject("general_Cancel_mnemonic")).intValue());
	cancelButton.setActionCommand (CANCEL);
	cancelButton.addActionListener (this);
	buttons.add (cancelButton);
	// add them all to the dialog
	getRootPane().setDefaultButton (applyButton);
	constraints.gridwidth = 4;
	constraints.gridheight = 1;
	constraints.gridy = 5;
	constraints.gridx = 0;
	layout.setConstraints(buttons, constraints);
	content.add (buttons);

	// finish the GUI
	getContentPane ().add (content);
	addComponentListener (listener);
	pack ();
    }

    /**
     *  This class handles its own action event. 
     */
    public void actionPerformed (ActionEvent e)
    {
	if (e.getActionCommand().equals (OK))
	{
	    if (verify ())
	    {
		if (source != null)
			performUpdateMacro ();
		    else
			performAddMacro ();
		cc.dm.hide (cc.dm.MACRODIALOG);
	    }
	    return;
	}
	if (e.getActionCommand().equals (APPLY))
	{
	    if (verify ())
	    {
		if (source != null)
		    performUpdateMacro ();
		else
		    performAddMacro ();
	    }
	    return;
	}
	if (e.getActionCommand().equals (CANCEL))
	{
	    cc.dm.hide (cc.dm.MACRODIALOG);
	    return;
	}
    }

    /**
     *  the verifivation code is all put into this method
     */
    private boolean verify ()
    {
	boolean allOK = true;
	try
	{
	    node1 = Integer.parseInt (node1TF.getText());
	    node1--;
	    n1 = cc.d.getNode(node1);
	}
	catch (Exception ex)
	{
	    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject("dialogs_macro_error_node1"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
	    allOK = false;
	}
	try
	{
	    node2 = Integer.parseInt (node2TF.getText());
	    node2--;
	    n2 = cc.d.getNode(node2);
	}
	catch (Exception ex)
	{
	    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject("dialogs_macro_error_node2") , (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
	    allOK = false;
	}
	try
	{
	    node3 = Integer.parseInt (node3TF.getText());
	    node3--;
	    n3 = cc.d.getNode(node3);
	}
	catch (Exception ex)
	{
	    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject("dialogs_macro_error_node3") , (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
	    allOK = false;
	}
	try
	{
	    node4 = Integer.parseInt (node4TF.getText());
	    node4--;
	    n4 = cc.d.getNode(node4);
	}
	catch (Exception ex)
	{
	    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject("dialogs_macro_error_node4") , (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
	    allOK = false;
	}
	// parse aniso checkboxes
	aiso = 0;
	if (x_left_rb.isSelected()) 
	    aiso += 1;
	if (x_right_rb.isSelected())
	    aiso += 5;
	if (y_down_rb.isSelected()) 
	    aiso += 2;
	if (y_up_rb.isSelected()) 
	    aiso += 10;
	// block tab edits
	try
	{
	    pb = (new Integer(pblockTF.getText())).intValue();
	}
	catch (Exception ex)
	{
	    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject("dialogs_macro_error_pb") , (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
	    allOK = false;
	}
	try
	{
	    ref = (new Integer(refinementTF.getText())).intValue();  
	}
	catch (Exception ex)
	{
	    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject("dialogs_macro_error_ref") , (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
	    allOK = false;
	}
	try
	{
	    mb = (new Integer(mblockTF.getText())).intValue(); 
	}
	catch (Exception ex)
	{
	    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject("dialogs_macro_error_pb") , (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
	    allOK = false;
	}
	try
	{
	    af = new double[3];
	    af[0] = (new Double(factor1TF.getText())).doubleValue();
	    af[1] = (new Double(factor2TF.getText())).doubleValue();
	    af[2] = (new Double(factor3TF.getText())).doubleValue();  
	}
	catch (Exception ex)
	{
	    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject("dialogs_macro_error_factors") , (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
	    allOK = false;
	}
	// check for node orientation
	if (allOK)
	{
	    // are all nodes different?
	    if (n1 == n2 || n1 == n3 || n1 == n4 ||
		n2 == n3 || n3 == n4 || n2 == n4)
	    {
		JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject("dialogs_tri_error_samenodes"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
		allOK = false;
	    }
	    // check for node orientation
	    Node[] nodes = {n1,n2,n3,n4};
	    double vek1 = cc.d.db.crossProduct (3,0,1,nodes);
	    double vek2 = cc.d.db.crossProduct (1,2,3,nodes);
	    if ((vek1>=0.0) || (vek2>=0.0))
	    {
		JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject("dialogs_tri_error_nodeorientation"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
		allOK = false;
	    }
	}
	return allOK;
    }

    /**
     *  this method adds a new Macro object to the domain with the four nodes
     *  set in the textfields
     */
    private void performAddMacro ()
    {
	DrawingArea da = parent.getDrawingArea ();
	// create and add edges
	Edge e1, e2, e3,e4;
	e1 = cc.d.adjustEdge (n1,n2);
	e2 = cc.d.adjustEdge (n2,n3);
	e3 = cc.d.adjustEdge (n3,n4);
	e4 = cc.d.adjustEdge (n4,n1);
	cc.d.updateEdgeNumbers ();
	// create and add Macro 
	Macro macro = new Macro ();
	macro.setCell (e1,e2,e3,e4);
	macro.setMatrixBlock(mb);
	macro.setParallelBlock(pb);
	macro.setRefinement(0,ref);
	macro.setRefinement(1,ref);
	macro.setAnisotropy(aiso,af[0],af[1],af[2]);
	cc.d.addCell (macro);
	cc.d.updateCellNumbers ();
	source = macro;
	Vector tmpvec = new Vector ();
	tmpvec.add (e1);
	tmpvec.add (e2);
	tmpvec.add (e3);
	tmpvec.add (e4);
	tmpvec.add (macro);
	cc.um.notifyUndo (cc.um.UNDO_ADDMACRO, tmpvec, null);
	da.repaint ();
    }

    /**
     *  updates the Macro element referenced by the source variable with
     *  the parameter values from the dialog textfields
     */
    private void performUpdateMacro ()
    {
	DrawingArea da = parent.getDrawingArea ();
	// kill old macro
	cc.d.delCell (source);
	// create & add edges
	Edge e1, e2, e3,e4;
	e1 = cc.d.adjustEdge (n1,n2);
	e2 = cc.d.adjustEdge (n2,n3);
	e3 = cc.d.adjustEdge (n3,n4);
	e4 = cc.d.adjustEdge (n4,n1);
	cc.d.updateEdgeNumbers ();
	// create and add macro 
	Macro macro = new Macro ();
	macro.setCell (e1,e2,e3,e4);
	macro.setMatrixBlock(mb);
	macro.setParallelBlock(pb);
	macro.setRefinement(0,ref);
	macro.setRefinement(1,ref);
	macro.setAnisotropy(aiso,af[0],af[1],af[2]);
	cc.d.addCell (macro);
	cc.d.updateCellNumbers ();
	Vector tmpvec = new Vector ();
	tmpvec.add (e1);
	tmpvec.add (e2);
	tmpvec.add (e3);
	tmpvec.add (e4);
	tmpvec.add (macro);
	cc.um.notifyUndo (cc.um.UNDO_ADDMACRO, tmpvec, null);
	da.repaint ();
    }


    // the integers corresponding to the textfields' string entries
    private int node1, node2, node3, node4;
    private int aiso;
    private int pb;
    private int ref;
    private int mb;
    private double[] af = new double[3];
    // and the corresponding objects
    private Node n1,n2,n3,n4;

    private static final String OK = "OK";
    private static final String CANCEL = "CANCEL";
    private static final String APPLY = "APPLY";


    // buttons
    private JButton okButton;
    private JButton cancelButton;
    private JButton applyButton;
    // textfields
    private JTextField node1TF;
    private JTextField node2TF;
    private JTextField node3TF;
    private JTextField node4TF;
    private JTextField pblockTF;
    private JTextField mblockTF;
    private JTextField refinementTF;
    private JRadioButton x_none_rb;
    private JRadioButton x_left_rb;
    private JRadioButton x_right_rb;
    private JRadioButton y_none_rb;
    private JRadioButton y_up_rb;
    private JRadioButton y_down_rb;
    private JTextField factor1TF;
    private JTextField factor2TF;
    private JTextField factor3TF;
    // tabs
    private JTabbedPane tabs;

   

}
