package devisor2.grid.GUI.dialogs;

import java.util.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

import devisor2.grid.options.*;
import devisor2.grid.GUI.framework.*;
import devisor2.foundation.elements.*;
import devisor2.foundation.base.*;
import devisor2.foundation.boundary.*;


/**
 *  See the GridListener for a detailed description of how the dialog
 *  is connected to the application.
 *
 *  @author Dominik Goeddeke
 *  @version yes
 *  
 */

public class MergeBoundariesDialog extends DockableDialog implements ActionListener
{

    /** 
     *  returns the initial Position of the dialog
     */
    public Point getInitialPosition ()
    {
	return new Point (parent.getLocationOnScreen().x - 50, parent.getLocationOnScreen().y);
    }


    /**
     *  the constructor just sets references correctly and creates the GUI
     */
    public MergeBoundariesDialog (MainFrame parent)
    {
	super (parent);
	setTitle (cc.wS("dialogs_mergeboundaries_caption"));
	JPanel content = new JPanel ();
	content.setBorder (new EmptyBorder (10,10,10,10));
	// just ordinary GUI layouting
	GridBagLayout layout = new GridBagLayout ();
	GridBagConstraints constraints = new GridBagConstraints ();
	content.setLayout (layout);
	constraints.fill = GridBagConstraints.BOTH;
	constraints.weightx = constraints.weighty = 1.0;
	// combo1
	JLabel label = new JLabel ((String)cc.rb.getObject ("dialogs_mergeboundaries_targetlabel"));
	constraints.gridy = 0;
	constraints.gridx = 0;
	layout.setConstraints (label, constraints);
	content.add (label);
	targetCombo = new JComboBox ();
	targetCombo.setToolTipText ((String)cc.rb.getObject("dialogs_mergeboundaries_targettooltip"));
	constraints.gridx = 1;
	layout.setConstraints (targetCombo, constraints);
	targetCombo.setEditable (false);
	content.add (targetCombo);
	// combo1
	label = new JLabel ((String)cc.rb.getObject ("dialogs_mergeboundaries_sourcelabel"));
	constraints.gridy = 1;
	constraints.gridx = 0;
	layout.setConstraints (label, constraints);
	content.add (label);
	sourceCombo = new JComboBox ();
	sourceCombo.setToolTipText ((String)cc.rb.getObject("dialogs_mergeboundaries_sourcetooltip"));
	constraints.gridx = 1;
	layout.setConstraints (sourceCombo, constraints);
	sourceCombo.setEditable (false);
	content.add (sourceCombo);
	// labelling
	//	updateCombo ();
	// buttons
	JPanel buttons = new JPanel ();
	buttons.setLayout (new FlowLayout ());
	okButton = new JButton ((String)cc.rb.getObject ("general_Ok"));
	okButton.setMnemonic ( ((Integer)cc.rb.getObject("general_Ok_mnemonic")).intValue());
	okButton.setActionCommand (OK);
	okButton.addActionListener (this);
	buttons.add (okButton);
	applyButton = new JButton ((String)cc.rb.getObject ("general_Apply"));
	applyButton.setMnemonic ( ((Integer)cc.rb.getObject("general_Apply_mnemonic")).intValue());
	applyButton.setActionCommand (APPLY);
	applyButton.addActionListener (this);
	buttons.add (applyButton);
	cancelButton = new JButton ((String)cc.rb.getObject ("general_Cancel"));
	cancelButton.setMnemonic ( ((Integer)cc.rb.getObject("general_Cancel_mnemonic")).intValue());
	cancelButton.setActionCommand (CANCEL);
	cancelButton.addActionListener (this);
	buttons.add (cancelButton);
	// add them all to the dialog
	getRootPane().setDefaultButton (applyButton);
	constraints.gridy = 3;
	constraints.gridx = 0;
	constraints.gridwidth = 2;
	layout.setConstraints(buttons, constraints);
	content.add (buttons);
	// finish the GUI
	getContentPane ().add (content);
	addComponentListener (listener);
	pack ();
    }

    /**
     *  updates the boundary comboboxes
     */
    public void refreshGUI ()
    {
	targetCombo.removeActionListener (this);
	sourceCombo.removeActionListener (this);
	targetCombo.removeAllItems ();
	sourceCombo.removeAllItems ();
	if (cc.d.db.getBoundaryCount () == 0)
	{
	    targetCombo.addItem ("-1");
	    targetCombo.setSelectedItem ("-1");
	    sourceCombo.addItem ("-1");
	    sourceCombo.setSelectedItem ("-1");
	}
	else
	{
	    for (int i=0;i<cc.d.db.getBoundaryCount();i++)
	    {
		targetCombo.addItem (Integer.toString (i+1));
		sourceCombo.addItem (Integer.toString (i+1));
	    }
	    targetCombo.setSelectedItem (Integer.toString(cc.gc.workingBoundary.getNumber ()+1));
	    sourceCombo.setSelectedItem (Integer.toString(cc.gc.workingBoundary.getNumber ()+1));
	}
	targetCombo.addActionListener (this);
	sourceCombo.addActionListener (this);

    }


    /**
     *  This class handles its own action event. 
     */
    public void actionPerformed (ActionEvent e)
    {
	if (e.getActionCommand().equals (OK))
	{
	    if (verify ())
	    {
		performMerge ();
		cc.dm.hide (cc.dm.MERGEBOUNDARIESDIALOG);
	    }
	    return;
	}
	if (e.getActionCommand().equals (APPLY))
	{
	    if (verify ())
	    {
		performMerge ();
	    }
	    return;
	}
	if (e.getActionCommand().equals (CANCEL))
	{
	    cc.dm.hide (cc.dm.MERGEBOUNDARIESDIALOG);
	    return;
	}
    }

    /**
     *  the verifivation code is all put into this method
     */
    private boolean verify ()
    {
	target = cc.d.db.getBoundary(Integer.parseInt ((String)targetCombo.getSelectedItem())-1);
	source = cc.d.db.getBoundary(Integer.parseInt ((String)sourceCombo.getSelectedItem())-1);
	if (target==source)
	{
	    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject("dialogs_mergeboundaries_error_sameboundaries"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
	    return false;
	}
	return true;
    }

    /**
     *  the actual merging is implemented in this method
     */
    private void performMerge ()
    {
	// segments are updated
	Enumeration segs = source.getSegments().elements();
	while (segs.hasMoreElements())
	{
	    Segment s = (Segment)segs.nextElement ();
	    s.setBoundary (target);
	}
	// and of course also bns
	Enumeration nodes = cc.d.getNodes().elements();
	while (nodes.hasMoreElements())
	{
	    Node n = (Node)nodes.nextElement ();
	    if (n instanceof BoundaryNode)
	    {
		BoundaryNode bn = (BoundaryNode)n;
		if (bn.getBoundary()==source)
		    bn.setBoundary (target);
	    }
	}
	// and after all. try to fix parametrisation
	int tmp = cc.dc.tolerance;
	cc.dc.tolerance=1;
	cc.d.db.continousBoundary();
	cc.dc.tolerance = tmp;
    }

    private static final String OK = "OK";
    private static final String CANCEL = "CANCEL";
    private static final String APPLY = "APPLY";

    private JButton okButton;
    private JButton cancelButton;
    private JButton applyButton;

    private JComboBox targetCombo;
    private JComboBox sourceCombo;

    private Boundary target;
    private Boundary source;

 
}
