package devisor2.grid.GUI.dialogs;

import java.util.*;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import javax.swing.*;
import javax.swing.border.*;

import devisor2.grid.options.*;
import devisor2.grid.GUI.framework.*;
import devisor2.grid.GUI.event.*;
import devisor2.foundation.elements.*;
import devisor2.grid.backend.*;



/**
 *  This class is the DockableDialog instance to get the line against which
 *  to mirror the selected elements. <br>
 *  See the GridListener and MainActionListener for a detailed description 
 *  of how the dialog is connected to the application.
 *
 *  @author Dominik Goeddeke
 *  @version done
 *  
 */

public class MirrorDialog extends DockableDialog implements ActionListener
{
    // the start and end point of the line in world coords
    private int x1,y1,x2,y2;

    /**
     *  this method can be used to update the GUI of the dialog dynamically.
     *  it is called by the DialogManager e.g. when some comboboxes containing
     *  boundary information need to be refreshed
     */
    public void refreshGUI ()
    {
	// no need fort this in this class :-)
	return;
    }


    /** 
     *  returns the initial Position of the dialog
     */
    public Point getInitialPosition ()
    {
	return new Point (parent.getLocationOnScreen().x - 50, parent.getLocationOnScreen().y);
    }

    /**
     *  the constructor just sets references correctly and creates the GUI
     */
    public MirrorDialog (MainFrame parent)
    {
	super (parent);
	setTitle ((String)cc.rb.getObject("dialogs_mirror_caption"));
	JPanel content = new JPanel ();
	content.setBorder (new EmptyBorder (10,10,10,10));
	// just ordinary GUI layouting
	GridBagLayout layout = new GridBagLayout ();
	GridBagConstraints constraints = new GridBagConstraints ();
	content.setLayout (layout);
	constraints.fill = GridBagConstraints.BOTH;
	constraints.weightx = constraints.weighty = 1.0;
	// edit fields
	JLabel label1 = new JLabel ((String)cc.rb.getObject ("dialogs_mirror_start"));
	constraints.gridy = 0;
	constraints.gridx = 0;
	layout.setConstraints (label1, constraints);
	content.add (label1);
	x1TF = new JTextField (6);
	constraints.gridx = 1;
	layout.setConstraints (x1TF, constraints);
	content.add (x1TF);
	y1TF = new JTextField (6);
	constraints.gridx = 2;
	layout.setConstraints (y1TF, constraints);
	content.add (y1TF);
	JLabel label2 = new JLabel ((String)cc.rb.getObject ("dialogs_mirror_end"));
	constraints.gridy = 1;
	constraints.gridx = 0;
	layout.setConstraints (label2, constraints);
	content.add (label2);
	x2TF = new JTextField (6);
	constraints.gridx = 1;
	layout.setConstraints (x2TF, constraints);
	content.add (x2TF);
	y2TF = new JTextField (6);
	constraints.gridx = 2;
	layout.setConstraints (y2TF, constraints);
	content.add (y2TF);

	// buttons
	JPanel buttons = new JPanel ();
	buttons.setLayout (new FlowLayout ());
	okButton = new JButton ((String)cc.rb.getObject ("general_Ok"));
	okButton.setMnemonic ( ((Integer)cc.rb.getObject("general_Ok_mnemonic")).intValue());
	okButton.setActionCommand (OK);
	okButton.addActionListener (this);
	buttons.add (okButton);
	applyButton = new JButton ((String)cc.rb.getObject ("general_Apply"));
	applyButton.setMnemonic ( ((Integer)cc.rb.getObject("general_Apply_mnemonic")).intValue());
	applyButton.setActionCommand (APPLY);
	applyButton.addActionListener (this);
	buttons.add (applyButton);
	cancelButton = new JButton ((String)cc.rb.getObject ("general_Cancel"));
	cancelButton.setMnemonic ( ((Integer)cc.rb.getObject("general_Cancel_mnemonic")).intValue());
	cancelButton.setActionCommand (CANCEL);
	cancelButton.addActionListener (this);
	buttons.add (cancelButton);
	// add them all to the dialog
	getRootPane().setDefaultButton (applyButton);
	constraints.gridy = 3;
	constraints.gridx = 0;
	constraints.gridwidth = 3;
	layout.setConstraints(buttons, constraints);
	content.add (buttons);
	// finish the GUI
	getContentPane ().add (content);
	addComponentListener (listener);
	pack ();
	//setSize (300,160);
    }

    /**
     *  This class handles its own action event. 
     */
    public void actionPerformed (ActionEvent e)
    {
	if (e.getActionCommand().equals (OK))
	{
	    if (verify ())
	    {
		ControlCenter.getMyself().gc.gtk.mirror (x1,y1,x2,y2);
		parent.getMainActionListener().disableMirrorMode ();
		int[] mirrorline = {x1,y1,x2,y2};
		Object[] info = new Object[2];
		info[0] = mirrorline;
		info[1] = parent.getDrawingArea().getListener().getSelector().getHotspotMask ();
		Vector selectedItems = parent.getDrawingArea().getListener().getSelectedItems();
		cc.um.notifyUndo (cc.um.UNDO_MIRROR, selectedItems, info);
		cc.dm.hide (cc.dm.MIRRORDIALOG);
	    }
	    return;
	}
	if (e.getActionCommand().equals (APPLY))
	{
	    if (verify ())
	    {
		ControlCenter.getMyself().gc.gtk.mirror (x1,y1,x2,y2);
		int[] mirrorline = {x1,y1,x2,y2};
		Object[] info = new Object[2];
		info[0] = mirrorline;
		info[1] = parent.getDrawingArea().getListener().getSelector().getHotspotMask ();
		Vector selectedItems = parent.getDrawingArea().getListener().getSelectedItems();
		cc.um.notifyUndo (cc.um.UNDO_MIRROR, selectedItems, info);
	    }
	    return;
	}
	if (e.getActionCommand().equals (CANCEL))
	{
	    parent.getMainActionListener().disableMirrorMode ();
	    cc.dm.hide (cc.dm.MIRRORDIALOG);
	    return;
	}
    }

    /**
     *  the verification code is all put into this method
     */
    private boolean verify ()
    {
	boolean allOK = true;
	try
	{
	    x1 = cc.gc.fromUserToWorld(Double.parseDouble (x1TF.getText()));
	}
	catch (Exception ex)
	{
	    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject("dialogs_mirror_x1_error"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
	    allOK = false;
	}
	try
	{
	    y1 = cc.gc.fromUserToWorld(Double.parseDouble (y1TF.getText()));
	}
	catch (Exception ex)
	{
	    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject("dialogs_mirror_y1_error"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
	    allOK = false;
	}
	try
	{
	    x2 = cc.gc.fromUserToWorld(Double.parseDouble (x2TF.getText()));
	}
	catch (Exception ex)
	{
	    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject("dialogs_mirror_x2_error"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
	    allOK = false;
	}
	try
	{
	    y2 = cc.gc.fromUserToWorld(Double.parseDouble (y2TF.getText()));
	}
	catch (Exception ex)
	{
	    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject("dialogs_mirror_y2_error"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
	    allOK = false;
	}

	return allOK;
    }


    // keys for the action commands

    private static final String OK = "OK";
    private static final String CANCEL = "CANCEL";
    private static final String APPLY = "APPLY";

    // buttons

    private JButton okButton;
    private JButton cancelButton;
    private JButton applyButton;
    private JTextField x1TF;
    private JTextField y1TF;
    private JTextField x2TF;
    private JTextField y2TF;


    
}




