package devisor2.grid.GUI.dialogs;

import java.util.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

import devisor2.grid.options.*;
import devisor2.grid.GUI.framework.*;
import devisor2.foundation.elements.*;


/**
 *  This class is the DockableDialog instance to set multiple boundary nodes.
 *  <br>
 *  <br>
 *  Take a look at our technical paper about dialogs if you want to learn
 *  more, to be found in the /manual/specs directory of your installation.
 *
 *  @author Dominik Goeddeke
 *  @version 1.0 fully tested
 *  
 */

public class MultipleBNDialog extends DockableDialog implements ActionListener
{
	
    /** 
     *  returns the initial Position of the dialog
     */
    public Point getInitialPosition ()
    {
	return new Point (parent.getLocationOnScreen().x + parent.getWidth(), parent.getLocationOnScreen().y);
    }


    /**
     *  the constructor just sets references correctly and creates the GUI
     */
    public MultipleBNDialog (MainFrame parent)
    {
	super (parent);
	setTitle ((String)cc.rb.getObject ("dialogs_multiplebn_title"));
	JPanel content = new JPanel ();
	content.setBorder (new EmptyBorder (10,10,10,10));
	GridBagLayout layout = new GridBagLayout ();
	GridBagConstraints constraints = new GridBagConstraints ();
	constraints.fill = GridBagConstraints.BOTH;
	constraints.weightx = constraints.weighty = 1.0;
	content.setLayout (layout);
	//////////////////////
	JLabel boundaryLabel = new JLabel ((String)cc.rb.getObject ("dialogs_multiplebn_boundary"));
	constraints.gridx = 0;
	constraints.gridy = 0;
	layout.setConstraints (boundaryLabel, constraints);
	content.add (boundaryLabel);
	boundaryCombo = new JComboBox ();
	boundaryCombo.setEditable (false);
	boundaryCombo.setActionCommand (BOUNDARY);
	boundaryCombo.addActionListener (this);
	constraints.gridx = 1;
	layout.setConstraints (boundaryCombo, constraints);
  	content.add (boundaryCombo);
	JLabel maxparLabel = new JLabel ((String)cc.rb.getObject ("dialogs_multiplebn_maxpar"));
	constraints.gridx = 2;
	layout.setConstraints (maxparLabel, constraints);
	content.add (maxparLabel);
	maxparTF = new JTextField (4);
	maxparTF.setEditable (false);
	constraints.gridx = 3;
	layout.setConstraints (maxparTF, constraints);
	content.add (maxparTF);
	/////////
	JLabel incrementLabel = new JLabel ((String)cc.rb.getObject ("dialogs_multiplebn_increment"));
	constraints.gridy = 1;
	constraints.gridx = 0;
	layout.setConstraints (incrementLabel, constraints);
	content.add (incrementLabel);
	incrementTF = new JTextField (4);
	constraints.gridx = 1;
	constraints.gridwidth = 3;
	layout.setConstraints (incrementTF, constraints);
	content.add (incrementTF);
	/////////
	JLabel startLabel = new JLabel ((String)cc.rb.getObject ("dialogs_multiplebn_start"));
	constraints.gridy = 2;
	constraints.gridx = 0;
	constraints.gridwidth = 1;
	layout.setConstraints (startLabel, constraints);
	content.add (startLabel);
	startTF = new JTextField (4);
	constraints.gridx = 1;
	constraints.gridwidth = 3;
	layout.setConstraints (startTF, constraints);
	content.add (startTF);
	/////////
	JLabel endLabel = new JLabel ((String)cc.rb.getObject ("dialogs_multiplebn_end"));
	constraints.gridy = 3;
	constraints.gridx = 0;
	constraints.gridwidth = 1;
	layout.setConstraints (endLabel, constraints);
	content.add (endLabel);
	endTF = new JTextField (4);
	constraints.gridx = 1;
	constraints.gridwidth = 3;
	layout.setConstraints (endTF, constraints);
	content.add (endTF);
	// buttons
	JPanel buttons = new JPanel ();
	buttons.setLayout (new FlowLayout ());
	okButton = new JButton ((String)cc.rb.getObject ("general_Ok"));
	okButton.setMnemonic ( ((Integer)cc.rb.getObject("general_Ok_mnemonic")).intValue());
	okButton.setActionCommand (OK);
	okButton.addActionListener (this);
	buttons.add (okButton);
	applyButton = new JButton ((String)cc.rb.getObject ("general_Apply"));
	applyButton.setMnemonic ( ((Integer)cc.rb.getObject("general_Apply_mnemonic")).intValue());
	applyButton.setActionCommand (APPLY);
	applyButton.addActionListener (this);
	buttons.add (applyButton);
	cancelButton = new JButton ((String)cc.rb.getObject ("general_Cancel"));
	cancelButton.setMnemonic ( ((Integer)cc.rb.getObject("general_Cancel_mnemonic")).intValue());
	cancelButton.setActionCommand (CANCEL);
	cancelButton.addActionListener (this);
	buttons.add (cancelButton);
	// add them all to the dialog
	getRootPane().setDefaultButton (applyButton);

	constraints.gridy = 4;
	constraints.gridx = 0;
	constraints.gridwidth = 4;
	layout.setConstraints(buttons, constraints);
	content.add (buttons);
	// finish the GUI
	getContentPane ().add (content);
	addComponentListener (listener);

	pack ();
    }


    /**
     *  updates the boundary comboboxes
     */
    public void refreshGUI ()
    {
	boundaryCombo.removeActionListener (this);
	boundaryCombo.removeAllItems ();
	if (cc.gc.workingBoundary == null)
	{
	    boundaryCombo.addItem ("-1");
	    boundaryCombo.setSelectedItem ("-1");
	}
	else
	{
	    for (int i=0; i<cc.d.db.getBoundaryCount(); i++)
		boundaryCombo.addItem (Integer.toString(i+1));
	    boundaryCombo.setSelectedItem (Integer.toString (cc.gc.workingBoundary.getNumber ()-1));
	}
	try
	{
	    boundary = Integer.parseInt ((String)boundaryCombo.getSelectedItem())-1;
	    maxparTF.setText (Double.toString (cc.d.db.getBoundary(boundary).getParaMax()));
	}
	catch (Exception ex)
	{
	    maxparTF.setText ("");
	}
	boundaryCombo.addActionListener (this);
    }

    /**
     *  This class handles its own action event. The actionPerformed method 
     *  just enables or disables the boundary condition radiobuttons
     */
    public void actionPerformed (ActionEvent e)
    {
	if (e.getActionCommand().equals (OK))
	{
	    if (verify ())
	    {
		cc.d.db.addMultipleBoundaryNodes (increment, start, end,cc.d.db.getBoundary(boundary));
		parent.getDrawingArea().repaint ();
		cc.dm.hide (cc.dm.MULTIPLEBNDIALOG);

	    }
	    return;
	}
	if (e.getActionCommand().equals (APPLY))
	{
	    if (verify ())
	    {
		cc.d.db.addMultipleBoundaryNodes (increment, start, end,cc.d.db.getBoundary(boundary));
		parent.getDrawingArea().repaint ();
	    }
	    return;
	}
	if (e.getActionCommand().equals (CANCEL))
	{
	    cc.dm.hide (cc.dm.MULTIPLEBNDIALOG);
	    return;
	}
	if (e.getActionCommand().equals (BOUNDARY))
	{
	    try
	    {
		boundary = Integer.parseInt ((String)boundaryCombo.getSelectedItem())-1;
	    }
	    catch (Exception ex)
	    {
		JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject ("dialogs_multiplebn_error_boundary"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
		return;
	    }
	    if (boundary < cc.d.db.getBoundaryCount() && boundary >= 0)
	    {
		maxparTF.setText (Double.toString (cc.d.db.getBoundary(boundary).getParaMax()));
	    }
	    else
	    {
		maxparTF.setText ("");
		JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject ("dialogs_multiplebn_error_nosuchboundary"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
	    }
	    return;
	}
    }



    /**
     *  the boring verification code is outsourced into this method
     */
    private boolean verify ()
    {
	boolean allOK = true;
	try
	{
	    increment = Double.parseDouble (incrementTF.getText ());
	}
	catch (NumberFormatException ex)
	{
	    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject ("dialogs_multiplebn_error_increment"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
		allOK = false;
	}
	try
	{
	    start = Double.parseDouble (startTF.getText ());
	}
	catch (NumberFormatException ex)
	{
	    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject ("dialogs_multiplebn_error_start"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
		allOK = false;
	}
	try
	{
	    end = Double.parseDouble (endTF.getText ());
	}
	catch (NumberFormatException ex)
	{
	    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject ("dialogs_multiplebn_error_end"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
		allOK = false;
	}
	if (allOK)
	{
	    if (start < 0)
	    {
		allOK = false;
		JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject ("dialogs_multiplebn_error_starttoosmall"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);

	    }
	    if (end > cc.d.db.getBoundary (boundary).getParaMax())
	    {
		allOK = false;
		JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject ("dialogs_multiplebn_error_endtoobig"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
	    }
	}
	return allOK;
    }

    // some storage for parametrisation (from textfields)
    private double increment;
    private int boundary;
    private double start;
    private double end;


    // keys for the action commands

    private static final String OK = "OK";;
    private static final String CANCEL = "CANCEL";
    private static final String APPLY = "APPLY";
    private static final String BOUNDARY = "BOUNDARY";
   
    // buttons 
    private JButton okButton;
    private JButton cancelButton;
    private JButton applyButton;
    private JComboBox boundaryCombo;
    private JTextField maxparTF;
    private JTextField incrementTF;
    private JTextField startTF;
    private JTextField endTF;
    
   
}
