package devisor2.grid.GUI.dialogs;


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.filechooser.*;
import java.io.*;
import java.util.*;

import devisor2.grid.options.*;
import devisor2.grid.GUI.framework.*;
import devisor2.grid.GUI.dialogs.*;
import devisor2.grid.GUI.event.*;

/** 
 *  This class provides the event handler for the options dialog
 *  (OptionsDialog.java), such as restoring factory defaults, 
 *  loading and saving user-defined settings, applying changes 
 *  (which is the most important bit here!) and closing the dialog.<br>
 *  <br>
 *  For every button in the OptionsDialog, a unique String 
 *  (an ActionCommand) is declared in this class and registered 
 *  with the corresponding button in the dialog. The actual event 
 *  handling code goes in a corresponding 
 *            <code>if e.getActionCommand().equals (blablabla)"<\code>
 *  block in the actionPerformed method.
 *
 *  @author Dominik Goeddeke
 *  @version 0.5 (for all options availible so far, event handlers have 
 *  been added)
 */

class OptionsActionListener implements ActionListener
{
    /** reference to the main window of the application */
    private MainFrame parent;

    /** reference to the OptionsDialog this listener is responsible for */
    private OptionsDialog dialog;

    /** reference to the ControlCenter */
    private ControlCenter cc;

    /** 
     *  The constructor just sets the reference correctly
     *
     *  @param parent - the mainframe instance, the link to the rest
     *                  of the application
     *  @param dialog - a reference to the OptionsDialog instance this
     *                  class listens to
     *  
     */
    public OptionsActionListener (MainFrame parent, OptionsDialog dialog)
    {
	this.parent = parent;
	this.dialog = dialog;
	cc = ControlCenter.getMyself ();
    }
    
    /** 
     *  The actionPerformed() method is just a king-size switch-block
     *  all actions this method performs must be declared as String
     *  constants in this class (see below)
     *  It is called by the JRE everytime an ActionEvent is raised.
     */
    public void actionPerformed(ActionEvent e) {
    	/* Action Command ACCEPT */
	if (e.getActionCommand ().equals(ACCEPT))
	{
	    String oldlf = cc.op.get (Options.other_lookandfeel);
	    dialog.updateOptions();
	    updateAll (oldlf);
	    // update DrawControl
	    
	    int ret1=cc.dc.screen_width;
	    int ret2=cc.dc.screen_height;
	    int ret3=cc.dc.used_screen_width;
	    int ret4=cc.dc.used_screen_height;
	    int ret5=cc.dc.world_x1;
	    int ret6=cc.dc.world_x2;
	    int ret7=cc.dc.world_y1;
	    int ret8=cc.dc.world_y2;
	    int ret9=cc.dc.act_x1;
	    int ret10=cc.dc.act_x2;
	    int ret11=cc.dc.act_y1;
	    int ret12=cc.dc.act_y2;
	    	    
	    cc.dc = new DrawControl (cc.op);
	    cc.dc.tolerance = cc.gc.SingleFromScreenToWorld(cc.dc.tolerance);

	    cc.dc.screen_width=ret1;
	    cc.dc.screen_height=ret2;
	    cc.dc.used_screen_width=ret3;
	    cc.dc.used_screen_height=ret4;
	    cc.dc.world_x1=ret5;
	    cc.dc.world_x2=ret6;
	    cc.dc.world_y1=ret7;
	    cc.dc.world_y2=ret8;
	    cc.dc.act_x1=ret9;
	    cc.dc.act_x2=ret10;
	    cc.dc.act_y1=ret11;
	    cc.dc.act_y2=ret12;

//	    cc.dc.main_width = parent.getWidth ();
//	    cc.dc.main_height = parent.getHeight ();
//	    cc.dc.screen_height = parent.getToolkit().getScreenSize().getSize().height;
//	    cc.dc.screen_width = parent.getToolkit().getScreenSize().getSize().width;
	    cc.gc.vsnapx = Integer.parseInt (cc.op.get(Options.snap_x));
	    cc.gc.vsnapy = Integer.parseInt (cc.op.get(Options.snap_y));
	    cc.gc.snapon = Boolean.valueOf (cc.op.get (Options.snap_enabled)).booleanValue ();
      
	    int eps = Integer.parseInt (cc.op.get (Options.epsilon));
	    double epsi = 0.01 + ((double)eps)/100 * 0.99;
	    cc.gc.eps = (int)(epsi*(cc.dc.world_x2 - cc.dc.world_x1));
	    // leave the house...
	    dialog.hide();
	    parent.getDrawingArea().repaint ();
	    return;
	}
	/* Action Command CANCEL */
	if (e.getActionCommand ().equals(CANCEL))
	{
	    dialog.hide();
	    return;
	}
	/* Action Command LOAD */
	if (e.getActionCommand ().equals(LOAD))
	{
	    // show a load dialog with supported formats optionfiles
	    FilePicker picker = new FilePicker (parent,(String)cc.rb.getObject ("dialogs_loadoptionsdialog_caption"), (String)cc.rb.getObject ("dialogs_loaddialog_button"), cc.op.get (Options.general_path), new FilePickerFilter (FilePickerFilter.FILTER_ALL), true);
	    int returnvalue = picker.showDialog ();
	    // if user accepted a file in the dialog...
	    if (returnvalue == JFileChooser.APPROVE_OPTION)
	    {
		// get the file
		File selectedFile = picker.getSelectedFile ();
		if (selectedFile != null)
		{
		    // eventueally add extension

		    // and load the file
		    String filepath = selectedFile.getPath ();
		    try
		    {
			cc.op.load (filepath);
		    }
		    catch (Exception ex)
		    {
			JOptionPane.showMessageDialog (dialog, ex.getMessage(),(String)cc.rb.getObject ("errors_messageboxheader"),JOptionPane.ERROR_MESSAGE);
		    }
		}
		((OptionsDialog)dialog).updateGUI();
		String oldlf = cc.op.get (Options.other_lookandfeel);
		updateAll ("12345");
		return;
	    }
	}

	/* Action Command SAVE */
	if (e.getActionCommand ().equals(SAVE))
	{
	    // first, accept without dialog closing
	    String oldlf = cc.op.get (Options.other_lookandfeel);
	    dialog.updateOptions ();
	    updateAll (oldlf);
	    // display dialog
	    FilePicker picker = new FilePicker (parent,(String)cc.rb.getObject ("dialogs_saveoptionsdialog_caption"), (String)cc.rb.getObject ("dialogs_savedialog_button"), cc.op.get (Options.general_path), new FilePickerFilter (FilePickerFilter.FILTER_ALL), true);
	    String os = System.getProperty ("os.name").toUpperCase();
	    if (os.indexOf ("WIN") != -1)
	    {
		picker.setSelectedFile (new File(cc.op.get (Options.general_devisorhome)+System.getProperty ("file.separator")+"devisorgrid.conf"));
	    }
	    else
	    {
		picker.setSelectedFile (new File(System.getProperty("user.home")+System.getProperty ("file.separator")+".devisorgrid"));
	    }
	    int returnvalue = picker.showDialog ();
	    // if user accepted a file in the dialog...
	    if (returnvalue == JFileChooser.APPROVE_OPTION)
	    {
		File selectedFile = picker.getSelectedFile ();
		if (selectedFile != null)
		    {
			String filepath = selectedFile.getPath ();
			String header = JOptionPane.showInputDialog(dialog, "Please enter the header comment for your Options" , "DeViSoRGrid2 Options file",JOptionPane.QUESTION_MESSAGE) ;
			cc.op.setHeader (header);
			try
			    {
				cc.op.save (filepath);
			    }
			catch (Exception ex)
			    {
				JOptionPane.showMessageDialog (dialog, ex.getMessage(),"ERROR",JOptionPane.ERROR_MESSAGE);
			    }
			
		    }
		else
		    {
			JOptionPane.showMessageDialog (dialog,"No file selected","Error",JOptionPane.ERROR_MESSAGE);
		    }
		return;
	    }
	    

	}

	/* Action Command RESET */
	if (e.getActionCommand().equals(RESET))
	    {
		cc.op.resetToDefaults ();
		((OptionsDialog)dialog).updateGUI();
		updateAll ("12345");
		// set DockableDialog positions

		return;
	    }


	/* Action Command HELP */
	if (e.getActionCommand ().equals(HELP))
	{
	    cc.help.show();
	    return;
	}

	/* Action Command DEVHOME */
	if (e.getActionCommand ().equals(DEVHOME))
	{
	    // show a load dialog which allows paths only
	    String initialDirectory = dialog.getDevHomeTextField().getText();
	    if (initialDirectory == null)
		initialDirectory = cc.op.get (Options.general_devisorhome);
	    FilePicker picker = new FilePicker (parent,(String)cc.rb.getObject ("options_OptionsDialog_generalPanel_devisorhome_dialogtitle"), (String)cc.rb.getObject ("general_Accept"), initialDirectory, new FilePickerFilter (FilePickerFilter.FILTER_DIRECTORIES), false);
	    int returnvalue = picker.showDialog ();
	    if (returnvalue == JFileChooser.APPROVE_OPTION)
	    {
		dialog.getDevHomeTextField().setText (picker.getSelectedFile().getAbsolutePath());
	    }
	    return;
	}
	/* Action Command TRIGEN */
	if (e.getActionCommand ().equals(TRIGEN))
	{
	    // show a load dialog 
	    String initialDirectory = dialog.getTrigenTextField().getText();
	    if (initialDirectory == null)
		initialDirectory = cc.op.get (Options.general_trigen);
	    FilePicker picker = new FilePicker (parent,(String)cc.rb.getObject ("options_OptionsDialog_generalPanel_trigen_dialogtitle"), (String)cc.rb.getObject ("general_Accept"), initialDirectory, new FilePickerFilter (FilePickerFilter.FILTER_BIN), true);
	    int returnvalue = picker.showDialog ();
	    if (returnvalue == JFileChooser.APPROVE_OPTION)
	    {
		dialog.getTrigenTextField().setText (picker.getSelectedFile().getAbsolutePath());
	    }
	    return;
	}
	/* Action Command FEATLIB */
	if (e.getActionCommand ().equals(FEATLIB))
	{
	    // show a load dialog which allows paths only
	    String initialDirectory = dialog.getFeatlibTextField().getText();
	    if (initialDirectory == null)
		initialDirectory = cc.op.get (Options.general_featlib);
	    FilePicker picker = new FilePicker (parent,(String)cc.rb.getObject ("options_OptionsDialog_generalPanel_featlib_dialogtitle"), (String)cc.rb.getObject ("general_Accept"), initialDirectory, new FilePickerFilter (FilePickerFilter.FILTER_DIRECTORIES), false);
	    int returnvalue = picker.showDialog ();
	    if (returnvalue == JFileChooser.APPROVE_OPTION)
	    {
		dialog.getFeatlibTextField().setText (picker.getSelectedFile().getAbsolutePath());
	    }
	    return;
	}

	/* Action Command PATH */
	if (e.getActionCommand ().equals(PATH))
	{
	    // show a load dialog which allows paths only
	    String initialDirectory = dialog.getPathTextField().getText();
	    if (initialDirectory == null)
		initialDirectory = cc.op.get (Options.general_path);
	    FilePicker picker = new FilePicker (parent,(String)cc.rb.getObject ("options_OptionsDialog_generalPanel_generalpath_dialogtitle"), (String)cc.rb.getObject ("general_Accept"), initialDirectory, new FilePickerFilter (FilePickerFilter.FILTER_DIRECTORIES), false);
	    int returnvalue = picker.showDialog ();
	    if (returnvalue == JFileChooser.APPROVE_OPTION)
	    {
		dialog.getPathTextField().setText (picker.getSelectedFile().getAbsolutePath());
	    }
	    return;
	}
	/* Action Command SNAP */
	if (e.getActionCommand ().equals (SNAP))
	{
	    if (dialog.getSnapCheck().isSelected())
	    {
		dialog.getSnapXTF().setEnabled (true);
		dialog.getSnapXTF().setText (Integer.toString (cc.gc.vsnapx));
		dialog.getSnapYTF().setEnabled (true);
		dialog.getSnapYTF().setText (Integer.toString (cc.gc.vsnapy));
	    }
	    else
	    {
		dialog.getSnapXTF().setEnabled (false);
		dialog.getSnapXTF().setText("");
		dialog.getSnapYTF().setEnabled (false);
		dialog.getSnapYTF().setText ("");
	    }
	}

	/* Action Command  APPLY_ACCELERATOR */
	if (e.getActionCommand().equals(APPLY_ACCELERATOR))
	    {
		String mod;
		String key;
		// step one: extract the selected modifier key
		if (dialog.getCTRLbutton().isSelected ())
		    {
			mod = "CTRL";
		    }
		else
		    {
			if (dialog.getALTbutton().isSelected())
			    {
				mod = "ALT";
			    }
			else
			    {
				if (dialog.getSHIFTbutton().isSelected())
				    {
					mod = "SHIFT";
				    }
				else
				    {
					mod = "NONE";
				    }
			    }
		    }
		// step two: extract key
		key = (String)dialog.getKeySelectionComboBox().getSelectedItem();
		// step three: long boring verification code
		// luckily implemented in the Accelerators class
		boolean correctAccelerator = Accelerators.isCorrectAccelerator (mod, key);
		if (!correctAccelerator)
		    {
			// a little error message
			JOptionPane.showMessageDialog (dialog, (String)cc.rb.getObject("options_OptionsDialog_acceleratorPanel_nocorrectaccelerator"),(String)cc.rb.getObject("errors_messageboxheader"), JOptionPane.ERROR_MESSAGE); 
		    }
		else
		    {
			// step four: is that accelerator already in use
			String accelerator = mod + "+" + key;
			if (accelerator.equals("NONE+NONE"))
			    accelerator = "NONE";
			boolean isInUse = cc.op.isAlreadyAccelerator (accelerator);
			if (isInUse)
			    {
				// a second error message
				JOptionPane.showMessageDialog (dialog, (String)cc.rb.getObject("options_OptionsDialog_acceleratorPanel_acceleratoralreadyinuse"),(String)cc.rb.getObject("errors_messageboxheader"), JOptionPane.ERROR_MESSAGE); 
			    }
			else
			    {
				// step five: update the table
				int row = dialog.getAcceleratorTable().getSelectedRow ();
				if (row != -1)
				    {
					((String[])(dialog.getAcceleratorTableData().elementAt(row)))[1] = accelerator;
					((AcceleratorTableModel)dialog.getAcceleratorTable().getModel()).fireTableDataChanged();
				    }
				else
				    {
					JOptionPane.showMessageDialog (dialog, (String)cc.rb.getObject("options_OptionsDialog_acceleratorPanel_youmustselectarow"),(String)cc.rb.getObject("errors_messageboxheader"), JOptionPane.ERROR_MESSAGE); 
				    }
			    }
		    }
	    }

	/* Action Command REMOVE_ACCELERATOR */
	if (e.getActionCommand ().equals(REMOVE_ACCELERATOR))
	{
	    int row = dialog.getAcceleratorTable().getSelectedRow ();
	    if (row != -1)
		{
		    ((String[])(dialog.getAcceleratorTableData().elementAt(row)))[1] = "NONE";
		    ((AcceleratorTableModel)dialog.getAcceleratorTable().getModel()).fireTableDataChanged();
		}
	    else
		{
		    JOptionPane.showMessageDialog (dialog, (String)cc.rb.getObject("options_OptionsDialog_acceleratorPanel_youmustselectarow"),(String)cc.rb.getObject("errors_messageboxheader"), JOptionPane.ERROR_MESSAGE); 
		}
	    return;
	}

	/* Action Command COLORNODE */
	if (e.getActionCommand ().equals(COLORNODE))
	{
	    dialog.getColornodeLabel().setBackground (ColorPicker.show ((String)cc.rb.getObject("options_OptionsDialog_colorPanel_objects_node_dialog"), dialog.getColornodeLabel().getBackground(), dialog));
	    return;
	}
	/* Action Command COLOREDGE */
	if (e.getActionCommand ().equals(COLOREDGE))
	{
	    dialog.getColoredgeLabel().setBackground (ColorPicker.show ((String)cc.rb.getObject("options_OptionsDialog_colorPanel_objects_edge_dialog"), dialog.getColoredgeLabel().getBackground(), dialog));
	    return;
	}
	/* Action Command COLOREDGEBASE */
	if (e.getActionCommand ().equals(COLOREDGEBASE))
	{
	    dialog.getColoredgebaseLabel().setBackground (ColorPicker.show ((String)cc.rb.getObject("options_OptionsDialog_colorPanel_objects_edgebase_dialog"), dialog.getColoredgebaseLabel().getBackground(), dialog));
	    return;
	}
	/* Action Command COLORELEMENT */
	if (e.getActionCommand ().equals(COLORELEMENT))
	{
	    dialog.getColorelementLabel().setBackground (ColorPicker.show ((String)cc.rb.getObject("options_OptionsDialog_colorPanel_objects_element_dialog"), dialog.getColorelementLabel().getBackground(), dialog));
	    return;
	}
	/* Action Command NUMBERBOUNDARY */
	if (e.getActionCommand ().equals(NUMBERBOUNDARY))
	{
	    dialog.getNumberboundaryLabel().setBackground (ColorPicker.show ((String)cc.rb.getObject("options_OptionsDialog_colorPanel_numbers_boundary_dialog"), dialog.getNumberboundaryLabel().getBackground(), dialog));
	    return;
	}
	/* Action Command NUMBERNODE */
	if (e.getActionCommand ().equals(NUMBERNODE))
	{
	    dialog.getNumbernodeLabel().setBackground (ColorPicker.show ((String)cc.rb.getObject("options_OptionsDialog_colorPanel_numbers_node_dialog"), dialog.getNumbernodeLabel().getBackground(), dialog));
	    return;
	}
	/* Action Command NUMBERREDGE */
	if (e.getActionCommand ().equals(NUMBEREDGE))
	{
	    dialog.getNumberedgeLabel().setBackground (ColorPicker.show ((String)cc.rb.getObject("options_OptionsDialog_colorPanel_numbers_edge_dialog"), dialog.getNumberedgeLabel().getBackground(), dialog));
	    return;
	}
	/* Action Command NUMBERELEMENT */
	if (e.getActionCommand ().equals(NUMBERELEMENT))
	{
	    dialog.getNumberelementLabel().setBackground (ColorPicker.show ((String)cc.rb.getObject("options_OptionsDialog_colorPanel_numbers_element_dialog"), dialog.getNumberelementLabel().getBackground(), dialog));
	    return;
	}
	/* Action Command HIGHLIGHTSELECTION */
	if (e.getActionCommand ().equals(HIGHLIGHTSELECTION))
	{
	    dialog.getHighlightselectionLabel().setBackground (ColorPicker.show ((String)cc.rb.getObject("options_OptionsDialog_colorPanel_highlight_selection_dialog"), dialog.getHighlightselectionLabel().getBackground(), dialog));
	    return;
	}
	/* Action Command HIGHLIGHTRECT */
	if (e.getActionCommand ().equals(HIGHLIGHTRECT))
	{
	    dialog.getHighlightrectLabel().setBackground (ColorPicker.show ((String)cc.rb.getObject("options_OptionsDialog_colorPanel_highlight_rect_dialog"), dialog.getHighlightrectLabel().getBackground(), dialog));
	    return;
	}
	/* Action Command COLORGRID */
	if (e.getActionCommand ().equals(COLORGRID))
	{
	    dialog.getMiscgridLabel().setBackground (ColorPicker.show ((String)cc.rb.getObject("options_OptionsDialog_colorPanel_misc_grid_dialog"), dialog.getMiscgridLabel().getBackground(), dialog));
	    return;
	}
	/* Action Command COLORBACKGROUND */
	if (e.getActionCommand ().equals(COLORBACKGROUND))
	{
	    dialog.getMiscbackgroundLabel().setBackground (ColorPicker.show ((String)cc.rb.getObject("options_OptionsDialog_colorPanel_misc_background_dialog"), dialog.getMiscbackgroundLabel().getBackground(), dialog));
	    return;
	}
	/* Action Command COLORLASSO */
	if (e.getActionCommand ().equals(COLORLASSO))
	{
	    dialog.getMisclassoLabel().setBackground (ColorPicker.show ((String)cc.rb.getObject("options_OptionsDialog_colorPanel_misc_lasso_dialog"), dialog.getMisclassoLabel().getBackground(), dialog));
	    return;
	}
	/* Action Command FILL0 */
	if (e.getActionCommand ().equals(FILL0))
	{
	    dialog.getFill0Label().setBackground (ColorPicker.show ((String)cc.rb.getObject("options_OptionsDialog_colorPanel_fill0_dialog"), dialog.getFill0Label().getBackground(), dialog));
	    return;
	}
	/* Action Command FILL1 */
	if (e.getActionCommand ().equals(FILL1))
	{
	    dialog.getFill1Label().setBackground (ColorPicker.show ((String)cc.rb.getObject("options_OptionsDialog_colorPanel_fill1_dialog"), dialog.getFill1Label().getBackground(), dialog));
	    return;
	}
	/* Action Command FILL2 */
	if (e.getActionCommand ().equals(FILL2))
	{
	    dialog.getFill2Label().setBackground (ColorPicker.show ((String)cc.rb.getObject("options_OptionsDialog_colorPanel_fill2_dialog"), dialog.getFill2Label().getBackground(), dialog));
	    return;
	}
	/* Action Command FILL3 */
	if (e.getActionCommand ().equals(FILL3))
	{
	    dialog.getFill3Label().setBackground (ColorPicker.show ((String)cc.rb.getObject("options_OptionsDialog_colorPanel_fill3_dialog"), dialog.getFill3Label().getBackground(), dialog));
	    return;
	}
	/* Action Command FILL4 */
	if (e.getActionCommand ().equals(FILL4))
	{
	    dialog.getFill4Label().setBackground (ColorPicker.show ((String)cc.rb.getObject("options_OptionsDialog_colorPanel_fill4_dialog"), dialog.getFill4Label().getBackground(), dialog));
	    return;
	}
	/* Action Command FILL5 */
	if (e.getActionCommand ().equals(FILL5))
	{
	    dialog.getFill5Label().setBackground (ColorPicker.show ((String)cc.rb.getObject("options_OptionsDialog_colorPanel_fill5_dialog"), dialog.getFill5Label().getBackground(), dialog));
	    return;
	}
	/* Action Command FILL6 */
	if (e.getActionCommand ().equals(FILL6))
	{
	    dialog.getFill6Label().setBackground (ColorPicker.show ((String)cc.rb.getObject("options_OptionsDialog_colorPanel_fill6_dialog"), dialog.getFill6Label().getBackground(), dialog));
	    return;
	}
	/* Action Command FILL7 */
	if (e.getActionCommand ().equals(FILL7))
	{
	    dialog.getFill7Label().setBackground (ColorPicker.show ((String)cc.rb.getObject("options_OptionsDialog_colorPanel_fill7_dialog"), dialog.getFill7Label().getBackground(), dialog));
	    return;
	}
	/* Action Command FILL8 */
	if (e.getActionCommand ().equals(FILL8))
	{
	    dialog.getFill8Label().setBackground (ColorPicker.show ((String)cc.rb.getObject("options_OptionsDialog_colorPanel_fill8_dialog"), dialog.getFill8Label().getBackground(), dialog));
	    return;
	}
	/* Action Command FILL9 */
	if (e.getActionCommand ().equals(FILL9))
	{
	    dialog.getFill9Label().setBackground (ColorPicker.show ((String)cc.rb.getObject("options_OptionsDialog_colorPanel_fill9_dialog"), dialog.getFill9Label().getBackground(), dialog));
	    return;
	}
	/* Action Command FILL10 */
	if (e.getActionCommand ().equals(FILL10))
	{
	    dialog.getFill10Label().setBackground (ColorPicker.show ((String)cc.rb.getObject("options_OptionsDialog_colorPanel_fill10_dialog"), dialog.getFill10Label().getBackground(), dialog));
	    return;
	}
	/* Action Command FILL11 */
	if (e.getActionCommand ().equals(FILL11))
	{
	    dialog.getFill11Label().setBackground (ColorPicker.show ((String)cc.rb.getObject("options_OptionsDialog_colorPanel_fill11_dialog"), dialog.getFill11Label().getBackground(), dialog));
	    return;
	}
	/* Action Command FILL12 */
	if (e.getActionCommand ().equals(FILL12))
	{
	    dialog.getFill12Label().setBackground (ColorPicker.show ((String)cc.rb.getObject("options_OptionsDialog_colorPanel_fill12_dialog"), dialog.getFill12Label().getBackground(), dialog));
	    return;
	}
	/* Action Command FILL13 */
	if (e.getActionCommand ().equals(FILL13))
	{
	    dialog.getFill13Label().setBackground (ColorPicker.show ((String)cc.rb.getObject("options_OptionsDialog_colorPanel_fill13_dialog"), dialog.getFill13Label().getBackground(), dialog));
	    return;
	}
	/* Action Command FILL14 */
	if (e.getActionCommand ().equals(FILL14))
	{
	    dialog.getFill14Label().setBackground (ColorPicker.show ((String)cc.rb.getObject("options_OptionsDialog_colorPanel_fill14_dialog"), dialog.getFill14Label().getBackground(), dialog));
	    return;
	}
	/* Action Command FILL15 */
	if (e.getActionCommand ().equals(FILL15))
	{
	    dialog.getFill15Label().setBackground (ColorPicker.show ((String)cc.rb.getObject("options_OptionsDialog_colorPanel_fill15_dialog"), dialog.getFill15Label().getBackground(), dialog));
	    return;
	}
	/* Action Command BOUND0 */
	if (e.getActionCommand ().equals(BOUND0))
	{
	    dialog.getBound0Label().setBackground (ColorPicker.show ((String)cc.rb.getObject("options_OptionsDialog_colorPanel_boundary0_dialog"), dialog.getBound0Label().getBackground(), dialog));
	    return;
	}
	/* Action Command BOUND1 */
	if (e.getActionCommand ().equals(BOUND1))
	{
	    dialog.getBound1Label().setBackground (ColorPicker.show ((String)cc.rb.getObject("options_OptionsDialog_colorPanel_boundary1_dialog"), dialog.getBound1Label().getBackground(), dialog));
	    return;
	}
	/* Action Command BOUND2 */
	if (e.getActionCommand ().equals(BOUND2))
	{
	    dialog.getBound2Label().setBackground (ColorPicker.show ((String)cc.rb.getObject("options_OptionsDialog_colorPanel_boundary2_dialog"), dialog.getBound2Label().getBackground(), dialog));
	    return;
	}
	/* Action Command BOUND3 */
	if (e.getActionCommand ().equals(BOUND3))
	{
	    dialog.getBound3Label().setBackground (ColorPicker.show ((String)cc.rb.getObject("options_OptionsDialog_colorPanel_boundary3_dialog"), dialog.getBound3Label().getBackground(), dialog));
	    return;
	}
	/* Action Command BOUND4 */
	if (e.getActionCommand ().equals(BOUND4))
	{
	    dialog.getBound4Label().setBackground (ColorPicker.show ((String)cc.rb.getObject("options_OptionsDialog_colorPanel_boundary4_dialog"), dialog.getBound4Label().getBackground(), dialog));
	    return;
	}
	/* Action Command BOUND5 */
	if (e.getActionCommand ().equals(BOUND5))
	{
	    dialog.getBound5Label().setBackground (ColorPicker.show ((String)cc.rb.getObject("options_OptionsDialog_colorPanel_boundary5_dialog"), dialog.getBound5Label().getBackground(), dialog));
	    return;
	}
	/* Action Command BOUND6 */
	if (e.getActionCommand ().equals(BOUND6))
	{
	    dialog.getBound6Label().setBackground (ColorPicker.show ((String)cc.rb.getObject("options_OptionsDialog_colorPanel_boundary6_dialog"), dialog.getBound6Label().getBackground(), dialog));
	    return;
	}
	/* Action Command BOUND7 */
	if (e.getActionCommand ().equals(BOUND7))
	{
	    dialog.getBound7Label().setBackground (ColorPicker.show ((String)cc.rb.getObject("options_OptionsDialog_colorPanel_boundary7_dialog"), dialog.getBound7Label().getBackground(), dialog));
	    return;
	}
	/* Action Command BOUND8 */
	if (e.getActionCommand ().equals(BOUND8))
	{
	    dialog.getBound8Label().setBackground (ColorPicker.show ((String)cc.rb.getObject("options_OptionsDialog_colorPanel_boundary8_dialog"), dialog.getBound8Label().getBackground(), dialog));
	    return;
	}
	/* Action Command BOUND9 */
	if (e.getActionCommand ().equals(BOUND9))
	{
	    dialog.getBound9Label().setBackground (ColorPicker.show ((String)cc.rb.getObject("options_OptionsDialog_colorPanel_boundary9_dialog"), dialog.getBound9Label().getBackground(), dialog));
	    return;
	}
	/* Action Command BOUND10 */
	if (e.getActionCommand ().equals(BOUND10))
	{
	    dialog.getBound10Label().setBackground (ColorPicker.show ((String)cc.rb.getObject("options_OptionsDialog_colorPanel_boundary10_dialog"), dialog.getBound10Label().getBackground(), dialog));
	    return;
	}
	/* Action Command BOUND11 */
	if (e.getActionCommand ().equals(BOUND11))
	{
	    dialog.getBound11Label().setBackground (ColorPicker.show ((String)cc.rb.getObject("options_OptionsDialog_colorPanel_boundary11_dialog"), dialog.getBound11Label().getBackground(), dialog));
	    return;
	}
	/* Action Command BOUND12 */
	if (e.getActionCommand ().equals(BOUND12))
	{
	    dialog.getBound12Label().setBackground (ColorPicker.show ((String)cc.rb.getObject("options_OptionsDialog_colorPanel_boundary12_dialog"), dialog.getBound12Label().getBackground(), dialog));
	    return;
	}
	/* Action Command BOUND13 */
	if (e.getActionCommand ().equals(BOUND13))
	{
	    dialog.getBound13Label().setBackground (ColorPicker.show ((String)cc.rb.getObject("options_OptionsDialog_colorPanel_boundary13_dialog"), dialog.getBound13Label().getBackground(), dialog));
	    return;
	}
	/* Action Command BOUND14 */
	if (e.getActionCommand ().equals(BOUND14))
	{
	    dialog.getBound14Label().setBackground (ColorPicker.show ((String)cc.rb.getObject("options_OptionsDialog_colorPanel_boundary14_dialog"), dialog.getBound14Label().getBackground(), dialog));
	    return;
	}
	/* Action Command BOUND15 */
	if (e.getActionCommand ().equals(BOUND15))
	{
	    dialog.getBound15Label().setBackground (ColorPicker.show ((String)cc.rb.getObject("options_OptionsDialog_colorPanel_boundary15_dialog"), dialog.getBound15Label().getBackground(), dialog));
	    return;
	}



	return;
    }
    

    /** 
     *  updates all GUI elements according to setting in current
     *  Options object in class ControlCenter
     */
    public void updateAll (String oldlookandfeel)
    {
	updateGeneral ();
	updateOther (oldlookandfeel);
	updateAccelerator ();
    }


    // updates everything on the GENERAL tab
    private void updateGeneral ()
    {

    }

    // update everything changed on the accelerators tab
    private void updateAccelerator ()
    {
	Vector data = dialog.getAcceleratorTableData ();
	// first the Options instance is updated
	Enumeration enum = data.elements ();
	while (enum.hasMoreElements ())
	    {
		String[] elem = (String[])enum.nextElement ();
		cc.op.set (elem[0], elem[1]);
	    }
	// and now the GUI menu itself
	parent.getMainMenu().setAccelerators ();
    }


    // updates everything changed on the OTHER tab
    private void updateOther (String oldlf)
    {
	try 
	{
	    if (!oldlf.equals(cc.op.get(Options.other_lookandfeel)))
	    {
		// set new l&f
		UIManager.setLookAndFeel(cc.op.get(Options.other_lookandfeel));
		// ... for the main frame
		SwingUtilities.updateComponentTreeUI(parent);
		parent.pack();
		SwingUtilities.updateComponentTreeUI(dialog);
		dialog.pack();
		SwingUtilities.updateComponentTreeUI(cc.help);
		cc.help.pack();
		// ... and all user-defined dialogs
		cc.dm.refreshDialogs ();
            }
	}
	catch (Exception ex) 
	{
	    if (!(ex instanceof NullPointerException))
	    {
		JOptionPane.showMessageDialog (parent,(String)cc.rb.getObject("errors_lookandfeelnotsupported"),(String)cc.rb.getObject("errors_messageboxheader"),JOptionPane.ERROR_MESSAGE);
	    }
	}
    }

    /**
     *  ActionCommand for the ACCEPT button
     */
    public static final String ACCEPT = "ACCEPT";

    /**
     *  ActionCommand for the CANCEL button
     */
    public static final String CANCEL = "CANCEL";
    
    /**
     *  ActionCommand for the LOAD button
     */
    public static final String LOAD = "LOAD";
   
    /**
     *  ActionCommand for the SAVE button
     */
    public static final String SAVE = "SAVE";
   
    /**
     *  ActionCommand for the RESETTODEFAULTS button
     */
    public static final String RESET = "RESET";
    
    /**
     *  ActionCommand for the HELP button
     */
    public static final String HELP = "HELP";
    
    /**
     *  ActionCommand for the DEVHOME button on tab GENERAL
     */
    public static final String DEVHOME = "DEVHOME";
    /**
     *  ActionCommand for the PATH button on tab GENERAL
     */
    public static final String PATH = "PATH";
    /**
     *  ActionCommand for the FEATLIB button on tab GENERAL
     */
    public static final String FEATLIB = "FEATLIB";
    /**
     *  ActionCommand for the TRIGEN button on tab GENERAL
     */
    public static final String TRIGEN = "TRIGEN";
    /**
     *  ActionCommand for the SNAP check box on DRAW tab
     */
    public static final String SNAP = "SNAP";
    /**
     *  ActionCommand for the NODE button on tab COLOR
     */
    public static final String COLORNODE = "COLORNODE";
    /**
     *  ActionCommand for the EDGE button on tab COLOR
     */
    public static final String COLOREDGE = "COLOREDGE";
    /**
     *  ActionCommand for the EDGEBASE button on tab COLOR
     */
    public static final String COLOREDGEBASE = "COLOREDGEBASE";
    /**
     *  ActionCommand for the ELEMENT button on tab COLOR
     */
    public static final String COLORELEMENT = "COLORELEMENT";
    /**
     *  ActionCommand for the BOUNDARYNUMBER button on tab COLOR
     */
    public static final String NUMBERBOUNDARY = "NUMBERBOUNDARY";
    /**
     *  ActionCommand for the NODENUMBER button on tab COLOR
     */
    public static final String NUMBERNODE = "NUMBERNODE";
    /**
     *  ActionCommand for the EDGENUMBER button on tab COLOR
     */
    public static final String NUMBEREDGE = "NUMBEREDGE";
    /**
     *  ActionCommand for the ELEMENTNUMBER button on tab COLOR
     */
    public static final String NUMBERELEMENT = "NUMBERELEMENT";
    /**
     *  ActionCommand for the HIGHLIGHTSELECTION button on tab COLOR
     */
    public static final String HIGHLIGHTSELECTION = "HIGHLIGHTSELECTION";
    /**
     *  ActionCommand for the HIGHLIGHTRECT button on tab COLOR
     */
    public static final String HIGHLIGHTRECT = "HIGHLIGHTRECT";
    /**
     *  ActionCommand for the GRID button on tab COLOR
     */
    public static final String COLORGRID = "COLORGRID";
    /**
     *  ActionCommand for the BACKGROUND button on tab COLOR
     */
    public static final String COLORBACKGROUND = "COLORBACKGROUND";
    /**
     *  ActionCommand for the LASSO button on tab COLOR
     */
    public static final String COLORLASSO = "COLORLASSO";
    /**
     *  ActionCommand for the FILL0 button on tab COLOR
     */
    public static final String FILL0 = "FILL0";
    /**
     *  ActionCommand for the FILL1 button on tab COLOR
     */
    public static final String FILL1 = "FILL1";
    /**
     *  ActionCommand for the FILL2 button on tab COLOR
     */
    public static final String FILL2 = "FILL2";
    /**
     *  ActionCommand for the FILL3 button on tab COLOR
     */
    public static final String FILL3 = "FILL3";
    /**
     *  ActionCommand for the FILL4 button on tab COLOR
     */
    public static final String FILL4 = "FILL4";
    /**
     *  ActionCommand for the FILL5 button on tab COLOR
     */
    public static final String FILL5 = "FILL5";
    /**
     *  ActionCommand for the FILL6 button on tab COLOR
     */
    public static final String FILL6 = "FILL6";
    /**
     *  ActionCommand for the FILL7 button on tab COLOR
     */
    public static final String FILL7 = "FILL7";
    /**
     *  ActionCommand for the FILL8 button on tab COLOR
     */
    public static final String FILL8 = "FILL8";
    /**
     *  ActionCommand for the FILL9 button on tab COLOR
     */
    public static final String FILL9 = "FILL9";
    /**
     *  ActionCommand for the FILL10 button on tab COLOR
     */
    public static final String FILL10 = "FILL10";
    /**
     *  ActionCommand for the FILL11 button on tab COLOR
     */
    public static final String FILL11 = "FILL11";
    /**
     *  ActionCommand for the FILL12 button on tab COLOR
     */
    public static final String FILL12 = "FILL12";
    /**
     *  ActionCommand for the FILL13 button on tab COLOR
     */
    public static final String FILL13 = "FILL13";
    /**
     *  ActionCommand for the FILL14 button on tab COLOR
     */
    public static final String FILL14 = "FILL14";
    /**
     *  ActionCommand for the FILL15 button on tab COLOR
     */
    public static final String FILL15 = "FILL15";

    /**
     *  ActionCommand for the APPLY button on the ACCELERATORS tab
     */
    public static final String APPLY_ACCELERATOR = "APPLY_ACCELERATOR";

    /**
     *  ActionCommand for the REMOVE button on the ACCELERATORS tab
     */
    public static final String REMOVE_ACCELERATOR = "REMOVE_ACCELERATOR";
    /**
     * ActionCommand for palette color 0 for boundaries
     */
    public static final String BOUND0 = "BOUND0";
    /**
     * ActionCommand for palette color 1 for boundaries
     */
    public static final String BOUND1 = "BOUND1";
    /**
     * ActionCommand for palette color 2 for boundaries
     */
    public static final String BOUND2 = "BOUND2";
    /**
     * ActionCommand for palette color 3 for boundaries
     */
    public static final String BOUND3 = "BOUND3";
    /**
     * ActionCommand for palette color 4 for boundaries
     */
    public static final String BOUND4 = "BOUND4";
    /**
     * ActionCommand for palette color 5 for boundaries
     */
    public static final String BOUND5 = "BOUND5";
    /**
     * ActionCommand for palette color 6 for boundaries
     */
    public static final String BOUND6 = "BOUND6";
    /**
     * ActionCommand for palette color 7 for boundaries
     */
    public static final String BOUND7 = "BOUND7";
    /**
     * ActionCommand for palette color 8 for boundaries
     */
    public static final String BOUND8 = "BOUND8";
    /**
     * ActionCommand for palette color 9 for boundaries
     */
    public static final String BOUND9 = "BOUND9";
    /**
     * ActionCommand for palette color 10 for boundaries
     */
    public static final String BOUND10 = "BOUND10";
    /**
     * ActionCommand for palette color 11 for boundaries
     */
    public static final String BOUND11 = "BOUND11";
    /**
     * ActionCommand for palette color 12 for boundaries
     */
    public static final String BOUND12 = "BOUND12";
    /**
     * ActionCommand for palette color 13 for boundaries
     */
    public static final String BOUND13 = "BOUND13";
    /**
     * ActionCommand for palette color 14 for boundaries
     */
    public static final String BOUND14 = "BOUND14";
    /**
     * ActionCommand for palette color 15 for boundaries
     */
    public static final String BOUND15 = "BOUND15";


}
