package devisor2.grid.GUI.dialogs;

import java.util.*;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

import devisor2.grid.options.*;
import devisor2.grid.GUI.framework.*;
import devisor2.foundation.elements.*;
import devisor2.foundation.*;


/**
 *  This class is the DockableDialog instance to set parallel blocks and 
 *  macro refinement parameters. <br>
 *  When it is displayed via the corresponding menu entry, the application
 *  switches to PARALLELBLOCK MODE, and every click on a macro element updates
 *  the displayed values in the dialog, which can be altered and in turn be
 *  written back to the macro element by clicking the APPLY button. <br>
 *  To avoid looking up all these values in the Element list every time, 
 *  they are stored in the ControlCenter instance. See there for details. <br>
 *
 *  @author Dominik Goeddeke
 *  @version beta
 *  
 */

public class ParallelBlockDialog extends DockableDialog implements ActionListener
{
    static Runtime RT = null;
    static Process PR = null;
  
    /**
     *  this method can be used to update the GUI of the dialog dynamically.
     *  it is called by the DialogManager e.g. when some comboboxes containing
     *  boundary information need to be refreshed
     */
    public void refreshGUI ()
    {
	// no need fort this in this class :-)
	return;
    }







 
    /** 
     *  returns the initial Position of the dialog
     */
    public Point getInitialPosition ()
    {
	return new Point (parent.getLocationOnScreen().x + parent.getWidth(), parent.getLocationOnScreen().y + (int)(parent.getHeight()/2));
    }

  
    /**
     *  the constructor sets references correctly and creates the GUI
     */
    public ParallelBlockDialog (MainFrame parent)
    {
	super (parent);
	setTitle ((String)cc.rb.getObject ("dialogs_parallelblock_title"));
	JPanel content = new JPanel ();
	// just ordinary GUI layouting
	GridBagLayout layout = new GridBagLayout ();
	GridBagConstraints constraints = new GridBagConstraints ();
	constraints.fill = GridBagConstraints.BOTH;
	constraints.weightx = constraints.weighty = 1.0;
	
	// parallel block edit
	JLabel PBLabel = new JLabel ((String)cc.rb.getObject ("dialogs_parallelblock_parallelblock"));
	pblockTF = new JTextField (10);
	
	// parallel block edit
	JLabel MBLabel = new JLabel ((String)cc.rb.getObject ("dialogs_parallelblock_matrixblock"));
	mblockTF = new JTextField (10);
	
	// macro refinement edit
	JLabel MRLabel = new JLabel ((String)cc.rb.getObject ("dialogs_parallelblock_macrorefinement"));
	refinementTF = new JTextField (10);
	// macro x anisotrophy
	JLabel xaniLabel = new JLabel ((String)cc.rb.getObject ("dialogs_parallelblock_anisotrophy_x"));
	ButtonGroup xanibg = new ButtonGroup ();
	x_none_rb = new JRadioButton ((String)cc.rb.getObject ("dialogs_parallelblock_anisotrophy_x_none"));
	xanibg.add (x_none_rb);
	x_left_rb = new JRadioButton ((String)cc.rb.getObject ("dialogs_parallelblock_anisotrophy_x_left"));
	xanibg.add (x_left_rb);
	x_right_rb = new JRadioButton ((String)cc.rb.getObject ("dialogs_parallelblock_anisotrophy_x_right"));
	xanibg.add (x_right_rb);
	// macro y anisotrophy
	JLabel yaniLabel = new JLabel ((String)cc.rb.getObject ("dialogs_parallelblock_anisotrophy_y"));
	ButtonGroup yanibg = new ButtonGroup ();
	y_none_rb = new JRadioButton ((String)cc.rb.getObject ("dialogs_parallelblock_anisotrophy_y_none"));
	yanibg.add (y_none_rb);
	y_down_rb = new JRadioButton ((String)cc.rb.getObject ("dialogs_parallelblock_anisotrophy_y_down"));
	yanibg.add (y_down_rb);
	y_up_rb = new JRadioButton ((String)cc.rb.getObject ("dialogs_parallelblock_anisotrophy_y_up"));
	yanibg.add (y_up_rb);
	// macro refinement factors
	JLabel factorLabel = new JLabel ((String)cc.rb.getObject ("dialogs_parallelblock_factors"));
	factor1TF = new JTextField (5);
	factor2TF = new JTextField (5);
	factor3TF = new JTextField (5);
	// the buttons
	JPanel buttons = new JPanel ();
	buttons.setLayout (new FlowLayout ());
	applyButton = new JButton ((String)cc.rb.getObject ("dialogs_parallelblock_apply"));
	applyButton.setToolTipText ((String)cc.rb.getObject ("dialogs_parallelblock_apply_tooltip"));
	applyButton.setMnemonic (((Integer)cc.rb.getObject ("dialogs_parallelblock_apply_mnemonic")).intValue ());
	applyButton.setActionCommand (APPLY);
	applyButton.addActionListener (this);
	buttons.add (applyButton);
	closeButton = new JButton ((String)cc.rb.getObject ("dialogs_parallelblock_close"));
	closeButton.setToolTipText ((String)cc.rb.getObject ("dialogs_parallelblock_close_tooltip"));
	closeButton.setMnemonic ( ((Integer)cc.rb.getObject("dialogs_parallelblock_close_mnemonic")).intValue());
	closeButton.setActionCommand (CLOSE);
	closeButton.addActionListener (this);
	buttons.add (closeButton);
	automaticButton = new JButton ((String)cc.rb.getObject ("dialogs_parallelblock_automatic"));
	automaticButton.setToolTipText ((String)cc.rb.getObject ("dialogs_parallelblock_automatic_tooltip"));
	automaticButton.setMnemonic ( ((Integer)cc.rb.getObject("dialogs_parallelblock_automatic_mnemonic")).intValue());
	automaticButton.setActionCommand (AUTOMATIC);
	automaticButton.addActionListener (this);
	buttons.add (automaticButton);
	// add them all to the dialog
	getRootPane().setDefaultButton (applyButton);
	content.setLayout (layout);
	content.setBorder (new EmptyBorder (10,10,10,10));
	constraints.gridy = 0;
	constraints.gridx = 0;
	layout.setConstraints(PBLabel, constraints);
	content.add (PBLabel);
	constraints.gridy = 0;
	constraints.gridx = 1;
	constraints.gridwidth = 3;
	layout.setConstraints(pblockTF, constraints);
	content.add (pblockTF);
	
	
	constraints.gridy = 1;
	constraints.gridx = 0;
	layout.setConstraints(MBLabel, constraints);
	content.add (MBLabel);
	constraints.gridy = 1;
	constraints.gridx = 1;
	constraints.gridwidth = 3;
	layout.setConstraints(mblockTF, constraints);
	content.add (mblockTF);
		
	constraints.gridwidth = 1;
	constraints.gridy = 2;
	constraints.gridx = 0;
	layout.setConstraints(MRLabel, constraints);
	content.add (MRLabel);
	constraints.gridy = 2;
	constraints.gridx = 1;
	constraints.gridwidth = 3;
	layout.setConstraints(refinementTF, constraints);
	content.add (refinementTF);
	constraints.gridwidth = 1;
	constraints.gridy = 3;
	constraints.gridx = 0;
	layout.setConstraints(xaniLabel, constraints);
	content.add (xaniLabel);
	constraints.gridx = 1;
	layout.setConstraints(x_none_rb, constraints);
	content.add (x_none_rb);
	constraints.gridx = 2;
	layout.setConstraints(x_left_rb, constraints);
	content.add (x_left_rb);
	constraints.gridx = 3;
	layout.setConstraints(x_right_rb, constraints);
	content.add (x_right_rb);
	constraints.gridy = 4;
	constraints.gridx = 0;
	layout.setConstraints(yaniLabel, constraints);
	content.add (yaniLabel);
	constraints.gridx = 1;
	layout.setConstraints(y_none_rb, constraints);
	content.add (y_none_rb);
	constraints.gridx = 2;
	layout.setConstraints(y_down_rb, constraints);
	content.add (y_down_rb);
	constraints.gridx = 3;
	layout.setConstraints(y_up_rb, constraints);
	content.add (y_up_rb);
	constraints.gridy = 5;
	constraints.gridx = 0;
	layout.setConstraints(factorLabel, constraints);
	content.add (factorLabel);
	constraints.gridx = 1;
	layout.setConstraints(factor1TF, constraints);
	content.add (factor1TF);
	constraints.gridx = 2;
	layout.setConstraints(factor2TF, constraints);
	content.add (factor2TF);
	constraints.gridx = 3;
	layout.setConstraints(factor3TF, constraints);
	content.add (factor3TF);
	// add buttons
	constraints.gridy = 6;
	constraints.gridx = 0;
	constraints.gridwidth = 4;
	layout.setConstraints (buttons, constraints);
	content.add (buttons);
	// completing the gui
	getContentPane().add (content);
	addComponentListener (listener);
	
	pblockTF.setText("1");
	mblockTF.setText("1");
        refinementTF.setText("1");
        factor1TF.setText("0.5");
        factor2TF.setText("0.5");
        factor3TF.setText("0.5");

        x_none_rb.setSelected(true);
        y_none_rb.setSelected(true);
	
	pack ();
    }

    public void setValues(int pb, int mb, int ref, int aiso, double[] anf)
    {
	pblockTF.setText(pb+"");
	mblockTF.setText(mb+"");
	refinementTF.setText(ref+"");
	factor1TF.setText(anf[0]+"");
	factor2TF.setText(anf[1]+"");
	factor3TF.setText(anf[2]+"");
	  
	if ((aiso & 1)==1)
	    x_left_rb.setSelected(true);
	else if ((aiso & 4)==4)
	    x_right_rb.setSelected(true);
	else
  	    x_none_rb.setSelected(true);
	if ((aiso & 2)==2)
	    y_down_rb.setSelected(true);
  	  else if ((aiso & 4)==4)
	    y_up_rb.setSelected(true);
	  else	  
	    y_none_rb.setSelected(true);
		  
	}
   
    
    private void automaticPar()
    {
	int maxpb = (new Integer(pblockTF.getText())).intValue();
	int ref = (new Integer(refinementTF.getText())).intValue();

	StringBuffer ges = new StringBuffer();
	
	int cco = cc.d.getCellCount();
	
	for(int index=0;index<cco;index++)
	{
	    Cell zell=cc.d.getCell(index);	    
	    zell.setParallelBlock(-1);
	    zell.setMatrixBlock(-1);
	}
	
	int aniso = 0;
	
	if ((maxpb>1) && (maxpb<cco))
	{
	    try
	    {
		FileOutputStream writeFile = new FileOutputStream("/tmp/metis.mesh");
		DataOutputStream out = new DataOutputStream( writeFile );
		
		out.writeBytes(cco+" 4\n");
		
		for(int index=0;index<cco;index++)
		{
		    Cell zell=cc.d.getCell(index);
		    
		    if ((zell instanceof Macro) || (zell instanceof Quad))
		    {
			out.writeBytes((((Cell2D)zell).getNode(0).getNumber()+1)+" "+
				       (((Cell2D)zell).getNode(3).getNumber()+1)+" "+
				       (((Cell2D)zell).getNode(2).getNumber()+1)+" "+
				       (((Cell2D)zell).getNode(1).getNumber()+1)+"\n");
		    }
		}
		
		out.close();
		
	    }
	    catch(IOException exep)
	    {
		System.err.println("IO-Error: "+exep);
	    }
	    
	    ges.append("CMD: "+cc.op.get (Options.metis_command)+" /tmp/metis.mesh "+maxpb+"\n");
	    
	    try
	    {
		RT=Runtime.getRuntime();
		PR=RT.exec(cc.op.get (Options.metis_command)+" /tmp/metis.mesh "+maxpb);
		
		if (PR != null)
		{
		    
		    BufferedReader IS = new BufferedReader(new InputStreamReader(PR.getInputStream()));
		    
		    String s;
		    
		    while ((s = IS.readLine()) != null)
			ges.append(s+"\n");
		} 
	    }
	    catch (IOException ee)
	    {
		System.out.println("Aufruffehler");
	    }
	    
	    String dateiname,buffer;
	    
	    try
	    {
		BufferedReader in;
		StreamTokenizer eingabe;
		
		in = new BufferedReader(new FileReader("/tmp/metis.mesh.epart."+maxpb));
		
		double[] testr = new double[16];
		int[] testi = new int[16];
		String[] tests = new String[16];
		
		int index=0;
		
		boolean[] binda = new boolean[maxpb];
		
		for (int i=0;i<maxpb;i++) binda[i]=false;
		
		while (in.ready())
		{
		    buffer=in.readLine();
		    cc.d.scanLine(buffer,"i",testr,testi,tests);
		    
		    binda[testi[0]]=true;
		    
		    cc.d.getCell(index).setParallelBlock(testi[0]+1);
		    cc.d.getCell(index).setMatrixBlock(testi[0]+1);
	cc.d.getCell(index).setMatrixBlock(cc.d.getCell(index).getNumber()+1);
		    
		    if ((cc.d.getCell(index) instanceof Macro) && (aniso==0))
		    {
			((Macro)cc.d.getCell(index)).setRefinement(0,ref);
			((Macro)cc.d.getCell(index)).setRefinement(1,ref);
			((Macro)cc.d.getCell(index)).setAnisotropy(0,0.0,0.0,0.0);
		    }
		    
		    index++;
		}
		
		in.close();
		
		int[] ifr = new int[maxpb];
		int[] ifrc = new int[maxpb];
		
		int ifridx = 0;
		
		for(int i=0;i<maxpb;i++)
		    if (binda[i])
		    {
			int fidx;
			
			for(index=0;index<cco;index++)
			{
			    Cell zell=cc.d.getCell(index);	    
			    int ipb = zell.getParallelBlock()-1;
			    
			    if (ipb==i) break;
			}
			
			fidx=index;
			
			ifrc[ifridx]=0;
			for(index=0;index<cco;index++)
			{
			    Cell zell=cc.d.getCell(index);	    
			    int ipb = zell.getParallelBlock()-1;
			    
			    if (ipb==i) ifrc[ifridx]=ifrc[ifridx]+1;
			}
			
			ifr[ifridx++]=fidx;
		    }
		
		/**	  System.out.println("Frr Blks");
			  for(index=0;index<ifridx;index++) System.out.print(ifr[index]+" ");
			  System.out.println(" ");
			  System.out.println("Frr Cnc");
			  for(index=0;index<ifridx;index++) System.out.print(ifrc[index]+" ");
			  System.out.println(" ");
		**/		  
		int ipool=0;
		
		for(int a=0; a<4;a++)
		{
		    for(int i=0;i<maxpb;i++)
		    {
			if (!binda[i])
			{
			    if (ifrc[ipool]<=1)
				while(ifrc[ipool]<=1) ipool++;
			    
			    if (ipool>=ifridx) break;
			    
			    Cell zell=cc.d.getCell(ifr[ipool++]);
			    
			    zell.setParallelBlock(i+1);
			    zell.setMatrixBlock(i+1);
			    zell.setMatrixBlock(zell.getNumber()+1);
			    binda[i]=true;
			    ifrc[ipool-1]--;
			    
			}
			
			if (ipool>=ifridx) break;
		    }
		    ipool=0;
		    ifridx=0;
		    
		    for(int i=0;i<maxpb;i++)
			if (binda[i])
			{
			    int fidx;
			    
			    for(index=0;index<cco;index++)
			    {
				Cell zell=cc.d.getCell(index);	    
				int ipb = zell.getParallelBlock()-1;
				
				if (ipb==i) break;
			    }
			    
			    fidx=index;
			    
			    ifrc[ifridx]=0;
			    for(index=0;index<cco;index++)
			    {
				Cell zell=cc.d.getCell(index);	    
				int ipb = zell.getParallelBlock()-1;
				
				if (ipb==i) ifrc[ifridx]=ifrc[ifridx]+1;
			    }
			    
			    ifr[ifridx++]=fidx;
			}	    
		}
		
		for(int i=0;i<maxpb;i++)
		    if (!binda[i]) 
			ges.append("WARNING!   Parallelblock "+
				   (i+1)+" not set!\n");
		
		for(index=0;index<cco;index++)
		{
		    Cell zell=cc.d.getCell(index);	    
		    int ipb = zell.getParallelBlock();
		    
		    if (ipb==-1)
			ges.append("WARNING!   Macro "+
				   (index+1)+" not assigned to any parallel block!\n");		 
		}		
		
	    }
	    catch(IOException exep2)
	    {
		System.out.println("Fehler auffetreten");
	    }
        }
        else
	{
	    for(int index=0;index<cco;index++)
	    {
		Cell zell=cc.d.getCell(index);
		
		Node n1 = ((Cell2D)zell).getNode(0);
		Node n2 = ((Cell2D)zell).getNode(1);
		Node n3 = ((Cell2D)zell).getNode(2);
		Node n4 = ((Cell2D)zell).getNode(3);
		
		double x1 = n1.getCoord(0); double y1 = n1.getCoord(1);
		double x2 = n2.getCoord(0); double y2 = n2.getCoord(1);
		double x3 = n3.getCoord(0); double y3 = n3.getCoord(1);
		double x4 = n4.getCoord(0); double y4 = n4.getCoord(1);
		
		double len1 = Math.sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2));		
		double len2 = Math.sqrt((x3-x2)*(x3-x2)+(y3-y2)*(y3-y2));
		
		if (((len1<len2) && (y2>y1)) || (len1==len2))
		    ges.append("AR coefficient macro "+(index+1)+" : + "+len1+"/"+len2+"\n");
		else		
		    if (len1>len2)
			ges.append("AR coefficient macro "+(index+1)+" : - "+len1+"/"+len2+" OneTurn\n");	  		
		    else
			ges.append("AR coefficient macro "+(index+1)+" : - "+len1+"/"+len2+" TwoTurn\n");
		
		if ((zell instanceof Macro) || (zell instanceof Quad))
		{
		    if (maxpb==1)
		    {
			cc.d.getCell(index).setParallelBlock(1);
			cc.d.getCell(index).setMatrixBlock(1);
		    }
		    else
		    {
			cc.d.getCell(index).setParallelBlock(index+1);
			cc.d.getCell(index).setMatrixBlock(index+1);
		    }
		    
		    if (aniso==0)
		    {
			((Macro)cc.d.getCell(index)).setRefinement(0,ref);
			((Macro)cc.d.getCell(index)).setRefinement(1,ref);
			((Macro)cc.d.getCell(index)).setAnisotropy(0,0.0,0.0,0.0);
		    }
		}
	    }
	    
        }
	
        TextFrame.initialize (cc.gc.mf, "AutomaticParallelization", "waiting for godot...");
	TextFrame.setText (ges.toString());
	TextFrame.showDialog ();      
    }
     
    /**
     *  This class handles its own action event. The actionPerformed method 
     *  just updates the ControlCenter's EdgeStatusInfo 
     */
    public void actionPerformed (ActionEvent e)
    {

	if (e.getActionCommand().equals (AUTOMATIC))
	{
	    automaticPar();
    	    parent.getDrawingArea().repaint();

	    return;
	}
	if (e.getActionCommand().equals (CLOSE))
	{
	    parent.getToolBar().getParallelblockToggle().doClick ();
	    return;
	}
	if (e.getActionCommand().equals (APPLY))
	{
	   Vector selectedItems = parent.getDrawingArea().getListener().getSelectedItems();
	 
	    Enumeration enum = selectedItems.elements();
	    
	    int aiso=0;
	    if (x_left_rb.isSelected()) aiso+=1;
	    if (x_right_rb.isSelected()) aiso+=5;
	    if (y_down_rb.isSelected()) aiso+=2;
	    if (y_up_rb.isSelected()) aiso+=10;
	    
	    int pb = (new Integer(pblockTF.getText())).intValue();
	    int ref = (new Integer(refinementTF.getText())).intValue();
	    int mb = (new Integer(mblockTF.getText())).intValue();
	   
	    double[] af = new double[3];
	    af[0] = (new Double(factor1TF.getText())).doubleValue();
	    af[1] = (new Double(factor2TF.getText())).doubleValue();
	    af[2] = (new Double(factor3TF.getText())).doubleValue();
	     
	    while (enum.hasMoreElements())
	    {
	      GridItem c = (GridItem)enum.nextElement ();
	      
	      if (c instanceof Macro)
	      {
	         Macro m = (Macro)c;
		 
                 m.setMatrixBlock(mb);

                 m.setParallelBlock(pb);
                 m.setRefinement(0,ref);
                 m.setRefinement(1,ref);
  
                 m.setAnisotropy(aiso,af[0],af[1],af[2]);
  		 

	      }
	    }
	    parent.getDrawingArea().repaint();
	    return;
	}
    }

    // keys for the action commands
    private static final String APPLY = "APPLY";
    private static final String CLOSE = "CLOSE";
    private static final String AUTOMATIC = "AUTOMATIC";

    // buttons
 
    private JButton applyButton;
    private JButton closeButton;
    private JButton automaticButton;
    private JTextField pblockTF;
    private JTextField mblockTF;
    private JTextField refinementTF;
    private JRadioButton x_none_rb;
    private JRadioButton x_left_rb;
    private JRadioButton x_right_rb;
    private JRadioButton y_none_rb;
    private JRadioButton y_up_rb;
    private JRadioButton y_down_rb;
    private JTextField factor1TF;
    private JTextField factor2TF;
    private JTextField factor3TF;

   

}
