package devisor2.grid.GUI.dialogs;

import java.util.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

import devisor2.grid.options.*;
import devisor2.grid.GUI.framework.*;
import devisor2.foundation.elements.*;
import devisor2.foundation.boundary.*;


/**
 *  This class is the DockableDialog instance for domain perimeter input. 
 *
 *  @author Dominik Goeddeke
 *  @version 0.9 (release candidate)
 *  
 */

public class PerimeterDialog extends DockableDialog implements ActionListener
{


    /**
     *  this method can be used to update the GUI of the dialog dynamically.
     *  it is called by the DialogManager e.g. when some comboboxes containing
     *  boundary information need to be refreshed
     */
    public void refreshGUI ()
    {
	// no need fort this in this class :-)
	return;
    }

    /** 
     *  returns the initial Position of the dialog
     */
    public Point getInitialPosition ()
    {
	return new Point (parent.getLocationOnScreen().x - 50, parent.getLocationOnScreen().y);
    }



    /**
     *  the constructor just sets references correctly and creates the GUI
     */
    public PerimeterDialog (MainFrame parent)
    {
	super (parent);
	setModal (true);
	JPanel content = new JPanel ();
	content.setBorder (new EmptyBorder (10,10,10,10));
	// just ordinary GUI layouting
	GridBagLayout layout = new GridBagLayout ();
	GridBagConstraints constraints = new GridBagConstraints ();
	content.setLayout (layout);
	setTitle((String)cc.rb.getObject ("dialogs_perimeter_title"));
	// x1
	JLabel x1Label = new JLabel ((String)cc.rb.getObject ("dialogs_perimeter_x1"));
	constraints.gridy = 0;
	constraints.gridx = 0;
	layout.setConstraints (x1Label, constraints);
	content.add (x1Label);
	x1TF = new JTextField (6);
	x1TF.setText (cc.gc.fromWorldToUser(cc.dc.world_x1)+"");
	constraints.gridx = 1;
	layout.setConstraints (x1TF, constraints);
	content.add (x1TF);
	// y1
	y1TF = new JTextField (6);
	y1TF.setText (cc.gc.fromWorldToUser(cc.dc.world_y1)+"");
	constraints.gridx = 2;
	layout.setConstraints (y1TF, constraints);
	content.add (y1TF);
	// x2
	JLabel x2Label = new JLabel ((String)cc.rb.getObject ("dialogs_perimeter_x2"));
	constraints.gridy = 1;
	constraints.gridx = 0;
	layout.setConstraints (x2Label, constraints);
	content.add (x2Label);
	x2TF = new JTextField (6);
	x2TF.setText (cc.gc.fromWorldToUser(cc.dc.world_x2)+"");
	constraints.gridx = 1;
	layout.setConstraints (x2TF, constraints);
	content.add (x2TF);
	// y2
	y2TF = new JTextField (6);
	y2TF.setText (cc.gc.fromWorldToUser(cc.dc.world_y2)+"");
	constraints.gridx = 2;
	layout.setConstraints (y2TF, constraints);
	content.add (y2TF);
	// buttons
	JPanel buttons = new JPanel ();
	buttons.setLayout (new FlowLayout ());
	okButton = new JButton ((String)cc.rb.getObject ("general_Ok"));
	okButton.setMnemonic ( ((Integer)cc.rb.getObject("general_Ok_mnemonic")).intValue());
	okButton.setActionCommand (OK);
	okButton.addActionListener (this);
	buttons.add (okButton);
	cancelButton = new JButton ((String)cc.rb.getObject ("general_Cancel"));
	cancelButton.setMnemonic ( ((Integer)cc.rb.getObject("general_Cancel_mnemonic")).intValue());
	cancelButton.setActionCommand (CANCEL);
	cancelButton.addActionListener (this);
	buttons.add (cancelButton);

	// add them all to the dialog
	getRootPane().setDefaultButton (okButton);
	constraints.gridy = 3;
	constraints.gridx = 0;
	constraints.gridwidth = 3;
	layout.setConstraints(buttons, constraints);
	content.add (buttons);
	// finish the GUI
	getContentPane ().add (content);
	addComponentListener (listener);
	pack ();
    }


    /**
     *  This class handles its own action event. The actionPerformed method 
     *  just updates the ControlCenter's EdgeStatusInfo 
     */
    public void actionPerformed (ActionEvent e)
    {
	if (e.getActionCommand().equals (OK))
	{
	    if (verify ())
	    {
		// verify parses the textfields and stores the values in local
		// doubles x1,y1,x2,y2
		// something like cc.d.setperimeter (x1,y1,x2,y2);
		
		cc.dc.world_x1=cc.gc.fromUserToWorld(x1);
		cc.dc.world_x2=cc.gc.fromUserToWorld(x2);
		cc.dc.world_y1=cc.gc.fromUserToWorld(y1);
		cc.dc.world_y2=cc.gc.fromUserToWorld(y2);
		
		cc.dc.act_x1=cc.dc.world_x1;
		cc.dc.act_x2=cc.dc.world_x2;
		cc.dc.act_y1=cc.dc.world_y1;
		cc.dc.act_y2=cc.dc.world_y2;
		
		cc.d.centerDomain();

		cc.dm.hide (cc.dm.PERIMETERDIALOG);
	    }
	    return;
	}
	if (e.getActionCommand().equals (CANCEL))
	{
	    cc.dm.hide (cc.dm.PERIMETERDIALOG);
	    return;
	}
	
    }

    /**
     *  the boring verification code is outsourced into this method
     */
    private boolean verify ()
    {
	boolean allOK = true;
	// the four text fields contain valid doubles
	try
	{
	    x1 = Double.parseDouble (x1TF.getText ());
	}
	catch (NumberFormatException ex)
	{
	    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject ("dialogs_perimeter_error_x1"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
		allOK = false;
	}
	try
	{
	    y1 = Double.parseDouble (y1TF.getText ());
	}
	catch (NumberFormatException ex)
	{
	    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject ("dialogs_perimeter_error_y1"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
		allOK = false;
	}
	try
	{
	    x2 = Double.parseDouble (x2TF.getText ());
	}
	catch (NumberFormatException ex)
	{
	    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject ("dialogs_perimeter_error_x2"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
		allOK = false;
	}
	try
	{
	    y2 = Double.parseDouble (y2TF.getText ());
	}
	catch (NumberFormatException ex)
	{
	    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject ("dialogs_perimeter_error_y2"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
		allOK = false;
	}
	return allOK;
    }



    // some storage
    private double x1,y1,x2,y2;

    // keys for the action commands

    private static final String OK = "OK";;
    private static final String CANCEL = "CANCEL";
  


    // GUI items
    private JButton okButton;
    private JButton cancelButton;
    private JTextField x1TF;
    private JTextField y1TF;
    private JTextField x2TF;
    private JTextField y2TF;

    
  
}
