package devisor2.grid.GUI.dialogs;

import java.util.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

import devisor2.grid.options.*;
import devisor2.grid.GUI.framework.*;
import devisor2.foundation.elements.*;


/**
 *  This class is the DockableDialog instance for Quad input. It is used both
 *  for direct input and for Property Queries.<br>
 *  The user can directly fill in four nodes (by their number) which form 
 *  the quad. <br>
 *  See the GridListener for a detailed description of how the dialog
 *  is connected to the application.
 *
 *  @author Dominik Goeddeke
 *  @version 0.5 (GUI, no real functionality)
 *  
 */

public class QuadDialog extends DockableDialog implements ActionListener
{

    // the Quad element acting as source for updating
    private Quad source;


    /**
     *  this method can be used to update the GUI of the dialog dynamically.
     *  it is called by the DialogManager e.g. when some comboboxes containing
     *  boundary information need to be refreshed
     */
    public void refreshGUI ()
    {
	// no need fort this in this class :-)
	return;
    }


    /** 
     *  returns the initial Position of the dialog
     */
    public Point getInitialPosition ()
    {
	return new Point (parent.getLocationOnScreen().x - 50, parent.getLocationOnScreen().y);
    }


    /**
     *  sets the given int as text for the Node1TF
     */
    public void setNode1 (int index)
    {
	node1TF.setText (Integer.toString (index));
    }
    
    /**
     *  sets the given int as text for the Node2TF
     */
    public void setNode2 (int index)
    {
	node2TF.setText (Integer.toString (index));
    }

    /**
     *  sets the given int as text for the Node3TF
     */
    public void setNode3 (int index)
    {
	node3TF.setText (Integer.toString (index));
    }

    /**
     *  sets the given int as text for the Node4TF
     */
    public void setNode4 (int index)
    {
	node4TF.setText (Integer.toString (index));
    }

    /**
     *  sets the source element (the Quad element the dialog works on.
     *  Note that if source is set explicitly to null, the dialog
     *  works for directinput, if a Quad element of the domain is referenced,
     *  the dialog works in property change mode
     *
     *  @param source - the source quad element for property mode
     */
    public void setSource (Quad source)
    {
	this.source = source;
	if (source != null)
	{
	    Node[] nodes = source.updateCellNodes ();
	    setNode1 (nodes[0].getNumber()+1);
	    setNode2 (nodes[1].getNumber()+1);
	    setNode3 (nodes[2].getNumber()+1);
	    setNode4 (nodes[3].getNumber()+1);
	}
    }

  

    /**
     *  sets the mode of the dialog: for coderecycling purposes, this
     *  dialog works on both QUADS and MACROS.
     *
     *  @param mode - QUADMODE or MACROMODE
     */
    public void setMode (boolean mode)
    {
	mode = mode;
    }



    /**
     *  the constructor just sets references correctly and creates the GUI
     */
    public QuadDialog (MainFrame parent)
    {
	super (parent);
	JPanel content = new JPanel ();
	content.setBorder (new EmptyBorder (10,10,10,10));
	// just ordinary GUI layouting
	GridBagLayout layout = new GridBagLayout ();
	GridBagConstraints constraints = new GridBagConstraints ();
	content.setLayout (layout);
	constraints.fill = GridBagConstraints.BOTH;
	constraints.weightx = constraints.weighty = 1.0;
	// first node
	JLabel node1Label = new JLabel ((String)cc.rb.getObject ("dialogs_quad_node1"));
	constraints.gridy = 0;
	constraints.gridx = 0;
	layout.setConstraints (node1Label, constraints);
	content.add (node1Label);
	node1TF = new JTextField (6);
	constraints.gridx = 1;
	layout.setConstraints (node1TF, constraints);
	content.add (node1TF);
	// second node
	JLabel node2Label = new JLabel ((String)cc.rb.getObject ("dialogs_quad_node2"));
	constraints.gridx = 2;
	layout.setConstraints (node2Label, constraints);
	content.add (node2Label);
	node2TF = new JTextField (6);
	constraints.gridx = 3;
	layout.setConstraints (node2TF, constraints);
	content.add (node2TF);
	// third node
	JLabel node3Label = new JLabel ((String)cc.rb.getObject ("dialogs_quad_node3"));
	constraints.gridy = 1;
	constraints.gridx = 0;
	layout.setConstraints (node3Label, constraints);
	content.add (node3Label);
	node3TF = new JTextField (6);
	constraints.gridx = 1;
	layout.setConstraints (node3TF, constraints);
	content.add (node3TF);
	// forth node
	JLabel node4Label = new JLabel ((String)cc.rb.getObject ("dialogs_quad_node4"));
	constraints.gridx = 2;
	layout.setConstraints (node4Label, constraints);
	content.add (node4Label);
	node4TF = new JTextField (6);
	constraints.gridx = 3;
	layout.setConstraints (node4TF, constraints);
	content.add (node4TF);

	// buttons
	JPanel buttons = new JPanel ();
	buttons.setLayout (new FlowLayout ());
	okButton = new JButton ((String)cc.rb.getObject ("general_Ok"));
	okButton.setMnemonic ( ((Integer)cc.rb.getObject("general_Ok_mnemonic")).intValue());
	okButton.setActionCommand (OK);
	okButton.addActionListener (this);
	buttons.add (okButton);
	applyButton = new JButton ((String)cc.rb.getObject ("general_Apply"));
	applyButton.setMnemonic ( ((Integer)cc.rb.getObject("general_Apply_mnemonic")).intValue());
	applyButton.setActionCommand (APPLY);
	applyButton.addActionListener (this);
	buttons.add (applyButton);
	cancelButton = new JButton ((String)cc.rb.getObject ("general_Cancel"));
	cancelButton.setMnemonic ( ((Integer)cc.rb.getObject("general_Cancel_mnemonic")).intValue());
	cancelButton.setActionCommand (CANCEL);
	cancelButton.addActionListener (this);
	buttons.add (cancelButton);
	// add them all to the dialog
	getRootPane().setDefaultButton (applyButton);
	constraints.gridy = 2;
	constraints.gridx = 0;
	constraints.gridwidth = 4;
	layout.setConstraints(buttons, constraints);
	content.add (buttons);
	// finish the GUI
	getContentPane ().add (content);
	addComponentListener (listener);
	pack ();
    }


    /**
     *  This class handles its own action event. 
     */
    public void actionPerformed (ActionEvent e)
    {
	if (e.getActionCommand().equals (OK))
	{
	    if (verify ())
	    {
		if (source != null)
			performUpdateQuad ();
		    else
			performAddQuad ();
		cc.dm.hide (cc.dm.QUADDIALOG);
	    }
	    return;
	}
	if (e.getActionCommand().equals (APPLY))
	{
	    if (verify ())
	    {
		if (source != null)
		    performUpdateQuad ();
		else
		    performAddQuad ();
	    }
	    return;
	}
	if (e.getActionCommand().equals (CANCEL))
	{
	    cc.dm.hide (cc.dm.QUADDIALOG);
	    return;
	}
    }

    /**
     *  the verifivation code is all put into this method
     */
    private boolean verify ()
    {
	boolean allOK = true;
	try
	{
	    node1 = Integer.parseInt (node1TF.getText());
	    node1--;
	    n1 = cc.d.getNode(node1);
	}
	catch (Exception ex)
	{
	    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject("dialogs_quad_error_node1"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
	    allOK = false;
	}
	try
	{
	    node2 = Integer.parseInt (node2TF.getText());
	    node2--;
	    n2 = cc.d.getNode(node2);
	}
	catch (Exception ex)
	{
	    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject("dialogs_quad_error_node2") , (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
	    allOK = false;
	}
	try
	{
	    node3 = Integer.parseInt (node3TF.getText());
	    node3--;
	    n3 = cc.d.getNode(node3);
	}
	catch (Exception ex)
	{
	    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject("dialogs_quad_error_node3") , (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
	    allOK = false;
	}
	try
	{
	    node4 = Integer.parseInt (node4TF.getText());
	    node4--;
	    n4 = cc.d.getNode(node4);
	}
	catch (Exception ex)
	{
	    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject("dialogs_quad_error_node4") , (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
	    allOK = false;
	}
	// check for node orientation
	if (allOK)
	{
	    // are all nodes different?
	    if (n1 == n2 || n1 == n3 || n1 == n4 ||
		n2 == n3 || n3 == n4 || n2 == n4)
	    {
		JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject("dialogs_tri_error_samenodes"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
		allOK = false;
	    }
	    // check for node orientation
	    Node[] nodes = {n1,n2,n3,n4};
	    double vek1 = cc.d.db.crossProduct (3,0,1,nodes);
	    double vek2 = cc.d.db.crossProduct (1,2,3,nodes);
	    if ((vek1>=0.0) || (vek2>=0.0))
	    {
		JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject("dialogs_tri_error_nodeorientation"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
		allOK = false;
	    }
	}
	return allOK;
    }

    /**
     *  this method adds a new Quad object to the domain with the four nodes
     *  set in the textfields
     */
    private void performAddQuad ()
    {
	DrawingArea da = parent.getDrawingArea ();
	// create and add edges
	Edge e1, e2, e3,e4;
	e1 = cc.d.adjustEdge (n1,n2);
	e2 = cc.d.adjustEdge (n2,n3);
	e3 = cc.d.adjustEdge (n3,n4);
	e4 = cc.d.adjustEdge (n4,n1);
	cc.d.updateEdgeNumbers ();
	// create and add quad
	Quad quad = new Quad ();
	quad.setCell (e1,e2,e3,e4);
	cc.d.addCell (quad);
	Vector tmpvec = new Vector ();
	tmpvec.add (e1);
	tmpvec.add (e2);
	tmpvec.add (e3);
	tmpvec.add (e4);
	tmpvec.add (quad);
	cc.um.notifyUndo (cc.um.UNDO_ADDQUAD, tmpvec, null);
	cc.d.updateCellNumbers ();
	da.repaint ();
    }

    /**
     *  updates the quad element referenced by the source variable with
     *  the parameter values from the dialog textfields
     */
    private void performUpdateQuad ()
    {
	DrawingArea da = parent.getDrawingArea ();
	// kill old quad
	cc.d.delCell (source);
	// create & add edges
	Edge e1, e2, e3,e4;
	e1 = cc.d.adjustEdge (n1,n2);
	e2 = cc.d.adjustEdge (n2,n3);
	e3 = cc.d.adjustEdge (n3,n4);
	e4 = cc.d.adjustEdge (n4,n1);
	cc.d.updateEdgeNumbers ();
	// create and add quad 
	Quad quad = new Quad ();
	quad.setCell (e1,e2,e3,e4);
	cc.d.addCell (quad);
	cc.d.updateCellNumbers ();
	Vector tmpvec = new Vector ();
	tmpvec.add (e1);
	tmpvec.add (e2);
	tmpvec.add (e3);
	tmpvec.add (e4);
	tmpvec.add (quad);
	cc.um.notifyUndo (cc.um.UNDO_ADDQUAD, tmpvec, null);
	source = quad;
	da.repaint ();
    }

    // the integers corresponding to the textfields' string entries
    private int node1, node2, node3, node4;
    // and the corresponding objects
    private Node n1,n2,n3,n4;

    private static final String OK = "OK";
    private static final String CANCEL = "CANCEL";
    private static final String APPLY = "APPLY";


    // buttons
    private JButton okButton;
    private JButton cancelButton;
    private JButton applyButton;
    // textfields
    private JTextField node1TF, node2TF, node3TF, node4TF;

    
    
}
