package devisor2.grid.GUI.dialogs;

import java.util.*;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

import devisor2.grid.options.*;
import devisor2.grid.GUI.framework.*;
import devisor2.foundation.elements.*;
import devisor2.foundation.*;
import devisor2.foundation.base.*;

/**
 *  This class is the DockableDialog instance to set the refinement 
 *  parameters for the external call of the TRIGEN2D tool. <br>
 *  When it is displayed via the corresponding menu entry, the application
 *  switches to REFINE MODE, and a click on the REFINE button performs the
 *  TRIGEN2D tool on all elements in the current selection. <br>
 *  <br>
 *  Take a look at our technical paper about dialogs if you want to learn
 *  more, to be found in the /manual/specs directory of your installation.
 *
 *  @author Dominik Goeddeke
 *  @version 1.0 fully tested
 *  
 */

public class RefineDialog extends DockableDialog implements ActionListener
{

    static Runtime RT = null;
    static Process PR = null;


    /**
     *  this method can be used to update the GUI of the dialog dynamically.
     *  it is called by the DialogManager e.g. when some comboboxes containing
     *  boundary information need to be refreshed
     */
    public void refreshGUI ()
    {
	// no need fort this in this class :-)
	return;
    }


    /** 
     *  returns the initial Position of the dialog
     */
    public Point getInitialPosition ()
    {
	return new Point (parent.getLocationOnScreen().x + parent.getWidth(), parent.getLocationOnScreen().y + parent.getHeight());
    }


    /**
     *  the constructor just sets references correctly and creates the GUI
     */
    public RefineDialog (MainFrame parent)
    {
	super (parent);
	setTitle ((String)cc.rb.getObject ("dialogs_refine_title"));
	JPanel content = new JPanel ();
	// the button panel
	JPanel buttons = new JPanel ();
	buttons.setLayout (new FlowLayout ());
	// the apply button
	applyButton = new JButton ((String)cc.rb.getObject ("dialogs_refine_apply"));
	applyButton.setToolTipText ((String)cc.rb.getObject ("dialogs_refine_apply_tooltip"));
	applyButton.setMnemonic (((Integer)cc.rb.getObject("dialogs_refine_apply_mnemonic")).intValue());
	applyButton.setActionCommand (APPLY);
	applyButton.addActionListener (this);
	buttons.add (applyButton);
	// the close button
	closeButton = new JButton ((String)cc.rb.getObject ("dialogs_refine_close"));
	closeButton.setToolTipText ((String)cc.rb.getObject ("dialogs_refine_close_tooltip"));
	closeButton.setMnemonic (((Integer)cc.rb.getObject("dialogs_refine_close_mnemonic")).intValue());
	closeButton.setActionCommand (CLOSE);
	closeButton.addActionListener (this);
	buttons.add (closeButton);
	// the radiobuttons
	JPanel radio = new JPanel ();
	radio.setLayout (new BoxLayout (radio, BoxLayout.Y_AXIS));
	radio.setBorder (BorderFactory.createTitledBorder ((String)cc.rb.getObject ("dialogs_refine_type")));
	ButtonGroup bg = new ButtonGroup ();
	// no button
	noButton = new JRadioButton ((String)cc.rb.getObject ("dialogs_refine_no"));
	bg.add (noButton);
	radio.add (noButton);
	// 3x3 DELMA button
	delma33Button = new JRadioButton ((String)cc.rb.getObject ("dialogs_refine_delma33"));
	bg.add (delma33Button);
	radio.add (delma33Button);
	// BOUNDARY3 button
	boundary3Button = new JRadioButton ((String)cc.rb.getObject ("dialogs_refine_boundary3"));
	bg.add (boundary3Button);
	radio.add (boundary3Button);
	// 2 BOUNDARY button
	boundary2Button = new JRadioButton ((String)cc.rb.getObject ("dialogs_refine_boundary2"));
	bg.add (boundary2Button);
	radio.add (boundary2Button);
	// BOUNDARY NORMAL button
	boundarynormalButton = new JRadioButton ((String)cc.rb.getObject ("dialogs_refine_boundarynormal"));
	bg.add (boundarynormalButton);
	radio.add (boundarynormalButton);
	// DELMA 5 button
	delma5Button = new JRadioButton ((String)cc.rb.getObject ("dialogs_refine_delma5"));
	bg.add (delma5Button);
	radio.add (delma5Button);
	// CORNER button
	cornerButton = new JRadioButton ((String)cc.rb.getObject ("dialogs_refine_corner"));
	bg.add (cornerButton);
	radio.add (cornerButton);
	//the parameters panel
	JPanel param = new JPanel ();
	GridBagLayout gbl= new GridBagLayout ();
	param.setLayout (gbl);
	GridBagConstraints gbc = new GridBagConstraints ();
	//gbc.fill = GridBagConstraints.BOTH;
	//gbc.weightx = gbc.weighty = 1.0;
	param.setBorder (BorderFactory.createTitledBorder ((String)cc.rb.getObject ("dialogs_refine_parameters")));
	JLabel levelLabel = new JLabel ((String)cc.rb.getObject ("dialogs_refine_level"));
	gbc.gridx = gbc.gridy = 0;
	gbl.setConstraints (levelLabel, gbc);
	param.add (levelLabel);
	levelTF = new JTextField (10);
	gbc.gridx = 1;
	gbl.setConstraints (levelTF, gbc);
	param.add (levelTF);
	JLabel param1Label = new JLabel ((String)cc.rb.getObject ("dialogs_refine_parameter1"));
	gbc.gridy = 1;
	gbc.gridx = 0;
	gbl.setConstraints (param1Label, gbc);
	param.add (param1Label);
	param1TF = new JTextField (10);
	gbc.gridx = 1;
	gbl.setConstraints (param1TF, gbc);
	param.add (param1TF);
	JLabel param2Label = new JLabel ((String)cc.rb.getObject ("dialogs_refine_parameter2"));
	gbc.gridy = 2;
	gbc.gridx = 0;
	gbl.setConstraints (param2Label, gbc);
	param.add (param2Label);
	param2TF = new JTextField (10);
	gbc.gridx = 1;
	gbl.setConstraints (param2TF, gbc);
	param.add (param2TF);
	// add all to the dialog
	GridBagLayout layout = new GridBagLayout ();
	GridBagConstraints constraints = new GridBagConstraints ();
	constraints.fill = GridBagConstraints.BOTH;
	constraints.weightx = constraints.weighty = 1.0;
	content.setLayout (layout);
	content.setBorder (new EmptyBorder (10,10,10,10));
	// type panel
	constraints.gridy = 0;
	layout.setConstraints(radio, constraints);
	content.add (radio);
	// parameters panel
	constraints.gridy = 1;
	layout.setConstraints(param, constraints);
	content.add (param);
	// button panel
	constraints.gridy = 2;
	layout.setConstraints(buttons, constraints);
	content.add (buttons);
	// finish the GUI
	getContentPane().add (content);
	addComponentListener (listener);
	pack ();
	getRootPane().setDefaultButton (applyButton);
	// set default values
	noButton.setSelected (true);
	levelTF.setText ("3");
	param1TF.setText ("0.3333333333");
	param2TF.setText ("0.6666666666");
	pack ();

    }

    /**
     *  teh actual refinement code
     */
    private void refine()
    {
	StringBuffer ges = new StringBuffer();
	Vector selectedItems = parent.getDrawingArea().getListener().getSelectedItems();
	int reflevel = (new Integer(levelTF.getText())).intValue();
	String trigenpath = cc.op.get (Options.general_trigen);
	int modus = 0;
	int cco;
	if (delma33Button.isSelected()) modus=1;
	if (boundary3Button.isSelected()) modus=2;
	if (boundary2Button.isSelected()) modus=3;
	if (boundarynormalButton.isSelected()) modus=4;
	if (delma5Button.isSelected()) modus=5;
	if (cornerButton.isSelected()) modus=6;
	double delma = (new Double(param1TF.getText())).doubleValue();
	double delmb = (new Double(param2TF.getText())).doubleValue();
	try
	{
	    FileOutputStream writeFile = new FileOutputStream(trigenpath+"/#data/trigen2d.dat");                
	    DataOutputStream out = new DataOutputStream( writeFile );
	    out.writeBytes ("===========================================================\n");
	    out.writeBytes ("input data file for trigen2d (generated by DeViSoR2)\n");
	    out.writeBytes ("===========================================================\n");
	    out.writeBytes ("2                     M\n"); 
	    out.writeBytes ("1                     MT\n"); 
	    out.writeBytes ("1                     ICHECK\n");
	    out.writeBytes ("1                     IMESH (0=FEAT-parametrization,1=OMEGA)\n"); 
	    out.writeBytes ("'#adc/devgridref.prm' CPARM  (name of parametrization file)\n");
	    out.writeBytes ("0                     IBDCHK (>0 check with #points=IBDCHK)\n");
	    out.writeBytes ("0                     IBYU   (level for BYU-output)\n");
	    out.writeBytes ("0                     IAVS   (level for AVS-output)\n");
	    out.writeBytes ((reflevel)+"                     NLEV (refined levels-1)\n"); 
	    out.writeBytes ((reflevel)+"                    IFMT   (level of output)\n");
	    out.writeBytes ("'#adc/devgridref.tri'       CFILEI (name of input file)\n"); 
	    out.writeBytes ("'#adc/devgridrefout.tri'       CFILEO (name of output  file)\n");
	    out.writeBytes (modus+"                     ITYPEL (0=no,>0=type of modification\n");
	    Enumeration enum = selectedItems.elements();
	    cco=0;
	    if (!enum.hasMoreElements())
		cco = 0;
	    else
	    {
		while (enum.hasMoreElements())
		{
		    GridItem c = (GridItem)enum.nextElement ();
		    
		    if (c instanceof Cell) cco++;
		}
	    }
	    out.writeBytes (cco+"                    NELMOD (number of modifoed elements)\n");
	    out.writeBytes (delma+"                DELMA  (distance od first line)\n");
	    out.writeBytes (delmb+"                DELMB  (distance od second line)\n"); 
	    enum = selectedItems.elements();
	    if (enum.hasMoreElements())
	    {
		while (enum.hasMoreElements())
		{
		    GridItem c = (GridItem)enum.nextElement ();
		    if (c instanceof Cell) 
			out.writeBytes(""+(((Cell)c).getNumber()+1)+"\n");
		}
	    }
	    out.close();
	}
	catch(IOException exep)
	{
	    System.err.println("IO-Error: "+exep);
	}
	cc.d.saveDomainFEAT(trigenpath+"/#adc/devgridref");
	ges.append("Start TRIGEN2D ...\n");
	try
	{
	    RT=Runtime.getRuntime();
	    PR=RT.exec("sh "+trigenpath+"/trigen_wrapper "+trigenpath);
	    ges.append("TRIGEN2D messages:\n");
	    ges.append("===========================================\n");
	    if (PR != null)
	    {
		BufferedReader IS = new BufferedReader(new InputStreamReader(PR.getInputStream()));
		String s;
		while ((s = IS.readLine()) != null)
		    ges.append(s+"\n");
	    } 
	}
	catch (IOException ee)
	{
	    System.out.println("Aufruffehler");
	}
	ges.append("TRIGEN2D finished!\n");
	ges.append("==============================================\n");
	cc.d = new Domain ();
	cc.gc.gtk.updateBoundaryCombos();
	cc.d.loadDomainFEAT(trigenpath+"/#adc/devgridrefout",true,true,false,0.0,0.0,false);
	//   cc.d.centerDomain();
	parent.getDrawingArea().repaint();
	TextFrame.initialize (cc.gc.mf, "Refine", "waiting for godot...");
	TextFrame.setText (ges.toString());
	TextFrame.showDialog ();      
    }
    
    /**
     */
    public void actionPerformed (ActionEvent e)
    {
	if (e.getActionCommand().equals (APPLY))
	{
	    refine();
	    parent.getMainMenu().getRefineItem().doClick ();
	    return;
	}
	if (e.getActionCommand().equals (CLOSE))
	{
	    parent.getMainMenu().getRefineItem().doClick ();
	    return;
	}
    }

    // keys for the action commands
    private static final String CLOSE = "CLOSE";
    private static final String APPLY = "APPLY";

    // buttons
    private JRadioButton noButton;
    private JRadioButton delma33Button;
    private JRadioButton boundary3Button;
    private JRadioButton boundary2Button;
    private JRadioButton boundarynormalButton;
    private JRadioButton delma5Button;
    private JRadioButton cornerButton;

    private JButton closeButton;
    private JButton applyButton;

    private JTextField param1TF;
    private JTextField param2TF;
    private JTextField levelTF;
    

}
