package devisor2.grid.GUI.dialogs;

import java.util.*;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import javax.swing.*;
import javax.swing.border.*;

import devisor2.grid.options.*;
import devisor2.grid.GUI.framework.*;
import devisor2.grid.GUI.event.*;
import devisor2.foundation.elements.*;
import devisor2.foundation.*;
import devisor2.grid.backend.*;



/**
 *  This class is the DockableDialog instance to get the scale factors. <br>
 *  See the GridListener and MainActionListener for a detailed description 
 *  of how the dialog is connected to the application.
 *
 *  @author Dominik Goeddeke
 *  @version done
 *  
 */

public class ScaleDialog extends DockableDialog implements ActionListener
{

    // factors in x and y direction
    private double factorx, factory;


   /**
     *  this method can be used to update the GUI of the dialog dynamically.
     *  it is called by the DialogManager e.g. when some comboboxes containing
     *  boundary information need to be refreshed
     */
    public void refreshGUI ()
    {
	// no need fort this in this class :-)
	return;
    }



    /** 
     *  returns the initial Position of the dialog
     */
    public Point getInitialPosition ()
    {
	return new Point (parent.getLocationOnScreen().x - 50, parent.getLocationOnScreen().y);
    }


    /**
     *  the constructor just sets references correctly and creates the GUI
     */
    public ScaleDialog (MainFrame parent)
    {
	super (parent);
	setTitle ((String)cc.rb.getObject("dialogs_scale_caption"));
	JPanel content = new JPanel ();
	content.setBorder (new EmptyBorder (10,10,10,10));
	// just ordinary GUI layouting
	GridBagLayout layout = new GridBagLayout ();
	GridBagConstraints constraints = new GridBagConstraints ();
	content.setLayout (layout);
	constraints.fill = GridBagConstraints.BOTH;
	constraints.weightx = constraints.weighty = 1.0;
	// edit fields
	JLabel label1 = new JLabel ((String)cc.rb.getObject ("dialogs_scale_x"));
	constraints.gridy = 0;
	constraints.gridx = 0;
	layout.setConstraints (label1, constraints);
	content.add (label1);
	xTF = new JTextField (6);
	constraints.gridx = 1;
	layout.setConstraints (xTF, constraints);
	content.add (xTF);
	JLabel label2 = new JLabel ((String)cc.rb.getObject ("dialogs_scale_y"));
	constraints.gridy = 1;
	constraints.gridx = 0;
	layout.setConstraints (label2, constraints);
	content.add (label2);
	yTF = new JTextField (6);
	constraints.gridx = 1;
	layout.setConstraints (yTF, constraints);
	content.add (yTF);
	// buttons
	JPanel buttons = new JPanel ();
	buttons.setLayout (new FlowLayout ());
	okButton = new JButton ((String)cc.rb.getObject ("general_Ok"));
	okButton.setMnemonic ( ((Integer)cc.rb.getObject("general_Ok_mnemonic")).intValue());
	okButton.setActionCommand (OK);
	okButton.addActionListener (this);
	buttons.add (okButton);
	applyButton = new JButton ((String)cc.rb.getObject ("general_Apply"));
	applyButton.setMnemonic ( ((Integer)cc.rb.getObject("general_Apply_mnemonic")).intValue());
	applyButton.setActionCommand (APPLY);
	applyButton.addActionListener (this);
	buttons.add (applyButton);
	cancelButton = new JButton ((String)cc.rb.getObject ("general_Cancel"));
	cancelButton.setMnemonic ( ((Integer)cc.rb.getObject("general_Cancel_mnemonic")).intValue());
	cancelButton.setActionCommand (CANCEL);
	cancelButton.addActionListener (this);
	buttons.add (cancelButton);
	// add them all to the dialog
	getRootPane().setDefaultButton (applyButton);
	constraints.gridy = 3;
	constraints.gridx = 0;
	constraints.gridwidth = 2;
	layout.setConstraints(buttons, constraints);
	content.add (buttons);
	// finish the GUI
	getContentPane ().add (content);
	addComponentListener (listener);
	pack ();
	//setSize (300,130);
    }

    /**
     *  This class handles its own action event. 
     */
    public void actionPerformed (ActionEvent e)
    {
	if (e.getActionCommand().equals (OK))
	{
	    if (verify ())
	    {
		scale ();
		cc.dm.hide (cc.dm.SCALEDIALOG);
	    }
	    return;
	}
	if (e.getActionCommand().equals (APPLY))
	{
	    if (verify ())
	    {
		scale ();
	    }
	    return;
	}
	if (e.getActionCommand().equals (CANCEL))
	{
	    cc.dm.hide (cc.dm.SCALEDIALOG);
	    return;
	}
    }

    private void scale ()
    {
	GridListener listener = parent.getDrawingArea().getListener();
  	GridSelector selector = listener.getSelector ();
	cc = ControlCenter.getMyself ();
	if (selector.isNodesOnly ())
	{
	    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject("dialogs_scale_nodes_error"), (String)cc.rb.getObject ("errors_selectionerror"), JOptionPane.ERROR_MESSAGE);
	    return;
	}
	if (selector.containsCircles() && factorx != factory)
        {
	     JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject("dialogs_scale_circles_error"), (String)cc.rb.getObject ("errors_selectionerror"), JOptionPane.ERROR_MESSAGE);
	    return;
	}
	// prepare undo
	Vector infos = new Vector ();
	infos.add (listener.getSelector().getHotspotMask ());
	Enumeration enum = listener.getSelectedItems().elements ();
	while (enum.hasMoreElements ())
	{
	    GridItem item = (GridItem)enum.nextElement ();
	    Object[] info = {item, item.getParameters()};
	    infos.add (info);
	}
	cc.um.notifyUndo (cc.um.UNDO_SCALE, listener.getSelectedItems(), infos);
	// perform trafo
	AffineTransform newtrafo1 = AffineTransform.getScaleInstance (factorx, factory);
	// get center coords of selection
	Point anchor = selector.getCenterHotspot ();
	GridTransformation trafo1 = new GridTransformation (newtrafo1,0,factorx, factory, 0,0,0,0);
	// apply trafo without repainting or anything
	listener.applyTransformation (trafo1, false);
	// calc neccessary translation which shifts the center hotspot
	// back to where it was
	int[] oldworld = cc.gc.fromScreenToWorld(anchor.x,anchor.y);
	anchor = selector.getCenterHotspot ();
	int[] world = cc.gc.fromScreenToWorld(anchor.x, anchor.y);
	AffineTransform newtrafo2 = AffineTransform.getTranslateInstance (oldworld[0] - world[0], oldworld[1] - world[1]);
	GridTransformation trafo2 = new GridTransformation (newtrafo2, 0, 0,0,oldworld[0]-world[0], oldworld[1]-world[1],0,0);
	// apply trafo and repaint
	listener.applyTransformation (trafo2,true);
    }

       

    /**
     *  the verification code is all put into this method
     */
    private boolean verify ()
    {
	boolean allOK = true;
	try
	{
	    factorx = Double.parseDouble (xTF.getText());
	}
	catch (Exception ex)
	{
	    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject("dialogs_scale_x_error"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
	    allOK = false;
	}
	try
	{
	    factory = Double.parseDouble (yTF.getText());
	}
	catch (Exception ex)
	{
	    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject("dialogs_scale_y_error"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
	    allOK = false;
	}
	return allOK;
    }


    // keys for the action commands

    private static final String OK = "OK";
    private static final String CANCEL = "CANCEL";
    private static final String APPLY = "APPLY";

    // buttons

    private JButton okButton;
    private JButton cancelButton;
    private JButton applyButton;
    private JTextField xTF;
    private JTextField yTF;

    
}
