package devisor2.grid.GUI.dialogs;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;


import devisor2.grid.options.*;
import devisor2.grid.GUI.framework.*;

/**
 *  This class acts as a container to display texts, which can be read from a
 *  file or passed to this class as a single String. This class does not 
 *  support HTML, it is designed for plain text only. <br>
 *  You can get a full scale example of how to use this class in the event 
 *  handling code for the DOMAINSTATISTICS action command. Note: this is a 
 *  dialog, so it is used just like any other dialog throughout the 
 *  application
 *
 *  @author Dominik Goeddeke
 *  @version 1.0 fully tested
 */
public class TextFrame extends JDialog implements ActionListener
{

    /**
     *  the text to be displayed
     */
    private String show;

    /**
     *  a reference to the MainFrame instance
     */
    private MainFrame parent;

    /**
     *  the obligatory static self-reference
     */
    private static TextFrame dialog;

    /**
     *  the controlcenter reference
     */
    private ControlCenter cc;

    /**
     *  creates a new instance of the dialog
     *  @param parent - the MainFrame reference
     *  @param title - the designated title for the dialog
     *  @param show - the text to be shown
     */
    public static void initialize (MainFrame parent, String title, String show)
    {
	dialog = new TextFrame (parent, title, show);
    }

    /**
     *  displays the dialog modally on the screen
     */
    public static void showDialog ()
    {
	if (dialog != null)
	    dialog.setVisible (true);
    }

    /**
     *  the constructor need not be called directly, the initialize()-
     *  method suffices
     */
    public TextFrame(MainFrame parent, String title, String show)
    {
	super (parent, true);
	cc = ControlCenter.getMyself ();
	setTitle(title);
	this.show = show;
	this.parent = parent;
	// layout gui
	getContentPane ().setLayout (new BorderLayout ());
	ta = new JTextArea(80,25);
	scroll = new JScrollPane (ta);
	scroll.setPreferredSize (new Dimension (200,200));
	getContentPane().add(scroll, BorderLayout.CENTER);
	okButton = new JButton ((String)cc.rb.getObject ("general_Ok"));
	okButton.setToolTipText ((String)cc.rb.getObject ("dialogs_textframe_tooltip"));
	okButton.setMnemonic ( ((Integer)cc.rb.getObject("general_Ok_mnemonic")).intValue());
	okButton.setActionCommand (OK);
	okButton.addActionListener (this);
	getRootPane().setDefaultButton (okButton);
	getContentPane().add (okButton, BorderLayout.SOUTH);
	setSize (new Dimension (400,400));
	setDefaultCloseOperation (DO_NOTHING_ON_CLOSE);
    }

    /**
     *  re-reads the text to be displayed
     */
    public static void update()
    {
	if (dialog != null)
	    dialog.readText(dialog.show);
    }

    /**
     *  adds the text to the container
     */
    private void readText(String s) 
    {
	StringBuffer app;
	String buffer;
	BufferedReader in;
	
	app = new StringBuffer();
	
	try
	{
	    in = new BufferedReader(new FileReader(s));
	    while (in.ready())
	    {
		buffer=in.readLine();
		app.append(buffer);
		app.append("\n");
	    }
	    in.close();
	    buffer=app.toString();
	    ta.setText(buffer);
	}
	catch (IOException e)
	{
	    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject ("dialogs_textframe_loaderror_message"), (String)cc.rb.getObject ("dialogs_textframe_loaderror_title"), JOptionPane.ERROR_MESSAGE );
	}
    }

    /**
     *  sets the text directly
     */
    public static void setText(String s)
    {
	if (dialog != null)
	{
	    dialog.ta.setText(s);
	    dialog.ta.setCaretPosition (0);
	}
    }

    /**
     *  the event handler for the ok button
     */
    public void actionPerformed ( ActionEvent e ) 
    {
	if (e.getActionCommand ().equals (OK))
	{
	    setVisible (false);
	}
    }

    // the gui elements
    private  JTextArea ta;
    private JButton okButton;
    private JScrollPane scroll;

    // action commands
    public static final String OK = "OK";


}
