package devisor2.grid.GUI.event;

import java.awt.event.*;
import java.awt.*;
import java.awt.geom.*;
import java.util.*;
import javax.swing.*;

import devisor2.grid.GUI.framework.*;
import devisor2.grid.options.*;
import devisor2.grid.backend.*;
import devisor2.grid.GUI.dialogs.*;
import devisor2.foundation.*;
import devisor2.foundation.boundary.*;
import devisor2.foundation.elements.*;

/**
 *  This is the adjustment listener for the scrollbars of the
 *  drawing area. <br>
 
 */
public class GridAdjustment implements AdjustmentListener
{
    // the MainFrame reference
    private MainFrame root;
    
    // the ControlCenter reference
    protected ControlCenter cc = ControlCenter.getMyself();

    // reference to the mainframe's drawing area
    private DrawingArea da;

    // reference to the GridSelector cooperating with this listener
    private GridSelector gs;

    int bugcount;

     Vector zl;
     int sxlen,sylen;
     
    public class ZoomLevel
    {
	int x1,y1,x2,y2;
	
	public ZoomLevel(int ux1, int ux2, int uy1, int uy2)
	{
	    x1=ux1;
	    x2=ux2;
	    y1=uy1;
	    y2=uy2;
	}
    }
  
      
    /**************************************************************/
    
    /**
     *  The constructor just sets references correctly
     *  and initializes some variables
     *
     *  @param mf - the MainFrame reference
     *  @param da - the drawing area this listener listens to
     */    
    public GridAdjustment(MainFrame mf, DrawingArea da) 
    {
	root = mf;
	this.da = da;
	bugcount=0;
	zl=new Vector();
    }


    public void setSliderDefault(JScrollBar egon)
    {
	bugcount=0;
	
	egon.setValue(0);
	egon.setMaximum(1024);
	egon.setMinimum(0);
	egon.setVisibleAmount(1024);
    }
    
    public void setDefault()
    {
	cc.d.centerDomain();
	root.getDrawingArea().repaint();
	zl=new Vector();
	
	setSliderDefault(root.getScrollBar(JScrollBar.HORIZONTAL));
	setSliderDefault(root.getScrollBar(JScrollBar.VERTICAL));
	if (!root.getDrawingArea().getListener().getSelectedItems().isEmpty())
  	{
  	    root.getDrawingArea().getListener().calcPerimeter (false);
  	}    
    }


    public void preZoom()
    {
	if (zl.size()>1)
	{
	    int idx=zl.size();
	    
	    ZoomLevel z = (ZoomLevel)zl.remove(idx-1);
	    
	    cc.dc.act_x2=z.x2;
	    cc.dc.act_x1=z.x1;
	    cc.dc.act_y1=z.y1;
	    cc.dc.act_y2=z.y2;
	    
	    cc.gc.setTransformation();
	    
	    root.getDrawingArea().repaint();  
	    
	    int[] perimeter = cc.d.getPerimeter();
	    
	    int xlen=perimeter[2]-perimeter[0];
	    int ylen=perimeter[3]-perimeter[1];
	    
	    int xadj=xlen/10;
	    int yadj=ylen/10;
	    
	    xlen=xlen+xlen/10;
	    ylen=ylen+ylen/10;
	    
	    perimeter[0]=perimeter[0]-xadj/2;
	    perimeter[1]=perimeter[1]-yadj/2;
	    
	    
	    int vxlen=cc.dc.act_x2-cc.dc.act_x1;
	    int vylen=cc.dc.act_y2-cc.dc.act_y1;
	    
	    sxlen=(int)(vxlen/(double)xlen*1024.0);	
	    sylen=(int)(vylen/(double)ylen*1024.0);
	    
	    int sxpos=(int)((cc.dc.act_x1-perimeter[0])/(double)xlen*1024.0);
	    int sypos=(int)(((cc.dc.act_y1-perimeter[1])/(double)ylen)*1024.0);
	    
	    bugcount=0;
	    
	    root.getScrollBar(JScrollBar.VERTICAL).setVisibleAmount(sxlen);
	    root.getScrollBar(JScrollBar.HORIZONTAL).setVisibleAmount(sylen);
	    
	    root.getScrollBar(JScrollBar.VERTICAL).setValue(sxpos);
	    root.getScrollBar(JScrollBar.HORIZONTAL).setValue(1024-(sypos+sylen));
	    if (!root.getDrawingArea().getListener().getSelectedItems().isEmpty())
  	    {
  		root.getDrawingArea().getListener().calcPerimeter (false);
  	    }
	}
	else
	    setDefault();
      		
    }
    
    
    public void newZoom(Rectangle worldbox)
    {
	zl.add(new ZoomLevel(cc.dc.act_x1,cc.dc.act_x2,cc.dc.act_y1,cc.dc.act_y2));

	cc.d.centerDomain(worldbox);
	root.getDrawingArea().repaint();  
	
	int[] perimeter = cc.d.getPerimeter();
	
	int xlen=perimeter[2]-perimeter[0];
	int ylen=perimeter[3]-perimeter[1];
	
	int xadj=xlen/10;
	int yadj=ylen/10;
	
	xlen=xlen+xlen/10;
	ylen=ylen+ylen/10;
	
	perimeter[0]=perimeter[0]-xadj/2;
	perimeter[1]=perimeter[1]-yadj/2;
	
	
	int vxlen=cc.dc.act_x2-cc.dc.act_x1;
	int vylen=cc.dc.act_y2-cc.dc.act_y1;
	
	sxlen=(int)(vxlen/(double)xlen*1024.0);	
	sylen=(int)(vylen/(double)ylen*1024.0);
	
	int sxpos=(int)((cc.dc.act_x1-perimeter[0])/(double)xlen*1024.0);
	int sypos=(int)(((cc.dc.act_y1-perimeter[1])/(double)ylen)*1024.0);
	
	bugcount=0;
	
	root.getScrollBar(JScrollBar.VERTICAL).setVisibleAmount(sxlen);
	root.getScrollBar(JScrollBar.HORIZONTAL).setVisibleAmount(sylen);
	
	root.getScrollBar(JScrollBar.VERTICAL).setValue(sxpos);
	root.getScrollBar(JScrollBar.HORIZONTAL).setValue(1024-(sypos+sylen));
	  if (!root.getDrawingArea().getListener().getSelectedItems().isEmpty())
  	{
  	    root.getDrawingArea().getListener().calcPerimeter (false);
  	}	
    }
    
    public void adjustZoom()
    {
	cc.gc.setTransformation();  
	root.getDrawingArea().repaint();  
	
	int[] perimeter = cc.d.getPerimeter();
	
	int xlen=perimeter[2]-perimeter[0];
	int ylen=perimeter[3]-perimeter[1];
	
	int xadj=xlen/10;
	int yadj=ylen/10;
	
	xlen=xlen+xlen/10;
	ylen=ylen+ylen/10;
	
	perimeter[0]=perimeter[0]-xadj/2;
	perimeter[1]=perimeter[1]-yadj/2;
	
	int vxlen=cc.dc.act_x2-cc.dc.act_x1;
	int vylen=cc.dc.act_y2-cc.dc.act_y1;
	
	sxlen=(int)(vxlen/(double)xlen*1024.0);	
	sylen=(int)(vylen/(double)ylen*1024.0);
	
	int sxpos=(int)((cc.dc.act_x1-perimeter[0])/(double)xlen*1024.0);
	int sypos=(int)(((cc.dc.act_y1-perimeter[1])/(double)ylen)*1024.0);
	
	bugcount=0;
	
	root.getScrollBar(JScrollBar.VERTICAL).setVisibleAmount(sxlen);
	root.getScrollBar(JScrollBar.HORIZONTAL).setVisibleAmount(sylen);
	
	root.getScrollBar(JScrollBar.VERTICAL).setValue(sxpos);
	root.getScrollBar(JScrollBar.HORIZONTAL).setValue(1024-(sypos+sylen));
	if (!root.getDrawingArea().getListener().getSelectedItems().isEmpty())
  	{
  	    root.getDrawingArea().getListener().calcPerimeter (false);
  	}
    }
    
    
    public void adjustmentValueChanged(AdjustmentEvent e)
    {
	if (bugcount>3)
	{
	    int vxlen=cc.dc.act_x2-cc.dc.act_x1;
	    int vylen=cc.dc.act_y2-cc.dc.act_y1;
	    
	    int[] perimeter = cc.d.getPerimeter();
	    int xlen=perimeter[2]-perimeter[0];
	    int ylen=perimeter[3]-perimeter[1];
	    
	    int xadj=xlen/10;
	    int yadj=ylen/10;
	    
	    xlen=xlen+xlen/10;
	    ylen=ylen+ylen/10;
	    
	    perimeter[0]=perimeter[0]-xadj/2;
	    perimeter[1]=perimeter[1]-yadj/2;
	    
	    if (e.getAdjustable()==root.getScrollBar(JScrollBar.VERTICAL))
	    {
		cc.dc.act_x1=perimeter[0]+(int)(e.getValue()/1024.0*(double)xlen);
		cc.dc.act_x2=cc.dc.act_x1+vxlen;
		//         System.out.println("SCROLL RECHTS "+e.getValue());
	    }
	    else
	    {
		cc.dc.act_y1=perimeter[1]+(int)(((1024-(e.getValue()+sylen)))/1024.0*(double)ylen);
		cc.dc.act_y2=cc.dc.act_y1+vylen;         
		//	   System.out.println("SCROLL UNTEN "+(e.getValue()));
	    }

        cc.dc.act_x1=((int)(cc.dc.act_x1*cc.globalscale2))/cc.globalscale2;  
        cc.dc.act_y1=((int)(cc.dc.act_y1*cc.globalscale2))/cc.globalscale2;
	//System.out.println("DMSX");

        cc.dc.act_x2=((int)(cc.dc.act_x2*cc.globalscale2))/cc.globalscale2;
        cc.dc.act_y2=((int)(cc.dc.act_y2*cc.globalscale2))/cc.globalscale2;
	    
	    cc.gc.setTransformation();
	    root.getDrawingArea().repaint ();
	    if (!root.getDrawingArea().getListener().getSelectedItems().isEmpty())
  	    {
  		root.getDrawingArea().getListener().calcPerimeter (false);
  	    }
	}
	bugcount++;
    }
}
