package devisor2.grid.GUI.framework;

import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import devisor2.grid.GUI.event.*;
import devisor2.grid.options.*;
import devisor2.grid.backend.*;

/**
 *  This class DrawingArea represent the place where the domains
 *  are to be drawn. It is part of the MainFrame and connected to
 *  the rest of the application via the centerfold reference.
 *  <br>
 *  It does not have any real functionality, this is left to the
 *  GridListener instance responsible attached to it. See there for
 *  details. the only thing supported is managing the horizontal and
 *  vertival scrollbars. <br>
 *
 *  @author Christian Becker, Dominik Goeddeke
 *  @version beta
 */
public class DrawingArea extends JPanel 
{

    public boolean redraw_enabled=false;
    
    /**
     *  the MainFrame reference
     */
    public MainFrame parent;
    
    /**
     *  reference to the ControlCenter and to all global variables
     */
    protected ControlCenter cc = ControlCenter.getMyself();
    
    /**
     * Helping objects
     */
    public double[] perimeter;

    /**
     *  the GridSelector connected with this DrawingArea
     */
    private GridSelector gs;

    /**
     *  the GridListener
     */
    private GridListener listener;

    /**
     *  the currently active GridCloner instance. might be null if no copy 
     *  and paste is active at the moment
     */
    private GridCloner cloner;

    public int screenx;
    public int screeny;
    
    /**
     * The constructur sets references correctly and initialises the 
     * GridListeners.
     *
     * @param mf - reference to the MainFrame
     */
    public DrawingArea(MainFrame mf) 
    {
	parent = mf;
	setBackground(Color.white);
//	cc.gc.setTransformation();
//       int x=mf.getDrawingArea().getWidth();
//       int y=mf.getDrawingArea().getHeight();

        screenx=Integer.parseInt(cc.op.get(Options.screenx));
	screeny=Integer.parseInt(cc.op.get(Options.screeny));
	
	Dimension dim = new Dimension (screenx,screeny);
	setMinimumSize (dim);
	setMaximumSize (dim);
	setPreferredSize (dim);
	setOpaque (true);
	
	listener = new GridListener (mf, this);
	gs = listener.getSelector ();
	
	addMouseMotionListener(listener);
	addMouseListener(listener);
    }

    /**
     *  returns the GridSelector connected to this DrawingArea
     */
    public GridSelector getSelector ()
    {
	return gs;
    }

    /**
     *  returns the GridListener 
     */
    public GridListener getListener ()
    {
	return listener;
    }

    /**
     *  Sets the GridCloner to the given instance
     */
    public void setCloner (GridCloner cloner)
    {
	this.cloner = cloner;
    }

    /**
     *  @return the current GridCloner instance
     */
    public GridCloner getCloner ()
    {
	return cloner;
    }

    /**
     *  updates the drawing area
     */
    public void update(Graphics g) 
    {
	paintComponent (g);
    }
    
    /**
     *  this method takes care of all the painting work
     */
    public void paintComponent(Graphics g) 
    {
	if (redraw_enabled)
	{
	    // paint background
	    super.paintComponent (g);
	    // redraw complete domain	
	    cc.d.draw(g);
	   //   // draw select box and hotspots
//  	    if (!listener.getSelectedItems().isEmpty())
//  	    {
//  		gs.updateHotspots (false);
//  	    }
	}

	// all that follows leads to our world-famous bug, but is left here
	// as a trophy :-)
	//  if (redraw_enabled)
//  	{
//  	    // paint background
//  	    super.paintComponent (gg);

//  	    Graphics g = (Graphics)gg;
//  	    System.out.println("REPAINT");
	    
//  	    cc.gc.setTransformation();
	    
//  	    g.setTransform(cc.gc.trf);
	    
//  	    //System.out.println("Scales "+cc.gc.trf.getScaleX()+" "+cc.gc.trf.getScaleY());
	    
//  	    //double[] p1 = {0,0};
//  	    //double[] p2 = new double[2];
	    
//  	    //trf.transform(p1,0,p2,0,1);
	    
//  	    //System.out.println("100,100 -> "+(int)p2[0]+" "+(int)p2[1]);
	    
//  	    g.setStroke(new BasicStroke((float)(1.0f/cc.gc.getScaleX())));
	    
//  	    g.clearRect( 0, 0, cc.dc.screen_width, cc.dc.screen_height);
	    
//  	    cc.d.draw(g);
//  	}
    }

}


