package devisor2.grid.GUI.framework;

import java.awt.*;
import java.awt.image.*;
import java.awt.geom.*;
import java.awt.event.*;
import java.io.*;
import java.text.*;
import java.util.*;
import javax.swing.*;
import devisor2.grid.GUI.event.*;
import devisor2.grid.options.*;
import devisor2.grid.backend.*;

/**
 *  This class DrawingArea represent the place where the domains
 *  are to be drawn. It is part of the MainFrame and connected to
 *  the rest of the application via the centerfold reference.
 *  <br>
 *  It does not have any real functionality, this is left to the
 *  GridListener instance responsible attached to it. See there for
 *  details. the only thing supported is managing the horizontal and
 *  vertival scrollbars. <br>
 *
 *  @author Christian Becker, Dominik Goeddeke
 *  @version beta
 */
public class PSGraphics extends Graphics 
{
  /**
   * stream object for the postscript output
   */
  DataOutputStream out;

  int FW,FH;
  
  int PSX;
  int PSY;
  
  double kx,ky;
  
  boolean landscape;
  
   public PSGraphics()
   {
   }

   public PSGraphics(String file,int ufw, int ufh, boolean ls)
   {
   
     FW=ufw;
     FH=ufh;
     landscape=ls;
     
     int rw=210;
     int rh=297;
     
   try
    {
      FileOutputStream writeFile = new FileOutputStream(file);
      out = new DataOutputStream( writeFile );

      out.writeBytes("%!PS-Adobe-2.0\n");  // write PS header 

      out.writeBytes("%%Title: "+file+"\n");
      out.writeBytes("%%Creator: DeViSoRGrid2\n");
      
      
      if (landscape)
      {
        kx=(0.8*rh)/FW;
	ky=kx;
	
	PSY=(int)0.1*rh;
	PSX=(int)(rh-(kx*FH))/2;
	
        out.writeBytes("%%BoundingBox: "+(PSX/25.4*72)+" "
          +(PSY/25.4*72)+" "+(PSX+kx*FH)/25.4*72+" "+(PSY+ky*FW)/25.4*72+"\n");
      }
      else
      {
        kx=(0.8*rw)/FW;
	ky=kx;
	
	PSX=(int)0.1*rw;
	PSY=(int)(rh-(kx*FH))/2;
	
        out.writeBytes("%%BoundingBox: "+(PSX/25.4*72)+" "
          +(PSY/25.4*72)+" "+(PSX+kx*FW)/25.4*72+" "+(PSY+ky*FH)/25.4*72+"\n");
      }	

      out.writeBytes("%%Pages: 1\n");
      out.writeBytes("%%DocumentFonts:\n");
      out.writeBytes("%%EndComments\n");
      out.writeBytes("%%EndProlog\n");
      out.writeBytes("\n");

//      out.writeBytes("newpath "+(PSX/25.4*72)+" "+(PSY/25.4*72)+" moveto\n");
//      out.writeBytes(((PSX+kx*FW)/25.4*72)+" "+(PSY/25.4*72)+" lineto\n");
//      out.writeBytes(((PSX+kx*FW)/25.4*72)+" "+(PSY+ky*FH)/25.4*72+" lineto\n");
//      out.writeBytes(((PSX)/25.4*72)+" "+(PSY+ky*FH)/25.4*72+" lineto\n");
//      out.writeBytes(((PSX)/25.4*72)+" "+(PSY)/25.4*72+" lineto\n");
//      out.writeBytes("clip closepath stroke\n");

      out.writeBytes("72 25.4 div dup scale\n");
      out.writeBytes("0.15 setlinewidth\n");
      out.writeBytes("/DeviceRGB setcolorspace\n");
          out.writeBytes("/Times-Roman findfont 2 scalefont setfont \n");


//drawLine(0,0,FW,FH);
//drawLine(0,FH,FW,0);
//drawLine(0,0,FW,0);
//drawLine(FW,0,FW,FH);
//drawLine(FW,FH,0,FH);
//drawLine(0,FH,0,0);

    }
    catch(IOException exp)
    {
      System.out.println("ERROR:Metagraph:constructor "+exp);
    }

     
   }
      


 /**
   * closes an output
   */
   public void close()
   {
     if (out!=null)
     {
      try
      {
        out.writeBytes("showpage");

        out.close();
      }
      catch(IOException exp)
      {
        System.err.println("ERROR:Metagraph:close "+exp);
      }

     }
   }
   
   public void drawLine(int x1, int y1, int x2, int y2)
   {
      try
      {
        out.writeBytes("newpath \n");
	if (landscape)
         out.writeBytes(""+(PSX+y1*kx)+" "+(x1*ky+PSY)+" moveto "
          +(PSX+y2*kx)+" "+(x2*ky+PSY)+" lineto \n");
	else
	 out.writeBytes(""+(PSX+x1*kx)+" "+((FH-y1)*ky+PSY)+" moveto "
          +(PSX+x2*kx)+" "+((FH-y2)*ky+PSY)+" lineto \n");
        out.writeBytes("stroke \n");

      }
      catch(IOException exp)
      {
        System.err.println("ERROR:Metagraph:drawLineM "+exp);
      }
   }
   
   public void dispose()
   {
   }
   
  public boolean drawImage(Image img, int x, int y,
                           ImageObserver observer)
  {
    return false;
  }
   
  public boolean drawImage(Image img, int x1, int y1,
                           int x2, int y2,
                           int x3, int y3, int x4, int y4,
                           ImageObserver observer)
  {
    return false;
  }
  
  public boolean drawImage(Image img, int x1, int y1,
                           int x2, int y2,
                           int x3, int y3, int x4, int y4,
                           Color c,
                           ImageObserver observer)
  {
    return false;
  }
  
  public boolean drawImage(Image img, int x, int y,
                           int width, int height,
                           ImageObserver observer)
  {
    return false;
  }
  
  public boolean drawImage(Image img, int x, int y, Color bgcolor,
                           ImageObserver observer)
  {
    return false;
  }
  
  public boolean drawImage(Image img, int x, int y,
                           int width, int height, Color bgcolor,
                           ImageObserver observer)
  {
    return false;
  }
  
  public void drawString(String str, int x, int y)
  {
     try
      {
        if (landscape)
        out.writeBytes(""+(PSX+y*kx)+" "+(PSY+x*ky)+" moveto 90 rotate ("+str+") show -90 rotate\n");
	else
        out.writeBytes(""+(PSX+x*kx)+" "+(PSY+(FH-y)*ky)+" moveto ("+str+") show \n");
      }
      catch(IOException exp)
      {
        System.err.println("ERROR:Metagraph:drawStringM "+exp);
      }
  }
  
  public void drawString(AttributedCharacterIterator str, int x, int y)
  {
  }

  public void fillPolygon(int xPoints[], int yPoints[], int nPoints)
  {
  }
  
  public void fillPolygon(Polygon p)
  {
       try
       {
        if (landscape)
        System.err.println("ERROR:yet not implemented");
 //       out.writeBytes(""+(PSX+y*kx)+" "+(PSY+x*kx)
   //       +" "+height*kx+" "+width*ky+" rectfill \n");
	else
	{
	out.writeBytes("newpath \n");
	out.writeBytes((PSX+p.xpoints[0]*kx)+" "+(PSY+(FH-p.ypoints[0])*ky)+" moveto\n");
        for (int i=1;i<p.npoints;i++)
	  out.writeBytes((PSX+p.xpoints[i]*kx)+" "+(PSY+(FH-p.ypoints[i])*ky)+" lineto\n");
	}
//	  out.writeBytes((PSX+p.xpoints[0]*kx)+" "+(PSY+(FH-p.ypoints[0])*ky)+" rlineto\n");
	out.writeBytes("closepath fill\n");
      }
      catch(IOException exp)
      {
        System.err.println("ERROR:Metagraph:drawStringM "+exp);
      }
    }
  
  public void drawPolygon(int xPoints[], int yPoints[], int nPoints)
  {
  }
  
  public void drawPolygon(Polygon p)
  {
  }
  
  public void drawPolyline(int xPoints[], int yPoints[], int nPoints)
  {
  }
  
  public void fillArc(int x, int y, int width, int height, int startAngle, int arcAngle)
  {
  }
  
  public void drawArc(int x, int y, int width, int height,
                      int sstartAngle, int sarcAngle)
  {
      int startAngle,arcAngle;
      
      if (sstartAngle<0) startAngle=sstartAngle+360; else startAngle=sstartAngle;
      if (sarcAngle<0) arcAngle=sarcAngle+360; else arcAngle=sarcAngle;

     if (startAngle>360) startAngle=startAngle-360;
      if (arcAngle>360) arcAngle=arcAngle-360; 
            
	    if ((startAngle+arcAngle)==startAngle) arcAngle=arcAngle-1;
      try
      {
        out.writeBytes("newpath \n");
	if (landscape)
         out.writeBytes(""+((PSX+y*kx)+(width*kx)/2)+" "+((x*ky+PSY)+(width*kx)/2)+" "
          +(width*kx)/2+" "+startAngle+" "+(startAngle+arcAngle)+" arc \n");
	else
	 out.writeBytes(""+((PSX+x*kx)+(width*kx)/2)+" "+(((FH-y)*ky+PSY)-(width*kx)/2)+" "
          +(width*kx)/2+" "+startAngle+" "+(startAngle+arcAngle)+" arc \n");
        out.writeBytes("stroke \n");

      }
      catch(IOException exp)
      {
        System.err.println("ERROR:Metagraph:drawLineM "+exp);
      }
      
  }
  
  public void fillOval(int x, int y, int width, int height)
  {
  }
  
  public void drawOval(int x, int y, int width, int height)
  {
  }
  
  public void fillRoundRect(int x, int y, int width, int height, int arcWidth, int arcHeight)
  {
  }
  
  public void drawRoundRect(int x, int y, int width, int height, int arcWidth, int arcHeight)
  {
  }
  
  public void clearRect(int x, int y, int width, int height)
  {
  }
  
  public void fillRect(int x, int y, int width, int height)
  {
      try
      {
        if (landscape)
        out.writeBytes(""+(PSX+y*kx)+" "+(PSY+x*kx)
          +" "+height*kx+" "+width*ky+" rectfill \n");
	else
        out.writeBytes(""+(PSX+x*kx)+" "+(PSY+(FH-y)*kx)
          +" "+width*kx+" "+height*ky+" rectfill \n");
      }
      catch(IOException exp)
      {
        System.err.println("ERROR:Metagraph:fillRectM "+exp);
      }
  }
  
  public void copyArea(int x, int y, int width, int height, int dx, int dy)
  {
  }
  
  public void setClip(Shape s)
  {
  }
  
  public void setClip(int x, int y, int width, int height)
  {
  }
  
  public Rectangle getClipBounds()
  {
    return null;
  }
  
  public Shape getClip()
  {
    return null;
  }
  
  public void clipRect(int x, int y, int width, int height)
  {
  }
  
  public FontMetrics getFontMetrics()
  {
    return null;
  }
  
  public FontMetrics getFontMetrics(Font f)
  {
    return null;
  }
  
  public void setFont(Font f)
  {
  }
  
  public Font getFont()
  {
    return null;
  }
  
  public void setXORMode(Color c1)
  {
  }
  
  public void setPaintMode()
  {
  }
  
  public Color getColor()
  {
    return null;
  }
  
  public void setColor(Color c)
  {
    
      try
      {
/**        if (c.equals(Color.white))
          out.writeBytes("1 1 1 setcolor \n");
        else if (c.equals(Color.red))
          out.writeBytes("1 0 0 setcolor \n");
        else if (c.equals(Color.blue))
          out.writeBytes("0 0 1 setcolor \n");
        else if (c.equals(Color.green))
          out.writeBytes("0 1 0 setcolor \n");
        else if (c.equals(Color.cyan))
          out.writeBytes("1 1 0 setcolor \n");
        else if (c.equals(Color.darkGray))
          out.writeBytes("0.8 0.8 0.8 setcolor \n");
        else if (c.equals(Color.gray))
          out.writeBytes("0.4 0.4 0.4 setcolor \n");
        else if (c.equals(Color.magenta))
          out.writeBytes("1 0 1 setcolor \n");
        else if (c.equals(Color.orange))
          out.writeBytes("0.5 0 0.2 setcolor \n");
        else if (c.equals(Color.yellow))
          out.writeBytes("0 1 1 setcolor \n");
        else **/
	
          out.writeBytes((c.getRed()/255.0)+" "+(c.getGreen()/255.0)+" "+
	  (c.getBlue()/255.0)+" setcolor \n");

      }
      catch(IOException exp)
      {
        System.err.println("ERROR:Metagraph:setColorM "+exp);
      }

  }
  
  public void translate(int x, int y)
  {
  }
  
  public Graphics create()
  {
    return null;
  }                
  
                                          
}


