package devisor2.grid.GUI.framework;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import devisor2.grid.options.*;
import devisor2.grid.GUI.event.*;



/** 
 *  This class represents the toolbar of the MainFrame instance.
 *  It is dockable so that the user can position it whereever he 
 *  wants to. <br>
 *  To link it to the application, the same ActionCommands
 *  as in the menu are used, and both classes use the same 
 *  ActionListener (i.e. MainActionListener class). So this class
 *  is pure GUI, definitely no functionality included.
 *  If you add further icons, please stick to the following: <br>
 *  (1) the gifs belong to the images directory, which can be
 *      accessed via the "general_devisorhome" key from Options. <br>
 *  (2) event handling code goes to the MainActionListener class, 
 *      which is where the ActionCommand String has to be declared.<br>
 *  (3) As always, do not declare tooltips directly, the text goes
 *      into the locale class. See there how this is accomplished.<br>
 *
 *  @author Dominik Goeddeke
 *  @version almost done
 */

public class ToolBar extends JToolBar
{
    private MainActionListener mal;
    private MainFrame parent;
    private ControlCenter cc;
    
    /* the whole lot of buttons declared here */
    private JButton newButton;
    private JButton openButton;
    private JButton saveButton;
    private JButton saveUCDButton;
    private JButton saveFEATButton;
    private JButton saveFEASTButton;
    private JButton undoButton;
    private JButton redoButton;
    private JButton cutButton;
    private JButton copyButton;
    private JToggleButton pasteToggle;
    private JToggleButton selectNodeToggle;
    private JToggleButton selectElementToggle;
    private JToggleButton selectSegmentToggle;
    private JToggleButton selectEdgesToggle;
    private JComboBox selectBoundaryCombo;
    private JToggleButton setZoomToggle;
    private JButton resetZoomButton;
    private JToggleButton zoomInToggle;
    private JButton zoomOutButton;
    private JToggleButton directInputToggle;
    private JButton newBoundaryButton;
    private JToggleButton copymodeToggle;
    private JComboBox addTypeCombo;
    private JToggleButton propertiesToggle;
    private JToggleButton edgeStatusToggle;
    private JToggleButton parallelblockToggle;
    private JToggleButton rectifyElementToggle;
    private JButton exitButton;
    private ImageIcon[] comboImages;

    /**
     *  The constructor just creates the toolbar and its buttons
     *  after setting some references
     *
     *  @param parent the MainFrame reference
     */
    public ToolBar (MainFrame parent)
    {
	super ();
	this.parent = parent;
	mal = parent.getMainActionListener ();
	cc = ControlCenter.getMyself ();
	initComboImages ();
	initButtons ();
    }


    /**
     *  creates the array of icons and their descriptions for the addTypecombo
     */
    private void initComboImages ()
    {
	comboImages = new ImageIcon[10];

	comboImages[0] = new ImageIcon (cc.op.get(Options.general_devisorhome)+"/images/AddNothing.gif");
	comboImages[0].setDescription ((String)cc.rb.getObject("GUI_framework_ToolBar_addnothingtooltip"));
	comboImages[1] = new ImageIcon (cc.op.get(Options.general_devisorhome)+"/images/Paste.gif");
	comboImages[1].setDescription ((String)cc.rb.getObject("GUI_framework_ToolBar_addclipboardtooltip"));
	comboImages[2] = new ImageIcon (cc.op.get(Options.general_devisorhome)+"/images/Line.gif");
	comboImages[2].setDescription ((String)cc.rb.getObject("GUI_framework_ToolBar_addlinetooltip"));
	comboImages[3] = new ImageIcon (cc.op.get(Options.general_devisorhome)+"/images/Multiline.gif");
	comboImages[3].setDescription ((String)cc.rb.getObject("GUI_framework_ToolBar_addmultilinetooltip"));
	comboImages[4] = new ImageIcon (cc.op.get(Options.general_devisorhome)+"/images/CirclePlus.gif");
	comboImages[4].setDescription ((String)cc.rb.getObject("GUI_framework_ToolBar_addcircleptooltip"));
	comboImages[5] = new ImageIcon (cc.op.get(Options.general_devisorhome)+"/images/CircleMinus.gif");
	comboImages[5].setDescription ((String)cc.rb.getObject("GUI_framework_ToolBar_addcirclemtooltip"));
	comboImages[6] = new ImageIcon (cc.op.get(Options.general_devisorhome)+"/images/Tri.gif");
	comboImages[6].setDescription ((String)cc.rb.getObject("GUI_framework_ToolBar_addtritooltip"));
	comboImages[7] = new ImageIcon (cc.op.get(Options.general_devisorhome)+"/images/Quad.gif");
	comboImages[7].setDescription ((String)cc.rb.getObject("GUI_framework_ToolBar_addquadtooltip"));
	comboImages[8] = new ImageIcon (cc.op.get(Options.general_devisorhome)+"/images/Macro.gif");
	comboImages[8].setDescription ((String)cc.rb.getObject("GUI_framework_ToolBar_addmacrotooltip"));
	comboImages[9] = new ImageIcon (cc.op.get(Options.general_devisorhome)+"/images/Node.gif");
	comboImages[9].setDescription ((String)cc.rb.getObject("GUI_framework_ToolBar_addnodetooltip"));
    }

    /**
     *  The whole work is done in this method, namely setting up the buttons,
     *  adding tooltips and connecting them to the ActionListener
     */
    private void initButtons ()
    {
	// NEW
	newButton = new JButton (new ImageIcon((cc.op.get(Options.general_devisorhome))+"/images/New.gif"));
	newButton.setActionCommand (MainActionListener.NEW);
	newButton.setToolTipText ((String)cc.rb.getObject("GUI_framework_ToolBar_newtooltip"));
	newButton.addActionListener (mal);
	newButton.setMaximumSize (new Dimension (31,31));
	newButton.setMinimumSize (new Dimension (31,31));
	newButton.setPreferredSize (new Dimension (31,31));
	add(newButton);
	// OPEN
	openButton = new JButton(new ImageIcon((cc.op.get(Options.general_devisorhome))+"/images/Open.gif"));
	openButton.setActionCommand (MainActionListener.OPEN);
	openButton.setToolTipText ((String)cc.rb.getObject("GUI_framework_ToolBar_opentooltip"));
	openButton.addActionListener (mal);
	openButton.setMaximumSize (new Dimension (31,31));
	openButton.setMinimumSize (new Dimension (31,31));
	openButton.setPreferredSize (new Dimension (31,31));
	add(openButton);
	// SAVE
	saveButton = new JButton(new ImageIcon((cc.op.get(Options.general_devisorhome))+"/images/Save.gif"));
	saveButton.setActionCommand (MainActionListener.SAVE);
	saveButton.setToolTipText ((String)cc.rb.getObject("GUI_framework_ToolBar_savetooltip"));
	saveButton.addActionListener (mal);
	saveButton.setMaximumSize (new Dimension (31,31));
	saveButton.setMinimumSize (new Dimension (31,31));
	saveButton.setPreferredSize (new Dimension (31,31));
	add(saveButton);
	// SAVE AS UCD
	saveUCDButton = new JButton(new ImageIcon((cc.op.get(Options.general_devisorhome))+"/images/SaveUCD.gif"));
	saveUCDButton.setActionCommand (MainActionListener.SAVEAS_UCD);
	saveUCDButton.setToolTipText ((String)cc.rb.getObject("GUI_framework_ToolBar_saveucdtooltip"));
	saveUCDButton.addActionListener (mal);
	saveUCDButton.setMaximumSize (new Dimension (31,31));
	saveUCDButton.setMinimumSize (new Dimension (31,31));
	saveUCDButton.setPreferredSize (new Dimension (31,31));
	saveUCDButton.setEnabled (false);
	add(saveUCDButton);
	// SAVE AS FEAT
	saveFEATButton = new JButton(new ImageIcon((cc.op.get(Options.general_devisorhome))+"/images/SaveFEAT.gif"));
	saveFEATButton.setActionCommand (MainActionListener.SAVEAS_FEAT);
	saveFEATButton.setToolTipText ((String)cc.rb.getObject("GUI_framework_ToolBar_savefeattooltip"));
	saveFEATButton.addActionListener (mal);
	saveFEATButton.setMaximumSize (new Dimension (31,31));
	saveFEATButton.setMinimumSize (new Dimension (31,31));
	saveFEATButton.setPreferredSize (new Dimension (31,31));
	add(saveFEATButton);
	// SAVE AS FEAST
	saveFEASTButton = new JButton(new ImageIcon((cc.op.get(Options.general_devisorhome))+"/images/SaveFEAST.gif"));
	saveFEASTButton.setActionCommand (MainActionListener.SAVEAS_FEAST);
	saveFEASTButton.setToolTipText ((String)cc.rb.getObject("GUI_framework_ToolBar_savefeasttooltip"));
	saveFEASTButton.addActionListener (mal);
	saveFEASTButton.setMaximumSize (new Dimension (31,31));
	saveFEASTButton.setMinimumSize (new Dimension (31,31));
	saveFEASTButton.setPreferredSize (new Dimension (31,31));
	add(saveFEASTButton);
	addSeparator ();
	// UNDO
	undoButton = new JButton(new ImageIcon((cc.op.get(Options.general_devisorhome))+"/images/Undo.gif"));
	undoButton.setActionCommand (MainActionListener.UNDO);
	undoButton.setToolTipText ((String)cc.rb.getObject("GUI_framework_ToolBar_undotooltip"));
	undoButton.addActionListener (mal);
	undoButton.setMaximumSize (new Dimension (31,31));
	undoButton.setMinimumSize (new Dimension (31,31));
	undoButton.setPreferredSize (new Dimension (31,31));
	add(undoButton);
	// REDO
	redoButton = new JButton(new ImageIcon((cc.op.get(Options.general_devisorhome))+"/images/Redo.gif"));
	redoButton.setActionCommand (MainActionListener.REDO);
	redoButton.setToolTipText ((String)cc.rb.getObject("GUI_framework_ToolBar_redotooltip"));
	redoButton.addActionListener (mal);
	redoButton.setMaximumSize (new Dimension (31,31));
	redoButton.setMinimumSize (new Dimension (31,31));
	redoButton.setPreferredSize (new Dimension (31,31));
	add(redoButton);
	// CUT
	cutButton = new JButton(new ImageIcon((cc.op.get(Options.general_devisorhome))+"/images/Cut.gif"));
	cutButton.setActionCommand (MainActionListener.CUT);
	cutButton.setToolTipText ((String)cc.rb.getObject("GUI_framework_ToolBar_cuttooltip"));
	cutButton.addActionListener (mal);
	cutButton.setMaximumSize (new Dimension (31,31));
	cutButton.setMinimumSize (new Dimension (31,31));
	cutButton.setPreferredSize (new Dimension (31,31));
	add(cutButton);
	// COPY
	copyButton = new JButton(new ImageIcon((cc.op.get(Options.general_devisorhome))+"/images/Copy.gif"));
	copyButton.setActionCommand (MainActionListener.COPY);
	copyButton.setToolTipText ((String)cc.rb.getObject("GUI_framework_ToolBar_copytooltip"));
	copyButton.addActionListener (mal);
	copyButton.setMaximumSize (new Dimension (31,31));
	copyButton.setMinimumSize (new Dimension (31,31));
	copyButton.setPreferredSize (new Dimension (31,31));
	add(copyButton);
	// COPY MODE
	copymodeToggle = new JToggleButton (new ImageIcon ((cc.op.get(Options.general_devisorhome))+"/images/Copymode.gif"));
	copymodeToggle.setActionCommand (MainActionListener.COPYMODE);
	copymodeToggle.setToolTipText ((String)cc.rb.getObject("GUI_framework_ToolBar_copymodetooltip"));
	copymodeToggle.addActionListener (mal);
	copymodeToggle.setMaximumSize (new Dimension (31,31));
	copymodeToggle.setMinimumSize (new Dimension (31,31));
	copymodeToggle.setPreferredSize (new Dimension (31,31));
	add(copymodeToggle);
	// PASTE
	pasteToggle = new JToggleButton(new ImageIcon((cc.op.get(Options.general_devisorhome))+"/images/Paste.gif"));
	pasteToggle.setActionCommand (MainActionListener.PASTE);
	pasteToggle.setToolTipText ((String)cc.rb.getObject("GUI_framework_ToolBar_pastetooltip"));
	pasteToggle.addActionListener (mal);
	pasteToggle.setMaximumSize (new Dimension (31,31));
	pasteToggle.setMinimumSize (new Dimension (31,31));
	pasteToggle.setPreferredSize (new Dimension (31,31));
	add(pasteToggle);
	addSeparator ();
	// NODE SELECTION
	selectNodeToggle = new JToggleButton (new ImageIcon ((cc.op.get(Options.general_devisorhome))+"/images/SelectNodes.gif"));
	selectNodeToggle.setActionCommand (MainActionListener.SELECTNODES);
	selectNodeToggle.setToolTipText ((String)cc.rb.getObject("GUI_framework_ToolBar_selectnodestooltip"));
	selectNodeToggle.addActionListener (mal);
	selectNodeToggle.setMaximumSize (new Dimension (31,31));
	selectNodeToggle.setMinimumSize (new Dimension (31,31));
	selectNodeToggle.setPreferredSize (new Dimension (31,31));
	add(selectNodeToggle);
	// ELEMENT SELECTION
	selectElementToggle = new JToggleButton (new ImageIcon ((cc.op.get(Options.general_devisorhome))+"/images/SelectElements.gif"));
	selectElementToggle.setActionCommand (MainActionListener.SELECTELEMENTS);
	selectElementToggle.setToolTipText ((String)cc.rb.getObject("GUI_framework_ToolBar_selectelementstooltip"));
	selectElementToggle.addActionListener (mal);
	selectElementToggle.setMaximumSize (new Dimension (31,31));
	selectElementToggle.setMinimumSize (new Dimension (31,31));
	selectElementToggle.setPreferredSize (new Dimension (31,31));
	add(selectElementToggle);
	// SEGMENT SELECTION
	selectSegmentToggle = new JToggleButton (new ImageIcon ((cc.op.get(Options.general_devisorhome))+"/images/SelectSegments.gif"));
	selectSegmentToggle.setActionCommand (MainActionListener.SELECTSEGMENTS);
	selectSegmentToggle.setToolTipText ((String)cc.rb.getObject("GUI_framework_ToolBar_selectsegmentstooltip"));
	selectSegmentToggle.addActionListener (mal);
	selectSegmentToggle.setMaximumSize (new Dimension (31,31));
	selectSegmentToggle.setMinimumSize (new Dimension (31,31));
	selectSegmentToggle.setPreferredSize (new Dimension (31,31));
	add(selectSegmentToggle);
	// EDGE SELECTION
	selectEdgesToggle = new JToggleButton (new ImageIcon ((cc.op.get(Options.general_devisorhome))+"/images/SelectEdges.gif"));
	selectEdgesToggle.setActionCommand (MainActionListener.SELECTEDGES);
	selectEdgesToggle.setToolTipText ((String)cc.rb.getObject("GUI_framework_ToolBar_selectedgestooltip"));
	selectEdgesToggle.addActionListener (mal);
	selectEdgesToggle.setMaximumSize (new Dimension (31,31));
	selectEdgesToggle.setMinimumSize (new Dimension (31,31));
	selectEdgesToggle.setPreferredSize (new Dimension (31,31));
	add(selectEdgesToggle);
	// BOUNDARY SELECTION
	selectBoundaryCombo = new JComboBox ();
	selectBoundaryCombo.setToolTipText ((String)cc.rb.getObject("GUI_framework_ToolBar_selectboundarytooltip"));
	selectBoundaryCombo.setActionCommand (MainActionListener.SELECTBOUNDARYCOMBO);
	selectBoundaryCombo.addActionListener (mal);
	selectBoundaryCombo.setMinimumSize (new Dimension (60,31));
	selectBoundaryCombo.setMaximumSize (new Dimension (60,31));
	selectBoundaryCombo.setPreferredSize (new Dimension (60,31));
	selectBoundaryCombo.setEditable (false);
	add (selectBoundaryCombo);
	addSeparator ();
	// TOGGLE ZOOMING
	setZoomToggle = new JToggleButton (new ImageIcon ((cc.op.get(Options.general_devisorhome))+"/images/Zoom.gif"));
	setZoomToggle.setActionCommand (MainActionListener.ZOOM_SETZOOM);
	setZoomToggle.setToolTipText ((String)cc.rb.getObject("GUI_framework_ToolBar_setzoomtooltip"));
	setZoomToggle.addActionListener (mal);
	setZoomToggle.setMaximumSize (new Dimension (31,31));
	setZoomToggle.setMinimumSize (new Dimension (31,31));
	setZoomToggle.setPreferredSize (new Dimension (31,31));
	add(setZoomToggle);
	// RESET ZOOM
	resetZoomButton = new JButton (new ImageIcon ((cc.op.get(Options.general_devisorhome))+"/images/ResetZoom.gif"));
	resetZoomButton.setActionCommand (MainActionListener.ZOOM_RESETZOOM);
	resetZoomButton.setToolTipText ((String)cc.rb.getObject("GUI_framework_ToolBar_resetzoomtooltip"));
	resetZoomButton.addActionListener (mal);
	resetZoomButton.setMaximumSize (new Dimension (31,31));
	resetZoomButton.setMinimumSize (new Dimension (31,31));
	resetZoomButton.setPreferredSize (new Dimension (31,31));
	add(resetZoomButton);
	// ZOOM IN
	zoomInToggle = new JToggleButton (new ImageIcon ((cc.op.get(Options.general_devisorhome))+"/images/ZoomIn.gif"));
	zoomInToggle.setActionCommand (MainActionListener.ZOOM_ZOOMPLUS);
	zoomInToggle.setToolTipText ((String)cc.rb.getObject("GUI_framework_ToolBar_zoomintooltip"));
	zoomInToggle.addActionListener (mal);
	zoomInToggle.setMaximumSize (new Dimension (31,31));
	zoomInToggle.setMinimumSize (new Dimension (31,31));
	zoomInToggle.setPreferredSize (new Dimension (31,31));
	add(zoomInToggle);
	// ZOOM OUT
	zoomOutButton = new JButton (new ImageIcon ((cc.op.get(Options.general_devisorhome))+"/images/ZoomOut.gif"));
	zoomOutButton.setActionCommand (MainActionListener.ZOOM_ZOOMMINUS);
	zoomOutButton.setToolTipText ((String)cc.rb.getObject("GUI_framework_ToolBar_zoomouttooltip"));
	zoomOutButton.addActionListener (mal);
	zoomOutButton.setMaximumSize (new Dimension (31,31));
	zoomOutButton.setMinimumSize (new Dimension (31,31));
	zoomOutButton.setPreferredSize (new Dimension (31,31));
	add(zoomOutButton);
	addSeparator ();
	// NEW BOUNDARY CONTAINER
	newBoundaryButton = new JButton (new ImageIcon ((cc.op.get(Options.general_devisorhome))+"/images/NewBoundary.gif"));
	newBoundaryButton.setActionCommand (MainActionListener.NEWBOUNDARY);
	newBoundaryButton.setToolTipText ((String)cc.rb.getObject("GUI_framework_ToolBar_newboundarytooltip"));
	newBoundaryButton.addActionListener (mal);
	newBoundaryButton.setMaximumSize (new Dimension (31,31));
	newBoundaryButton.setMinimumSize (new Dimension (31,31));
	newBoundaryButton.setPreferredSize (new Dimension (31,31));
	add(newBoundaryButton);
	// DIRECT INPUT TOGGLE
	directInputToggle = new JToggleButton (new ImageIcon ((cc.op.get(Options.general_devisorhome))+"/images/DirectInput.gif"));
	directInputToggle.setActionCommand (MainActionListener.DIRECTINPUT);
	directInputToggle.setToolTipText ((String)cc.rb.getObject("GUI_framework_ToolBar_directinputtooltip"));
	directInputToggle.addActionListener (mal);
	directInputToggle.setMaximumSize (new Dimension (31,31));
	directInputToggle.setMinimumSize (new Dimension (31,31));
	directInputToggle.setPreferredSize (new Dimension (31,31));
	add(directInputToggle);
	// ADD TYPE COMBO
        addTypeCombo = new JComboBox(comboImages);
	addTypeCombo.setMaximumSize (new Dimension (55,31));
	addTypeCombo.setMinimumSize (new Dimension (55,31));
	addTypeCombo.setPreferredSize (new Dimension (55,31));
        addTypeCombo.setMaximumRowCount(comboImages.length);
	addTypeCombo.setEditable(false);
	addTypeCombo.setToolTipText ((String)cc.rb.getObject ("GUI_framework_ToolBar_addtypetooltip"));
	addTypeCombo.addActionListener (mal);
	addTypeCombo.setActionCommand (MainActionListener.ADDTYPECHANGED);
	add (addTypeCombo);
	addSeparator ();
	//PROPERTIES	
	propertiesToggle = new JToggleButton(new ImageIcon(((String)cc.op.get(Options.general_devisorhome))+"/images/Properties.gif"));
	propertiesToggle.setActionCommand (MainActionListener.PROPERTIES);
	propertiesToggle.setToolTipText ((String)cc.rb.getObject("GUI_framework_ToolBar_propertiestooltip"));
	propertiesToggle.addActionListener (mal);
	propertiesToggle.setMaximumSize (new Dimension (31,31));
	propertiesToggle.setMinimumSize (new Dimension (31,31));
	propertiesToggle.setPreferredSize (new Dimension (31,31));
	add(propertiesToggle);
	//EDGE STATUS	
	edgeStatusToggle = new JToggleButton(new ImageIcon(((String)cc.op.get(Options.general_devisorhome))+"/images/EdgeStatus.gif"));
	edgeStatusToggle.setActionCommand (MainActionListener.EDGESTATUS);
	edgeStatusToggle.setToolTipText ((String)cc.rb.getObject("GUI_framework_ToolBar_edgestatustooltip"));
	edgeStatusToggle.addActionListener (mal);
	edgeStatusToggle.setMaximumSize (new Dimension (31,31));
	edgeStatusToggle.setMinimumSize (new Dimension (31,31));
	edgeStatusToggle.setPreferredSize (new Dimension (31,31));
	add(edgeStatusToggle);
	// PARALLELBLOCK	
	parallelblockToggle = new JToggleButton(new ImageIcon(((String)cc.op.get(Options.general_devisorhome))+"/images/Parallelblock.gif"));
	parallelblockToggle.setActionCommand (MainActionListener.PARALLELBLOCK);
	parallelblockToggle.setToolTipText ((String)cc.rb.getObject("GUI_framework_ToolBar_parallelblocktooltip"));
	parallelblockToggle.addActionListener (mal);
	parallelblockToggle.setMaximumSize (new Dimension (31,31));
	parallelblockToggle.setMinimumSize (new Dimension (31,31));
	parallelblockToggle.setPreferredSize (new Dimension (31,31));
	add(parallelblockToggle);
	// RECTIFY ELEMENT	
	rectifyElementToggle = new JToggleButton(new ImageIcon(((String)cc.op.get(Options.general_devisorhome))+"/images/RectifyElement.gif"));
	rectifyElementToggle.setActionCommand (MainActionListener.RECTIFYELEMENT);
	rectifyElementToggle.setToolTipText ((String)cc.rb.getObject("GUI_framework_ToolBar_rectifyelementtooltip"));
	rectifyElementToggle.addActionListener (mal);
	rectifyElementToggle.setMaximumSize (new Dimension (31,31));
	rectifyElementToggle.setMinimumSize (new Dimension (31,31));
	rectifyElementToggle.setPreferredSize (new Dimension (31,31));
	rectifyElementToggle.setEnabled (false);
	add(rectifyElementToggle);
	addSeparator();
	//EXIT	
	exitButton = new JButton(new ImageIcon(((String)cc.op.get(Options.general_devisorhome))+"/images/Exit.gif"));
	exitButton.setActionCommand (MainActionListener.EXIT);
	exitButton.setToolTipText ((String)cc.rb.getObject("GUI_framework_ToolBar_exittooltip"));
	exitButton.addActionListener (mal);
	exitButton.setMaximumSize (new Dimension (31,31));
	exitButton.setMinimumSize (new Dimension (31,31));
	exitButton.setPreferredSize (new Dimension (31,31));
	add(exitButton);
    }
    
    /* gets and sets */
    public JToggleButton getPasteToggle ()
    {
	return pasteToggle;
    }
    public JComboBox getSelectBoundaryCombo ()
    {
	return selectBoundaryCombo;
    }
    public JToggleButton getSelectNodeToggle ()
    {
	return selectNodeToggle;
    }
    public JToggleButton getSelectElementToggle ()
    {
	return selectElementToggle;
    }
    public JToggleButton getSelectSegmentToggle ()
    {
	return selectSegmentToggle;
    }
    public JToggleButton getSelectEdgesToggle ()
    {
	return selectEdgesToggle;
    }
    public JToggleButton getSetZoomToggle ()
    {
	return setZoomToggle;
    }
    public JToggleButton getZoomInToggle ()
    {
	return zoomInToggle;
    }
    public JToggleButton getDirectInputToggle ()
    {
	return directInputToggle;
    }
    public JToggleButton getCopymodeToggle ()
    {
	return copymodeToggle;
    }
    public JComboBox getAddTypeCombo ()
    {
	return addTypeCombo;
    }
    public JToggleButton getEdgeStatusToggle ()
    {
	return edgeStatusToggle;
    }
    public JToggleButton getParallelblockToggle ()
    {
	return parallelblockToggle;
    }
    public JToggleButton getRectifyElementToggle ()
    {
	return rectifyElementToggle;
    }
    public JToggleButton getPropertiesToggle ()
    {
	return propertiesToggle;
    }



}// end of class



