package devisor2.grid.backend;

import java.awt.event.*;
import java.awt.*;
import java.awt.geom.*;
import java.util.*;

import devisor2.grid.GUI.framework.*;
import devisor2.grid.options.*;
import devisor2.grid.GUI.event.*;
import devisor2.foundation.elements.*;
import devisor2.foundation.base.*;
import devisor2.foundation.boundary.*;
import devisor2.foundation.*;


/**
 *  This class finally adds the GUI feedback for scaling, translating and 
 *  rotating to the BaseSelector. Only use this class for full support, 
 *  the other ones are not instanciable.
 *
 *  @author Dominik Goeddeke
 *  @version 0.9 release candidate
 */
public class GridSelector extends GridHotspotter
{
  
    // the baseline relative to which rotating is calculated
    protected int baseline_x;
    protected int baseline_y;
    // the line which defines the angle
    protected int vector_x;
    protected int vector_y;
    // the angle between the baseline and the vector
    protected double angle;
    // a backup of the perimeter
    protected Rectangle rotateBackup;
    // and the explicit corners of it
    protected Point topleft, topright, bottomleft, bottomright;
    protected Point2D newtopleft, newtopright, newbottomleft, newbottomright;
    // the rotate matrix (homogenous coordinates, screen coord system)
    protected AffineTransform rotmatrix;
 

    /**
     *  The constructor just sets references correctly
     *  and initializes some variables
     *
     *  @param mf - the MainFrame reference
     *  @param da - the drawing area where the select box will be painted on
     */    
    public GridSelector (MainFrame mf, DrawingArea da) 
    {
	super (mf,da);
    }


    //////////////////////////////////////////////////////////
    //
    //  SCALE, TRANSLATE, ROTATE
    //
    /////////////////////////////////////////////////////////


    /**
     *  performs the update of the select box for the move operation
     *
     *  @param x,y - the event point which marks the new center of the box
     *  @param first - hack to indicate that a third drawing is neccessary
     */
    public void translateSelectBox (int x, int y, boolean first)
    {
	g.setXORMode (da.getBackground ());
	g.setColor (cc.dc.color_lasso);
	if (first)
	{
	    if (nodesonly)
	    {
		g.drawRect (perimeter.x-4, perimeter.y-4, perimeter.width +7, perimeter.height +7);
	    }
	    else
	    {
		Stroke savestroke = g.getStroke ();
		g.setStroke (dashed);
		g.drawRect (perimeter.x, perimeter.y, perimeter.width - 1, perimeter.height - 1);
		g.setStroke (savestroke);
	    }
	}
	// paint over old box
	if (nodesonly)
	{
	    g.drawRect (perimeter.x-4, perimeter.y-4, perimeter.width +7, perimeter.height +7);
	}
	else
	{
	    Stroke savestroke = g.getStroke ();
	    g.setStroke (dashed);
	    g.drawRect (perimeter.x, perimeter.y, perimeter.width - 1, perimeter.height - 1);
	    g.setStroke (savestroke);
	}
	// update rectangle
	perimeter.x = x - perimeter.width/2;
	perimeter.y = y - perimeter.height/2;
	// and paint it 
	if (nodesonly)
	{
	    g.drawRect (perimeter.x-4, perimeter.y-4, perimeter.width +7, perimeter.height +7);
	}
	else
	{
	    Stroke savestroke = g.getStroke ();
	    g.setStroke (dashed);
	    g.drawRect (perimeter.x, perimeter.y, perimeter.width - 1, perimeter.height - 1);
	    g.setStroke (savestroke);
	}
	g.setPaintMode ();
    }

    /**
     *  a private delegation of the drawing process, which due to some 
     *  strange results is performed manually to allow for negative rectangle
     *  width and height
     */
    private void drawPerimeter ()
    {
	Stroke savestroke = g.getStroke();
	g.setStroke (dashed);
	g.drawLine (perimeter.x, perimeter.y, perimeter.x+perimeter.width, perimeter.y);
	g.drawLine (perimeter.x+perimeter.width, perimeter.y, perimeter.x+perimeter.width, perimeter.y+perimeter.height);
	g.drawLine (perimeter.x+perimeter.width, perimeter.y+perimeter.height, perimeter.x, perimeter.y+perimeter.height);
	g.drawLine (perimeter.x, perimeter.y+perimeter.height, perimeter.x, perimeter.y);
	g.setStroke (savestroke);
    }

    /**
     *  starts the scaling GUI feedback by setting the neccessary parameters
     *
     *  @param hotspot - the hotspot with which to scale
     */
    public void startScaleSelectBox (int hotspot)
    {
	switch (hotspot)
	    {
	    case HOTSPOT_TOPLEFT :
		{ 
		    perimeter.y = perimeter.y + perimeter.height;
		    perimeter.x = perimeter.x + perimeter.width;
		    perimeter.height = -perimeter.height;
		    perimeter.width = -perimeter.width;
		    break;
		}
	    case HOTSPOT_TOPCENTER :
		{
		    perimeter.y = perimeter.y + perimeter.height;
		    perimeter.height = -perimeter.height;
		    break;
		}
	    case HOTSPOT_TOPRIGHT :
		{
		    perimeter.y = perimeter.y + perimeter.height;
		    perimeter.height = -perimeter.height;
		    break;
		}
	    case HOTSPOT_CENTERLEFT :
		{
		    perimeter.x = perimeter.x + perimeter.width;
		    perimeter.width = -perimeter.width;
		    break;
		}
	    case HOTSPOT_BOTTOMLEFT :
		{
		    perimeter.x = perimeter.x + perimeter.width;
		    perimeter.width = -perimeter.width;
		}
	    }
    }
    

    /**
     *  performs the update of the select box for the scale operation
     *
     *  @param x,y - the event point which marks the new position of the ...
     *  @param hotspot - the hotspot through which scaling is applied
     *  @param first - hack to indicate that a third drawing is neccessary
     */
    public void scaleSelectBox (int x, int y, int hotspot, boolean first)
    {
	g.setXORMode (da.getBackground ());
	g.setColor (cc.dc.color_lasso);
	if (first)
	{
	    drawPerimeter ();
	}
	// paint over old box
	drawPerimeter ();
	// update rectangle: according to what hotspot is used and where we 
	// are, the rectangle's position and size is updated
	// things are getting complicated, because the user can do all sorts
	// of nasty things, like crossing the zero height or width border
	// thus, some tedious special cases have to be considered
	switch (hotspot)
	    {
	    case HOTSPOT_TOPLEFT :
		{ 
		    perimeter.width = x - perimeter.x;
		    perimeter.height = y - perimeter.y;
		    break;
		}
	    case HOTSPOT_TOPCENTER :
		{
		    perimeter.height = y - perimeter.y;
		    break;
		}
	    case HOTSPOT_TOPRIGHT :
		{  
		    perimeter.width = x - perimeter.x;
		    perimeter.height = y - perimeter.y;
		}
	    case HOTSPOT_CENTERLEFT :
		{
		    perimeter.width = x - perimeter.x;
		    break;
		}
	    case HOTSPOT_CENTERRIGHT :
		{
		    perimeter.width = x - perimeter.x;
		    break;
		}
	    case HOTSPOT_BOTTOMLEFT :
		{
		    perimeter.width = x - perimeter.x;
		    perimeter.height = y - perimeter.y;
		}
	    case HOTSPOT_BOTTOMCENTER :
		{
		    perimeter.height = y - perimeter.y;
		    break;
		}
	    case HOTSPOT_BOTTOMRIGHT :
		{
		    perimeter.width = x - perimeter.x;
		    perimeter.height = y - perimeter.y;
		    break;
		}
	    }
	// and paint it 
	drawPerimeter ();
	g.setPaintMode ();
    }

    /**
     *  fixes negative orientation of the perimeter variable
     */
    public void endScaleSelectBox ()
    {
	if (perimeter.width < 0)
	{
	    perimeter.x = perimeter.x + perimeter.width;
	    perimeter.width = -perimeter.width;
	}
	if (perimeter.height < 0)
	{
	    perimeter.y = perimeter.y + perimeter.height;
	    perimeter.height = -perimeter.height;
	}

    }

    /**
     *  draws the perimeter given by the four points
     */
    private void drawPerimeter (Point2D tl, Point2D tr, Point2D bl, Point2D br)
    {
	Stroke savestroke = g.getStroke ();
	g.setStroke (dashed);
	g.drawLine ((int)tl.getX(), (int)tl.getY(), (int)tr.getX(), (int)tr.getY());
	g.drawLine ((int)tr.getX(), (int)tr.getY(), (int)br.getX(), (int)br.getY());
	g.drawLine ((int)br.getX(), (int)br.getY(), (int)bl.getX(), (int)bl.getY());
	g.drawLine ((int)bl.getX(), (int)bl.getY(), (int)tl.getX(), (int)tl.getY());
	g.setStroke (savestroke);
    }


    /**
     *  starts the rotate GUI feedback by defining the baseline against
     *  which to rotate
     *
     *  @param hotspot - the hotspot clicked on
     */
    public void startRotateSelectBox (int hotspot)
    {
	// get a copy of the perimeter to calculate on
	rotateBackup = new Rectangle (perimeter);
	// calc the Point representation of the interesting coordinates
	newtopleft = topleft = new Point (rotateBackup.x, rotateBackup.y);
	newtopright = topright = new Point (rotateBackup.x+rotateBackup.width, rotateBackup.y);
	newbottomleft = bottomleft = new Point (rotateBackup.x, rotateBackup.y+rotateBackup.height);
	newbottomright = bottomright = new Point (rotateBackup.x+rotateBackup.width, rotateBackup.y+rotateBackup.height);
	// define baseline
	switch (hotspot)
	{
	    case HOTSPOT_TOPCENTER :
	    {
		baseline_x = 0;
		baseline_y = -perimeter.height/2;
		//System.out.println ("baseline "+baseline_x+" "+baseline_y);
		break;
	    }
	    case HOTSPOT_CENTERLEFT :
	    {
		baseline_x = -perimeter.width/2;
		baseline_y = 0;
		//System.out.println ("baseline "+baseline_x+" "+baseline_y);
		break;
	    }
	    case HOTSPOT_CENTERRIGHT :
	    {
		baseline_x = perimeter.width/2;
		baseline_y = 0;
		//System.out.println ("baseline "+baseline_x+" "+baseline_y);
		break;
	    }
	    case HOTSPOT_BOTTOMCENTER :
	    {
		baseline_x = 0;
		baseline_y = perimeter.height/2;
		//System.out.println ("baseline "+baseline_x+" "+baseline_y);
		break;
	    }
	}
   }

    /**
     *  rotates the select box around the angle defined by the baseline and 
     *  and the line from the perimeter's center to the current mouse position
     *
     *  @param x,y - the current mouse position
     *  @param hotspot - the hotspot originally defining the baseline
     *  @param first - flag to hack, same as above
     */
    public void rotateSelectBox (int x, int y, int hotspot, boolean first)
    {
	g.setXORMode (da.getBackground ());
	g.setColor (cc.dc.color_lasso);
	// paint over old box
	if (first)
	{
	    drawPerimeter (newtopleft, newtopright, newbottomleft, newbottomright);
	}
	drawPerimeter (newtopleft, newtopright, newbottomleft, newbottomright);
	// fix bad user behaviour
	if (x == rotateBackup.x+rotateBackup.width/2)
	    x += 10;
	if (y == rotateBackup.y+rotateBackup.height/2)
	    y += 10;
	// calc second vector
	vector_x = x - rotateBackup.x - (rotateBackup.width/2);
	vector_y = y - rotateBackup.y - (rotateBackup.height/2);
	//System.out.println ("vector : "+vector_x +" "+vector_y);
	// calculate angle 
	double baselength = Math.sqrt (baseline_x*baseline_x + baseline_y*baseline_y);
	double veclength = Math.sqrt (vector_x*vector_x + vector_y*vector_y);
	double scalarproduct = baseline_x*vector_x + baseline_y*vector_y;
	angle = Math.acos (scalarproduct / (baselength*veclength));
	// make angle 2pi-periodic
	switch (hotspot)
	{
	    case HOTSPOT_TOPCENTER :
	    {
		if (x > rotateBackup.x+rotateBackup.width/2)
		    angle = 2*Math.PI - angle;
		break;
	    }
	    case HOTSPOT_CENTERLEFT :
	    {
		if (y > rotateBackup.y+rotateBackup.height/2)
		    angle = 2*Math.PI - angle; 
		break;
	    }
	    case HOTSPOT_CENTERRIGHT :
	    {
		if (y < rotateBackup.y+rotateBackup.height/2)
		    angle = 2*Math.PI - angle;
		break;
	    }
	    case HOTSPOT_BOTTOMCENTER :
	    { if (x < rotateBackup.x+rotateBackup.width/2)
		angle = 2*Math.PI - angle;
	    break;
	    }
	}
	// transform a bit (for GUI support)
	if (hotspot == HOTSPOT_TOPCENTER || hotspot == HOTSPOT_BOTTOMCENTER)
	    angle = (-angle);
	rotmatrix = AffineTransform.getRotateInstance (angle,rotateBackup.x+rotateBackup.width/2, rotateBackup.y+rotateBackup.height/2);
	newtopleft = rotmatrix.transform (topleft, null);
	newtopright = rotmatrix.transform (topright, null);
	newbottomleft = rotmatrix.transform (bottomleft, null);
	newbottomright = rotmatrix.transform (bottomright, null);
	perimeter.x = (int)newtopleft.getX();
	perimeter.y = (int)newtopleft.getY();
	perimeter.width = (int)(newbottomright.getX() - newtopleft.getX());
	perimeter.height = (int)(newbottomright.getY() - newtopleft.getY());
	// and draw again
	drawPerimeter (newtopleft, newtopright, newbottomleft, newbottomright);
	g.setPaintMode ();
    }

    /**
     *  @return the angle which defines the rotation
     */
    public double getRotateAngle ()
    {
	return angle;
    }

 
}






