package devisor2.grid.backend.spellchecker;


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.filechooser.*;
import java.io.*;
import java.util.*;

import devisor2.grid.options.*;
import devisor2.grid.GUI.framework.*;
import devisor2.grid.GUI.dialogs.*;
import devisor2.grid.GUI.event.*;
import devisor2.foundation.*;
import devisor2.foundation.elements.*;
import devisor2.foundation.boundary.*;
import devisor2.foundation.base.*;
/** 
 *
 *  This class is just the Action Listener for the SpellCheckerDialog.
 *  As usual, string liter
 *  @author Dominik Goeddeke
 *  @version 0.9
 */
class SpellCheckerActionListener implements ActionListener
{
    /** reference to the main window of the application */
    private MainFrame parent;

    /** reference to the SpellCheckerDialog this listener is responsible for */
    private SpellCheckerDialog dialog;

    /** reference to the ControlCenter */
    private ControlCenter cc;

    /** 
     *  The constructor just sets the reference correctly
     *
     *  @param parent - the mainframe instance, the link to the rest
     *                  of the application
     *  @param dialog - a reference to the SpellCheckerDialog instance this
     *                  class listens to
     *  
     */
    public SpellCheckerActionListener (MainFrame parent, SpellCheckerDialog dialog)
    {
	this.parent = parent;
	this.dialog = dialog;
	cc = ControlCenter.getMyself ();
    }
    
    /** 
     *  The actionPerformed() method is just a king-size switch-block
     *  all actions this method performs must be declared as String
     *  constants in this class (see below)
     *  It is called by the JRE everytime an ActionEvent is raised.
     */
    public void actionPerformed(ActionEvent e) 
    {
    	/* Action Command CLOSE */
	if (e.getActionCommand ().equals(CLOSE))
	{
	    cc.dm.hide (cc.dm.SPELLCHECKERDIALOG);
	    return;
	}

	/* Action Command REMOVE */
	if (e.getActionCommand ().equals(REMOVE))
	{
	    int selectedRow = dialog.getTable().getSelectedRow();
	    dialog.getTableData().removeElementAt (selectedRow);
	    dialog.repaint ();
	    return;
	}


	/* Action Command ZOOM */
	if (e.getActionCommand ().equals(ZOOM))
	{
	    GridItem item = dialog.getSelectedItem ();
	    int[] peri = cc.d.getPerimeter ();
	    int margin_x = (peri[2]-peri[0])/5;
	    int margin_y = (peri[3]-peri[1])/5;
	    if (item instanceof Node)
	    {
		Node n = (Node)item;
		parent.getAdjustment().newZoom (new Rectangle(n.getCoord(0)-margin_x, n.getCoord(1)+margin_y, 2*margin_x, 2*margin_y));
		parent.getDrawingArea().repaint();
		return;
	    }
	    if (item instanceof Cell2D)
	    {
		Node n = ((Cell2D)item).updateCellNodes()[0];
		parent.getAdjustment().newZoom (new Rectangle(n.getCoord(0)-margin_x, n.getCoord(1)+margin_y, 2*margin_x, 2*margin_y));
		parent.getDrawingArea().repaint();
		return;
	    }

	    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject("spellchecker_dialog_error_noselection"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
	    return;
	}

	/* Action Command REFRESH */
	if (e.getActionCommand ().equals(REFRESH))
	{
	    dialog.updateTable ();
	    dialog.repaint ();
	    return;
	}

	/* Action Command MOREINFO */
	if (e.getActionCommand ().equals(MOREINFO))
	{
	    GridItem item = dialog.getSelectedItem ();
	    if (item instanceof BoundaryNode)
	    {
		BoundaryNode n = (BoundaryNode)item;
		BoundaryNodeDialog dialog = (BoundaryNodeDialog)cc.dm.get (cc.dm.BOUNDARYNODEDIALOG);
		cc.dm.hide (cc.dm.BOUNDARYNODEDIALOG);
		dialog.setTitle ((String)cc.rb.getObject("dialogs_boundarynode_caption_property"));
		dialog.setSource (n);
		dialog.setSelectedTab (1);
		dialog.setVisible (true);
		return;
	    }
	    if (item instanceof Node)
	    {
		Node n = (Node)item;
		BoundaryNodeDialog dialog = (BoundaryNodeDialog)cc.dm.get (cc.dm.BOUNDARYNODEDIALOG);
		cc.dm.hide (cc.dm.BOUNDARYNODEDIALOG);
		dialog.setTitle ((String)cc.rb.getObject("dialogs_boundarynode_caption_property"));
		dialog.setSource (n);
		dialog.setSelectedTab (0);
		dialog.setVisible (true);
		return;
	    }
	    if (item instanceof Tri)
	    {
		Tri t = (Tri)item;
		TriDialog dialog = (TriDialog)cc.dm.get (cc.dm.TRIDIALOG);
		cc.dm.hide (cc.dm.TRIDIALOG);
		dialog.setTitle ((String)cc.rb.getObject("dialogs_tri_caption_property"));
		dialog.setSource (t);
		dialog.setVisible (true);
		return;
	    }
	    if (item instanceof Macro)
	    {
		Macro m = (Macro)item;
		MacroDialog dialog = (MacroDialog)cc.dm.get (cc.dm.MACRODIALOG);
		cc.dm.hide (cc.dm.MACRODIALOG);
		dialog.setTitle ((String)cc.rb.getObject("dialogs_macro_caption_property"));
		dialog.setSource (m);
		dialog.setSelectedTab (1);
		dialog.setVisible (true);
		return;
	    }
	    if (item instanceof Quad)
	    {
		Quad q = (Quad)item;
		QuadDialog dialog = (QuadDialog)cc.dm.get (cc.dm.QUADDIALOG);
		cc.dm.hide (cc.dm.QUADDIALOG);
		dialog.setTitle ((String)cc.rb.getObject("dialogs_quad_caption_property"));
		dialog.setSource (q);
		dialog.setVisible (true);
		return;
	    }


	    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject("spellchecker_dialog_error_noselection"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
	    return;
	}


	return;
    }
    

    /**
     *  ActionCommand for the CLOSE button
     */
    public static final String CLOSE = "CLOSE";
    /**
     *  ActionCommand for the ZOOM button
     */
    public static final String ZOOM = "ZOOM";
    /**
     *  ActionCommand for the REMOVE button
     */
    public static final String REMOVE = "REMOVE";    
    /**
     *  ActionCommand for the REFRESH button
     */
    public static final String REFRESH = "REFRESH";
    /**
     *  ActionCommand for the MOREINFO button
     */
    public static final String MOREINFO = "MOREINFO";


}
