package devisor2.grid.backend.spellchecker;

import java.util.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

import devisor2.grid.options.*;
import devisor2.grid.GUI.framework.*;
import devisor2.grid.GUI.event.*;
import devisor2.grid.GUI.dialogs.*;
import devisor2.foundation.*;
import devisor2.foundation.elements.*;
import devisor2.foundation.boundary.*;
import devisor2.foundation.base.*;

/** 
 *  This class represents the SpellChecker dialog which provides access to the
 *  spellchecking functionalities: <br>
 *  - searching for bad or malformed elements etc and presenting them in 
 *    a table <br>
 *  - zooming in onto these elements by single selection in the table
 *  - configuring the spellchecker
 *
 *  @author Dominik Goeddeke
 *  @version 0.5 (half of the functions added) 
 */
public class SpellCheckerDialog extends DockableDialog
{

    // the parent frame where the dialog is centered on
    private MainFrame parent;
    // the actionlistener object
    private SpellCheckerActionListener actionlistener;


    /**
     *  this method can be used to update the GUI of the dialog dynamically.
     *  it is called by the DialogManager e.g. when some comboboxes containing
     *  boundary information need to be refreshed
     */
    public void refreshGUI ()
    {
	// no need for this in this class :-)
	return;
    }


    /** 
     *  returns the initial Position of the dialog
     */
    public Point getInitialPosition ()
    {
	return new Point (parent.getLocationOnScreen().x - 50, parent.getLocationOnScreen().y);
    }


    /**
     *  the constructor just sets references correctly and creates the GUI
     */
    public SpellCheckerDialog (MainFrame parent)
    {
	// basics and attribute initialisation
	super (parent);
	this.parent = parent;
	actionlistener = new SpellCheckerActionListener (parent,this);
	cc = ControlCenter.getMyself();
	addComponentListener (listener);
	// gui and event handling
	initDialog ();
	initTablePanel ();
	initSettingsPanel ();
	// ok all done
	getContentPane().add(tabs,BorderLayout.CENTER);
	pack();
    }

    /**
     * creates basic layout, the south panel for the buttons, 
     * positioning etc. 
     */
    private void initDialog ()
    {
	tabs = new JTabbedPane ();
	getContentPane().setLayout (new BorderLayout());
	setTitle ((String)cc.rb.getObject ("spellchecker_dialog_title"));
	setDefaultCloseOperation (DO_NOTHING_ON_CLOSE);
	// south panel includes a couple of buttons
	JPanel buttons = new JPanel ();
	buttons.setLayout (new FlowLayout ());
	closeButton = new JButton ((String)cc.rb.getObject ("general_Close"));
	closeButton.setToolTipText ((String)cc.rb.getObject ("spellchecker_dialog_closebutton_tooltip"));
	closeButton.setMnemonic (((Integer)cc.rb.getObject("general_Close_mnemonic")).intValue());
	closeButton.addActionListener (actionlistener);
	closeButton.setActionCommand (SpellCheckerActionListener.CLOSE);
	buttons.add(closeButton);
	getContentPane().add (buttons,BorderLayout.SOUTH);
    }

    /** creates the GUI for the settings tab */
    private void initSettingsPanel ()
    {
	settingsPanel = new JPanel ();
	settingsPanel.setBorder (new EmptyBorder (10,10,10,10));
	GridBagLayout layout = new GridBagLayout ();
	GridBagConstraints constraints = new GridBagConstraints ();
	settingsPanel.setLayout (layout);
	constraints.fill = GridBagConstraints.HORIZONTAL;
	constraints.weightx = 1.0;
	// isolated nodes
	JPanel isolatednodes = new JPanel ();
	isolatednodes.setLayout (new FlowLayout ());
	isolatednodesCheck = new JCheckBox ((String)cc.rb.getObject ("spellchecker_dialog_isolatednodes"));
	isolatednodes.add (isolatednodesCheck);
	constraints.gridy = 0;
	constraints.gridx = 0;
	layout.setConstraints(isolatednodes, constraints);
	settingsPanel.add (isolatednodes);
	isolatednodesCheck.setSelected (true);
	// inner angles
	JPanel innerangles = new JPanel ();
	innerangles.setLayout (new FlowLayout ());
	inneranglesCheck = new JCheckBox ((String)cc.rb.getObject ("spellchecker_dialog_innerangles"));
	innerangles.add (inneranglesCheck);
	inneranglesEdit = new JTextField (5);
	inneranglesEdit.setText ("45.0");
	innerangles.add (inneranglesEdit);
	constraints.gridy = 1;
	layout.setConstraints(innerangles, constraints);
	settingsPanel.add (innerangles);
	inneranglesCheck.setSelected (false);
	// aspect ratios
	JPanel aspectratios = new JPanel ();
	aspectratios.setLayout (new FlowLayout ());
	aspectratiosCheck = new JCheckBox ((String)cc.rb.getObject ("spellchecker_dialog_aspectratios"));
	aspectratios.add (aspectratiosCheck);
	aspectratiosEdit1 = new JTextField (5);
	aspectratiosEdit1.setText ("1");
	aspectratios.add (aspectratiosEdit1);
	aspectratios.add (new JLabel(":"));
	aspectratiosEdit2 = new JTextField (5);
	aspectratiosEdit2.setText ("5");
	aspectratios.add (aspectratiosEdit2);
	constraints.gridy = 2;
	layout.setConstraints(aspectratios, constraints);
	settingsPanel.add (aspectratios);
	aspectratiosCheck.setSelected (false);
	// neighbouring size
	JPanel neighbouring = new JPanel ();
	neighbouring.setLayout (new FlowLayout ());
	neighbouringCheck = new JCheckBox ((String)cc.rb.getObject ("spellchecker_dialog_neighbouring"));
	neighbouring.add (neighbouringCheck);
	neighbouringEdit1 = new JTextField (5);
	neighbouringEdit1.setText ("1");
	neighbouring.add (neighbouringEdit1);
	aspectratios.add (new JLabel(":"));
	neighbouringEdit2 = new JTextField (5);
	neighbouringEdit2.setText ("4");
	neighbouring.add (neighbouringEdit2);
	constraints.gridy = 3;
	layout.setConstraints(neighbouring, constraints);
	settingsPanel.add (neighbouring);
	neighbouringCheck.setSelected (false);
	// discontinous boundaries
	JPanel discontinous = new JPanel ();
	discontinous.setLayout (new FlowLayout ());
	discontinousCheck = new JCheckBox ((String)cc.rb.getObject ("spellchecker_dialog_discontinous"));
	discontinous.add (discontinousCheck);
	constraints.gridy = 4;
	layout.setConstraints(discontinous, constraints);
	settingsPanel.add (discontinous);
	// nonconforming elements
	JPanel nonconforming = new JPanel ();
	nonconforming.setLayout (new FlowLayout ());
	nonconformingCheck = new JCheckBox ((String)cc.rb.getObject ("spellchecker_dialog_nonconforming"));
	nonconforming.add (nonconformingCheck);
	constraints.gridy = 5;
	layout.setConstraints(nonconforming, constraints);
	settingsPanel.add (nonconforming);
	// done
	tabs.addTab ((String)cc.rb.getObject ("spellchecker_dialog_settings_tabheader"),null,settingsPanel,(String)cc.rb.getObject ("spellchecker_dialog_settings_tabheader_tooltip"));
    }


    /** creates the GUI for the table tab */
    private void initTablePanel ()
    {
	tablePanel = new JPanel ();
	tablePanel.setBorder (new EmptyBorder (10,10,10,10));
	GridBagLayout layout = new GridBagLayout ();
	GridBagConstraints constraints = new GridBagConstraints ();
	tablePanel.setLayout (layout);
	constraints.fill = GridBagConstraints.HORIZONTAL;
	constraints.weightx = 1.0;
	constraints.weighty = 1.0;
	// one JTable, one column
	JPanel tablecontainer = new JPanel ();
	table = new JTable ();
	updateTable ();
	table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	JScrollPane tablescroll = new JScrollPane (table);
	tablescroll.setPreferredSize (new Dimension (200,400));
	tablescroll.setMinimumSize (new Dimension (200,400));
	tablescroll.setMaximumSize (new Dimension (200,400));
	ListSelectionModel rowSM = table.getSelectionModel();
	lsListener = new SpellCheckerTableListSelectionListener(this);
	rowSM.addListSelectionListener(lsListener); 
	tablecontainer.add(tablescroll);
	constraints.gridx = 0;
	constraints.gridy = 0;
	layout.setConstraints(tablecontainer, constraints);
	tablePanel.add (tablecontainer);

	// ...and the buttons to zoom in etc
	JPanel buttons = new JPanel ();
	buttons.setLayout (new BoxLayout (buttons, BoxLayout.Y_AXIS));
	// zoom
	zoomButton = new JButton ((String)cc.rb.getObject ("spellchecker_dialog_zoom"));
	zoomButton.setToolTipText((String)cc.rb.getObject ("spellchecker_dialog_zoom_tooltip"));
	zoomButton.setMnemonic (((Integer)cc.rb.getObject("spellchecker_dialog_zoom_mnemonic")).intValue());
	zoomButton.addActionListener (actionlistener);
	zoomButton.setActionCommand (SpellCheckerActionListener.ZOOM);
	buttons.add (zoomButton);
	// remove
	removeButton = new JButton ((String)cc.rb.getObject ("spellchecker_dialog_remove"));
	removeButton.setToolTipText((String)cc.rb.getObject ("spellchecker_dialog_remove_tooltip"));
	removeButton.setMnemonic (((Integer)cc.rb.getObject("spellchecker_dialog_remove_mnemonic")).intValue());
	removeButton.addActionListener (actionlistener);
	removeButton.setActionCommand (SpellCheckerActionListener.REMOVE);
	buttons.add (removeButton);
	// refresh
	refreshButton = new JButton ((String)cc.rb.getObject ("spellchecker_dialog_refresh"));
	refreshButton.setToolTipText((String)cc.rb.getObject ("spellchecker_dialog_refresh_tooltip"));
	refreshButton.setMnemonic (((Integer)cc.rb.getObject("spellchecker_dialog_refresh_mnemonic")).intValue());
	refreshButton.addActionListener (actionlistener);
	refreshButton.setActionCommand (SpellCheckerActionListener.REFRESH);
	buttons.add (refreshButton);
	// more info
	moreinfoButton = new JButton ((String)cc.rb.getObject ("spellchecker_dialog_moreinfo"));
	moreinfoButton.setToolTipText((String)cc.rb.getObject ("spellchecker_dialog_moreinfo_tooltip"));
	moreinfoButton.setMnemonic (((Integer)cc.rb.getObject("spellchecker_dialog_moreinfo_mnemonic")).intValue());
	moreinfoButton.addActionListener (actionlistener);
	moreinfoButton.setActionCommand (SpellCheckerActionListener.MOREINFO);
	buttons.add (moreinfoButton);
	constraints.gridx = 1;
	constraints.gridy = 0;
	layout.setConstraints(buttons, constraints);
	tablePanel.add (buttons);
	// ok all done
	tabs.addTab ((String)cc.rb.getObject ("spellchecker_dialog_table_tabheader"),null,tablePanel,(String)cc.rb.getObject ("spellchecker_dialog_table_tabheader_tooltip"));
    }

    /**
     *  reads the settings tab and updates the table accordingly
     */
    public void updateTable ()
    {
	SpellCheckerFilter filter = new SpellCheckerFilter (this);
	try
	{
	    filter.setIsolatedNodes (isolatednodesCheck.isSelected());
	}
	catch (NullPointerException ex)
	{
	    filter.setIsolatedNodes (false);
	}
	try
	{
	    filter.setDiscontinous (discontinousCheck.isSelected());
	}
	catch (NullPointerException ex)
	{
	    filter.setDiscontinous (false);
	}
	try
	{
	    filter.setNonconforming (nonconformingCheck.isSelected()); 
	}
	catch (NullPointerException ex)
	{
	    filter.setNonconforming (false); 
	}
	try
	{
	    if (inneranglesCheck.isSelected ())
	    {
		double value = 0;
		try
		{
		    value = Double.parseDouble(inneranglesEdit.getText());
		    filter.setInnerAngles (true, value);
		}
		catch (Exception ex)
		{
		    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject("spellchecker_dialog_error_innerangles"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
		    filter.setInnerAngles (false, 0);
		}
	    }
	    else
		filter.setInnerAngles (false, 0);
	}
	catch (NullPointerException es)
	{
	    filter.setInnerAngles (false, 0);
	}
	try
	{
	    if (aspectratiosCheck.isSelected ())
	    {
		double value = 0;
		try
		{
		    value = Double.parseDouble(aspectratiosEdit1.getText());
		    double value2 = Double.parseDouble(aspectratiosEdit2.getText());
		    filter.setAspectRatio (true, value, value2);
		}
		catch (Exception ex)
		{
		    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject("spellchecker_dialog_error_aspectratio"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
		    filter.setAspectRatio (false, 0, 0);
		}
	    }
	    else
		filter.setAspectRatio (false, 0, 0);
	}
	catch (NullPointerException ex)
	{
	    filter.setAspectRatio (false, 0, 0);
	}
	try
	{
	    if (neighbouringCheck.isSelected ())
	    {
		double value = 0;
		try
		{
		    value = Double.parseDouble(neighbouringEdit1.getText());
		    double value2 = Double.parseDouble(neighbouringEdit2.getText());
		    filter.setNeighbouring (true, value, value2);
		}
		catch (Exception ex)
		{
		    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject("spellchecker_dialog_error_neighbouring"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
		    filter.setNeighbouring (false, 0, 0);
		}
	    }
	    else
		filter.setNeighbouring (false, 0,0);
	}
	catch (NullPointerException ex)
	{
	    filter.setNeighbouring (false, 0, 0);
	}
	///
	tablemodel = new SpellCheckerTableModel (filter);
	table.setModel (tablemodel);
	table.doLayout ();
    }

    /** 
     *  sets the focus to the Table tab 
     */
    public void setFocusToTable ()
    {
	tabs.setSelectedIndex (tabs.indexOfComponent (tablePanel));
    }

    /** 
     *  sets the focus to the Settings tab 
     */
    public void setFocusToSettings ()
    {
	tabs.setSelectedIndex (tabs.indexOfComponent (settingsPanel));
    }

    /**
     * shows the dialog with initial focus to the table tab
     */
    public void showTable() 
    {
	setFocusToTable ();
	repaint ();
    }

    /**
     * shows the dialog with initial focus to the settings tab
     */
    public void showSettings() 
    {
	setFocusToSettings ();
	repaint ();
    }

    JTable getTable ()
    {
	return table;
    }

    /**
     *  sets the inner variable which stores the selected item
     */
    void setSelectedItem (GridItem selected)
    {
	selectedItem = selected;
    }

    GridItem getSelectedItem ()
    {
	return selectedItem;
    }

    Vector getTableData ()
    {
	return ((SpellCheckerTableModel)(table.getModel())).getData ();
    }

    /**
     *  reference to the selected item in the list, updated by the 
     *  SelectionListener
     */
    private GridItem selectedItem;


    /** components of the dialog */
    private JTabbedPane tabs;
    private JButton closeButton;
    // table panel
    private JPanel tablePanel;
    private JTable table;
    private SpellCheckerTableModel tablemodel;
    private SpellCheckerTableListSelectionListener lsListener;
    private JButton zoomButton;
    private JButton removeButton;
    private JButton refreshButton;
    private JButton moreinfoButton;

    // settings panel
    private JPanel settingsPanel;
    private JCheckBox isolatednodesCheck;
    private JCheckBox inneranglesCheck;
    private JTextField inneranglesEdit;
    private JCheckBox aspectratiosCheck;
    private JTextField aspectratiosEdit1;
    private JTextField aspectratiosEdit2;
    private JCheckBox neighbouringCheck;
    private JTextField neighbouringEdit1;
    private JTextField neighbouringEdit2;
    private JCheckBox discontinousCheck;
    private JCheckBox nonconformingCheck;


   



}


