package devisor2.grid.backend.spellchecker;

import javax.swing.*;
import javax.swing.event.*;

import java.awt.*;
import java.awt.event.*;

import java.util.*;

import devisor2.grid.options.*;
import devisor2.foundation.*;
import devisor2.foundation.elements.*;
import devisor2.foundation.boundary.*;
import devisor2.foundation.base.*;

/**
 *  This class encapsulates the various filter possibilities for the
 *  table in the SpellChecker module. It is created by the dialog and used to
 *  create the TableModel. See there for details.
 *
 *  @author Dominik Goeddeke
 *  @version 1.0 (fully tested)
 */
public class SpellCheckerFilter
{
    /**
     *  reference to the SpellCheckerDialog where the JTable is located
     */
    private SpellCheckerDialog parent;

    private ControlCenter cc;


    // local storage of filter options
    private boolean isolatednodes = false;
    private boolean innerangles = false;
    private double inneranglesthreshold = Math.PI/10.0;
    private boolean aspectratios = false;
    private double[] aspectratiothreshold = {1.0, 5.0};
    private boolean neighbouring = false;
    private double[] neighbouringthreshold = {1.0, 5.0};
    private boolean discontinous = false;
    private boolean nonconforming = false;


    /**
     *  the constructor just sets references correctly
     *
     *  @param parent - a SpellCheckerDialog reference
     */
    public SpellCheckerFilter (SpellCheckerDialog parent)
    {
	super ();
	this.parent = parent;
	this.cc = ControlCenter.getMyself ();
    }

    public void setIsolatedNodes (boolean nodes)
    {
	isolatednodes = nodes;
    }

    public boolean isIsolatedNodes ()
    {
	return isolatednodes;
    }

    public void setInnerAngles (boolean angles, double threshold)
    {
	innerangles = angles;
	inneranglesthreshold = threshold;
    }

    public boolean isInnerAngles ()
    {
	return innerangles;
    }

    public double getInnerAnglesThreshold ()
    {
	return inneranglesthreshold;
    }

    public void setAspectRatio (boolean ratio, double from, double to)
    {
	aspectratios = ratio;
	aspectratiothreshold = new double[2];
	aspectratiothreshold[0] = from;
	aspectratiothreshold[1] = to;
    }

    public boolean isAspectRatio ()
    {
	return aspectratios;
    }

    public double getAspectRatioThreshold ()
    {
	return aspectratiothreshold[0]/ aspectratiothreshold[1];
    }

    public void setNeighbouring (boolean neigh, double from, double to)
    {
	neighbouring = neigh;
	neighbouringthreshold = new double[2];
	neighbouringthreshold[0] = from;
	neighbouringthreshold[1] = to;
    }

    public boolean isNeighbouring ()
    {
	return neighbouring;
    }

    public double getNeighbouringThreshold ()
    {
	return neighbouringthreshold[0]/neighbouringthreshold[1];
    }

    public void setDiscontinous (boolean cont)
    {
	discontinous = cont;
    }

    public boolean isDiscontinous ()
    {
	return discontinous;
    }

    public void setNonconforming (boolean non)
    {
	nonconforming = non;
    }

    public boolean isNonconforming ()
    {
	return nonconforming;
    }





}
