package devisor2.grid.backend.undo;


import devisor2.grid.options.*;
import devisor2.foundation.*;

import java.util.*;

/**
 *  This class encapsulates the UndoContainer for the undo functionality
 *  in the DeViSoRGrid 2 application. It basically stores all information
 *  neccessary to perform one undo step: a unique ID for the operation which
 *  needs to be taken back, a reference to all affected GridItems, and
 *  all neccessary additional information, e.g. GridTransformation objects.
 *
 *  Take a look at the technical paper about UNDO if you want to learn more
 *  about the full undo implementation.
 *
 *  @see UndoManager, UndoStack
 *  @author Dominik Goeddeke
 *
 */
public class UndoContainer
{

    /**
     *  a unique ID for the operation to be taken back
     *  use constants declared in the UndoManager class.
     */
    private int opCode;

    /**
     *  all affected GridItems, stored in a Vector for easier access
     */
    private Vector items;

    /**
     *  reference to additional information neccessary to perform the undo step
     */
    private Object info;

    /**
     *  basic constructor, just sets references
     */
    public UndoContainer (int opCode, Vector items, Object info)
    {
	this.opCode = opCode;
	this.items = items;
	this.info = info;
    }

    /**
     *  sets the opCode for this UndoContainer
     */
    public void setOpCode (int opCode)
    {
	this.opCode = opCode;
    }

    /**
     *  returns the opCode of this UndoContainer
     */
    public int getOpCode ()
    {
	return opCode;
    }

    /**
     *  sets the affected items for this UndoContainer
     */
    public void setItems (Vector items)
    {
	this.items = items;
    }

    /**
     *  returns the affected items of this UndoContainer
     */
    public Vector getItems ()
    {
	return items;
    }

    /**
     *  sets the additonal info for this UndoContainer
     */
    public void setInfo (Object info)
    {
	this.info = info;
    }

    /**
     *  returns the additional info of this UndoContainer
     */
    public Object getInfo ()
    {
	return info;
    }

    /**
     *  returns an Enumeration of the affected items of this UndoContainer
     */
    public Enumeration getItemEnum ()
    {
	return items.elements ();
    }


}
