package devisor2.grid.backend.undo;


import devisor2.grid.options.*;
import devisor2.foundation.*;

import java.util.*;

/**
 *  This class encapsulates the Stack which stores the UndoContainers.
 *  For easier implementation, it extends the Stack provided by Java and
 *  adds the capacity limit and automatic casting of the push and pop
 *  operations.
 *
 *  Take a look at the technical paper about UNDO if you want to learn more
 *  about the full undo implementation.
 *
 *  @see UndoManager, UndoContainer
 *  @author Dominik Goeddeke
 *
 */
public class UndoStack extends Stack
{

    /**
     *  the capacity limit of the Stack
     */
    private int capacity;

    /**
     *  a counter for the number of items already pushed on the Stack
     */
    private int elementCount;

    /**
     *  the constructor reads the capacity limit from the Options instance
     *  and creates a new ampty stack with that limit
     */
    public UndoStack ()
    {
	super ();
	this.elementCount = 0;
	ControlCenter cc = ControlCenter.getMyself ();
	this.capacity = Integer.parseInt (cc.op.get (cc.op.other_undosteps));
    }


    /**
     *  overrides the push method of the java Stack by adding the capacity 
     *  limit
     */
    public UndoContainer push (UndoContainer item)
    {
	if (elementCount == capacity)
	{
	    removeElement (lastElement ());
	    elementCount--;
	}
	super.push (item);
	elementCount++;
	return item;
    }

    /**
     *  overrides the pop method of the java Stack by adding automated casting
     */
    public UndoContainer popp ()
    {
	return (UndoContainer)(super.pop());
    }


    /**
     *  sets the capacity limit to a new value
     *  and eventually deletes items too far down in the stack.
     */
    public void setCapacity (int capacity)
    {
	this.capacity = capacity;
	if (capacity < elementCount)
	{
	    elementCount = capacity;
	    Stack tempStack = new Stack ();
	    for (int i=0; i<=capacity; i++)
	    {
		tempStack.push (pop());
	    }
	    removeAllElements ();
	    while (!tempStack.empty ())
		super.push (tempStack.pop());
	}
    }



}

