package devisor2.grid.options;

import java.util.*;
import java.awt.event.*;
import javax.swing.*;

/**
 *  This class maps the GUI representation of accelerator keys to the
 *  Java object representation. <br>
 *  So, nothing spectacular here, just a long, boring multi-if-else-statement.
 *  <br>
 *  The application-wide accelerator settings are managed and saved in the 
 *  Options class. See there for details. <br>
 *
 *  @see devisor2.options.Options
 *  @author Dominik Goeddeke
 *  @version 1.0 (fully tested)
 */
public class Accelerators
{

    /**
     *  Returns the Swing representation of the accelerator defined by
     *  modifier + key
     *
     *  @param modifier - one of the constants CRTL, ALT, SHIFT
     *  @param key - one of the constants A-Z, 0-9, F-keys or INS, DEL etc.
     *
     *  @return a KeyStroke representation of the combination modifiers + key
     *  @see javax.swing.KeyStroke
     */
    public static KeyStroke getAccelerator (String modifier, String key)
    {
	if (modifier.equals ("CTRL"))
	    return KeyStroke.getKeyStroke (getKeyConstant(key), ActionEvent.CTRL_MASK);
	if (modifier.equals ("ALT"))
	    return KeyStroke.getKeyStroke (getKeyConstant(key), ActionEvent.ALT_MASK);
	if (modifier.equals ("SHIFT"))
	    return KeyStroke.getKeyStroke (getKeyConstant(key), ActionEvent.SHIFT_MASK);
	if (modifier.equals ("NONE"))
	    return KeyStroke.getKeyStroke (getKeyConstant(key), 0); 
	return null;
    }


    /**
     *  Returns the Swing representation of the given (internal) string
     *  representing of an accelerator.
     *  The input format is as follows: "modifier+key".
     *
     *  @param a String representing an accelerator
     *  @return a Keystroke object representing the accelerator
     */
    public static KeyStroke getAccelerator (String accString)
    {
	int pluspos = accString.indexOf ('+');
	String mod, key;
	if (pluspos == -1)
	    {
		mod = "NONE";
		key = accString;
	    }
	else
	    {
		mod = accString.substring (0,pluspos);
		key = accString.substring (pluspos+1); 
	    }

	return getAccelerator (mod, key);
    }


    /**
     *  Returns a String representation of the given mmemonic
     *  specified by the KeyEvent constant
     *
     *  @param key - a KeyEvent constant
     *  @returns a String representation of the given constant
     */
    public static String getKeyString (int key)
    {
	if (key == KeyEvent.VK_A) return "A";
	if (key == KeyEvent.VK_A) return "A";
	if (key == KeyEvent.VK_B) return "B";
	if (key == KeyEvent.VK_C) return "C";
	if (key == KeyEvent.VK_D) return "D";
	if (key == KeyEvent.VK_E) return "E";
	if (key == KeyEvent.VK_F) return "F";
	if (key == KeyEvent.VK_G) return "G";
	if (key == KeyEvent.VK_H) return "H";
	if (key == KeyEvent.VK_I) return "I";
	if (key == KeyEvent.VK_J) return "J";
	if (key == KeyEvent.VK_K) return "K";
	if (key == KeyEvent.VK_L) return "L";
	if (key == KeyEvent.VK_M) return "M";
	if (key == KeyEvent.VK_N) return "N";
	if (key == KeyEvent.VK_O) return "O";
	if (key == KeyEvent.VK_P) return "P";
	if (key == KeyEvent.VK_Q) return "Q";
	if (key == KeyEvent.VK_R) return "R";
	if (key == KeyEvent.VK_S) return "S";
	if (key == KeyEvent.VK_T) return "T";
	if (key == KeyEvent.VK_U) return "U";
	if (key == KeyEvent.VK_V) return "V";
	if (key == KeyEvent.VK_W) return "W";
	if (key == KeyEvent.VK_X) return "X";
	if (key == KeyEvent.VK_Y) return "Y";
	if (key == KeyEvent.VK_Z) return "Z";
	if (key == KeyEvent.VK_0) return "0";
	if (key == KeyEvent.VK_1) return "1";
	if (key == KeyEvent.VK_2) return "2";
	if (key == KeyEvent.VK_3) return "3";
	if (key == KeyEvent.VK_4) return "4";
	if (key == KeyEvent.VK_5) return "5";
	if (key == KeyEvent.VK_6) return "6";
	if (key == KeyEvent.VK_7) return "7";
	if (key == KeyEvent.VK_8) return "8";
	if (key == KeyEvent.VK_9) return "9";
	if (key == KeyEvent.VK_F1) return "F1";
	if (key == KeyEvent.VK_F2) return "F2";
	if (key == KeyEvent.VK_F3) return "F3";
	if (key == KeyEvent.VK_F4) return "F4";
	if (key == KeyEvent.VK_F5) return "F5";
	if (key == KeyEvent.VK_F6) return "F6";
	if (key == KeyEvent.VK_F7) return "F7";
	if (key == KeyEvent.VK_F8) return "F8";
	if (key == KeyEvent.VK_F9) return "F9";
	if (key == KeyEvent.VK_F10) return "F10";
	if (key == KeyEvent.VK_F11) return "F11";
	if (key == KeyEvent.VK_F12) return "F12";
	if (key == KeyEvent.VK_INSERT) return "INS";
	if (key == KeyEvent.VK_DELETE) return "DEL";
	if (key == KeyEvent.VK_HOME) return "HOME";
	if (key == KeyEvent.VK_END) return "END";
	if (key == KeyEvent.VK_PAGE_UP) return "PGUP";
	if (key == KeyEvent.VK_PAGE_DOWN) return "PGDN";
	return "NONE";
   }

    /**
     *  returns the KEY_VK constant assigned to the given key
     *  @see javax.swing.KeyEvent
     *
     *  @param key - an A-Z, 0-9, alphanumerical character
     *  @returns the corresponding KeyEvent constant
     */
    public static int getKeyConstant (String key)
    {
	if (key.equals("A")) return KeyEvent.VK_A;
	if (key.equals("B")) return KeyEvent.VK_B;
	if (key.equals("C")) return KeyEvent.VK_C;
	if (key.equals("D")) return KeyEvent.VK_D;
	if (key.equals("E")) return KeyEvent.VK_E;
	if (key.equals("F")) return KeyEvent.VK_F;
	if (key.equals("G")) return KeyEvent.VK_G;
	if (key.equals("H")) return KeyEvent.VK_H;
	if (key.equals("I")) return KeyEvent.VK_I;
	if (key.equals("J")) return KeyEvent.VK_J;
	if (key.equals("K")) return KeyEvent.VK_K;
	if (key.equals("L")) return KeyEvent.VK_L;
	if (key.equals("M")) return KeyEvent.VK_M;
	if (key.equals("N")) return KeyEvent.VK_N;
	if (key.equals("O")) return KeyEvent.VK_O;
	if (key.equals("P")) return KeyEvent.VK_P;
	if (key.equals("Q")) return KeyEvent.VK_Q;
	if (key.equals("R")) return KeyEvent.VK_R;
	if (key.equals("S")) return KeyEvent.VK_S;
	if (key.equals("T")) return KeyEvent.VK_T;
	if (key.equals("U")) return KeyEvent.VK_U;
	if (key.equals("V")) return KeyEvent.VK_V;
	if (key.equals("W")) return KeyEvent.VK_W;
	if (key.equals("X")) return KeyEvent.VK_X;
	if (key.equals("Y")) return KeyEvent.VK_Y;
	if (key.equals("Z")) return KeyEvent.VK_Z;
	if (key.equals("0")) return KeyEvent.VK_0;
	if (key.equals("1")) return KeyEvent.VK_1;
	if (key.equals("2")) return KeyEvent.VK_2;
	if (key.equals("3")) return KeyEvent.VK_3;
	if (key.equals("4")) return KeyEvent.VK_4;
	if (key.equals("5")) return KeyEvent.VK_5;
	if (key.equals("6")) return KeyEvent.VK_6;
	if (key.equals("7")) return KeyEvent.VK_7;
	if (key.equals("8")) return KeyEvent.VK_8;
	if (key.equals("9")) return KeyEvent.VK_9;
	if (key.equals("F1")) return KeyEvent.VK_F1;
	if (key.equals("F2")) return KeyEvent.VK_F2;
	if (key.equals("F3")) return KeyEvent.VK_F3;
	if (key.equals("F4")) return KeyEvent.VK_F4;
	if (key.equals("F5")) return KeyEvent.VK_F5;
	if (key.equals("F6")) return KeyEvent.VK_F6;
	if (key.equals("F7")) return KeyEvent.VK_F7;
	if (key.equals("F8")) return KeyEvent.VK_F8;
	if (key.equals("F9")) return KeyEvent.VK_F9;
	if (key.equals("F10")) return KeyEvent.VK_F10;
	if (key.equals("F11")) return KeyEvent.VK_F11;
	if (key.equals("F12")) return KeyEvent.VK_F12;
	if (key.equals("INS")) return KeyEvent.VK_INSERT;
	if (key.equals("DEL")) return KeyEvent.VK_DELETE;
	if (key.equals("HOME")) return KeyEvent.VK_HOME;
	if (key.equals("END")) return KeyEvent.VK_END;
	if (key.equals("PGUP")) return KeyEvent.VK_PAGE_UP;
	if (key.equals("PGDN")) return KeyEvent.VK_PAGE_DOWN;
	return KeyEvent.VK_UNDEFINED;
    }

    /**
     *  This method "just" checks if the given combination of modifier
     *  and key is a correct accelerator. 
     *  Accepted combinations are:
     *  - if the modifier is NONE, only function keys and INS,DEL,HOME,END
     *    PGUP and PGDN are accepted
     *  - if the modifier is ALT, no letters are accepted (because these are
     *    reserved for the menu)
     *
     *  @param modifier - the strings CTRL, ALT, SHIFT, NONE
     *  @param key - a String representation of a key (see above)
     *
     *  @returns true if the given combination represents a correct 
     *           accelerator, false otherwise
     */
    public static boolean isCorrectAccelerator (String modifier, String key)
    {
	// the CTRL part: as keys can only be those from the combobox,
	// everything is allowed
	if (modifier.equals("CTRL") & !(key.equals("NONE"))) return true;
	// same with the SHIFT part
	if (modifier.equals("SHIFT") & !(key.equals("NONE"))) return true;
	// the NONE part
	if (modifier.equals("NONE"))
	    {
		if (key.equals ("F1")) return true;
		if (key.equals ("F1")) return true;
		if (key.equals ("F2")) return true;
		if (key.equals ("F3")) return true;
		if (key.equals ("F4")) return true;
		if (key.equals ("F5")) return true;
		if (key.equals ("F6")) return true;
		if (key.equals ("F7")) return true;
		if (key.equals ("F8")) return true;
		if (key.equals ("F9")) return true;
		if (key.equals ("F10")) return true;
		if (key.equals ("F11")) return true;
		if (key.equals ("F12")) return true;
		if (key.equals ("INS")) return true;
		if (key.equals ("DEL")) return true;
		if (key.equals ("HOME")) return true;
		if (key.equals ("END")) return true;
		if (key.equals ("PGUP")) return true;
		if (key.equals ("PGDN")) return true;
		if (key.equals ("NONE")) return true;
	    }
	// the ALT part
	if (modifier.equals("ALT"))
	    {
		if (key.equals ("F1")) return true;
		if (key.equals ("F1")) return true;
		if (key.equals ("F2")) return true;
		if (key.equals ("F3")) return true;
		if (key.equals ("F4")) return true;
		if (key.equals ("F5")) return true;
		if (key.equals ("F6")) return true;
		if (key.equals ("F7")) return true;
		if (key.equals ("F8")) return true;
		if (key.equals ("F9")) return true;
		if (key.equals ("F10")) return true;
		if (key.equals ("F11")) return true;
		if (key.equals ("F12")) return true;
		if (key.equals ("INS")) return true;
		if (key.equals ("DEL")) return true;
		if (key.equals ("HOME")) return true;
		if (key.equals ("END")) return true;
		if (key.equals ("PGUP")) return true;
		if (key.equals ("PGDN")) return true;
		if (key.equals ("1")) return true;
		if (key.equals ("2")) return true;
		if (key.equals ("3")) return true;
		if (key.equals ("4")) return true;
		if (key.equals ("5")) return true;
		if (key.equals ("6")) return true;
		if (key.equals ("7")) return true;
		if (key.equals ("8")) return true;
		if (key.equals ("9")) return true;
		if (key.equals ("0")) return true;
	    }
	return false;
    }
}






