package devisor2.grid.options;

import java.awt.*;
import devisor2.grid.GUI.framework.*;

/** 
 *  This is the convenience class for easy access of all draw options
 *  such as colors, fonts and paint flags. <br>
 *  These options are still being stored in the Options class, therefore
 *  also made persistent with it, and this class is updated every time
 *  any of these options are changed. Updating is taken care of by the 
 *  constructor only.<br>
 *  For easy access, there is a reference to the most recent instance of this
 *  class in the ControlCenter, which you can access as usual.
 *
 *  @author Dominik Goeddeke
 *  @version 1.0
 */
public class DrawControl
{

    /**
     *  The constructor just initializes all the class variables, which are 
     *  public for easier access. Feel free to make them private and provide
     *  get methods :-)
     *
     *  @param op - the Options instance to read values from
     */
    public DrawControl (Options op)
    {
	// screen and MainFrame size updated from within startup class.
	displaycoords = Boolean.valueOf(op.get (Options.draw_usercoords)).booleanValue();
	draw_orientationmarkers = Boolean.valueOf(op.get (Options.draw_orientationmarkers)).booleanValue();

	// init draw flags
	draw_boundary = Boolean.valueOf(op.get (Options.draw_drawboundary)).booleanValue();
	number_boundary = Boolean.valueOf(op.get (Options.draw_numberboundary)).booleanValue();
	draw_node = Boolean.valueOf(op.get (Options.draw_drawnode)).booleanValue();
	number_node = Boolean.valueOf(op.get (Options.draw_numbernode)).booleanValue();
	draw_edge = Boolean.valueOf(op.get (Options.draw_drawedge)).booleanValue();
	number_edge = Boolean.valueOf(op.get (Options.draw_numberedge)).booleanValue();
	draw_element = Boolean.valueOf(op.get (Options.draw_drawelement)).booleanValue();
	number_element = Boolean.valueOf(op.get (Options.draw_numberelement)).booleanValue();
	cross_element = Boolean.valueOf(op.get (Options.draw_crosselement)).booleanValue();
	fill_element = Boolean.valueOf(op.get (Options.draw_fillelement)).booleanValue();
	draw_grid = Boolean.valueOf(op.get (Options.draw_drawgrid)).booleanValue();
	draw_perimeter = Boolean.valueOf(op.get (Options.draw_drawperimeter)).booleanValue();
	highlight_rect = Boolean.valueOf(op.get (Options.draw_drawrect)).booleanValue();
	// colors
	color_node = new Color (Integer.parseInt(op.get(Options.color_node)));
	color_edge = new Color (Integer.parseInt(op.get(Options.color_edge)));
	color_edgebase = new Color (Integer.parseInt(op.get(Options.color_edgebase)));
	color_element = new Color (Integer.parseInt(op.get(Options.color_element)));
	color_boundarynumber = new Color (Integer.parseInt(op.get(Options.color_boundarynumber)));
	color_nodenumber = new Color (Integer.parseInt(op.get(Options.color_nodenumber)));
	color_edgenumber = new Color (Integer.parseInt(op.get(Options.color_edgenumber)));
	color_elementnumber = new Color (Integer.parseInt(op.get(Options.color_elementnumber)));
	color_selection = new Color (Integer.parseInt(op.get(Options.color_selection)));
	color_rect = new Color (Integer.parseInt(op.get(Options.color_rect)));
	color_grid = new Color (Integer.parseInt(op.get(Options.color_grid)));
	color_background = new Color (Integer.parseInt(op.get(Options.color_background)));
	color_lasso = new Color (Integer.parseInt(op.get(Options.color_lasso)));

	// fill colors for parallel blocks palette
	parallel_palette = new Color[16];
	parallel_palette[0] = new Color (Integer.parseInt(op.get(Options.color_fill0)));
	parallel_palette[1] = new Color (Integer.parseInt(op.get(Options.color_fill1)));
	parallel_palette[2] = new Color (Integer.parseInt(op.get(Options.color_fill2)));
	parallel_palette[3] = new Color (Integer.parseInt(op.get(Options.color_fill3)));
	parallel_palette[4] = new Color (Integer.parseInt(op.get(Options.color_fill4)));
	parallel_palette[5] = new Color (Integer.parseInt(op.get(Options.color_fill5)));
	parallel_palette[6] = new Color (Integer.parseInt(op.get(Options.color_fill6)));
	parallel_palette[7] = new Color (Integer.parseInt(op.get(Options.color_fill7)));
	parallel_palette[8] = new Color (Integer.parseInt(op.get(Options.color_fill8)));
	parallel_palette[9] = new Color (Integer.parseInt(op.get(Options.color_fill9)));
	parallel_palette[10] = new Color (Integer.parseInt(op.get(Options.color_fill10)));
	parallel_palette[11] = new Color (Integer.parseInt(op.get(Options.color_fill11)));
	parallel_palette[12] = new Color (Integer.parseInt(op.get(Options.color_fill12)));
	parallel_palette[13] = new Color (Integer.parseInt(op.get(Options.color_fill13)));
	parallel_palette[14] = new Color (Integer.parseInt(op.get(Options.color_fill14)));
	parallel_palette[15] = new Color (Integer.parseInt(op.get(Options.color_fill15)));

	// boundary color palette
	boundary_palette = new Color[16];
	boundary_palette[0] = new Color (Integer.parseInt(op.get(Options.color_boundary0)));
	boundary_palette[1] = new Color (Integer.parseInt(op.get(Options.color_boundary1)));
	boundary_palette[2] = new Color (Integer.parseInt(op.get(Options.color_boundary2)));
	boundary_palette[3] = new Color (Integer.parseInt(op.get(Options.color_boundary3)));
	boundary_palette[4] = new Color (Integer.parseInt(op.get(Options.color_boundary4)));
	boundary_palette[5] = new Color (Integer.parseInt(op.get(Options.color_boundary5)));
	boundary_palette[6] = new Color (Integer.parseInt(op.get(Options.color_boundary6)));
	boundary_palette[7] = new Color (Integer.parseInt(op.get(Options.color_boundary7)));
	boundary_palette[8] = new Color (Integer.parseInt(op.get(Options.color_boundary8)));
	boundary_palette[9] = new Color (Integer.parseInt(op.get(Options.color_boundary9)));
	boundary_palette[10] = new Color (Integer.parseInt(op.get(Options.color_boundary10)));
	boundary_palette[11] = new Color (Integer.parseInt(op.get(Options.color_boundary11)));
	boundary_palette[12] = new Color (Integer.parseInt(op.get(Options.color_boundary12)));
	boundary_palette[13] = new Color (Integer.parseInt(op.get(Options.color_boundary13)));
	boundary_palette[14] = new Color (Integer.parseInt(op.get(Options.color_boundary14)));
	boundary_palette[15] = new Color (Integer.parseInt(op.get(Options.color_boundary15)));

	
	// fonts
	font_numbering = new Font (op.get(Options.font_numbering), Integer.parseInt(op.get(Options.font_numbering_style)), Integer.parseInt (op.get(Options.font_numbering_size)));


//	world_x1=-1*100000;
//	world_y1=-1*100000;
//	world_x2= 1*100000;
//	world_y2= 1*100000;
	
//	act_x1=world_x1;
//	act_x2=world_x2;
//	act_y1=world_y1;
//	act_y2=world_y2;

	// click tolerance
	tolerance = Integer.parseInt (op.get(Options.click_tolerance));
    }

    /**
     *  the physical screen width
     */
    public int screen_width;
    /**
     *  the physical screen heigth
     */
    public int screen_height;
    /**
     *  the actual MainFrame width
     */

    public int used_screen_width;
    /**
     *  the physical screen heigth
     */
    public int used_screen_height;
    /**
     *  the actual MainFrame width
     */
    public int world_x1,world_y1,world_x2,world_y2;

    /**
     *  the actual MainFrame width
     */
    public int act_x1,act_y1,act_x2,act_y2;


    /**
     *  flag to toggle user coords of mouse position feedback
     */
    public boolean displaycoords;

    /**
     *  flag to toggle drawing of orientation markers
     */
    public boolean draw_orientationmarkers;

    /**
     * flag to indicate if boundaries are painted
     */
    public boolean draw_boundary;
    /**
     *  flag to indicate if boundary numbers are painted
     */
    public boolean number_boundary;
    /**
     * flag to indicate if nodes are painted
     */
    public boolean draw_node;
    /**
     *  flag to indicate if node numbers are painted
     */
    public boolean number_node;
    /**
     * flag to indicate if edges are painted
     */
    public boolean draw_edge;
    /**
     *  flag to indicate if edge numbers are painted
     */
    public boolean number_edge;
    /**
     * flag to indicate if elements are painted
     */
    public boolean draw_element;
    /**
     *  flag to indicate if element numbers are painted
     */
    public boolean number_element;
    /**
     * flag to indicate if elements are crossed
     */
    public boolean cross_element;
    /**
     *  flag to indicate if elements are filled
     */
    public boolean fill_element;
    /**
     * flag to indicate if the grid is painted
     */
    public boolean draw_grid;
    /**
     * flag to indicate if the perimeter is painted
     */
    public boolean draw_perimeter;
    /**
     * flag to indicate if rectangular elements are highlighted
     */
    public boolean highlight_rect;





    /**
     *  color nodes are painted with
     */
    public Color color_node;
    /**
     *  base color edges are painted with
     */
    public Color color_edgebase;
    /**
     *  color edges are painted with
     */
    public Color color_edge;
    /**
     *  color elements are painted with
     */
    public Color color_element;
    /**
     *  boundarynumbers are painted with
     */
    public Color color_boundarynumber;
    /**
     *  color nodenumbers are painted with
     */
    public Color color_nodenumber;
    /**
     *  color edgenumbers are painted with
     */
    public Color color_edgenumber;
    /**
     *  color elementnumbers are painted with
     */
    public Color color_elementnumber;
    /**
     *  highlight color for selections
     */
    public Color color_selection;
    /**
     *  highlight color for rectangular status
     */
    public Color color_rect;
    /**
     *  color of the grid
     */
    public Color color_grid;
    /**
     *  color of the background
     */
    public Color color_background;
    /**
     *  color of the lasso
     */
    public Color color_lasso;

    /**
     *  all fillxx colors in one palette
     */
    public Color[] parallel_palette;

    /**
     *  the palette for boundary colors
     */
    public Color[] boundary_palette;

    /**
     *  font used to display numbering
     */
    public Font font_numbering;

    /**
     *  click tolerance (i.e. the distance added in all directions
     *  to the click point so that all objects inside this rectangle
     *  count as clicked on)
     */
    public int tolerance;
   

   
}
