package devisor2.grid.options;

import java.awt.*;
import devisor2.grid.GUI.framework.*;
import devisor2.grid.backend.*;
import devisor2.foundation.boundary.*;
import java.awt.geom.*;

/** 
 *  This is the convenience class for easier access to transformation options
 *  and everything that affects working with the domain.
 *  For easy access, there is a reference to the most recent instance of this
 *  class in the ControlCenter, which you can access as usual.
 *
 *  @author Dominik Goeddeke
 *  @version 1.0
 */
public class GridControl
{
    private ControlCenter cc = ControlCenter.getMyself ();

    public GridToolkit gtk;


    /**
     *  The number of the selected boundary, default is the first boundary,
     *  created during domain creation
     */
    public Boundary workingBoundary = null;
    
    /**
     *  reference to the MainFrame of the application
     */
    public MainFrame mf;
    
    /**
     *  The grid transformation matrix and its inverse
     */
    public AffineTransform trf;
    public AffineTransform invtrf;

    /**
     *  snap variables
     */
    public int vsnapx,vsnapy;
    public int vsnapoffx,vsnapoffy;
    public boolean snapon;

    /**
     *  the global eps used in various methods
     */
    public int eps;

    public GridControl (MainFrame uf, ControlCenter uc)
    {
	setMainFrame(uf, uc);
    }


    public void setMainFrame(MainFrame uf, ControlCenter uc)
    {
      mf=uf;
      cc=uc;
      gtk = new GridToolkit (uf);
      vsnapx = Integer.parseInt (cc.op.get(Options.snap_x));
      vsnapy = Integer.parseInt (cc.op.get(Options.snap_y));
      snapon =  Boolean.valueOf(cc.op.get (Options.snap_enabled)).booleanValue();
      int eps = Integer.parseInt (cc.op.get (Options.epsilon));
      double epsi = 0.01 + ((double)eps)/100 * 0.99;
      eps = (int)(epsi*(cc.dc.world_x2 - cc.dc.world_x1));
    }
    
    public double runden(double d)
    {
    
      String s = (new Double(d)).toString();
      
      StringBuffer ss = new StringBuffer(s);
     if (ss.indexOf("E-")>=0)
     {
// 	 System.out.println("DIGIMON "+s.substring(0,1));

         int nk = (new Integer(ss.substring(ss.indexOf("E-")+2))).intValue();

 //	 System.out.println("DIGIMON "+nk);

         ss=new StringBuffer("0.");
	 for (int i=0;i<nk-1;i++) ss=ss.append("0");
	 ss=ss.append(s.substring(0,1));
	 ss=ss.append("00");
     }
      
      int l=ss.length();
      int i;
      
      int[] digi= new int[l];
 
// 	 System.out.println("DIGI "+ss);
     
     
      for(i=0; i<ss.length(); i++)
      {
         String s1 = ss.substring(i,i+1);
	 
//	 System.out.println("DIGI "+s1);
	 
	 if (s1.equals("."))
	   digi[i]=-1;
	 else
	   digi[i]=(new Integer(s1)).intValue(); 
	 
      }
      
            
      for(i=l-1; i>1;i--)
      {
        if (digi[i]==0) break;
	
     //   if (digi[i]<=4)
//	{
//	  digi[i]=0;
//	  digi[i-1]--;
//	}
		
	if (digi[i]==10)
	{
	  digi[i]=0;
	  digi[i-1]++;
	}
	else
	{
        if (digi[i]>4)
	{
	  digi[i]=0;
	  digi[i-1]++;
	}	
        }
	  
      }

int flag = 0;
      
        for(i=0; i<l-1;i++)
	{
//	System.out.println("FLAGGG "+flag);
	
	  if ((flag==0) && (digi[i]==-1)) { flag=1; continue;}
	  
	  if ((flag==1) && (digi[i]==0) && (digi[i+1]!=0)) { flag=2; continue; }
	  
	  if (flag==2) { flag=3; continue; }
//	  if (flag==3) { flag=4; continue; }
	  
	  if (flag==3) digi[i]=0;
	  
	  
         }
	 
	 digi[l-1]=0;
	 
      ss = new StringBuffer();
 
      for(i=0; i<l; i++)
      {
         if (digi[i]==-1)
	   ss=ss.append(".");
	 else
	   ss=ss.append((new Integer(digi[i])).toString());
	 	 
      } 
      
      s=new String(ss);
      
       double r = (new Double(s)).doubleValue();    
   //       System.out.println("GERUNDET : von "+d+" zu "+r);  

   return r;    
    }
    
    
    
    public void setTransformation()
    {
    if (mf!=null)
    {
       cc.dc.screen_width=mf.getDrawingArea().getWidth();
       cc.dc.screen_height=mf.getDrawingArea().getHeight();

//runden(1.2343546);
//runden(1.23499999998);
//runden(1.2349939999);
//runden(1.2349949999);
//runden(0.00234995999);
//runden(0.00234993999);
//runden(0.00234994999);


        int iff = cc.globalscale2;
	int tv = ((int)(cc.dc.act_x1/cc.globalscale2))*cc.globalscale2;
	while (tv==0)
	{
	   iff=iff/10;
	   tv = ((int)(cc.dc.act_x1/iff))*iff;
	}
	cc.dc.act_x1=tv;


        iff = cc.globalscale2;
	tv = ((int)(cc.dc.act_y1/cc.globalscale2))*cc.globalscale2;
	while (tv==0)
	{
	   iff=iff/10;
	   tv = ((int)(cc.dc.act_y1/iff))*iff;
	}
	cc.dc.act_y1=tv;
		   
        iff = cc.globalscale2;
	tv = ((int)(cc.dc.act_x2/cc.globalscale2))*cc.globalscale2;
	while (tv==0)
	{
	   iff=iff/10;
	   tv = ((int)(cc.dc.act_x2/iff))*iff;
	}
	cc.dc.act_x2=tv;

        iff = cc.globalscale2;
	tv = ((int)(cc.dc.act_y2/cc.globalscale2))*cc.globalscale2;
	while (tv==0)
	{
	   iff=iff/10;
	   tv = ((int)(cc.dc.act_y2/iff))*iff;
	}
	cc.dc.act_y2=tv;
		
	    
       //System.out.println("setTransfo "+cc.dc.screen_width);
       //System.out.println("setTransfo "+cc.dc.screen_height);

       double kx = (double)cc.dc.screen_width/(double)(cc.dc.act_x2-cc.dc.act_x1);
       double ky = (double)cc.dc.screen_height/(double)(cc.dc.act_y2-cc.dc.act_y1);
       
 //      double k = Math.round(Math.min(kx,ky)*100000000)/100000000.0;
       double k = runden(Math.min(kx,ky));

//System.out.println("KMXK "+k);
       
       //if ((k>0.0020) && (k<0.0021)) k=0.002;
       
//System.out.println("KMX "+k);

       cc.dc.used_screen_width=(int)(cc.dc.screen_width*k/kx);
       cc.dc.used_screen_height=(int)(cc.dc.screen_height*k/ky);
    
       kx = runden((double)cc.dc.used_screen_width/(double)(cc.dc.act_x2-cc.dc.act_x1));
       ky = runden((double)cc.dc.used_screen_height/(double)(cc.dc.act_y2-cc.dc.act_y1));
       
      kx=k;
      ky=k;
     //  if ((k>0.0019) && (k<0.0021)) kx=0.002;
     //  if ((k>0.0019) && (k<0.0021)) ky=0.002;
// System.out.println("KMK "+kx+" "+ky);
         
       trf = new AffineTransform();
       trf.concatenate(new AffineTransform(1.,0.,0.,-1.,0.,(double)cc.dc.used_screen_height));
       trf.scale(kx,ky);
       
   //    System.out.println("DRMS "+(-(double)cc.dc.act_x1)+" "+(-(double)cc.dc.act_y1));
       
       trf.translate(-(double)cc.dc.act_x1,-(double)cc.dc.act_y1);
       
    	try
	{
	    invtrf = trf.createInverse();
	}
	catch (NoninvertibleTransformException ex)
	{
		System.err.println("Fehler");
	}
	}              
    }
    
    public Rectangle boxFromScreenToWorld(Rectangle box)
    {
      int[] xy = fromScreenToWorld(box.x,box.y);
      
      Rectangle ret = new Rectangle();
      
      ret.width = SingleFromScreenToWorld(box.width);
      ret.height = SingleFromScreenToWorld(box.height);
      
      ret.x=xy[0];
      ret.y=xy[1];
      
      return ret;
      
    }
      
    public double fromWorldToUser(int x)
    {
      return (double)x/cc.globalscale;
    }
    
    
    public int fromUserToWorld(double x)
    {
      return (int)(x*cc.globalscale);
    }
    
          
    public int[] fromScreenToWorld(int x, int y)
    {
	int[] tcoord = new int[2];
	
	double[] scr = {x,y};
	double[] user = new double[2];
	
	invtrf.transform(scr,0,user,0,1);
	
	tcoord[0]=(int)user[0]; 
	tcoord[1]=(int)user[1]; 
	
	return tcoord;
    }
    
    public int[] fromWorldToScreen(int x, int y)
    {
	int[] tcoord = new int[2];
	
	double[] scr = {x,y};
	double[] user = new double[2];
	
	trf.transform(scr,0,user,0,1);
	
	tcoord[0]=(int)user[0];
	tcoord[1]=(int)user[1];
	
	return tcoord;
    }

    public double getScaleX ()
    {
	return trf.getScaleX();
    }

    public double getScaleY ()
    {
	return trf.getScaleY();
    }
    /**
     *  a single value representing some length or some distance is 
     *  transformed into world space
     */
    public int SingleFromScreenToWorld (int value)
    {
	return (int)(value / getScaleX());
    }

    /**
     *  a single value representing some length or some distance is 
     *  transformed from world space
     */
    public int SingleFromWorldToScreen (int value)
    {
	return (int)(value * getScaleX());
    }


}

