package devisor2.grid.options;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/** 
 *  This class provides the localization information for english language
 *  and US country scheme (e.g. keyboard layout).
 *  All text messages for the whole Application are stored in this class's
 *  rather giant key-value Object[][] attribute with the key being the first
 *  argument (a constant String) and the value the second argument. <br>
 *  <br>
 *  As the locale is dynamically linked to the application, a 
 *  MissingRessourceException is thrown during startup if you use a key
 *  not specified in this class. So please NEVER modify the keys! <br>
 *  <br>
 *  If you want to create your own localized version of the application, just
 *  create a copy of this class in the same directory and change the en to 
 *  your target language (a pair of lowercase letters that conform to 
 *  ISO-639), and the US to your target country scheme (two uppercase letters
 *  that conform to ISO-3166) in both filename and classname. 
 *  After that, just try not to get bored while translating all the 
 *  right-hand String values assigned to the keys. Again: DO NOT TRANSLATE
 *  THE KEYS! <br>
 *  By convention, the first part of each key consists of the full qualified
 *  package name, followed by an underscore (_), the class name, underscore,
 *  method- or attribute name, underscore and then your identifier.
 *  Some general keys like the captions of buttons used everywhere throughout
 *  the application etc. are stored with the prefix general_. <br>
 *  <br>
 *  This class is also made globally accessible in the ControlCenter class. 
 *  <br> <br>
 *  Throughout the application, the contents of this class are accessible
 *  via the getObject(key) method. Note that an Object is returned which
 *  normally has to be casted to a String. On the other hand, also
 *  menu mnemonics are saved in here, so no specific casting takes
 *  place in this class. <br>
 *  <br>
 *  Example: <br>
 *  Let's say you want to extract the caption for an APPLY-button.
 *  For this application, it is stored under the key "general_Apply"
 *  Via the ControlCenter instance set up during program startup (suppose you
 *  have a reference in your class called cc), you have access to the 
 *  ResourceBundle for your locale (in fact, you are just reading the class 
 *  info of the ResourceBundle for english language...), named rb in the 
 *  ControlCenter class and declared as public there.
 *  You want to extract a String, but as said before, an Object is returned,
 *  so you might have to cast to avoid an error during compilation. <br>
 *  So all you have to do is: <br>
 *  (1) get the object stored under the specified key you are looking for:<br>
 *      <code>Object obj = cc.op.getObject ("general_apply"): <\code> <br>
 *  (2) cast it to a String:<br>
 *      <code> String myCaption = (String)obj; <\code> <br>
 *  (3) use it whichever way you want: <br>
 *      <code>JButton myApplyButton = new JButton (myCaption);<\code> <br>
 *  (4) or, to save time writing code, you can do this all in one step: <br>
 *      <code>JButton myApplyButton = new JButton ((String)cc.op.get ("general_apply"));<\code> <br>
 *  Analogously, the Mnemonics are stored as objects of class Integer, so 
 *  you have to cast to an Integer then and get the int value by invoking
 *  the Integer.intValue() method. <br>
 *
 *  @author Dominik Gddeke
 *  @translated by Patrick Otto
 *  @see devisor2.GUI.base.dialog.OptionDialog
 *  @see java.util.Locale
 *  @see java.util.ListRessourceBundle
 *  @version 1.0 english
 *
 */

public class Locale_de_DE extends ListResourceBundle
{
    /**
     *  returns the whole two-dimensional object array
     *
     *  @return the complete locales stored in this class
     */
    public Object[][] getContents ()
    {
	return contents;
    }

    /**
     *  ugly but neccessary...
     */
    private Object[][] contents = 
    {
	/* General button captions */
	{"general_Ok", "OK"},
	{"general_Ok_mnemonic", new Integer (KeyEvent.VK_O)},
	{"general_Accept", "Besttigen"},
	{"general_Cancel", "Abbrechen"},
	{"general_Cancel_mnemonic", new Integer (KeyEvent.VK_B)},
	{"general_Apply", "bernehmen"},
	{"general_Apply_mnemonic", new Integer (KeyEvent.VK_N)},
	{"general_Quit", "Beenden"},
	{"general_Close", "Schliessen"},
	{"general_Close_mnemonic", new Integer (KeyEvent.VK_S)},

	/* General messagebox texts */
	{"general_message_unsaveddata","Nicht gesicherte nderungen gehen verloren.\n Wirklich beenden ?"},
	{"general_message_unsaveddata_header","Nicht gesicherte nderungen"},
	{"general_message_quit","Programm wirklich beenden ?"},
	{"general_message_quit_header","Beenden"},
	
	/* error messages */
	{"errors_lookandfeelnotsupported","Diese Auswahl des Look&Feel wird nicht von ihrem System untersttzt \n Das Look&Feel wird nicht gendert."},
	{"errors_messageboxheader", "FEHLER"},
	{"errors_parseerror", "Parsing Fehler"},
	{"errors_notfounderror", "Reference not found!"},
	{"errors_gridinputerror", "Maus Eingabefehler!"},
	{"errors_directerror", "Eingabefehler"},
	{"errors_directnotsupported", "Direkte Eingabe wird fr diese \n Art von Element oder Segment nicht untersttzt \n Wechseln sie den Eingabe Modus."},
	{"errors_selectionerror", "Auswahlfehler"},
	{"errors_emptyselection", "Dieser Befehl kann nur ausgefhrt werden, \n wenn Gegenstnde markiert sind."},
	{"errors_cloneerror", "Clone Fehler"},

	/* General dialog captions and dialog button captions */
	{"dialogs_loaddialog_caption", "vorhandenes Gebiet ffnen"},
	{"dialogs_loaddialog_button", "ffnen"},
	{"dialogs_savedialog_button", "Speichern"},
	{"dialogs_saveucddialog_caption", "Speichern unter ... im UCD Format"},
	{"dialogs_savefeatdialog_caption", "Speichern unter ... im FEAT Format"},
	{"dialogs_savefeastdialog_caption", "Speichern unter ... im FEAST Format"},
	{"dialogs_printdialog_caption", "Speichern unter ... im PostScript Format"},
	{"dialogs_printlsdialog_caption", "Speichern unter ... im PostScript Querformat Format"},
	{"dialogs_saveoptionsdialog_caption", "Speichern der DeViSoRGrid2 Einstellungen"},
	{"dialogs_loadoptionsdialog_caption", "DeViSoRGrid2 Einstellungen laden"},
	{"dialogs_mergedialog_caption", "Auswahl eines Gebiets welches untergemischt werden soll  ..."},
	{"dialogs_mergedialog_button", "Mischen"},
	{"dialogs_printdialog_button", "Speichern unter"},

	/* menu items and corresponding shortcuts */
	{"menu_file_header", "Datei"},
	{"menu_file_header_mnemonic", new Integer(KeyEvent.VK_D) },
	{"menu_file_new", "Neu" },
	{"menu_file_new_mnemonic", new Integer(KeyEvent.VK_N)},
	{"menu_file_open", "ffnen ..." },
	{"menu_file_open_mnemonic", new Integer(KeyEvent.VK_O)},
	{"menu_file_save", "Speichern" },
	{"menu_file_save_mnemonic", new Integer(KeyEvent.VK_S)},
	{"menu_file_saveas", "Speichern unter..." },
	{"menu_file_saveas_mnemonic", new Integer(KeyEvent.VK_U)},
	{"menu_file_saveas_ucd", "UCD ... " },
	{"menu_file_saveas_ucd_mnemonic", new Integer(KeyEvent.VK_C)},
	{"menu_file_saveas_feat", "FEAT ... " },
	{"menu_file_saveas_feat_mnemonic", new Integer(KeyEvent.VK_E)},
	{"menu_file_saveas_feast", "FEAST ... " },
	{"menu_file_saveas_feast_mnemonic", new Integer(KeyEvent.VK_F)},
	{"menu_file_merge", "Gebiete vermischen ..." },
	{"menu_file_merge_mnemonic", new Integer(KeyEvent.VK_V)},
	{"menu_file_printls", "Drucken im Querformat..." },
	{"menu_file_printls_mnemonic", new Integer(KeyEvent.VK_L)},
	{"menu_file_print", "Drucken ..." },
	{"menu_file_print_mnemonic", new Integer(KeyEvent.VK_D)},
        {"menu_file_exit", "Beenden" },
	{"menu_file_exit_mnemonic", new Integer(KeyEvent.VK_B)},
	{"menu_edit_header", "Bearbeiten"},
	{"menu_edit_header_mnemonic", new Integer(KeyEvent.VK_B) },
	{"menu_edit_undo", "Rckgngig" },
	{"menu_edit_undo_mnemonic", new Integer(KeyEvent.VK_R)},
	{"menu_edit_redo", "Redo" },
	{"menu_edit_redo_mnemonic", new Integer(KeyEvent.VK_R)},
	{"menu_edit_cut", "Ausschneiden" },
	{"menu_edit_cut_mnemonic", new Integer(KeyEvent.VK_D)},
	{"menu_edit_copy", "Kopieren" },
	{"menu_edit_copy_mnemonic", new Integer(KeyEvent.VK_K)},
	{"menu_edit_paste", "Einfgen" },
	{"menu_edit_paste_mnemonic", new Integer(KeyEvent.VK_I)},
	{"menu_edit_copymode", "Einfge Modus" },
	{"menu_edit_copymode_mnemonic", new Integer(KeyEvent.VK_M)},
	{"menu_edit_delete", "Lschen" },
	{"menu_edit_delete_mnemonic", new Integer(KeyEvent.VK_L)},
	{"menu_edit_deletewb", "aktuellen Rand lschen" },
	{"menu_edit_deletewb_mnemonic", new Integer(KeyEvent.VK_A)},
	{"menu_edit_scale", "Skalieren ..." },
	{"menu_edit_scale_mnemonic", new Integer(KeyEvent.VK_S)},
	{"menu_edit_rotate", "Drehen ..." },
	{"menu_edit_rotate_mnemonic", new Integer(KeyEvent.VK_R)},
	{"menu_edit_mirror", "Spiegeln ..." },
	{"menu_edit_mirror_mnemonic", new Integer(KeyEvent.VK_P)},
	{"menu_edit_selectboundary", "aktuellen Rand auswhlen..." },
	{"menu_edit_selectboundary_mnemonic", new Integer(KeyEvent.VK_W)},
	{"menu_view_header", "Ansicht" },
	{"menu_view_header_mnemonic", new Integer(KeyEvent.VK_A)},
	{"menu_view_zoom_setzoom", "Bereich vergrern" },
	{"menu_view_zoom_setzoom_mnemonic", new Integer(KeyEvent.VK_B)},
	{"menu_view_zoom_resetzoom", "Normalansicht (1:1)" },
	{"menu_view_zoom_resetzoom_mnemonic", new Integer(KeyEvent.VK_N)},
	{"menu_view_zoom_zoomplus", "Vergrern" },
	{"menu_view_zoom_zoomplus_mnemonic", new Integer(KeyEvent.VK_G)},
	{"menu_view_zoom_zoomminus", "Verkleinern" },
	{"menu_view_zoom_zoomminus_mnemonic", new Integer(KeyEvent.VK_K)},
	{"menu_view_selectnodes", "Knoten auswhlen" },
	{"menu_view_selectnodes_mnemonic", new Integer(KeyEvent.VK_N)},
	{"menu_view_selectelements", "Elemente auswhlen" },
	{"menu_view_selectelements_mnemonic", new Integer(KeyEvent.VK_E)},
	{"menu_view_selectsegments", "Randteile auswhlen" },
	{"menu_view_selectsegments_mnemonic", new Integer(KeyEvent.VK_S)},
	{"menu_view_selectedges", "Kanten auswhlen" },
	{"menu_view_selectedges_mnemonic", new Integer(KeyEvent.VK_N)},
	{"menu_view_properties", "Eigenschaften anzeigen" },
	{"menu_view_properties_mnemonic", new Integer(KeyEvent.VK_Z)},
	{"menu_domain_header", "Gebiet" },
	{"menu_domain_header_mnemonic", new Integer(KeyEvent.VK_D)},
	{"menu_domain_newboundary", "neuer Rand" },
	{"menu_domain_newboundary_mnemonic", new Integer(KeyEvent.VK_R)},
	{"menu_domain_newanalytic", "analytische Gebietsbeschreibung ..." },
	{"menu_domain_newanalytic_mnemonic", new Integer(KeyEvent.VK_A)},
	{"menu_domain_directinput", "direkte Eingabe" },
	{"menu_domain_directinput_mnemonic", new Integer(KeyEvent.VK_D)},
	{"menu_domain_newsegment", "neues Randteil" },
	{"menu_domain_newsegment_mnemonic", new Integer(KeyEvent.VK_T)},
	{"menu_domain_newline", "Linie" },
	{"menu_domain_newline_mnemonic", new Integer(KeyEvent.VK_L)},
	{"menu_domain_newmultiline", "Mehrfachlinie" },
	{"menu_domain_newmultiline_mnemonic", new Integer(KeyEvent.VK_M)},
	{"menu_domain_newcirclep", "Kreis +" },
	{"menu_domain_newcirclep_mnemonic", new Integer(KeyEvent.VK_PLUS)},
	{"menu_domain_newcirclem", "Kreis -" },
	{"menu_domain_newcirclem_mnemonic", new Integer(KeyEvent.VK_MINUS)},
	{"menu_domain_newelement", "neues Element" },
	{"menu_domain_newelement_mnemonic", new Integer(KeyEvent.VK_E)},
	{"menu_domain_newtri", "Dreick" },
	{"menu_domain_newtri_mnemonic", new Integer(KeyEvent.VK_D)},
	{"menu_domain_newquad", "Viereck" },
	{"menu_domain_newquad_mnemonic", new Integer(KeyEvent.VK_Q)},
	{"menu_domain_newmacro", "Makroelement" },
	{"menu_domain_newmacro_mnemonic", new Integer(KeyEvent.VK_M)},
	{"menu_domain_newnode", "neuer Knoten" },
	{"menu_domain_newnode_mnemonic", new Integer(KeyEvent.VK_N)},
	{"menu_domain_adjustboundaries", "Rnder angleichen" },
	{"menu_domain_adjustboundaries_mnemonic", new Integer(KeyEvent.VK_G)},
	{"menu_domain_linkboundaries", "Rnder verbinden" },
	{"menu_domain_linkboundaries_mnemonic", new Integer(KeyEvent.VK_L)},
	{"menu_domain_fromtri2prm", "PRM Datei aus TRI erzeugen" },
	{"menu_domain_fromtri2prm_mnemonic", new Integer(KeyEvent.VK_G)},	
        {"menu_domain_swapboundaryorientation", "Orientierung der Rnder vertauschen" },
	{"menu_domain_swapboundaryorientation_mnemonic", new Integer(KeyEvent.VK_O)},
	{"menu_domain_domainperimeter", "Gebietsumfang ..." },
	{"menu_domain_domainperimeter_mnemonic", new Integer(KeyEvent.VK_U)},
	{"menu_domain_domainstatistics", "Gebietsinformationen" },
	{"menu_domain_domainstatistics_mnemonic", new Integer(KeyEvent.VK_I)},
	{"menu_grid_header", "Gitter"},
	{"menu_grid_header_mnemonic", new Integer(KeyEvent.VK_G) },
	{"menu_grid_multipleboundarynodes", "mehrfach Randknoten hinzufgen ..."},
	{"menu_grid_multipleboundarynodes_mnemonic", new Integer(KeyEvent.VK_M) },
	{"menu_grid_edgestatus", "Eckeneigenschaften setzen ..."},
	{"menu_grid_edgestatus_mnemonic", new Integer(KeyEvent.VK_E) },
	{"menu_grid_parallelblock", "parallele Blcke setzen ..."},
	{"menu_grid_parallelblock_mnemonic", new Integer(KeyEvent.VK_P) },
	{"menu_grid_loadbalancing", "Loadbalancing ..."},
	{"menu_grid_loadbalancing_mnemonic", new Integer(KeyEvent.VK_L) },
	{"menu_grid_rectifyelement", "Element rektifizieren"},
	{"menu_grid_rectifyelement_mnemonic", new Integer(KeyEvent.VK_R) },
	{"menu_grid_refine", "Verfeinern ..."},
	{"menu_grid_refine_mnemonic", new Integer(KeyEvent.VK_F) },
	{"menu_grid_spellchecker", "Rechtschreibpruefung ..."},
	{"menu_grid_spellchecker_mnemonic", new Integer(KeyEvent.VK_P) },
	{"menu_grid_mergeboundaries", "Zwei Raender verbinden"},
	{"menu_grid_mergeboundaries_mnemonic", new Integer(KeyEvent.VK_Z) },
	{"menu_options_header", "Optionen"},
	{"menu_options_header_mnemonic", new Integer(KeyEvent.VK_O) },
	{"menu_options_general","allgemeine Optionen"},
	{"menu_options_general_mnemonic",new Integer(KeyEvent.VK_G)},
	{"menu_options_draw","Zeichenoptionen"},
	{"menu_options_draw_mnemonic",new Integer(KeyEvent.VK_D)},
	{"menu_options_color","Farboptionen"},
	{"menu_options_color_mnemonic",new Integer(KeyEvent.VK_C)},
	{"menu_options_accelerator","Tastaturkrzel"},
	{"menu_options_accelerator_mnemonic",new Integer(KeyEvent.VK_A)},
	{"menu_options_misc","verschiedene Optionen"},
	{"menu_options_misc_mnemonic",new Integer(KeyEvent.VK_M)},
	{"menu_help_header", "Hilfe"},
	{"menu_help_header_mnemonic", new Integer(KeyEvent.VK_H) },
	{"menu_help_manual", "Handbuch ..."},
	{"menu_help_manual_mnemonic", new Integer(KeyEvent.VK_B) },
	{"menu_help_faq", "FAQ ..."},
	{"menu_help_faq_mnemonic", new Integer(KeyEvent.VK_F) },
	{"menu_help_readme", "Liesmich ..."},
	{"menu_help_readme_mnemonic", new Integer(KeyEvent.VK_L) },
	{"menu_help_about", "Info ..."},
	{"menu_help_about_mnemonic", new Integer(KeyEvent.VK_I) },



	/* keys for class devisor2.GUI.dialogs.OptionsDialog */
	// general stuff
	{"options_OptionsDialog_general_title", "DeViSoRGrid2 Optionen"},
	// buttons
	{"options_OptionsDialog_acceptButton", "Besttigen"},
	{"options_OptionsDialog_acceptButton_tooltip", "nderungen besttigen, Fenster schliessen"},
	{"options_OptionsDialog_acceptButton_mnemonic", new Integer (KeyEvent.VK_A)},
	{"options_OptionsDialog_cancelButton", "Abbrechen"},
	{"options_OptionsDialog_cancelButton_tooltip", "Abbrechen und Fenster schliessen"},
	{"options_OptionsDialog_cancelButton_mnemonic", new Integer (KeyEvent.VK_B)},
	{"options_OptionsDialog_loadButton", "Einstellungen laden..."},
	{"options_OptionsDialog_loadButton_tooltip", "letzte gespeicherte Einstellungen laden"},
	{"options_OptionsDialog_loadButton_mnemonic", new Integer (KeyEvent.VK_L)},
	{"options_OptionsDialog_saveButton", "Einstellungen speichern..."},
	{"options_OptionsDialog_saveButton_tooltip", "Einstellungen in einer Datei Speichern."},
	{"options_OptionsDialog_saveButton_mnemonic", new Integer (KeyEvent.VK_S)},
	{"options_OptionsDialog_resetButton", "Standardeinstellung wiederherstellen"},
	{"options_OptionsDialog_resetButton_tooltip", "Auf Standardeinstellungen zurcksetzen."},
	{"options_OptionsDialog_resetButton_mnemonic", new Integer (KeyEvent.VK_R)},
	{"options_OptionsDialog_helpButton", "Hilfe"},
	{"options_OptionsDialog_helpButton_tooltip", "Hilfefenster anzeigen"},
	{"options_OptionsDialog_helpButton_mnemonic", new Integer (KeyEvent.VK_H)},
	// general tab
	{"options_OptionsDialog_generalPanel_tabheader", "Allgemein"},
	{"options_OptionsDialog_generalPanel_tabheader_tooltip", "nderungen der allgemeinen DeViSoR2 Optionen"},
	{"options_OptionsDialog_generalPanel_devisorhome","DEVISORHOME Verzeichnis"},
	{"options_OptionsDialog_generalPanel_devisorhome_tooltip","Anklicken um das Dateiauswahlfenster anzuzeigen"},
	{"options_OptionsDialog_generalPanel_devisorhome_dialogtitle","DEVISORHOME Verzeichnissauswahl"},
	{"options_OptionsDialog_generalPanel_generalpath","Standard Laden- und Speichernpfad"},
	{"options_OptionsDialog_generalPanel_generalpath_tooltip","Anklicken um das Dateiauswahlfenster anzuzeigen"},
	{"options_OptionsDialog_generalPanel_generalpath_dialogtitle","Standard Pfadauswahl"},
	{"options_OptionsDialog_generalPanel_trigen","TriGen2D Pfad"},
	{"options_OptionsDialog_generalPanel_trigen_tooltip","Anklicken um das Dateiauswahlfenster anzuzeigen"},
	{"options_OptionsDialog_generalPanel_trigen_dialogtitle","Trigen Pfadauswahl"},
	{"options_OptionsDialog_generalPanel_featlib","Feat Library Pfad"},
	{"options_OptionsDialog_generalPanel_featlib_tooltip","Anklicken um das Dateiauswahlfenster anzuzeigen"},
	{"options_OptionsDialog_generalPanel_featlib_dialogtitle","Feat Library Pfadauswahl"},

	// draw tab
	{"options_OptionsDialog_drawPanel_tabheader", "Zeicheneinstellungen"},
	{"options_OptionsDialog_drawPanel_tabheader_tooltip", "Ansicht der Zeichenflche einstellen"},
	{"options_OptionsDialog_drawPanel_boundary_header", "Rand"},
	{"options_OptionsDialog_drawPanel_boundary_draw", "Rand zeichnen"},
	{"options_OptionsDialog_drawPanel_boundary_number", "Randknotennummern anzeigen"},
	{"options_OptionsDialog_drawPanel_node_header", "Knoten"},
	{"options_OptionsDialog_drawPanel_node_draw", "Knoten zeichnen"},
	{"options_OptionsDialog_drawPanel_node_number", "Knotennummern anzeigen"},
	{"options_OptionsDialog_drawPanel_edge_header", "Ecken"},
	{"options_OptionsDialog_drawPanel_edge_draw", "Ecken zeichnen"},
	{"options_OptionsDialog_drawPanel_edge_number", "Eckennummern anzeigen"},
	{"options_OptionsDialog_drawPanel_element_header", "Elemente"},
	{"options_OptionsDialog_drawPanel_element_draw", "Elemente zeichnen"},
	{"options_OptionsDialog_drawPanel_element_number", "Elementenummern anzeigen"},
	{"options_OptionsDialog_drawPanel_element_cross", "Elemente kreuzen"},
	{"options_OptionsDialog_drawPanel_element_fill", "Elemente fllen"},
	{"options_OptionsDialog_drawPanel_misc_header", "Verschiedenes"},
	{"options_OptionsDialog_drawPanel_misc_grid", "Gitter zeichnen"},
	{"options_OptionsDialog_drawPanel_misc_perimeter", "Umfang zeichnen"},
	{"options_OptionsDialog_drawPanel_misc_rectangular", "rechteckige Elemente hervorheben"},
	{"options_OptionsDialog_drawPanel_edit_header", "Maus Eingabe"},
	{"options_OptionsDialog_drawPanel_edit_snapon", "Schnappmodus einschalten"},
	{"options_OptionsDialog_drawPanel_edit_snapx", "Schnappen in x Richtung"},
	{"options_OptionsDialog_drawPanel_edit_snapy", "Schnappen in y Richtung"},
	{"options_OptionsDialog_drawPanel_edit_tolerance", "Click Toleranz"},
	{"options_OptionsDialog_drawPanel_edit_drawcoords", "Benutzerkoordinaten anzeigen"},
	{"options_OptionsDialog_drawPanel_edit_draworientationmarkers", "Orientierungsmarkierungen zeichnen"},
	{"options_OptionsDialog_drawPanel_edit_epsilon", "Globales Epsilon"},
	{"options_OptionsDialog_drawPanel_edit_epsilon_sliderlow", "niedrig"},
	{"options_OptionsDialog_drawPanel_edit_epsilon_slidermedium", "mittel"},
	{"options_OptionsDialog_drawPanel_edit_epsilon_sliderhigh", "hoch"},

	// color tab
	{"options_OptionsDialog_colorPanel_tabheader", "Farbeinstellungen"},
	{"options_OptionsDialog_colorPanel_tabheader_tooltip", "Feineinstellung der Farbeinstellungen fr Zeichengegenstnde ndern"},
	{"options_OptionsDialog_colorPanel_objects_header", "Objekt"},
	{"options_OptionsDialog_colorPanel_objects_node", "Knoten:"},
	{"options_OptionsDialog_colorPanel_objects_node_dialog", "Knotenfarbe whlen"},
	{"options_OptionsDialog_colorPanel_objects_edge", "Ecken:"},
	{"options_OptionsDialog_colorPanel_objects_edge_dialog", "Eckenfarbe whlen"},
	{"options_OptionsDialog_colorPanel_objects_edgebase", "Kantenbasis:"},
	{"options_OptionsDialog_colorPanel_objects_edgebase_dialog", "Farbe fr Kantenbasis whlen"},
	{"options_OptionsDialog_colorPanel_objects_element", "Elemente:"},
	{"options_OptionsDialog_colorPanel_objects_element_dialog", "Elementenfarbe whlen"},
	{"options_OptionsDialog_colorPanel_numbers_header", "Beschriftung"},
	{"options_OptionsDialog_colorPanel_numbers_boundary", "Rndernummerierung:"},
	{"options_OptionsDialog_colorPanel_numbers_boundary_dialog", "Farbe fr Rndernummern whlen"},
	{"options_OptionsDialog_colorPanel_numbers_node", "Knotennummerierung:"},
	{"options_OptionsDialog_colorPanel_numbers_node_dialog", "Farbe fr Knotennummern whlen"},
	{"options_OptionsDialog_colorPanel_numbers_edge", "Eckennummerierung:"},
	{"options_OptionsDialog_colorPanel_numbers_edge_dialog", "Farbe fr Kantennummerierung auswhlen"},
	{"options_OptionsDialog_colorPanel_numbers_element", "Elementenummerierung:"},
	{"options_OptionsDialog_colorPanel_numbers_element_dialog", "Farbe fr Elementenummerierung auswhlen"},
	{"options_OptionsDialog_colorPanel_highlight_header", "Hervorheben"},
	{"options_OptionsDialog_colorPanel_highlight_selection", "Auswahl:"},
	{"options_OptionsDialog_colorPanel_highlight_selection_dialog", "Farbe fr hervorgehobene Auswahl whlen"},
	{"options_OptionsDialog_colorPanel_highlight_rect", "rektifizierte Elemente:"},
	{"options_OptionsDialog_colorPanel_highlight_rect_dialog", "Farbe fr hervorgehobene rektifizierte Elemente whlen"},
	{"options_OptionsDialog_colorPanel_misc_header", "Verschiedenes"},
	{"options_OptionsDialog_colorPanel_misc_grid", "Gitter:"},
	{"options_OptionsDialog_colorPanel_misc_grid_dialog", "Farbe fr Hintegrundgitter whlen"},
	{"options_OptionsDialog_colorPanel_misc_background", "Hintergrund:"},
	{"options_OptionsDialog_colorPanel_misc_background_dialog", "Hintegrundfarbe wahlen"},
	{"options_OptionsDialog_colorPanel_misc_lasso", "Lasso:"},
	{"options_OptionsDialog_colorPanel_misc_lasso_dialog", "Farbe fr das Lasso whlen"},
	{"options_OptionsDialog_colorPanel_fill_header", "Fllfarbe fr parallele Blcke"},
	{"options_OptionsDialog_colorPanel_fill0_dialog", "erste Farbe fr parallele Blcke auswhlen"},
	{"options_OptionsDialog_colorPanel_fill1_dialog", "zeite Farbe fr parallele Blcke auswhlen"},
	{"options_OptionsDialog_colorPanel_fill2_dialog", "dritte Farbe fr parallele Blcke auswhlen"},
	{"options_OptionsDialog_colorPanel_fill3_dialog", "vierte Farbe fr parallele Blcke auswhlen"},
	{"options_OptionsDialog_colorPanel_fill4_dialog", "fnfte Farbe fr parallele Blcke auswhlen"},
	{"options_OptionsDialog_colorPanel_fill5_dialog", "sechste Farbe fr parallele Blcke auswhlen"},
	{"options_OptionsDialog_colorPanel_fill6_dialog", "siebte Farbe fr parallele Blcke auswhlen"},
	{"options_OptionsDialog_colorPanel_fill7_dialog", "achte Farbe fr parallele Blcke auswhlen"},
	{"options_OptionsDialog_colorPanel_fill8_dialog", "neunte Farbe fr parallele Blcke auswhlen"},
	{"options_OptionsDialog_colorPanel_fill9_dialog", "zehnte Farbe fr parallele Blcke auswhlen"},
	{"options_OptionsDialog_colorPanel_fill10_dialog", "elfte Farbe fr parallele Blcke auswhlen"},
	{"options_OptionsDialog_colorPanel_fill11_dialog", "zwlfte Farbe fr parallele Blcke auswhlen"},
	{"options_OptionsDialog_colorPanel_fill12_dialog", "dreizehnte Farbe fr parallele Blcke auswhlen"},
	{"options_OptionsDialog_colorPanel_fill13_dialog", "vierzehnte Farbe fr parallele Blcke auswhlen"},
	{"options_OptionsDialog_colorPanel_fill14_dialog", "fnfzehnte Farbe fr parallele Blcke auswhlen"},
	{"options_OptionsDialog_colorPanel_fill15_dialog", "sechzehnte Farbe fr parallele Blcke auswhlen"},
	{"options_OptionsDialog_colorPanel_boundarypalette_header", "Rnder Farbpalette"},
	{"options_OptionsDialog_colorPanel_boundary0_dialog", "erste Farbe der Palette fr Rnder auswhlen"},
	{"options_OptionsDialog_colorPanel_boundary1_dialog", "zweite Farbe der Palette fr Rnder auswhlen"},
	{"options_OptionsDialog_colorPanel_boundary2_dialog", "dritte Farbe der Palette fr Rnder auswhlen"},
	{"options_OptionsDialog_colorPanel_boundary3_dialog", "vierte Farbe der Palette fr Rnder auswhlen"},
	{"options_OptionsDialog_colorPanel_boundary4_dialog", "fnfte Farbe der Palette fr Rnder auswhlen"},
	{"options_OptionsDialog_colorPanel_boundary5_dialog", "sechste Farbe der Palette fr Rnder auswhlen"},
	{"options_OptionsDialog_colorPanel_boundary6_dialog", "siebte Farbe der Palette fr Rnder auswhlen"},
	{"options_OptionsDialog_colorPanel_boundary7_dialog", "achte Farbe der Palette fr Rnder auswhlen"},
	{"options_OptionsDialog_colorPanel_boundary8_dialog", "neunte Farbe der Palette fr Rnder auswhlen"},
	{"options_OptionsDialog_colorPanel_boundary9_dialog", "zehnte Farbe der Palette fr Rnder auswhlen"},
	{"options_OptionsDialog_colorPanel_boundary10_dialog", "elfte Farbe der Palette fr Rnder auswhlen"},
	{"options_OptionsDialog_colorPanel_boundary11_dialog", "zwlfte Farbe der Palette fr Rnder auswhlen"},
	{"options_OptionsDialog_colorPanel_boundary12_dialog", "dreizehnte Farbe der Palette fr Rnder auswhlen"},
	{"options_OptionsDialog_colorPanel_boundary13_dialog", "vierzehnte Farbe der Palette fr Rnder auswhlen"},
	{"options_OptionsDialog_colorPanel_boundary14_dialog", "fnfzehnte Farbe der Palette fr Rnder auswhlen"},
	{"options_OptionsDialog_colorPanel_boundary15_dialog", "sechzehnte Farbe der Palette fr Rnder auswhlen"},

	// Accelerators tab
	{"options_OptionsDialog_acceleratorPanel_tabheader", "Tastaturkrzel"},
	{"options_OptionsDialog_acceleratorPanel_tabheader_tooltip", "Erstellen der eigenen Tastaturkrzlen"},
	{"options_OptionsDialog_acceleratorPanel_southpaneltitle", "ndern der aktuellen Auswahl"},
	{"options_OptionsDialog_acceleratorPanel_tabletitle", "aktuelle Einstellungen"},
	{"options_OptionsDialog_acceleratorPanel_tableleftcolumn", "Beschreibung"},
	{"options_OptionsDialog_acceleratorPanel_tablerightcolumn", "Krzel"},
	{"options_OptionsDialog_acceleratorPanel_applybuttontooltip", "bernehmen des Krzels fr die oben aufgefhrte Funktion"},
	{"options_OptionsDialog_acceleratorPanel_applybuttonmnemonic", new Integer (KeyEvent.VK_N)},
	{"options_OptionsDialog_acceleratorPanel_removebutton", "Krzel entfernen"},
	{"options_OptionsDialog_acceleratorPanel_removebuttontooltip", "Keine Krzel bernehmen"},
	{"options_OptionsDialog_acceleratorPanel_removebuttonmnemonic", new Integer (KeyEvent.VK_E)},
	{"options_OptionsDialog_acceleratorPanel_nocorrectaccelerator", "Ausgewhltes Krzel ist ungltig. \n Lesen sie das Handbuch um eine komplette Liste gltiger Krzel zu bekommen."},
	{"options_OptionsDialog_acceleratorPanel_acceleratoralreadyinuse", "Das aktuelle Krzel wird bereits verwendet. \n Bitte whlen sie ein anderes."},
	{"options_OptionsDialog_acceleratorPanel_youmustselectarow", "Whlen sie bitte zuerst eine Zeile in der oben aufgefhrten Tabelle. \n Ansonsten kann kein Krzel einer Funktion zugewiesen werden."},
	// other tab
	{"options_OptionsDialog_otherPanel_tabheader", "Anderes"},
	{"options_OptionsDialog_otherPanel_tabheader_tooltip", "Alles was nirgendwo hin passte"},
	{"options_OptionsDialog_otherPanel_lookandfeel","Look&Feel auswhlen"},
	{"options_OptionsDialog_otherPanel_lookandfeel_mnemonic", new Character ('K')},
	{"options_OptionsDialog_otherPanel_screenx","Fensterbreite"},
	{"options_OptionsDialog_otherPanel_screenx_mnemonic", new Character ('W')},
	{"options_OptionsDialog_otherPanel_screeny","Fensterhhe"},
	{"options_OptionsDialog_otherPanel_screeny_mnemonic", new Character ('H')},
	{"options_OptionsDialog_otherPanel_optionfileextension","Endung fr die Options Datei"},
	{"options_OptionsDialog_otherPanel_optionfileextension_mnemonic", new Character ('O')},
	
	
	/* keys for class devisor2.GUI.dialogs.InfoDialog */
	{"GUI_dialogs_InfoDialog_frametitle", "ber DeViSoRGrid2"},
	{"GUI_dialogs_InfoDialog_version", "Version"},
	{"GUI_dialogs_InfoDialog_developers", "Entwickler"},
	{"GUI_dialogs_InfoDialog_about", "Allgemeine Information"},
	{"GUI_dialogs_InfoDialog_abouttext1", "DeViSoRGrid 2 ist das the preprocessing tool in der "},
	{"GUI_dialogs_InfoDialog_abouttext2", "FEAST software Familie. Es erstellt Grobgitter gibt ihnen"},
	{"GUI_dialogs_InfoDialog_abouttext3", "die Mglichkeit Domains und Grobgitter durch ein"},
	{"GUI_dialogs_InfoDialog_abouttext4", "einfaches point&click Interface zu erstellen."},
	{"GUI_dialogs_InfoDialog_contact", "Wie man uns erreichen kann"},
	{"GUI_dialogs_InfoDialog_contact_text1", "Fr mehr Informationen und die letzten Updates,"},
	{"GUI_dialogs_InfoDialog_contact_text2", "besuchen sie bitte unsere Homepage unter"},
	{"GUI_dialogs_InfoDialog_contact_text3", "Oder, senden sie eine Email an:"},

	/* tooltips for class devisor2.GUI.framework.ToolBar */
	{"GUI_framework_ToolBar_newtooltip", "Erstellen eines neuen, leeren Gebiets"},
	{"GUI_framework_ToolBar_opentooltip", "Ein vorhandenes Gebiet"},
	{"GUI_framework_ToolBar_savetooltip", "Gebiet im aktuellen Format speichern"},
	{"GUI_framework_ToolBar_saveucdtooltip", "Gebiet im UCD Format speichern"},
	{"GUI_framework_ToolBar_savefeattooltip", "Gebiet im FEAT Format speichern"},
	{"GUI_framework_ToolBar_savefeasttooltip", "Gebiet im FEAST Format speichern"},
	{"GUI_framework_ToolBar_undotooltip", "letzte nderung rckgngig machen"},
	{"GUI_framework_ToolBar_redotooltip", "letzte nderung doch nicht rckgngig machen"},
	{"GUI_framework_ToolBar_cuttooltip", "ausgewhlte Gegenstnde auschneiden"},
	{"GUI_framework_ToolBar_copytooltip", "ausgewhlte Gegenstnde in die Zwischenablage kopieren"},
	{"GUI_framework_ToolBar_pastetooltip", "ausgewhlte Gegenstnde aus der Zwischenablage einfgen"},
	{"GUI_framework_ToolBar_selectnodestooltip", "Knoten auswhlen"},
	{"GUI_framework_ToolBar_selectelementstooltip", "Elemente auswhlen"},
	{"GUI_framework_ToolBar_selectsegmentstooltip", "Randteile auswhlen"},
	{"GUI_framework_ToolBar_selectedgestooltip", "Kanten auswhlen"},
	{"GUI_framework_ToolBar_selectboundarytooltip", "zu bearbeitenden Rand auswhlen"},
	{"GUI_framework_ToolBar_setzoomtooltip", "Vergrerungsfaktor eingeben"},
	{"GUI_framework_ToolBar_resetzoomtooltip", "Standard Vergrerung"},
	{"GUI_framework_ToolBar_zoomintooltip", "Vergrern"},
	{"GUI_framework_ToolBar_zoomouttooltip", "Verkleinern"},
	{"GUI_framework_ToolBar_directinputtooltip", "Eingabemodus wechseln"},
	{"GUI_framework_ToolBar_copymodetooltip", "Kopiermodus fr Randteile ndern"},
	{"GUI_framework_ToolBar_newboundarytooltip", "neuen Rand erstellen"},
	{"GUI_framework_ToolBar_addtypetooltip", "hinzuzufgenden Gegestand auswhlen"},
	{"GUI_framework_ToolBar_addnothingtooltip", "nichts hinzuzufgen"},
	{"GUI_framework_ToolBar_addclipboardtooltip", "Gegenstnde aus der Arbeitsablage hinzufgen."},
	{"GUI_framework_ToolBar_addlinetooltip", "Linien hinzufgen."},
	{"GUI_framework_ToolBar_addmultilinetooltip", "Mehrfachlinie hinzufgen."},
	{"GUI_framework_ToolBar_addcircleptooltip", "positivorientierten Kreis hinzufgen."},
	{"GUI_framework_ToolBar_addcirclemtooltip", "negativorientierten Kreis hinzufgen."},
	{"GUI_framework_ToolBar_addtritooltip", "Dreieckelemente hinzufgen."},
	{"GUI_framework_ToolBar_addquadtooltip", "Viereckelemente  hinzufgen.."},
	{"GUI_framework_ToolBar_addmacrotooltip", "MaKroelemente  hinzufgen.."},
	{"GUI_framework_ToolBar_addnodetooltip", "Knoten hinzufgen"},
	{"GUI_framework_ToolBar_addtypestooltip", "Auswahl des Elementtyps oder Randkomponententyps welches zum Gebiet hinzugefgt werden soll"},
	{"GUI_framework_ToolBar_propertiestooltip", "zum Eigenschaftsmodus wechseln"},
	{"GUI_framework_ToolBar_edgestatustooltip", "Eckenstatus setzten"},
	{"GUI_framework_ToolBar_parallelblocktooltip", "parallele Blcke setzen"},
	{"GUI_framework_ToolBar_rectifyelementtooltip", "Elemente rektifizieren"},
	{"GUI_framework_ToolBar_exittooltip", "DeViSoRGrid 2 beenden"},

	/* keys for class devisor2.GUI.framework.MainFrame */
	{"GUI_framework_Mainframe_frametitle", "DeViSoRGrid2 - Preprocessing fr die FEAST Software Familie"},

	/* keys for class devisor2.GUI.help.HelpFrame */
	{"GUI_help_HelpFrame_frametitle", "DeViSoRGrid2 online Handbuch"},

	/* keys for class devisor2.GUI.help.FAQFrame */
	{"GUI_help_FAQFrame_frametitle", "DeViSoRGrid2 FAQs"},

	/* keys for the FilePickerFilter class */
	{"GUI_dialogs_FilePickerFilter_description_ucd", "Dateien im UCD format (.inp)"},
	{"GUI_dialogs_FilePickerFilter_description_feat", "Dateien im FEAT format (.prm, .tri)"},
	{"GUI_dialogs_FilePickerFilter_description_feast", "Dateien im FEAST format (.feast)"},
	{"GUI_dialogs_FilePickerFilter_description_all", "Alle Datein"},
	{"GUI_dialogs_FilePickerFilter_description_directories", "Es sind nur Verzeichnisse erlaubt"},
	{"GUI_dialogs_FilePickerFilter_description_binaries", "ausfhrbare Dateien"},
	{"GUI_dialogs_FilePickerFilter_description_ps", "PostScript-Dateien"},
	{"GUI_dialogs_FilePickerFilter_description_default", "Alle untersttzten Dateiformate (.inp, .prm, .tri, .feast)"},

	/* keys for the EDGE STATUS dialog */
	{"dialogs_edgestatus_title", "Kantenstatus"},
	{"dialogs_edgestatus_status_caption", "Kantenstatus"},
	{"dialogs_edgestatus_status_none", "kein"},
	{"dialogs_edgestatus_status_real", "echt"},
	{"dialogs_edgestatus_status_inner", "innerer"},
	{"dialogs_edgestatus_status_mirror", "gespiegelt"},
	{"dialogs_edgestatus_bc_caption", "Randbedingungen"},
	{"dialogs_edgestatus_bc_dirichlet", "Dirichlet"},
	{"dialogs_edgestatus_bc_neumann", "Neumann"},
	{"dialogs_edgestatus_bc_none", "keine"},
	{"dialogs_edgestatus_close", "Schliessen"},
	{"dialogs_edgestatus_close_tooltip", "Statussetzen beenden"},
	{"dialogs_edgestatus_close_mnemonic", new Integer (KeyEvent.VK_C)},
	{"dialogs_edgestatus_apply", "bernehmen"},
	{"dialogs_edgestatus_apply_tooltip", "Kantenstatus bernehmen"},
	{"dialogs_edgestatus_apply_mnemonic", new Integer (KeyEvent.VK_A)},	{
         "dialogs_edgestatus_automatic", "Einstellungen automatisch vornehmen"},
	{"dialogs_edgestatus_automatic_tooltip", "Eckenstatus automatisch setzen"},
	{"dialogs_edgestatus_automatic_mnemonic", new Integer (KeyEvent.VK_A)},
	/* keys for the PARALLELBLOCK dialog */
	{"dialogs_parallelblock_title", "parallele Blcke & Makrozerlegung"},
	{"dialogs_parallelblock_apply", "bernehmen"},
	{"dialogs_parallelblock_apply_tooltip", "Diese Einstellungen fr das ausgewahlte Makroelement bernehmen"},
	{"dialogs_parallelblock_apply_mnemonic", new Integer (KeyEvent.VK_A)},
	{"dialogs_parallelblock_close", "Schliessen"},
	{"dialogs_parallelblock_close_tooltip", "Das Fenster schliessen"},
	{"dialogs_parallelblock_close_mnemonic", new Integer (KeyEvent.VK_C)},
	{"dialogs_parallelblock_automatic", "Automatisch"},
	{"dialogs_parallelblock_automatic_tooltip", "parallele Blcke automatisch setzen"},
	{"dialogs_parallelblock_automatic_mnemonic", new Integer (KeyEvent.VK_U)},
	{"dialogs_parallelblock_parallelblock", "parallele Blcke"},
	{"dialogs_parallelblock_matrixblock", "Matrix Block"},
	{"dialogs_parallelblock_macrorefinement", "Makro Zerlegungslevel"},
	{"dialogs_parallelblock_anisotrophy_x", "Makro X Anisotrophy"},
	{"dialogs_parallelblock_anisotrophy_x_none", "keine"},
	{"dialogs_parallelblock_anisotrophy_x_left", "links"},
	{"dialogs_parallelblock_anisotrophy_x_right", "rechts"},
	{"dialogs_parallelblock_anisotrophy_y", "Makro Y Anisotrophy"},
	{"dialogs_parallelblock_anisotrophy_y_none", "keine"},
	{"dialogs_parallelblock_anisotrophy_y_down", "unten"},
	{"dialogs_parallelblock_anisotrophy_y_up", "oben"},
	{"dialogs_parallelblock_factors", "Makro Verfeinerungsfaktor  "},

	/* keys for the REFINE dialog */
	{"dialogs_refine_title", "Automatische Verfeinerungsparameter"},
	{"dialogs_refine_apply", "bernehmen"},
	{"dialogs_refine_apply_tooltip", "TRIGEN2D mit diesen Parametern starten"},
	{"dialogs_refine_apply_mnemonic", new Integer (KeyEvent.VK_A)},
	{"dialogs_refine_close", "Schliessen"},
	{"dialogs_refine_close_tooltip", "Automatische Verfeinerung beenden"},
	{"dialogs_refine_close_mnemonic", new Integer (KeyEvent.VK_B)},
	{"dialogs_refine_type", "Verfeinerungstechnik"},
	{"dialogs_refine_no", "Keine adaptive Verfeinerung"},
	{"dialogs_refine_delma33", "3x3 Verfeinerung mit DELMA"},
	{"dialogs_refine_boundary3", "3 Randverfeinerung mit DELMA/DELMB"},
	{"dialogs_refine_boundary2", "2 Randverfeinerung mit DELMA"},
	{"dialogs_refine_boundarynormal", "3 Randnormalverfeimerumg mit DELMA"},
	{"dialogs_refine_delma5", "5 Verfeinerung mit DELMA"},
	{"dialogs_refine_corner", "3 Eckenverfeinerung mit DELMA"},
	{"dialogs_refine_parameters", "Verfeinerungsparameters"},
	{"dialogs_refine_parameter1", "erster Parameter: "},
	{"dialogs_refine_parameter2", "zweiter Parameter: "},
	{"dialogs_refine_level", "Verfeinerungslevel: "},

	/* keys for the loadbalancing dialog */
	{"dialogs_loadbalancing_title", "LoadBalancing"},
	{"dialogs_loadbalancing_mpi2sol", "MPI2SOL Pfad:"},
	{"dialogs_loadbalancing_filename", "Dateiname:"},
	{"dialogs_loadbalancing_calc", "Berechnen"},
	{"dialogs_loadbalancing_calc_mnemonic", new Integer (KeyEvent.VK_E)},
	{"dialogs_loadbalancing_perform", "Perform"},
	{"dialogs_loadbalancing_perform_mnemonic", new Integer (KeyEvent.VK_P)},

	/* keys for the line dialog */
	{"dialogs_line_caption_direct", "direkte Eingabe fr Linien"},
	{"dialogs_line_caption_property", "Linieneigenschaften"},
	{"dialogs_line_startpoint", "Startkoordinaten:"},
	{"dialogs_line_endpoint", "Endkoordinaten:"},
	{"dialogs_line_error_x1", "Das Eingabefeld der X-Koordinate fr den Sartpunkt \n enthlt keinen gltigen Wert."},
	{"dialogs_line_error_y1", "Das Eingabefeld der Y-Koordinate fr den Sartpunkt \n enthlt keinen gltigen Wert."},
	{"dialogs_line_error_x2", "Das Eingabefeld der X-Koordinate fr den Endpunkt \n enthlt keinen gltigen Wert"},
	{"dialogs_line_error_y2", "Das Eingabefeld der Y-Koordinate fr den Endpunkt \n enthlt keinen gltigen Wert."},

	/* keys for the MultipleBNDialog */
	{"dialogs_multiplebn_title","Mehrfachrandknoten"},
	{"dialogs_multiplebn_boundary","Rand:"},
	{"dialogs_multiplebn_maxpar","MaxPar:"},
	{"dialogs_multiplebn_increment","Erhhung um:"},
	{"dialogs_multiplebn_start","StartPar:"},
	{"dialogs_multiplebn_end","EndPar:"},
	{"dialogs_multiplebn_error_boundary", "Das 'Rand'-Textfeld enthlt keinen Integerwert !"},
	{"dialogs_multiplebn_error_increment","Das 'Erhhen um'-Textfeld enthlte keinen Doublewert."},
	{"dialogs_multiplebn_error_start","Das 'Startparameter'-Textfeld enthlte keinen Doublewert."},
	{"dialogs_multiplebn_error_end","Das 'Endparameter'-Textfeld enthlt keinen Doublewert."},
	{"dialogs_multiplebn_error_nosuchboundary","Das Gebiet enhlt keinen Rand mit dieser Nummer !"},
	{"dialogs_multiplebn_error_starttoosmall","Der Startparameterwert ist zu klein.\n Der Wert darf nicht kleiner als 0 sein."},
	{"dialogs_multiplebn_error_endtoobig", "Der Endparameterwert ist zu gross.\n Der eingegebene Wert darf nicht grer als der angezeige Maximalwert sein."},


	/* keys for the circle dialog */
	{"dialogs_circle_caption_direct_p", "Direkte Eingabe fr positiv orientierten Kreis"},
	{"dialogs_circle_caption_direct_m", "Direkte Eingabe fr negativ orientierten Kreis"},
	{"dialogs_circle_caption_property_p", "p Kreiseigenschaften"},
	{"dialogs_circle_caption_property_m", "n Kreiseigenschaften"},
	{"dialogs_circle_center", "Mittelpunkt: "},
	{"dialogs_circle_radius", "Radius: "},
	{"dialogs_circle_startangle", "Startwinkel: "},
	{"dialogs_circle_endangle", "Endwinkel: "},
	{"dialogs_circle_error_x", "Das Eingabefeld der X-Koordinate fr den Mittelpunkt \n enthlt keinen gltigen Wert."},
	{"dialogs_circle_error_y", "Das Eingabefeld der Y-Koordinate fr den Sartpunkt \n enthlt keinen gltigen Wert."},
	{"dialogs_circle_error_radius", "Das Eingabefeld fr den Radius \n enthlt keinen gltigen Wert."},
	{"dialogs_circle_error_start", "Das Eingabefeld fr den Sartwinkel \n enthlt keinen gltigen Wert"},
	{"dialogs_circle_error_end", "Das Eingabefeld fr den Endwinkel \n enthlt keinen gltigen Wert"},
	{"dialogs_circle_error_orientation", "Dieser Kreis wre falsch orientiert.\n Bitte berprfen sie den Start- und Endwinkel."},

	/* keys for the BoundaryNodeDialog */
	{"dialogs_boundarynode_caption_direct", "direkte Eingabe fr Knoten"},
	{"dialogs_boundarynode_caption_property", "Knoteneigenschaften"},
	{"dialogs_boundarynode_boundary", "Rand: "},
	{"dialogs_boundarynode_segment", "Randteil: "},
	{"dialogs_boundarynode_maxpar", "t-Max: "},
	{"dialogs_boundarynode_actpar", "t-Act "},
	{"dialogs_boundarynode_editcheck", "letzter hinzugefhgter Rand"},
	{"dialogs_boundarynode_coord", "Koordinaten: "},
	{"dialogs_boundarynode_nodeprop", "Knoteneigenschaft:"},
	{"dialogs_boundarynode_createboundary", "Randknoten erstellen"},
	{"dialogs_boundarynode_fastforward", ">>"},
	{"dialogs_boundarynode_fastforward_tooltip", "grosse Erhhung"},
	{"dialogs_boundarynode_forward", ">"},
	{"dialogs_boundarynode_forward_tooltip", "kleine Erhhung"},
	{"dialogs_boundarynode_rewind", "<"},
	{"dialogs_boundarynode_rewind_tooltip", "kleine Erniedrigung"},
	{"dialogs_boundarynode_fastrewind", "<<"},
	{"dialogs_boundarynode_fastrewind_tooltip", "grosse Erniedrigung"},
	{"dialogs_boundarynode_para_tabheader", "parametrische Beschreibung"},
	{"dialogs_boundarynode_coord_tabheader", "kartesische Beschreibung"},
	{"dialogs_boundarynode_para_tooltip", "Parameterbeschreibung der Randknoten"},
	{"dialogs_boundarynode_coord_tooltip", "kartesische Beschreibung der Randknoten"},
	{"dialogs_boundarynode_error_coord_x", "Das Eingabefeld fr die X-Koordinate \n enthlt keinen gltigen Wert."},
	{"dialogs_boundarynode_error_coord_y", "Das Eingabefeld fr die Y-Koordinate \n enthlt keinen gltigen Wert."},
	{"dialogs_boundarynode_error_nodeprop", "Das Eingabefeld fr die Knoteneigenschaft \n enthlt keinen gltigen Wert."},
	{"dialogs_boundarynode_error_actpar", "Das Eingabefeld fr den aktuellen Parameterwert \n enthlt keinen gltigen Wert."},
	{"dialogs_boundarynode_error_boundnum", "Das Eingabefeld fr die Randnummer \n enthlt keinen gltigen Wert."},
	{"dialogs_boundarynode_error_boundrange", "Das Eingabefeld fr die Y-Koordinate \n enthlt keinen gltigen Wert. \n Es gibt keinen Rand mit dieser Nummer!"},
	{"dialogs_boundarynode_error_actrange", "Das Eingabefeld fr den aktuellen Parameter \n enthlt keinen gltigen Wert. \n Der Wert ist nicht erlaubten Bereich !"},
	{"dialogs_boundarynode_error_editlast", "Es gibt keinen Knoten um die nderungen vorzunehmen. \n Es mssen erst Knoten hinzugefgt werden, bevor sie verndert werden!."},
	{"dialogs_boundarynode_error_usepara", "Bitte Parametrisation benutzen"},
	{"dialogs_boundarynode_error_useparatext", "Es ist nicht erlaubt Randknoten \n durch kartesische Beschreibung zu erstellen oder zu verndern."},
	{"dialogs_boundarynode_error_para", "Parametrisationserror"},
	{"dialogs_boundarynode_error_regularparametric", "Nur Randknoten besitzten eine parametrische Beschreibung !"},

	/* keys for PerimeterDialog */
	{"dialogs_perimeter_title", "Gebietsumfang"},
	{"dialogs_perimeter_x1", "linke obere Ecke   "},
	{"dialogs_perimeter_x2", "rechte untere Ecke "},
	{"dialogs_perimeter_error_x1", "Das Eingabefeld fr die X Koordinate der linken oberen Ecke \n enthlt keinen gltigen Wert."},
	{"dialogs_perimeter_error_y1", "Das Eingabefeld fr die Y Koordinate der linken oberen Ecke \n enthlt keinen gltigen Wert."},
	{"dialogs_perimeter_error_x2", "Das Eingabefeld fr die X Koordinate der rechten unteren Ecke \n enthlt keinen gltigen Wert."},
	{"dialogs_perimeter_error_y2", "Das Eingabefeld fr die Y Koordinate der rechten unteren Ecke \n enthlt keinen gltigen Wert."},

	/* keys for the QuadDialog */
	{"dialogs_quad_caption_direct", "direkte Eingabe fr Quadrate"},
	{"dialogs_quad_caption_property", "Quadrateingeschaften"},
	{"dialogs_quad_node1", "Knoten 1:"},
	{"dialogs_quad_node2", "Knoten 2:"},
	{"dialogs_quad_node3", "Knoten 3:"},
	{"dialogs_quad_node4", "Knoten 4:"},
	{"dialogs_quad_error_node1", "Das Eingabefeld fr den ersten Knoten \n enthlt keinen gltigen Wert."},
	{"dialogs_quad_error_node2", "Das Eingabefeld fr den zweiten Knoten \n enthlt keinen gltigen Wert."},
	{"dialogs_quad_error_node3", "Das Eingabefeld fr den dritten Knoten \n enthlt keinen gltigen Wert."},
	{"dialogs_quad_error_node4", "Das Eingabefeld fr den vierten Knoten \n enthlt keinen gltigen Wert."},

	/* keys for the Macro Dialog */
	{"dialogs_macro_caption_direct", "Direkte Macro Eingabe"},
	{"dialogs_macro_caption_property", "Macro Eigenschaften"},
	{"dialogs_macro_nodes_tabheader", "Knoten"},
	{"dialogs_macro_nodes_tooltip", "Knoten eingeben, die das Macro aufspannen"},
	{"dialogs_macro_blocks_tabheader", "Parallel"},
	{"dialogs_macro_blocks_tooltip", "Parallelverarbeitung fuer dieses Macro festlegen"},
	{"dialogs_macro_parallelblock", "Parallelblock"},
	{"dialogs_macro_matrixblock", "Matrixblock"},
	{"dialogs_macro_macrorefinement", "Macro Verfeinerungslevel"},
	{"dialogs_macro_anisotrophy_x", "Macro X Anisotrophie"},
	{"dialogs_macro_anisotrophy_x_none", "Keine"},
	{"dialogs_macro_anisotrophy_x_left", "Links"},
	{"dialogs_macro_anisotrophy_x_right", "Rechts"},
	{"dialogs_macro_anisotrophy_y", "Macro Y Anisotrophie"},
	{"dialogs_macro_anisotrophy_y_none", "Keine"},
	{"dialogs_macro_anisotrophy_y_down", "Runter"},
	{"dialogs_macro_anisotrophy_y_up", "Hoch"},
	{"dialogs_macro_factors", "Macro Verfeinerungsfaktoren"},
	{"dialogs_macro_node1", "Knoten 1:"},
	{"dialogs_macro_node2", "Knoten 2:"},
	{"dialogs_macro_node3", "Knoten 3:"},
	{"dialogs_macro_node4", "Knoten 4:"},
	{"dialogs_macro_error_node1", "Das Eingabefeld fuer den ersten Knoten \n enthaelt eine ungueltige Eingabe."},
	{"dialogs_macro_error_node2", "Das Eingabefeld fuer den zweiten Knoten \n enthaelt eine ungueltige Eingabe."},
	{"dialogs_macro_error_node3", "Das Eingabefeld fuer den dritten Knoten \n enthaelt eine ungueltige Eingabe."},
	{"dialogs_macro_error_node4", "Das Eingabefeld fuer den vierten Knoten \n enthaelt eine ungueltige Eingabe."},
	{"dialogs_macro_error_pb", "Das Eingabefeld fuer den Parallelblock \n enthaelt eine ungueltige Eingabe."},
	{"dialogs_macro_error_ref", "Das Eingabefeld fuer den Verfeinerungslevel \n enthaelt eine ungueltige Eingabe."},
	{"dialogs_macro_error_mb", "Das Eingabefeld fuer den Matrixblock \n enthaelt eine ungueltige Eingabe."},
	{"dialogs_macro_error_factors", "Die Eingabefelder fuer die Verfeinerungsfaktoren \n enthalten eine ungueltige Eingabe."},

	/* keys for the TriDialog */
	{"dialogs_tri_caption_direct", "direkte Eingabe fr Dreiecke"},
	{"dialogs_tri_caption_property", "Dreickseigenschaften"},
	{"dialogs_tri_node1", "Knoten 1:"},
	{"dialogs_tri_node2", "Knoten 2:"},
	{"dialogs_tri_node3", "Knoten 3:"},
	{"dialogs_tri_error_node1", "Das Eingabefeld fr den ersten Knoten \n enthlt keinen gltigen Wert."},
	{"dialogs_tri_error_node2", "Das Eingabefeld fr den zweiten Knoten \n enthlt keinen gltigen Wert."},
	{"dialogs_tri_error_node3", "Das Eingabefeld fr den dritten Knoten \n enthlt keinen gltigen Wert."},
	{"dialogs_tri_error_nodeorientation", "Die ausgewhlten Knoten sind nicht richtig orientiert !"},
	{"dialogs_tri_error_samenodes", "Achtung: Es mssen unterschiedliche Knoten sein !"},

	/* keys for the RotateDialog */
	{"dialogs_rotate_caption", "Gegenstand drehen"},
	{"dialogs_rotate_label", "Drehwinkel:"},
	{"dialogs_rotate_error", "Geben sie einen gltigen Winkel in Grad an!"},
	{"dialogs_rotate_nodes_error", "Es knnen nicht nur ausschliesslich Knoten gedreht werden"},

	/* keys for the ScaleDialog */
	{"dialogs_scale_caption", "Gegenstand skalieren"},
	{"dialogs_scale_x", "Faktor in X Richtung:"},
	{"dialogs_scale_y", "Faktor in Y Richtung:"},
	{"dialogs_scale_x_error", "Kein gltiger Wert fr die Skalierung in X Richtung"},
	{"dialogs_scale_y_error", "Kein gltiger Wert fr die Skalierung in Y Richtung"},
	{"dialogs_scale_nodes_error", "Es knnen nicht nur ausschliesslich Knoten rotiert werden"},
	{"dialogs_scale_circles_error", "Kreise drfen nur einheitlich skaliert werden !"},

	/* keys for MirrorDialog */
	{"dialogs_mirror_caption", "Gegenstnde spiegeln"},
	{"dialogs_mirror_start", "Startkoordinaten der Spiegelgeraden "},
	{"dialogs_mirror_end", "Endkoordinaten der Spiegelgeraden"},
	{"dialogs_mirror_x1_error", "Fehler bei der Startkoordinate.\n berprfen sie die Benutzerkoordinaten."},
	{"dialogs_mirror_y1_error", "Fehler bei der Startkoordinate.\n berprfen sie die Benutzerkoordinaten."},
	{"dialogs_mirror_x2_error", "Fehler bei der Endkoordinate.\n berprfen sie die Benutzerkoordinaten.."},
	{"dialogs_mirror_y2_error", "Fehler bei der Endkoordinate.\n berprfen sie die Benutzerkoordinaten.."},

	/* keys for the ChangeBoundaryDialog */
	{"dialogs_changeb_caption", "Randauswahl"},
	{"dialogs_changeb_label", "aktueller Rand: "},
	{"dialogs_changeb_error_nobounds", "Das Gebiet enthlt keine Rnder .\n Es knnen keine Rnder ausgewhlt werden"},

	/* keys for the TEXTFRAME dialog */
	{"dialogs_textframe_tooltip", "Anklicken um das Fenster zu schliessen"},
	{"dialogs_textframe_loaderror_title", "Textfehler"},
	{"dialogs_textframe_loaderror_message", "Parsingfehler"},

	/* keys for the GRIDLISTENER */
	{"gridlistener_error_nonodehit", "Es wurde kein Knoten angeklickt. \n Um Elemente zu erstellen mssen erst Knoten existieren"},
	{"gridlistener_error_radiuszero", "Kreise mit Radius gleich Null sind nicht erlaubt !"},
	{"gridlistener_error_noboundary", "Es gibt keinen Rand um Randteile hinzuzufgen. \n Erstellen sie erst einen Rand um Randteile hinzuzufgen."},
	{"gridlistener_error_boundarymove", "Randknoten drfen nicht auf dem Gitter verschoben werden.!"},
	{"gridlistener_error_nosegments", "Es gibt keine Randteile um Randknoten hinzuzufgen.\n Erstellen sie erst einen Rand und Ranteile um Randknoten hinzuzufgen."},
	{"gridlistener_error_nodeorientation", "Die ausgewhlten Knoten sind nicht richtig orientiert.\n Es wurde kein Element hinzugefgt."},
	{"gridlistener_error_nodetwicehit", "Ein Knoten wurde doppelt angeklickt!\n Es wurde kein Element hinzugefgt."},
	{"gridlistener_error_samepointtwice", "Eine Linie ist kein Punkt \n Derselbe Punkt darf nicht zweimal angeklickt werden!"},
	{"gridlistener_error_clone", "Unbehebbarer Fehler bein Randknoten einfgen."},

	/* keys for the ContBoundary */
	{"continousBoundary_startmessage", "stetige Randfortsetzung: starten ..."},
	{"continousBoundary_procboundary", "Rnder bearbeiten "},
	{"continousBoundary_procsegment", "  Randteile bearbeiten "},
	{"continousBoundary_bonderror", "  Rand konnte nicht stetig fortgesetzt werden, Rnder werden umorientiert"},
	{"continousBoundary_segfound", "  Kein entsprechendes Randteil gefunden, Umorientierung"},
	{"continousBoundary_error", "Fehler: keine stetige Fortsetzung mglich!"},
	{"continousBoundary_action1", "  Fortsetzungsaktion: Startpunkt angleichen"},
	{"continousBoundary_action2", "  Fortsetzungsaktion: Angleichen und Umorienteirung des Startpunktes"},
	{"continousBoundary_ok", "Erfolg: Rnder angeglichen !"},

	/* keys for the swapOrientation */
	{"swapOrientation_startmessage", "Orientierungsnderung: starten ..."},
	{"swapOrientation_procsegment", "  Randteile bearbeiten "},
	{"swapOrientation_change1", "  Randteile ndern "},
	{"swapOrientation_change2", " mit "},
	{"swapOrientation_procnode", "Knoten bearbeiten "},
	{"swapOrientation_ok", "Erfolg: Orientierung des aktuellen Randes wurde gendert !"},

	/* keys for the mode display in the statusbar */
	{"modedisplay_mode", "Modus"},
	{"modedisplay_zooming", "Zoomen"},
	{"modedisplay_drawing", "Zeichnen"},
	{"modedisplay_edgestatus", "Spezifikation der Kantenstati"},
	{"modedisplay_parallelblocks", "Spezifikation der Parallelblcke"},
	{"modedisplay_propertyquery", "Abfragen"},
	{"modedisplay_refine", "Verfeinern"}, 
	{"modedisplay_rectify", "Rektifizieren"},
	{"modedisplay_mirror", "Spiegeln"},  
	{"modedisplay_none", "kein Modus"},
	{"modedisplay_adding", "Hinzugefgt werden"},
	{"modedisplay_nodes", "Knoten"},
	{"modedisplay_lines", "Linien"},
	{"modedisplay_multilines", "Multilinien"},
	{"modedisplay_circlep", "pos. Kreissegmente"},
	{"modedisplay_circlem", "neg. Kreissegmente"},
	{"modedisplay_clipboard", "Inhalte aus der Zwischenablage"},
	{"modedisplay_tris", "Tris"},
	{"modedisplay_quads", "Quads"},
	{"modedisplay_macros", "Macros"},
	{"modedisplay_selecting", "Selektierbar sind"},
	{"modedisplay_edges", "Kanten"},
	{"modedisplay_elements", "Elemente"},
	{"modedisplay_segments", "Randteile"},
	{"modedisplay_copymode_append", "Einfgemodus: Anhngen"},
	{"modedisplay_copymode_keep", "Einfgemodus: Struktur erhalten"},

	/* keys for the spellchecker package */
	{"spellchecker_dialog_title", "Rechtschreibpruefung"},
	{"spellchecker_dialog_settings_tabheader", "Einstellungen"},
	{"spellchecker_dialog_settings_tabheader_tooltip", "Konfiguration der Fehlerparameter."},
	{"spellchecker_dialog_table_tabheader", "Fehlerliste"},
	{"spellchecker_dialog_table_tabheader_tooltip", "Darstellung und Editierung der Fehlerliste."},
	{"spellchecker_dialog_closebutton_tooltip", "Rechtschreibpruefung beenden."},
	{"spellchecker_dialog_isolatednodes", "Isolierte Knoten"},
	{"spellchecker_dialog_innerangles", "Elemente mit kleineren Innenwinkeln als "},
	{"spellchecker_dialog_aspectratios", "Elemente mit groesserem Seitenverhaeltnis als "},
	{"spellchecker_dialog_neighbouring", "Benachbarte Elemente mit groesserem Flaechenverhaeltnis als "},
	{"spellchecker_dialog_discontinous", "Unstetige Randbeschreibung"},
	{"spellchecker_dialog_nonconforming", "Nichtkonforme Elemente"},
	{"spellchecker_dialog_zoom", "Zentrieren"},
	{"spellchecker_dialog_zoom_tooltip", "Zentriert die Ansicht auf das markierte Element."},
	{"spellchecker_dialog_zoom_mnemonic", new Integer (KeyEvent.VK_Z)},
	{"spellchecker_dialog_remove", "Loeschen"},
	{"spellchecker_dialog_remove_tooltip", "Entfernt das markierte Element aus der Liste."},
	{"spellchecker_dialog_remove_mnemonic", new Integer (KeyEvent.VK_L)},
	{"spellchecker_dialog_refresh", "Aktualisieren"},
	{"spellchecker_dialog_refresh_tooltip", "Aktualisiert die Fehlerliste."},
	{"spellchecker_dialog_refresh_mnemonic", new Integer (KeyEvent.VK_A)},
	{"spellchecker_dialog_moreinfo", "Elementeigenschaften ..."},
	{"spellchecker_dialog_moreinfo_tooltip", "Zeigt den passenden Eigenschaftsdialog zu dem markierten Element an."},
	{"spellchecker_dialog_moreinfo_mnemonic", new Integer (KeyEvent.VK_E)},
	{"spellchecker_dialog_error_innerangles", "Das Eingabefeld fuer den Schwellwert fuer Innenwinkel\n enthaelt keine gueltige Eingabe.\nInnenwinkel werden nicht beruecksichtigt."},
	{"spellchecker_dialog_error_aspectratio", "Das Eingabefeld fuer den Schwellwert fuer Seitenverhaeltnisse\n enthaelt keine gueltige Eingabe.\nSeitenverhaeltnisse werden nicht beruecksichtigt."},
	{"spellchecker_dialog_error_neighbouring", "Das Eingabefeld fuer den Schwellwert fuer Flaechenverhaeltnisse\n enthaelt keine gueltige Eingabe.\nFlaechenverhaeltnisse werden nicht beruecksichtigt."},
	{"spellchecker_dialog_error_noselection", "In der Liste ist kein Eintrag markiert."},

	{"spellchecker_tablemodel_columnheader_type", "Beschreibung"},
	{"spellchecker_tablemodel_columnheader_number", "Nummer"},
	{"spellchecker_tablemodel_isolatednode", "Isolierter Knoten"},
	{"spellchecker_tablemodel_innerangle", "Innenwinkel"},
	{"spellchecker_tablemodel_aspectratio", "Seitenverhaeltnis"},
	{"spellchecker_tablemodel_neighbours", "Flaechenverhaeltnis"},

	/* keys for the MergeBoundariesDialog */
	{"dialogs_mergeboundaries_caption", "Zwei Raender vereinigen"},
	{"dialogs_mergeboundaries_targetlabel", "Zielrand:"},
	{"dialogs_mergeboundaries_targettooltip", "Rand eingeben, der ueberlebt"},
	{"dialogs_mergeboundaries_sourcelabel", "Quellrand:"},
	{"dialogs_mergeboundaries_sourcetooltip","Rand eingeben, der aufgeloest werden soll."},
	{"dialogs_mergeboundaries_error_sameboundaries", "Ein Rand kann nicht mit sich selbst vereinigt werden."},


   };

    
}






















