package devisor2.grid.options;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/** 
 *  This class provides the localization information for english language
 *  and US country scheme (e.g. keyboard layout).
 *  All text messages for the whole Application are stored in this class's
 *  rather giant key-value Object[][] attribute with the key being the first
 *  argument (a constant String) and the value the second argument. <br>
 *  <br>
 *  As the locale is dynamically linked to the application, a 
 *  MissingRessourceException is thrown during startup if you use a key
 *  not specified in this class. So please NEVER modify the keys! <br>
 *  <br>
 *  If you want to create your own localized version of the application, just
 *  create a copy of this class in the same directory and change the en to 
 *  your target language (a pair of lowercase letters that conform to 
 *  ISO-639), and the US to your target country scheme (two uppercase letters
 *  that conform to ISO-3166) in both filename and classname. 
 *  After that, just try not to get bored while translating all the 
 *  right-hand String values assigned to the keys. Again: DO NOT TRANSLATE
 *  THE KEYS! <br>
 *  By convention, the first part of each key consists of the full qualified
 *  package name, followed by an underscore (_), the class name, underscore,
 *  method- or attribute name, underscore and then your identifier.
 *  Some general keys like the captions of buttons used everywhere throughout
 *  the application etc. are stored with the prefix general_. <br>
 *  <br>
 *  This class is also made globally accessible in the ControlCenter class. 
 *  <br> <br>
 *  Throughout the application, the contents of this class are accessible
 *  via the getObject(key) method. Note that an Object is returned which
 *  normally has to be casted to a String. On the other hand, also
 *  menu mnemonics are saved in here, so no specific casting takes
 *  place in this class. <br>
 *  <br>
 *  Example: <br>
 *  Let's say you want to extract the caption for an APPLY-button.
 *  For this application, it is stored under the key "general_Apply"
 *  Via the ControlCenter instance set up during program startup (suppose you
 *  have a reference in your class called cc), you have access to the 
 *  ResourceBundle for your locale (in fact, you are just reading the class 
 *  info of the ResourceBundle for english language...), named rb in the 
 *  ControlCenter class and declared as public there.
 *  You want to extract a String, but as said before, an Object is returned,
 *  so you might have to cast to avoid an error during compilation. <br>
 *  So all you have to do is: <br>
 *  (1) get the object stored under the specified key you are looking for:<br>
 *      <code>Object obj = cc.op.getObject ("general_apply"): <\code> <br>
 *  (2) cast it to a String:<br>
 *      <code> String myCaption = (String)obj; <\code> <br>
 *  (3) use it whichever way you want: <br>
 *      <code>JButton myApplyButton = new JButton (myCaption);<\code> <br>
 *  (4) or, to save time writing code, you can do this all in one step: <br>
 *      <code>JButton myApplyButton = new JButton ((String)cc.op.get ("general_apply"));<\code> <br>
 *  Analogously, the Mnemonics are stored as objects of class Integer, so 
 *  you have to cast to an Integer then and get the int value by invoking
 *  the Integer.intValue() method. <br>
 *
 *  @author Dominik Goeddeke
 *  @see devisor2.GUI.base.dialog.OptionDialog
 *  @see java.util.Locale
 *  @see java.util.ListRessourceBundle
 *  @version 1.0 english
 *
 */

public class Locale_en_US extends ListResourceBundle
{
    /**
     *  returns the whole two-dimensional object array
     *
     *  @return the complete locales stored in this class
     */
    public Object[][] getContents ()
    {
	return contents;
    }

    /**
     *  ugly but neccessary...
     */
    private Object[][] contents = 
    {
	/* General button captions */
	{"general_Ok", "OK"},
	{"general_Ok_mnemonic", new Integer (KeyEvent.VK_O)},
	{"general_Accept", "Accept"},
	{"general_Cancel", "Cancel"},
	{"general_Cancel_mnemonic", new Integer (KeyEvent.VK_A)},
	{"general_Apply", "Apply"},
	{"general_Apply_mnemonic", new Integer (KeyEvent.VK_A)},
	{"general_Quit", "Quit"},
	{"general_Close", "Close"},
	{"general_Close_mnemonic", new Integer (KeyEvent.VK_C)},

	/* General messagebox texts */
	{"general_message_unsaveddata","You have unsaved data.\n Do you really want to proceed?"},
	{"general_message_unsaveddata_header","Unsaved Data"},
	{"general_message_quit","Are you sure you want to exit?"},
	{"general_message_quit_header","Quit"},
	
	/* error messages */
	{"errors_lookandfeelnotsupported","Your Look&Feel selection is not supported on your system! \n No changes to the Look&Feel are made."},
	{"errors_messageboxheader", "ERROR"},
	{"errors_parseerror", "Parsing Error"},
	{"errors_notfounderror", "Reference not found!"},
	{"errors_gridinputerror", "Mouse input error!"},
	{"errors_directerror", "Direct input error"},
	{"errors_directnotsupported", "Direct Input is not supported \n for this kind of element or segment! \n Try switching direct input off before trying again."},
	{"errors_selectionerror", "Selection Error"},
	{"errors_emptyselection", "This operation can only be performed \n when some items are selected."},
	{"errors_cloneerror", "Clone Error"},

	/* General dialog captions and dialog button captions */
	{"dialogs_loaddialog_caption", "Open an existing domain"},
	{"dialogs_loaddialog_button", "Open"},
	{"dialogs_savedialog_button", "Save"},
	{"dialogs_saveucddialog_caption", "Save as UCD"},
	{"dialogs_savefeatdialog_caption", "Save as FEAT"},
	{"dialogs_savefeastdialog_caption", "Save as FEAST"},
	{"dialogs_printdialog_caption", "Save as PostScript"},
	{"dialogs_printlsdialog_caption", "Save as PostScript Landscape"},
	{"dialogs_saveoptionsdialog_caption", "Save DeViSoRGrid2 Options"},
	{"dialogs_loadoptionsdialog_caption", "Load DeViSoRGrid2 Options"},
	{"dialogs_mergedialog_caption", "Choose domain to merge with ..."},
	{"dialogs_mergedialog_button", "Merge"},
	{"dialogs_printdialog_button", "SaveAs"},

	/* menu items and corresponding shortcuts */
	{"menu_file_header", "File"},
	{"menu_file_header_mnemonic", new Integer(KeyEvent.VK_F) },
	{"menu_file_new", "New" },
	{"menu_file_new_mnemonic", new Integer(KeyEvent.VK_N)},
	{"menu_file_open", "Open ..." },
	{"menu_file_open_mnemonic", new Integer(KeyEvent.VK_O)},
	{"menu_file_save", "Save" },
	{"menu_file_save_mnemonic", new Integer(KeyEvent.VK_S)},
	{"menu_file_saveas", "Save As" },
	{"menu_file_saveas_mnemonic", new Integer(KeyEvent.VK_A)},
	{"menu_file_saveas_ucd", "UCD ... " },
	{"menu_file_saveas_ucd_mnemonic", new Integer(KeyEvent.VK_U)},
	{"menu_file_saveas_feat", "FEAT ... " },
	{"menu_file_saveas_feat_mnemonic", new Integer(KeyEvent.VK_E)},
	{"menu_file_saveas_feast", "FEAST ... " },
	{"menu_file_saveas_feast_mnemonic", new Integer(KeyEvent.VK_F)},
	{"menu_file_merge", "Merge Domain ..." },
	{"menu_file_merge_mnemonic", new Integer(KeyEvent.VK_M)},
	{"menu_file_printls", "Print Landscape..." },
	{"menu_file_printls_mnemonic", new Integer(KeyEvent.VK_Z)},
	{"menu_file_print", "Print ..." },
	{"menu_file_print_mnemonic", new Integer(KeyEvent.VK_P)},
        {"menu_file_exit", "Exit" },
	{"menu_file_exit_mnemonic", new Integer(KeyEvent.VK_X)},
	{"menu_edit_header", "Edit"},
	{"menu_edit_header_mnemonic", new Integer(KeyEvent.VK_E) },
	{"menu_edit_undo", "Undo" },
	{"menu_edit_undo_mnemonic", new Integer(KeyEvent.VK_U)},
	{"menu_edit_redo", "Redo" },
	{"menu_edit_redo_mnemonic", new Integer(KeyEvent.VK_R)},
	{"menu_edit_cut", "Cut" },
	{"menu_edit_cut_mnemonic", new Integer(KeyEvent.VK_T)},
	{"menu_edit_copy", "Copy" },
	{"menu_edit_copy_mnemonic", new Integer(KeyEvent.VK_C)},
	{"menu_edit_paste", "Paste" },
	{"menu_edit_paste_mnemonic", new Integer(KeyEvent.VK_P)},
	{"menu_edit_copymode", "Copy Mode" },
	{"menu_edit_copymode_mnemonic", new Integer(KeyEvent.VK_M)},
	{"menu_edit_delete", "Delete" },
	{"menu_edit_delete_mnemonic", new Integer(KeyEvent.VK_D)},
	{"menu_edit_deletewb", "Delete working boundary" },
	{"menu_edit_deletewb_mnemonic", new Integer(KeyEvent.VK_W)},
	{"menu_edit_scale", "Scale ..." },
	{"menu_edit_scale_mnemonic", new Integer(KeyEvent.VK_L)},
	{"menu_edit_rotate", "Rotate ..." },
	{"menu_edit_rotate_mnemonic", new Integer(KeyEvent.VK_R)},
	{"menu_edit_mirror", "Mirror ..." },
	{"menu_edit_mirror_mnemonic", new Integer(KeyEvent.VK_I)},
	{"menu_edit_selectboundary", "Select current Boundary ..." },
	{"menu_edit_selectboundary_mnemonic", new Integer(KeyEvent.VK_B)},
	{"menu_view_header", "View" },
	{"menu_view_header_mnemonic", new Integer(KeyEvent.VK_V)},
	{"menu_view_zoom_setzoom", "Set Zoom" },
	{"menu_view_zoom_setzoom_mnemonic", new Integer(KeyEvent.VK_Z)},
	{"menu_view_zoom_resetzoom", "Reset Zoom" },
	{"menu_view_zoom_resetzoom_mnemonic", new Integer(KeyEvent.VK_R)},
	{"menu_view_zoom_zoomplus", "Zoom in" },
	{"menu_view_zoom_zoomplus_mnemonic", new Integer(KeyEvent.VK_I)},
	{"menu_view_zoom_zoomminus", "Zoom out" },
	{"menu_view_zoom_zoomminus_mnemonic", new Integer(KeyEvent.VK_O)},
	{"menu_view_selectnodes", "Select Nodes" },
	{"menu_view_selectnodes_mnemonic", new Integer(KeyEvent.VK_N)},
	{"menu_view_selectelements", "Select Elements" },
	{"menu_view_selectelements_mnemonic", new Integer(KeyEvent.VK_E)},
	{"menu_view_selectsegments", "Select Segments" },
	{"menu_view_selectsegments_mnemonic", new Integer(KeyEvent.VK_S)},
	{"menu_view_selectedges", "Select Edges" },
	{"menu_view_selectedges_mnemonic", new Integer(KeyEvent.VK_D)},
	{"menu_view_properties", "View Properties" },
	{"menu_view_properties_mnemonic", new Integer(KeyEvent.VK_P)},
	{"menu_domain_header", "Domain" },
	{"menu_domain_header_mnemonic", new Integer(KeyEvent.VK_D)},
	{"menu_domain_newboundary", "New Boundary Container" },
	{"menu_domain_newboundary_mnemonic", new Integer(KeyEvent.VK_C)},
	{"menu_domain_newanalytic", "Analytic Domain Description ..." },
	{"menu_domain_newanalytic_mnemonic", new Integer(KeyEvent.VK_A)},
	{"menu_domain_directinput", "Direct Input" },
	{"menu_domain_directinput_mnemonic", new Integer(KeyEvent.VK_D)},
	{"menu_domain_newsegment", "New Segment" },
	{"menu_domain_newsegment_mnemonic", new Integer(KeyEvent.VK_S)},
	{"menu_domain_newline", "Line" },
	{"menu_domain_newline_mnemonic", new Integer(KeyEvent.VK_L)},
	{"menu_domain_newmultiline", "MultiLine" },
	{"menu_domain_newmultiline_mnemonic", new Integer(KeyEvent.VK_M)},
	{"menu_domain_newcirclep", "Circle+" },
	{"menu_domain_newcirclep_mnemonic", new Integer(KeyEvent.VK_PLUS)},
	{"menu_domain_newcirclem", "Circle-" },
	{"menu_domain_newcirclem_mnemonic", new Integer(KeyEvent.VK_MINUS)},
	{"menu_domain_newelement", "New Element" },
	{"menu_domain_newelement_mnemonic", new Integer(KeyEvent.VK_E)},
	{"menu_domain_newtri", "Tri" },
	{"menu_domain_newtri_mnemonic", new Integer(KeyEvent.VK_T)},
	{"menu_domain_newquad", "Quad" },
	{"menu_domain_newquad_mnemonic", new Integer(KeyEvent.VK_Q)},
	{"menu_domain_newmacro", "Macro" },
	{"menu_domain_newmacro_mnemonic", new Integer(KeyEvent.VK_M)},
	{"menu_domain_newnode", "New Node" },
	{"menu_domain_newnode_mnemonic", new Integer(KeyEvent.VK_N)},
	{"menu_domain_adjustboundaries", "Adjust Boundaries" },
	{"menu_domain_adjustboundaries_mnemonic", new Integer(KeyEvent.VK_J)},
	{"menu_domain_linkboundaries", "Link Boundaries" },
	{"menu_domain_linkboundaries_mnemonic", new Integer(KeyEvent.VK_L)},
	{"menu_domain_fromtri2prm", "Generate PRM from TRI" },
	{"menu_domain_fromtri2prm_mnemonic", new Integer(KeyEvent.VK_G)},	{"menu_domain_swapboundaryorientation", "Swap Boundary Orientation" },
	{"menu_domain_swapboundaryorientation_mnemonic", new Integer(KeyEvent.VK_O)},
	{"menu_domain_domainperimeter", "Domain Perimeter ..." },
	{"menu_domain_domainperimeter_mnemonic", new Integer(KeyEvent.VK_P)},
	{"menu_domain_domainstatistics", "Domain Statistics" },
	{"menu_domain_domainstatistics_mnemonic", new Integer(KeyEvent.VK_I)},
	{"menu_grid_header", "Grid"},
	{"menu_grid_header_mnemonic", new Integer(KeyEvent.VK_G) },
	{"menu_grid_multipleboundarynodes", "Add Multiple Boundary Nodes..."},
	{"menu_grid_multipleboundarynodes_mnemonic", new Integer(KeyEvent.VK_M) },
	{"menu_grid_edgestatus", "Set Edge Status ..."},
	{"menu_grid_edgestatus_mnemonic", new Integer(KeyEvent.VK_E) },
	{"menu_grid_parallelblock", "Set Parallel Block ..."},
	{"menu_grid_parallelblock_mnemonic", new Integer(KeyEvent.VK_P) },
	{"menu_grid_loadbalancing", "Loadbalancing ..."},
	{"menu_grid_loadbalancing_mnemonic", new Integer(KeyEvent.VK_L) },
	{"menu_grid_rectifyelement", "Rectify Element"},
	{"menu_grid_rectifyelement_mnemonic", new Integer(KeyEvent.VK_R) },
	{"menu_grid_refine", "Refine ..."},
	{"menu_grid_refine_mnemonic", new Integer(KeyEvent.VK_F) },
	{"menu_grid_spellchecker", "SpellChecker ..."},
	{"menu_grid_spellchecker_mnemonic", new Integer(KeyEvent.VK_S) },
	{"menu_grid_mergeboundaries", "Merge two Boundaries ..."},
	{"menu_grid_mergeboundaries_mnemonic", new Integer(KeyEvent.VK_M) },
	{"menu_options_header", "Options"},
	{"menu_options_header_mnemonic", new Integer(KeyEvent.VK_O) },
	{"menu_options_general","General Options"},
	{"menu_options_general_mnemonic",new Integer(KeyEvent.VK_G)},
	{"menu_options_draw","Draw Options"},
	{"menu_options_draw_mnemonic",new Integer(KeyEvent.VK_D)},
	{"menu_options_color","Color Options"},
	{"menu_options_color_mnemonic",new Integer(KeyEvent.VK_C)},
	{"menu_options_accelerator","Accelerator Assignment"},
	{"menu_options_accelerator_mnemonic",new Integer(KeyEvent.VK_A)},
	{"menu_options_misc","Miscellaneous Options"},
	{"menu_options_misc_mnemonic",new Integer(KeyEvent.VK_M)},
	{"menu_help_header", "Help"},
	{"menu_help_header_mnemonic", new Integer(KeyEvent.VK_H) },
	{"menu_help_manual", "Manual ..."},
	{"menu_help_manual_mnemonic", new Integer(KeyEvent.VK_M) },
	{"menu_help_faq", "FAQ ..."},
	{"menu_help_faq_mnemonic", new Integer(KeyEvent.VK_F) },
	{"menu_help_readme", "Readme ..."},
	{"menu_help_readme_mnemonic", new Integer(KeyEvent.VK_R) },
	{"menu_help_about", "About ..."},
	{"menu_help_about_mnemonic", new Integer(KeyEvent.VK_A) },



	/* keys for class devisor2.GUI.dialogs.OptionsDialog */
	// general stuff
	{"options_OptionsDialog_general_title", "DeViSoRGrid2 Options Control Center"},
	// buttons
	{"options_OptionsDialog_acceptButton", "Accept"},
	{"options_OptionsDialog_acceptButton_tooltip", "Accept changes, apply and close the dialog"},
	{"options_OptionsDialog_acceptButton_mnemonic", new Integer (KeyEvent.VK_A)},
	{"options_OptionsDialog_cancelButton", "Cancel"},
	{"options_OptionsDialog_cancelButton_tooltip", "Cancel and close the dialog"},
	{"options_OptionsDialog_cancelButton_mnemonic", new Integer (KeyEvent.VK_C)},
	{"options_OptionsDialog_loadButton", "Load settings..."},
	{"options_OptionsDialog_loadButton_tooltip", "Load previously saved settings"},
	{"options_OptionsDialog_loadButton_mnemonic", new Integer (KeyEvent.VK_L)},
	{"options_OptionsDialog_saveButton", "Save settings..."},
	{"options_OptionsDialog_saveButton_tooltip", "Save settings to file."},
	{"options_OptionsDialog_saveButton_mnemonic", new Integer (KeyEvent.VK_S)},
	{"options_OptionsDialog_resetButton", "Restore Defaults"},
	{"options_OptionsDialog_resetButton_tooltip", "Set all options to factory defaults."},
	{"options_OptionsDialog_resetButton_mnemonic", new Integer (KeyEvent.VK_R)},
	{"options_OptionsDialog_helpButton", "Help"},
	{"options_OptionsDialog_helpButton_tooltip", "Show help screen"},
	{"options_OptionsDialog_helpButton_mnemonic", new Integer (KeyEvent.VK_H)},
	// general tab
	{"options_OptionsDialog_generalPanel_tabheader", "General"},
	{"options_OptionsDialog_generalPanel_tabheader_tooltip", "Change general DeViSoR2 options"},
	{"options_OptionsDialog_generalPanel_devisorhome","DEVISORHOME Directory"},
	{"options_OptionsDialog_generalPanel_devisorhome_tooltip","Click to show the File Chooser Dialog"},
	{"options_OptionsDialog_generalPanel_devisorhome_dialogtitle","DEVISORHOME directory selection"},
	{"options_OptionsDialog_generalPanel_generalpath","Default load and save path"},
	{"options_OptionsDialog_generalPanel_generalpath_tooltip","Click to show the File Chooser Dialog"},
	{"options_OptionsDialog_generalPanel_generalpath_dialogtitle","Default path selection"},
	{"options_OptionsDialog_generalPanel_trigen","TriGen2D Path"},
	{"options_OptionsDialog_generalPanel_trigen_tooltip","Click to show the File Chooser Dialog"},
	{"options_OptionsDialog_generalPanel_trigen_dialogtitle","Trigen Path selection"},
	{"options_OptionsDialog_generalPanel_featlib","Feat Library Path"},
	{"options_OptionsDialog_generalPanel_featlib_tooltip","Click to show the File Chooser Dialog"},
	{"options_OptionsDialog_generalPanel_featlib_dialogtitle","Feat Library Path selection"},

	// draw tab
	{"options_OptionsDialog_drawPanel_tabheader", "Draw Settings"},
	{"options_OptionsDialog_drawPanel_tabheader_tooltip", "Configure the appearance of the drawing area"},
	{"options_OptionsDialog_drawPanel_boundary_header", "Boundaries"},
	{"options_OptionsDialog_drawPanel_boundary_draw", "Draw Boundaries"},
	{"options_OptionsDialog_drawPanel_boundary_number", "Display Boundary Numbers"},
	{"options_OptionsDialog_drawPanel_node_header", "Nodes"},
	{"options_OptionsDialog_drawPanel_node_draw", "Draw Nodes"},
	{"options_OptionsDialog_drawPanel_node_number", "Display Node Numbers"},
	{"options_OptionsDialog_drawPanel_edge_header", "Edges"},
	{"options_OptionsDialog_drawPanel_edge_draw", "Draw edges"},
	{"options_OptionsDialog_drawPanel_edge_number", "Display Edge Numbers"},
	{"options_OptionsDialog_drawPanel_element_header", "Elements"},
	{"options_OptionsDialog_drawPanel_element_draw", "Draw Elements"},
	{"options_OptionsDialog_drawPanel_element_number", "Display Element Numbers"},
	{"options_OptionsDialog_drawPanel_element_cross", "Cross Elements"},
	{"options_OptionsDialog_drawPanel_element_fill", "Fill Elements"},
	{"options_OptionsDialog_drawPanel_misc_header", "Miscellaneous"},
	{"options_OptionsDialog_drawPanel_misc_grid", "Draw Grid"},
	{"options_OptionsDialog_drawPanel_misc_perimeter", "Draw Perimeter"},
	{"options_OptionsDialog_drawPanel_misc_rectangular", "Highlight Rectangular Elements"},
	{"options_OptionsDialog_drawPanel_edit_header", "Configure Mouse Input"},
	{"options_OptionsDialog_drawPanel_edit_snapon", "Snap Mode enabled"},
	{"options_OptionsDialog_drawPanel_edit_snapx", "Snap in x direction"},
	{"options_OptionsDialog_drawPanel_edit_snapy", "Snap in y direction"},
	{"options_OptionsDialog_drawPanel_edit_tolerance", "Click Tolerance"},
	{"options_OptionsDialog_drawPanel_edit_drawcoords", "Display user coordinates"},
	{"options_OptionsDialog_drawPanel_edit_draworientationmarkers", "Draw orientation markers"},
	{"options_OptionsDialog_drawPanel_edit_epsilon", "Global epsilon"},
	{"options_OptionsDialog_drawPanel_edit_epsilon_sliderlow", "low"},
	{"options_OptionsDialog_drawPanel_edit_epsilon_slidermedium", "average"},
	{"options_OptionsDialog_drawPanel_edit_epsilon_sliderhigh", "high"},

	// color tab
	{"options_OptionsDialog_colorPanel_tabheader", "Color Settings"},
	{"options_OptionsDialog_colorPanel_tabheader_tooltip", "Fine-tune the colors used for painting items"},
	{"options_OptionsDialog_colorPanel_objects_header", "Objects"},
	{"options_OptionsDialog_colorPanel_objects_node", "Nodes:"},
	{"options_OptionsDialog_colorPanel_objects_node_dialog", "Select the Color for Node objects"},
	{"options_OptionsDialog_colorPanel_objects_edge", "Edges:"},
	{"options_OptionsDialog_colorPanel_objects_edge_dialog", "Select the Color for Edge objects"},
	{"options_OptionsDialog_colorPanel_objects_edgebase", "Edge base:"},
	{"options_OptionsDialog_colorPanel_objects_edgebase_dialog", "Select the Edge Base Color"},
	{"options_OptionsDialog_colorPanel_objects_element", "Elements:"},
	{"options_OptionsDialog_colorPanel_objects_element_dialog", "Select the Color for Element objects"},
	{"options_OptionsDialog_colorPanel_numbers_header", "Labelling"},
	{"options_OptionsDialog_colorPanel_numbers_boundary", "Boundary numbering:"},
	{"options_OptionsDialog_colorPanel_numbers_boundary_dialog", "Select the Color for Boundary numbers"},
	{"options_OptionsDialog_colorPanel_numbers_node", "Node Numbering:"},
	{"options_OptionsDialog_colorPanel_numbers_node_dialog", "Select the Color for Node numbers"},
	{"options_OptionsDialog_colorPanel_numbers_edge", "Edge numbering:"},
	{"options_OptionsDialog_colorPanel_numbers_edge_dialog", "Select the Color for Edge numbers"},
	{"options_OptionsDialog_colorPanel_numbers_element", "Element numbering:"},
	{"options_OptionsDialog_colorPanel_numbers_element_dialog", "Select the Color for Element number"},
	{"options_OptionsDialog_colorPanel_highlight_header", "Highlighting"},
	{"options_OptionsDialog_colorPanel_highlight_selection", "Selections:"},
	{"options_OptionsDialog_colorPanel_highlight_selection_dialog", "Select the Color for highlighting selections"},
	{"options_OptionsDialog_colorPanel_highlight_rect", "Rectified Elements:"},
	{"options_OptionsDialog_colorPanel_highlight_rect_dialog", "Select the Color for highlighting rectified elements"},
	{"options_OptionsDialog_colorPanel_misc_header", "Miscellaeous"},
	{"options_OptionsDialog_colorPanel_misc_grid", "Grid:"},
	{"options_OptionsDialog_colorPanel_misc_grid_dialog", "Select the Color for the background grid"},
	{"options_OptionsDialog_colorPanel_misc_background", "Background:"},
	{"options_OptionsDialog_colorPanel_misc_background_dialog", "Select the Color for the background"},
	{"options_OptionsDialog_colorPanel_misc_lasso", "Lasso:"},
	{"options_OptionsDialog_colorPanel_misc_lasso_dialog", "Select the Color for the lasso pointer"},
	{"options_OptionsDialog_colorPanel_fill_header", "Parallel Block Fill Colors"},
	{"options_OptionsDialog_colorPanel_fill0_dialog", "Select the first palette color for parallel blocks"},
	{"options_OptionsDialog_colorPanel_fill1_dialog", "Select the second palette color for parallel blocks"},
	{"options_OptionsDialog_colorPanel_fill2_dialog", "Select the third palette color for parallel blocks"},
	{"options_OptionsDialog_colorPanel_fill3_dialog", "Select the forth palette color for parallel blocks"},
	{"options_OptionsDialog_colorPanel_fill4_dialog", "Select the fifth palette color for parallel blocks"},
	{"options_OptionsDialog_colorPanel_fill5_dialog", "Select the sixth palette color for parallel blocks"},
	{"options_OptionsDialog_colorPanel_fill6_dialog", "Select the seventh palette color for parallel blocks"},
	{"options_OptionsDialog_colorPanel_fill7_dialog", "Select the eighth palette color for parallel blocks"},
	{"options_OptionsDialog_colorPanel_fill8_dialog", "Select the ninth palette color for parallel blocks"},
	{"options_OptionsDialog_colorPanel_fill9_dialog", "Select the tenth palette color for parallel blocks"},
	{"options_OptionsDialog_colorPanel_fill10_dialog", "Select the eleventh palette color for parallel blocks"},
	{"options_OptionsDialog_colorPanel_fill11_dialog", "Select the twelveth palette color for parallel blocks"},
	{"options_OptionsDialog_colorPanel_fill12_dialog", "Select the thirteenth palette color for parallel blocks"},
	{"options_OptionsDialog_colorPanel_fill13_dialog", "Select the fourteenth palette color for parallel blocks"},
	{"options_OptionsDialog_colorPanel_fill14_dialog", "Select the fifteenth palette color for parallel blocks"},
	{"options_OptionsDialog_colorPanel_fill15_dialog", "Select the sixteenth palette color for parallel blocks"},
	{"options_OptionsDialog_colorPanel_boundarypalette_header", "Boundary Palette Colors"},
	{"options_OptionsDialog_colorPanel_boundary0_dialog", "Select the first palette color for boundaries"},
	{"options_OptionsDialog_colorPanel_boundary1_dialog", "Select the second palette color for boundaries"},
	{"options_OptionsDialog_colorPanel_boundary2_dialog", "Select the third palette color for boundaries"},
	{"options_OptionsDialog_colorPanel_boundary3_dialog", "Select the fourth palette color for boundaries"},
	{"options_OptionsDialog_colorPanel_boundary4_dialog", "Select the fifth palette color for boundaries"},
	{"options_OptionsDialog_colorPanel_boundary5_dialog", "Select the sixth palette color for boundaries"},
	{"options_OptionsDialog_colorPanel_boundary6_dialog", "Select the seventh palette color for boundaries"},
	{"options_OptionsDialog_colorPanel_boundary7_dialog", "Select the eighth palette color for boundaries"},
	{"options_OptionsDialog_colorPanel_boundary8_dialog", "Select the nineth palette color for boundaries"},
	{"options_OptionsDialog_colorPanel_boundary9_dialog", "Select the tenth palette color for boundaries"},
	{"options_OptionsDialog_colorPanel_boundary10_dialog", "Select the eleventh palette color for boundaries"},
	{"options_OptionsDialog_colorPanel_boundary11_dialog", "Select the twelveth palette color for boundaries"},
	{"options_OptionsDialog_colorPanel_boundary12_dialog", "Select the thirteenth palette color for boundaries"},
	{"options_OptionsDialog_colorPanel_boundary13_dialog", "Select the fourteenth palette color for boundaries"},
	{"options_OptionsDialog_colorPanel_boundary14_dialog", "Select the fifteenth palette color for boundaries"},
	{"options_OptionsDialog_colorPanel_boundary15_dialog", "Select the sixteenth palette color for boundaries"},

	// Accelerators tab
	{"options_OptionsDialog_acceleratorPanel_tabheader", "Keyboard Accelerators"},
	{"options_OptionsDialog_acceleratorPanel_tabheader_tooltip", "Set your own keyboard accelerators for application functions"},
	{"options_OptionsDialog_acceleratorPanel_southpaneltitle", "Modify current selection"},
	{"options_OptionsDialog_acceleratorPanel_tabletitle", "Current settings"},
	{"options_OptionsDialog_acceleratorPanel_tableleftcolumn", "Description"},
	{"options_OptionsDialog_acceleratorPanel_tablerightcolumn", "Accelerator"},
	{"options_OptionsDialog_acceleratorPanel_applybuttontooltip", "Assign the specified accelerator to the function above"},
	{"options_OptionsDialog_acceleratorPanel_applybuttonmnemonic", new Integer (KeyEvent.VK_P)},
	{"options_OptionsDialog_acceleratorPanel_removebutton", "Remove Accelerator"},
	{"options_OptionsDialog_acceleratorPanel_removebuttontooltip", "Assign NONE to the selected function, thus removing the accelerator assignment."},
	{"options_OptionsDialog_acceleratorPanel_removebuttonmnemonic", new Integer (KeyEvent.VK_E)},
	{"options_OptionsDialog_acceleratorPanel_nocorrectaccelerator", "The Accelerator you selected is not a valid one. \n See documentation for a full list of valid Accelerators."},
	{"options_OptionsDialog_acceleratorPanel_acceleratoralreadyinuse", "The Accelerator you selected is already assigned to another function. \n Please select another one."},
	{"options_OptionsDialog_acceleratorPanel_youmustselectarow", "Please select a row in the table above first. \n Otherwise no Accelerator can be mapped to a function."},
	// other tab
	{"options_OptionsDialog_otherPanel_tabheader", "Other"},
	{"options_OptionsDialog_otherPanel_tabheader_tooltip", "Everything that did't fit in somewhere else"},
	{"options_OptionsDialog_otherPanel_lookandfeel","Select Look&Feel"},
	{"options_OptionsDialog_otherPanel_lookandfeel_mnemonic", new Character ('K')},
	{"options_OptionsDialog_otherPanel_screenx","Window width"},
	{"options_OptionsDialog_otherPanel_screenx_mnemonic", new Character ('W')},
	{"options_OptionsDialog_otherPanel_screeny","Window height"},
	{"options_OptionsDialog_otherPanel_screeny_mnemonic", new Character ('H')},
	{"options_OptionsDialog_otherPanel_optionfileextension","Extension for the Options file"},
	{"options_OptionsDialog_otherPanel_optionfileextension_mnemonic", new Character ('O')},
	
	
	/* keys for class devisor2.GUI.dialogs.InfoDialog */
	{"GUI_dialogs_InfoDialog_frametitle", "About DeViSoRGrid2"},
	{"GUI_dialogs_InfoDialog_version", "Version"},
	{"GUI_dialogs_InfoDialog_developers", "Developers"},
	{"GUI_dialogs_InfoDialog_about", "General Information"},
	{"GUI_dialogs_InfoDialog_abouttext1", "DeViSoRGrid 2 is the preprocessing tool in the "},
	{"GUI_dialogs_InfoDialog_abouttext2", "FEAST software family. It works as a coarse grid "},
	{"GUI_dialogs_InfoDialog_abouttext3", "and mesh generator and allows you to define domains"},
	{"GUI_dialogs_InfoDialog_abouttext4", "and coarse grids via a simple point&click interface."},
	{"GUI_dialogs_InfoDialog_contact", "How to contact us"},
	{"GUI_dialogs_InfoDialog_contact_text1", "For further information and the latest updates,"},
	{"GUI_dialogs_InfoDialog_contact_text2", "please visit our homepage at "},
	{"GUI_dialogs_InfoDialog_contact_text3", "Or, send an email to the following adress:"},

	/* tooltips for class devisor2.GUI.framework.ToolBar */
	{"GUI_framework_ToolBar_newtooltip", "Create a new, empty domain"},
	{"GUI_framework_ToolBar_opentooltip", "Open an existing domain"},
	{"GUI_framework_ToolBar_savetooltip", "Save Domain in current format"},
	{"GUI_framework_ToolBar_saveucdtooltip", "Save Domain in UCD format"},
	{"GUI_framework_ToolBar_savefeattooltip", "Save Domain in FEAT format"},
	{"GUI_framework_ToolBar_savefeasttooltip", "Save Domain in FEAST format"},
	{"GUI_framework_ToolBar_undotooltip", "Undo last action"},
	{"GUI_framework_ToolBar_redotooltip", "Redo last Undo"},
	{"GUI_framework_ToolBar_cuttooltip", "Cut selected items to clipboard"},
	{"GUI_framework_ToolBar_copytooltip", "Copy selected items to clipboard"},
	{"GUI_framework_ToolBar_pastetooltip", "Paste items from clipboard"},
	{"GUI_framework_ToolBar_selectnodestooltip", "Select nodes mask"},
	{"GUI_framework_ToolBar_selectelementstooltip", "Select elements mask"},
	{"GUI_framework_ToolBar_selectsegmentstooltip", "Selecting segments mask"},
	{"GUI_framework_ToolBar_selectedgestooltip", "Selecting edges mask"},
	{"GUI_framework_ToolBar_selectboundarytooltip", "Select the boundary to work on"},
	{"GUI_framework_ToolBar_setzoomtooltip", "Set Zoom Level"},
	{"GUI_framework_ToolBar_resetzoomtooltip", "Default Zoom Level"},
	{"GUI_framework_ToolBar_zoomintooltip", "Zoom In"},
	{"GUI_framework_ToolBar_zoomouttooltip", "Zoom Out"},
	{"GUI_framework_ToolBar_directinputtooltip", "Toggle Direct Input Mode"},
	{"GUI_framework_ToolBar_copymodetooltip", "Change Copy Mode for Segments"},
	{"GUI_framework_ToolBar_newboundarytooltip", "Create a new Boundary Container"},
	{"GUI_framework_ToolBar_addtypetooltip", "Choose the item to be added"},
	{"GUI_framework_ToolBar_addnothingtooltip", "Currently adding nothing."},
	{"GUI_framework_ToolBar_addclipboardtooltip", "Currently adding items from the clipboard."},
	{"GUI_framework_ToolBar_addlinetooltip", "Currently adding Line segments."},
	{"GUI_framework_ToolBar_addmultilinetooltip", "Currently adding MultiLine segments."},
	{"GUI_framework_ToolBar_addcircleptooltip", "Currently adding Circle+ segments."},
	{"GUI_framework_ToolBar_addcirclemtooltip", "Currently adding Circle- segments."},
	{"GUI_framework_ToolBar_addtritooltip", "Currently adding Tri elements."},
	{"GUI_framework_ToolBar_addquadtooltip", "Currently adding Quad elements."},
	{"GUI_framework_ToolBar_addmacrotooltip", "Currently adding Macro elements."},
	{"GUI_framework_ToolBar_addnodetooltip", "Currently adding Nodes."},

	{"GUI_framework_ToolBar_addtypestooltip", "Select the Type of Segments or Elements you want to add to the Domain"},
	{"GUI_framework_ToolBar_propertiestooltip", "Switch to Property Query Mode"},
	{"GUI_framework_ToolBar_edgestatustooltip", "Set Edge Status"},
	{"GUI_framework_ToolBar_parallelblocktooltip", "Set Parallel Blocks"},
	{"GUI_framework_ToolBar_rectifyelementtooltip", "Rectify Elements"},
	{"GUI_framework_ToolBar_exittooltip", "Exit DeViSoRGrid 2"},

	/* keys for class devisor2.GUI.framework.MainFrame */
	{"GUI_framework_Mainframe_frametitle", "DeViSoRGrid2 - Preprocessing for the FEAST Software Family"},

	/* keys for class devisor2.GUI.help.HelpFrame */
	{"GUI_help_HelpFrame_frametitle", "DeViSoRGrid2 Online Manual"},

	/* keys for class devisor2.GUI.help.FAQFrame */
	{"GUI_help_FAQFrame_frametitle", "DeViSoRGrid2 Frequently Asked Questions"},

	/* keys for the FilePickerFilter class */
	{"GUI_dialogs_FilePickerFilter_description_ucd", "Files in UCD format (.inp)"},
	{"GUI_dialogs_FilePickerFilter_description_feat", "Files in FEAT format (.prm, .tri)"},
	{"GUI_dialogs_FilePickerFilter_description_feast", "Files in FEAST format (.feast)"},
	{"GUI_dialogs_FilePickerFilter_description_all", "All files"},
	{"GUI_dialogs_FilePickerFilter_description_directories", "Only directories are accepted"},
	{"GUI_dialogs_FilePickerFilter_description_binaries", "executable Files"},
	{"GUI_dialogs_FilePickerFilter_description_ps", "PostScript-Files"},

	{"GUI_dialogs_FilePickerFilter_description_default", "All supported files (.inp, .prm, .tri, .feast)"},

	/* keys for the EDGE STATUS dialog */
	{"dialogs_edgestatus_title", "Edge Status"},
	{"dialogs_edgestatus_status_caption", "Edge Status"},
	{"dialogs_edgestatus_status_none", "None"},
	{"dialogs_edgestatus_status_real", "Real"},
	{"dialogs_edgestatus_status_inner", "Inner"},
	{"dialogs_edgestatus_status_mirror", "Mirror"},
	{"dialogs_edgestatus_bc_caption", "Boundary Condition"},
	{"dialogs_edgestatus_bc_dirichlet", "Dirichlet"},
	{"dialogs_edgestatus_bc_neumann", "Neumann"},
	{"dialogs_edgestatus_bc_none", "None"},
	{"dialogs_edgestatus_close", "Close"},
	{"dialogs_edgestatus_close_tooltip", "Finish setting edge stati"},
	{"dialogs_edgestatus_close_mnemonic", new Integer (KeyEvent.VK_C)},
	{"dialogs_edgestatus_apply", "Apply"},
	{"dialogs_edgestatus_apply_tooltip", "Applying edge stati"},
	{"dialogs_edgestatus_apply_mnemonic", new Integer (KeyEvent.VK_A)},	{"dialogs_edgestatus_automatic", "Automatic Setting"},
	{"dialogs_edgestatus_automatic_tooltip", "Set edge stati automatically"},
	{"dialogs_edgestatus_automatic_mnemonic", new Integer (KeyEvent.VK_A)},
	/* keys for the PARALLELBLOCK dialog */
	{"dialogs_parallelblock_title", "ParallelBlock & Macro Refinement"},
	{"dialogs_parallelblock_apply", "Apply"},
	{"dialogs_parallelblock_apply_tooltip", "Apply these settings to the selected macro element"},
	{"dialogs_parallelblock_apply_mnemonic", new Integer (KeyEvent.VK_A)},
	{"dialogs_parallelblock_close", "Close"},
	{"dialogs_parallelblock_close_tooltip", "Close the dialog"},
	{"dialogs_parallelblock_close_mnemonic", new Integer (KeyEvent.VK_C)},
	{"dialogs_parallelblock_automatic", "Automatic"},
	{"dialogs_parallelblock_automatic_tooltip", "Set Parallel Blocks automatically"},
	{"dialogs_parallelblock_automatic_mnemonic", new Integer (KeyEvent.VK_U)},
	{"dialogs_parallelblock_parallelblock", "Parallel Block"},
	{"dialogs_parallelblock_matrixblock", "Matrix Block"},
	{"dialogs_parallelblock_macrorefinement", "Macro Refinement Level"},
	{"dialogs_parallelblock_anisotrophy_x", "Macro X Anisotrophy"},
	{"dialogs_parallelblock_anisotrophy_x_none", "None"},
	{"dialogs_parallelblock_anisotrophy_x_left", "Left"},
	{"dialogs_parallelblock_anisotrophy_x_right", "Right"},
	{"dialogs_parallelblock_anisotrophy_y", "Macro Y Anisotrophy"},
	{"dialogs_parallelblock_anisotrophy_y_none", "None"},
	{"dialogs_parallelblock_anisotrophy_y_down", "Down"},
	{"dialogs_parallelblock_anisotrophy_y_up", "Up"},
	{"dialogs_parallelblock_factors", "Macro Refinement Factors"},

	/* keys for the REFINE dialog */
	{"dialogs_refine_title", "Automatic Refinement Parameters"},
	{"dialogs_refine_apply", "Apply"},
	{"dialogs_refine_apply_tooltip", "Start TRIGEN2D with these parameters"},
	{"dialogs_refine_apply_mnemonic", new Integer (KeyEvent.VK_A)},
	{"dialogs_refine_close", "Close"},
	{"dialogs_refine_close_tooltip", "Finish automatic refinement"},
	{"dialogs_refine_close_mnemonic", new Integer (KeyEvent.VK_C)},
	{"dialogs_refine_type", "Refinement Technique"},
	{"dialogs_refine_no", "No adaptive Refinement"},
	{"dialogs_refine_delma33", "3x3 Refinement with DELMA"},
	{"dialogs_refine_boundary3", "3 Boundary Refinement with DELMA/DELMB"},
	{"dialogs_refine_boundary2", "2 Boundary Refinement with DELMA"},
	{"dialogs_refine_boundarynormal", "3 Boundary Normal Refinement with DELMA"},
	{"dialogs_refine_delma5", "5 Refinement with DELMA"},
	{"dialogs_refine_corner", "3 Corner Refinement with DELMA"},
	{"dialogs_refine_parameters", "Refinement Parameters"},
	{"dialogs_refine_parameter1", "First Parameter:"},
	{"dialogs_refine_parameter2", "Second Parameter:"},
	{"dialogs_refine_level", "Level to Resize:"},

	/* keys for the loadbalancing dialog */
	{"dialogs_loadbalancing_title", "LoadBalancing"},
	{"dialogs_loadbalancing_mpi2sol", "MPI2SOL path:"},
	{"dialogs_loadbalancing_filename", "Filename:"},
	{"dialogs_loadbalancing_calc", "Calculate"},
	{"dialogs_loadbalancing_calc_mnemonic", new Integer (KeyEvent.VK_A)},
	{"dialogs_loadbalancing_perform", "Perform"},
	{"dialogs_loadbalancing_perform_mnemonic", new Integer (KeyEvent.VK_P)},

	/* keys for the line dialog */
	{"dialogs_line_caption_direct", "Direct Line Input"},
	{"dialogs_line_caption_property", "Line Properties"},
	{"dialogs_line_startpoint", "Start Coordinates:"},
	{"dialogs_line_endpoint", "End Coordinates:"},
	{"dialogs_line_error_x1", "The edit field for the start point X coordinate \n does not contain a valid input."},
	{"dialogs_line_error_y1", "The edit field for the start point Y coordinate \n does not contain a valid input."},
	{"dialogs_line_error_x2", "The edit field for the end point X coordinate \n does not contain a valid input."},
	{"dialogs_line_error_y2", "The edit field for the end point Y coordinate \n does not contain a valid input."},

	/* keys for the MultipleBNDialog */
	{"dialogs_multiplebn_title","Multiple Boundary Nodes"},
	{"dialogs_multiplebn_boundary","Boundary:"},
	{"dialogs_multiplebn_maxpar","MaxPar:"},
	{"dialogs_multiplebn_increment","Increment:"},
	{"dialogs_multiplebn_start","StartPar:"},
	{"dialogs_multiplebn_end","EndPar:"},
	{"dialogs_multiplebn_error_boundary", "The Boundary textfield does not contain an Integer!"},
	{"dialogs_multiplebn_error_increment","The increment texifield does not contain a valid Double."},
	{"dialogs_multiplebn_error_start","The start parameter value texifield does not contain a valid Double."},
	{"dialogs_multiplebn_error_end","The end parameter value texifield does not contain a valid Double."},
	{"dialogs_multiplebn_error_nosuchboundary","The Domain does not contain a Boundary with that Number!"},
	{"dialogs_multiplebn_error_starttoosmall","The start parameter value you requested is too small.\nRemember: Only numbers from 0 upwards are possible."},
	{"dialogs_multiplebn_error_endtoobig", "The end parameter value you requested is too big.\nRemember: Only numbers up to the displayed maximal value are possible."},


	/* keys for the circle dialog */
	{"dialogs_circle_caption_direct_p", "Direct CircleP Input"},
	{"dialogs_circle_caption_direct_m", "Direct CircleM Input"},
	{"dialogs_circle_caption_property_p", "CircleP Properties"},
	{"dialogs_circle_caption_property_m", "CircleM Properties"},
	{"dialogs_circle_center", "Center: "},
	{"dialogs_circle_radius", "Radius: "},
	{"dialogs_circle_startangle", "Start Angle: "},
	{"dialogs_circle_endangle", "End Angle: "},
	{"dialogs_circle_error_x", "The edit field for the center X coordinate \n does not contain a valid input."},
	{"dialogs_circle_error_y", "The edit field for the center Y coordinate\n does not contain a valid input."},
	{"dialogs_circle_error_radius", "The edit field for the radius \n does not contain a valid input."},
	{"dialogs_circle_error_start", "The edit field for the start angle \n does not contain a valid input."},
	{"dialogs_circle_error_end", "The edit field for the end angle \n does not contain a valid input."},
	{"dialogs_circle_error_orientation", "This circle would have the wrong orientation.\n Make sure the start and end angles are correct."},

	/* keys for the BoundaryNodeDialog */
	{"dialogs_boundarynode_caption_direct", "Direct Node Input"},
	{"dialogs_boundarynode_caption_property", "Node Properties"},
	{"dialogs_boundarynode_boundary", "Boundary: "},
	{"dialogs_boundarynode_segment", "Segment: "},
	{"dialogs_boundarynode_maxpar", "t-Max: "},
	{"dialogs_boundarynode_actpar", "t-Act "},
	{"dialogs_boundarynode_editcheck", "Edit last added BN"},
	{"dialogs_boundarynode_coord", "Coordinates: "},
	{"dialogs_boundarynode_nodeprop", "Nodal Property: "},
	{"dialogs_boundarynode_createboundary", "Create Boundary Node"},
	{"dialogs_boundarynode_fastforward", ">>"},
	{"dialogs_boundarynode_fastforward_tooltip", "large increment"},
	{"dialogs_boundarynode_forward", ">"},
	{"dialogs_boundarynode_forward_tooltip", "small increment"},
	{"dialogs_boundarynode_rewind", "<"},
	{"dialogs_boundarynode_rewind_tooltip", "small decrement"},
	{"dialogs_boundarynode_fastrewind", "<<"},
	{"dialogs_boundarynode_fastrewind_tooltip", "large decrement"},
	{"dialogs_boundarynode_para_tabheader", "Parametric Description"},
	{"dialogs_boundarynode_coord_tabheader", "Cartesian Description"},
	{"dialogs_boundarynode_para_tooltip", "Edit the parametric description of this BoundaryNode"},
	{"dialogs_boundarynode_coord_tooltip", "Edit the cartesian description of this BoundaryNode"},
	{"dialogs_boundarynode_error_coord_x", "The edit field for the X coordinate \n does not contain a valid input."},
	{"dialogs_boundarynode_error_coord_y", "The edit field for the Y coordinate \n does not contain a valid input."},
	{"dialogs_boundarynode_error_nodeprop", "The edit field for the nodal property \n does not contain a valid input."},
	{"dialogs_boundarynode_error_actpar", "The edit field for the act. parameter value \n does not contain a valid input."},
	{"dialogs_boundarynode_error_boundnum", "The edit field for the boundary number \n does not contain a valid input."},
	{"dialogs_boundarynode_error_boundrange", "The edit field for the boundary number \n does not contain a valid input. \n There is no boundary with that number!"},
	{"dialogs_boundarynode_error_actrange", "The edit field for the act. parameter \n does not contain a valid input. \n The value is not within the allowed range!"},
	{"dialogs_boundarynode_error_editlast", "There is no node to be perform changes on. \n Try adding a node first before changing its values again!."},
	{"dialogs_boundarynode_error_usepara", "Use Parametrisation"},
	{"dialogs_boundarynode_error_useparatext", "It is not allowed to create or change \n Boundary Nodes via cartesian description."},
	{"dialogs_boundarynode_error_para", "Parametrisation Error"},
	{"dialogs_boundarynode_error_regularparametric", "Only boundary nodes have a parametric description!"},

	/* keys for PerimeterDialog */
	{"dialogs_perimeter_title", "Domain perimeter"},
	{"dialogs_perimeter_x1", "Topleft corner     "},
	{"dialogs_perimeter_x2", "Bottomright corner "},
	{"dialogs_perimeter_error_x1", "The edit field for the topleft x value \n does not contain a valid input."},
	{"dialogs_perimeter_error_y1", "The edit field for the topleft y value \n does not contain a valid input."},
	{"dialogs_perimeter_error_x2", "The edit field for the bottomright x value \n does not contain a valid input."},
	{"dialogs_perimeter_error_y2", "The edit field for the bottomright y value \n does not contain a valid input."},




	/* keys for the QuadDialog */
	{"dialogs_quad_caption_direct", "Direct Quad Input"},
	{"dialogs_quad_caption_property", "Quad Properties"},
	{"dialogs_quad_node1", "Node 1:"},
	{"dialogs_quad_node2", "Node 2:"},
	{"dialogs_quad_node3", "Node 3:"},
	{"dialogs_quad_node4", "Node 4:"},
	{"dialogs_quad_error_node1", "The edit field for the first node \n does not contain a valid input."},
	{"dialogs_quad_error_node2", "The edit field for the second node \n does not contain a valid input."},
	{"dialogs_quad_error_node3", "The edit field for the third node \n does not contain a valid input."},
	{"dialogs_quad_error_node4", "The edit field for the fourth node \n does not contain a valid input."},

	/* keys for the Macro Dialog */
	{"dialogs_macro_caption_direct", "Direct Macro Input"},
	{"dialogs_macro_caption_property", "Macro Properties"},
	{"dialogs_macro_nodes_tabheader", "Nodes"},
	{"dialogs_macro_nodes_tooltip", "Edit Nodes that define this Macro"},
	{"dialogs_macro_blocks_tabheader", "Parallel"},
	{"dialogs_macro_blocks_tooltip", "Edit Parallel Computing Information of this Macro"},
	{"dialogs_macro_parallelblock", "Parallel Block"},
	{"dialogs_macro_matrixblock", "Matrix Block"},
	{"dialogs_macro_macrorefinement", "Macro Refinement Level"},
	{"dialogs_macro_anisotrophy_x", "Macro X Anisotrophy"},
	{"dialogs_macro_anisotrophy_x_none", "None"},
	{"dialogs_macro_anisotrophy_x_left", "Left"},
	{"dialogs_macro_anisotrophy_x_right", "Right"},
	{"dialogs_macro_anisotrophy_y", "Macro Y Anisotrophy"},
	{"dialogs_macro_anisotrophy_y_none", "None"},
	{"dialogs_macro_anisotrophy_y_down", "Down"},
	{"dialogs_macro_anisotrophy_y_up", "Up"},
	{"dialogs_macro_factors", "Macro Refinement Factors"},
	{"dialogs_macro_node1", "Node 1:"},
	{"dialogs_macro_node2", "Node 2:"},
	{"dialogs_macro_node3", "Node 3:"},
	{"dialogs_macro_node4", "Node 4:"},
	{"dialogs_macro_error_node1", "The edit field for the first node \n does not contain a valid input."},
	{"dialogs_macro_error_node2", "The edit field for the second node \n does not contain a valid input."},
	{"dialogs_macro_error_node3", "The edit field for the third node \n does not contain a valid input."},
	{"dialogs_macro_error_node4", "The edit field for the fourth node \n does not contain a valid input."},
	{"dialogs_macro_error_pb", "The edit field for the parallel block \n does not contain a valid input."},
	{"dialogs_macro_error_ref", "The edit field for the refinement level \n does not contain a valid input."},
	{"dialogs_macro_error_mb", "The edit field for the matrix block \n does not contain a valid input."},
	{"dialogs_macro_error_factors", "The edit fields for the refinement factors \n do not contain valid inputs."},

	/* keys for the TriDialog */
	{"dialogs_tri_caption_direct", "Direct Tri Input"},
	{"dialogs_tri_caption_property", "Tri Properties"},
	{"dialogs_tri_node1", "Node 1:"},
	{"dialogs_tri_node2", "Node 2:"},
	{"dialogs_tri_node3", "Node 3:"},
	{"dialogs_tri_error_node1", "The edit field for the first node \n does not contain a valid input."},
	{"dialogs_tri_error_node2", "The edit field for the second node \n does not contain a valid input."},
	{"dialogs_tri_error_node3", "The edit field for the third node \n does not contain a valid input."},
	{"dialogs_tri_error_nodeorientation", "The nodes you selected are not orientated correctly!"},
	{"dialogs_tri_error_samenodes", "Beware: All nodes must be different!"},

	/* keys for the RotateDialog */
	{"dialogs_rotate_caption", "Rotate Selected Items"},
	{"dialogs_rotate_label", "Rotate angle:"},
	{"dialogs_rotate_error", "Enter a valid angle in DEGREES!"},
	{"dialogs_rotate_nodes_error", "You are not trying to rotate only nodes aren't you?"},

	/* keys for the ScaleDialog */
	{"dialogs_scale_caption", "Scale Selected Items"},
	{"dialogs_scale_x", "Factor in x direction:"},
	{"dialogs_scale_y", "Factor in y direction:"},
	{"dialogs_scale_x_error", "No valid scale factor in x direction"},
	{"dialogs_scale_y_error", "No valid scale factor in y direction"},
	{"dialogs_scale_nodes_error", "You are not trying to scale only nodes aren't you?"},
	{"dialogs_scale_circles_error", "Circles must only be scaled uniformly!"},

	/* keys for MirrorDialog */
	{"dialogs_mirror_caption", "Mirror Selected Items"},
	{"dialogs_mirror_start", "Start Coordinates of Mirror Line"},
	{"dialogs_mirror_end", "End Coordinates of Mirror Line"},
	{"dialogs_mirror_x1_error", "Something is wrong with the start coordinates.\nMake sure to use user coordinates."},
	{"dialogs_mirror_y1_error", "Something is wrong with the start coordinates.\nMake sure to use user coordinates."},
	{"dialogs_mirror_x2_error", "Something is wrong with the end coordinates.\nMake sure to use user coordinates."},
	{"dialogs_mirror_y2_error", "Something is wrong with the end coordinates.\nMake sure to use user coordinates."},

	/* keys for the ChangeBoundaryDialog */
	{"dialogs_changeb_caption", "Select working Boundary"},
	{"dialogs_changeb_label", "Working Boundary: "},
	{"dialogs_changeb_error_nobounds", "The domain does not contain any bondaries at all.\nSo where's the point in changing the working boundary?"},

	/* keys for the TEXTFRAME dialog */
	{"dialogs_textframe_tooltip", "Click to Close the Dialog"},
	{"dialogs_textframe_loaderror_title", "Text Error"},
	{"dialogs_textframe_loaderror_message", "error Parsing the designated text"},

	/* keys for the GRIDLISTENER */
	{"gridlistener_error_nonodehit", "No node has been hit. \n Remember: To add elements, you have to add nodes first"},
	{"gridlistener_error_radiuszero", "Circles with radius zero are not allowed!"},
	{"gridlistener_error_noboundary", "Beware: There is no Boundary to add Segments to.\n To add some, add a Boundary first."},
	{"gridlistener_error_boundarymove", "Boundary Nodes must not be moved around on the Grid. \nUse the dialog instead!"},
	{"gridlistener_error_nosegments", "Beware: There are no Segments to add BoundaryNodes to.\n To add some, add a Boundary and Segments first."},
	{"gridlistener_error_nodeorientation", "The nodes you selected are not correctly orientated.\nNo element has been added."},
	{"gridlistener_error_nodetwicehit", "Beware: You hit the same node twice!\nThis is very very illegal.\nNo element has been added."},
	{"gridlistener_error_samepointtwice", "Beware: A line is not a point! \nPlease don't hit the same point twice!"},
	{"gridlistener_error_clone", "Unrecoverable Error while pasting BoundaryNodes.\nSorry, fix nodes and elements manually."},

	/* keys for the ContBoundary */
	{"continousBoundary_startmessage", "continuousBoundary: starting ..."},
	{"continousBoundary_procboundary", "Processing Boundary "},
	{"continousBoundary_procsegment", "  Processing Segment "},
	{"continousBoundary_bonderror", "  Couldn't continousize boundary, try to swap"},
	{"continousBoundary_segfound", "  Corresponding segment found, swapping"},
	{"continousBoundary_error", "ERROR: No continous conjunction possible!"},
	{"continousBoundary_action1", "  Cont.Action: Adjust startpoint"},
	{"continousBoundary_action2", "  Cont.Action: Adjust and swap startpoint"},
	{"continousBoundary_ok", "SUCCESS: Boundaries adjusted!"},

	/* keys for the swapOrientation */
	{"swapOrientation_startmessage", "Swap orientation: starting ..."},
	{"swapOrientation_procsegment", "  Processing Segment "},
	{"swapOrientation_change1", "  Changing Segment "},
	{"swapOrientation_change2", " with "},
	{"swapOrientation_procnode", "Processing Node "},
	{"swapOrientation_ok", "SUCCESS: Working boundary orientation changed!"},

	/* keys for the mode display in the statusbar */
	{"modedisplay_mode", "Mode"},
	{"modedisplay_zooming", "Zooming"},
	{"modedisplay_drawing", "Drawing"},
	{"modedisplay_edgestatus", "Setting Edge Stati"},
	{"modedisplay_parallelblocks", "Setting Parallelblocks"},
	{"modedisplay_propertyquery", "Property Queries"},
	{"modedisplay_refine", "Refining"}, 
	{"modedisplay_rectify", "Rectifying"},
	{"modedisplay_mirror", "Mirroring"},  
	{"modedisplay_none", "no Mode"},
	{"modedisplay_adding", "Adding"},
	{"modedisplay_nodes", "Nodes"},
	{"modedisplay_lines", "Lines"},
	{"modedisplay_multilines", "Multilines"},
	{"modedisplay_circlep", "Circle+"},
	{"modedisplay_circlem", "Circle-"},
	{"modedisplay_clipboard", "Clipboard Contents"},
	{"modedisplay_tris", "Tris"},
	{"modedisplay_quads", "Quads"},
	{"modedisplay_macros", "Macros"},
	{"modedisplay_selecting", "Selecting"},
	{"modedisplay_edges", "Edges"},
	{"modedisplay_elements", "Elements"},
	{"modedisplay_segments", "Segments"},
	{"modedisplay_copymode_append", "Copymode: Append"},
	{"modedisplay_copymode_keep", "Copymode: Keep Structure"},

	/* keys for the spellchecker package */
	{"spellchecker_dialog_title", "SpellChecker"},
	{"spellchecker_dialog_settings_tabheader", "Settings"},
	{"spellchecker_dialog_settings_tabheader_tooltip", "Select which types of spelling errors are corrected."},
	{"spellchecker_dialog_table_tabheader", "List"},
	{"spellchecker_dialog_table_tabheader_tooltip", "View list of spelling errors."},
	{"spellchecker_dialog_closebutton_tooltip", "Close the SpellChecker dialog"},
	{"spellchecker_dialog_isolatednodes", "Isolated nodes"},
	{"spellchecker_dialog_innerangles", "Elements with inner angles less than "},
	{"spellchecker_dialog_aspectratios", "Elements with aspect ratios greater than "},
	{"spellchecker_dialog_neighbouring", "Neighbouring elements with size difference greater than "},
	{"spellchecker_dialog_discontinous", "Discontinous boundary parametrisation"},
	{"spellchecker_dialog_nonconforming", "Nonconforming elements"},
	{"spellchecker_dialog_zoom", "Zoom in"},
	{"spellchecker_dialog_zoom_tooltip", "This centers the view on the selected item."},
	{"spellchecker_dialog_zoom_mnemonic", new Integer (KeyEvent.VK_Z)},
	{"spellchecker_dialog_remove", "Remove"},
	{"spellchecker_dialog_remove_tooltip", "Remove the selected item from the list."},
	{"spellchecker_dialog_remove_mnemonic", new Integer (KeyEvent.VK_R)},
	{"spellchecker_dialog_refresh", "Refresh"},
	{"spellchecker_dialog_refresh_tooltip", "Refresh the list."},
	{"spellchecker_dialog_refresh_mnemonic", new Integer (KeyEvent.VK_F)},
	{"spellchecker_dialog_moreinfo", "More info ..."},
	{"spellchecker_dialog_moreinfo_tooltip", "Show detailed information about the selected item."},
	{"spellchecker_dialog_moreinfo_mnemonic", new Integer (KeyEvent.VK_I)},
	{"spellchecker_dialog_error_innerangles", "The threshold field for inner angles does not contain a valid angle (in degrees).\nInner angles will be omitted."},
	{"spellchecker_dialog_error_aspectratio", "The threshold fields for aspect ratios do not contain valid double values.\nAspect ratios will be omitted."},
	{"spellchecker_dialog_error_neighbouring", "The threshold field for size difference does not contain a valid double value.\nNeighbouring elements will be omitted."},
	{"spellchecker_dialog_error_noselection", "Nothing is selected in the list!\nSelect an entry first."},


	{"spellchecker_tablemodel_columnheader_type", "Type"},
	{"spellchecker_tablemodel_columnheader_number", "Number"},
	{"spellchecker_tablemodel_isolatednode", "Isolated Node"},
	{"spellchecker_tablemodel_innerangle", "Critical Inner Angle"},
	{"spellchecker_tablemodel_aspectratio", "Large Aspect Ratio"},
	{"spellchecker_tablemodel_neighbours", "Area Difference"},

	/* keys for the MergeBoundariesDialog */
	{"dialogs_mergeboundaries_caption", "Merge two boundaries"},
	{"dialogs_mergeboundaries_targetlabel", "Target boundary:"},
	{"dialogs_mergeboundaries_targettooltip", "Select the boundary you want to insert INTO"},
	{"dialogs_mergeboundaries_sourcelabel", "Source boundary"},
	{"dialogs_mergeboundaries_sourcetooltip","Select the boundary you want to dissolve."},
	{"dialogs_mergeboundaries_error_sameboundaries", "It is impossible to merge identical boundaries!"},

    };

    
}

