import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import java.util.*;
import java.io.*;

import devisor2.grid.GUI.framework.*;
import devisor2.grid.options.*;
import devisor2.grid.GUI.help.*;
import devisor2.foundation.elements.*;
import devisor2.foundation.boundary.*;
import devisor2.foundation.base.*;
import devisor2.grid.GUI.dialogs.*;
import devisor2.grid.backend.undo.*;

/**
 *  This is the startup program for the DeViSoRGriD 2 application.
 *  In order to work properly, before initializing any GUI, the options
 *  and locale are loaded and linked to a new ControlCenter object.
 *  From there, they can be accessed anytime at runtime. <br>
 *  <br>
 *  But before that, the saved options are loaded, according to the
 *  following search hierarchy (switching to the next step whenever loading 
 *  errors occur): <br>
 *  (1) if the user specifies the path to his devisor2settings file
 *      as first argument when executing this class, the given file is loaded
 *      and used to create the Options instance. <br>
 *      Example: <code>java devisorgrid myoptions.conf<\code><br>
 *  (2) the default options file <code>.devisorgrid<\code> or 
 *      <code> devisorgrid.conf<\code> is loaded.<br>
 *  (4) the build-in default values are used, using the current directory
 *      as DEVISORHOME. <br>
 *  <br>
 *  Thus, a valid Options instance is created, especially with a valid
 *  DEVISORHOME path. <br>
 *  <br>
 *  After that, the ControlCenter instance (which holds global variables)
 *  is created, thus instanciating the ResourceBundle. To stay conform with
 *  the java styleguide, which options instance to load depends on the
 *  language settings of your operating system. See there for details. <br>
 *  <br>
 *  Finally, the MainFrame instance is created and displayed, and
 *  the user can start to work.<br>
 *  <br>
 *  @see everything else, especially the java tutorial on localization support
 *       and the ControlCenter and MainFrame classes.
 *
 *  @author Dominik Goeddeke
 *  @version beta, search hierarchy not implemented
 */

public class devisorgrid
{

    private static Options options;


    public static void main (String[] args)
    {
	long start = System.currentTimeMillis ();
	System.out.println ("Starting the DeViSoRGriD2 application ...");
	System.out.println ("Version: "+InfoDialog.version_level1+"."+InfoDialog.version_level2+"."+InfoDialog.version_level3+" of "+InfoDialog.release_date);
	System.out.println ("Please stand by while loading Options ...");
	boolean optionssuccess = false;
	// options from command line
	if (args.length > 0)
	{
	    options = new Options ();
	    try
	    {
		options.load (args[0]);
		optionssuccess = true;
	    }
	    catch (Exception ex1)
	    {
		System.out.println ("Error while loading the given Options file!");
	    }

	}
	// users conf file
	if (!optionssuccess)
	{
	    String os = System.getProperty ("os.name").toUpperCase();
	    if (os.indexOf ("WIN") != -1)
	    {
		options = new Options ();
		try
		{
		    System.out.println (System.getProperty ("user.home")+System.getProperty ("file.separator")+"devisorgrid.conf");
		    options.load (System.getProperty ("user.home")+System.getProperty ("file.separator")+"devisorgrid.conf");
		    optionssuccess = true;
		}
		catch (Exception ex)
		{
		    System.out.println ("Could not load user specific options file!");
		}
	    }
	    else
	    {
		options = new Options ();
		try
		{
		    options.load (System.getProperty ("user.home")+System.getProperty ("file.separator")+".devisorgrid");
		    optionssuccess = true;
		}
		catch (Exception ex)
		{
		    System.out.println ("Using factory defaults. Please check paths in OptionsDialog!");
		}
	    }
	}
	// factory options
	if (!optionssuccess)
	{
	    options = new Options ();
	    options.resetToDefaults ();
	    options.set (Options.general_devisorhome, System.getProperty ("devisorgrid.home"));
	}
	System.out.println ("Options loaded in " + (System.currentTimeMillis()-start) + "ms.");
	// control center
	start = System.currentTimeMillis ();
	System.out.println ("Loading ControlCenter ...");
	start = System.currentTimeMillis ();
	ControlCenter cc = new ControlCenter (options);
	System.out.println ("ControlCenter loaded in " + (System.currentTimeMillis()-start) + "ms.");
	// help frame
	System.out.println ("Loading the help system ...");
	try
	{
	    cc.help = new HelpFrame (cc.op.get(Options.general_helpfile));
	}
	catch (IOException ex)
	{
	    System.out.println ("Could not load help file!");
	    System.out.print ("Make sure the following file exists: ");
	    System.out.println (cc.op.get(Options.general_helpfile));
	}
	try
	{
	    cc.faq = new FAQFrame (cc.op.get(Options.general_faqfile));
	}
	catch (IOException ex)
	{
	    System.out.println ("Could not load faq file!");
	    System.out.print ("Make sure the following file exists: ");
	    System.out.println (cc.op.get(Options.general_faqfile));
	}
	System.out.println ("Help loaded in " + (System.currentTimeMillis()-start) + "ms.");
	System.out.println ("Creating the GUI and starting up the MainFrame ...");
	start = System.currentTimeMillis ();
	MainFrame frame = new MainFrame ();
	cc.gc.setMainFrame(frame,cc);
	frame.pack ();
	// last but not least, update DrawControl
	cc.dc.screen_width = frame.getToolkit().getScreenSize().getSize().width;
	cc.dc.screen_height = frame.getToolkit().getScreenSize().getSize().height;
	// ok thats it
	cc.dm = new DialogManager (frame);
	cc.um = new UndoManager (frame);
	frame.getMainActionListener().restoreStartUpMode ();
	cc.gc.setMainFrame(frame,cc);

        frame.getDrawingArea().redraw_enabled=true;

	cc.dc.world_x1=-1*100000;
	cc.dc.world_y1=-1*100000;
	cc.dc.world_x2= 1*100000;
	cc.dc.world_y2= 1*100000;
	
	cc.dc.act_x1=cc.dc.world_x1;
	cc.dc.act_x2=cc.dc.world_x2;
	cc.dc.act_y1=cc.dc.world_y1;
	cc.dc.act_y2=cc.dc.world_y2;
	
	cc.gc.setTransformation();
	
	cc.d.centerDomain();

	frame.show ();

	cc.dm.initDialogs ();
	cc.gc.gtk.updateBoundaryCombos();
	cc.dc.tolerance = cc.gc.SingleFromScreenToWorld(cc.dc.tolerance);
	System.out.println ("MainFrame is up after " + (System.currentTimeMillis()-start) + "ms.");


        ReadmeFrame.initialize(cc.gc.mf, "README", "waiting for godot...");

        int revlevel = 2;
	String revfile = System.getProperty("user.home")+System.getProperty("file.separator")+".dgm";
	
	int cf=DomainIO.checkFile(revfile);
	if (cf==0)
	{	   

          ReadmeFrame.showDialog ();      
      
	  try
	  {
    	    FileOutputStream writeFile = new FileOutputStream(revfile);
    	    DataOutputStream out = new DataOutputStream( writeFile );
	    out.writeBytes(revlevel+"");	  
	    out.close();
	  }
	  catch (IOException ex)
	  {
	  }
	}
	else
	{
    	  try
    	  {
    	    BufferedReader in;
    	    StreamTokenizer eingabe;
	    
            in = new BufferedReader(new FileReader(revfile));
		
            eingabe = new StreamTokenizer(in);
		
            eingabe.ordinaryChar('/');
            eingabe.slashSlashComments(true);
            eingabe.slashStarComments(true);
            eingabe.parseNumbers();
		
            int erg=eingabe.nextToken();

	    int number=(int)eingabe.nval; 
	    	    
	    in.close();
	    
	    if (number<revlevel)
	    {
              ReadmeFrame.showDialog ();      
    	      FileOutputStream writeFile = new FileOutputStream(revfile);
    	      DataOutputStream out = new DataOutputStream( writeFile );
	      out.writeBytes(revlevel+"");	  
	      out.close();
	    }
	  }
	  catch (IOException ex)
	  {
	  }	  
	}
    }

}


